      subroutine angles

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  angles   Updated     2004 November 23 13:30.
c               angles   Originated  1993 November 2 10:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To specify angle units, and to convert one or more values from
c               one angle unit to another.
c
c     Logic:    Entry points:  angles.
c angles        Called by:  helpall, main.
c
c angles        Calls:  angout, compsubs, varfind, varout, angles.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Initialize.

 8100 format ('angles ',a)
 8140 format ('coord  ',a,5x,a,4x,a,4x,a)

      if (nword .eq. 0) then
        angunit = 'degrees'
        go to 999
      endif

c===============================================================================

c.... Input line:  angles

      if (nword .eq. 1) then
        write (   6, 8100) angunit
        write (iout, 8100) angunit
        write (   6, 8140) asys, acoordu, acoordv, acoordw
        write (iout, 8140) asys, acoordu, acoordv, acoordw
      endif

c===============================================================================

c.... Input line:  angles [help]

 8110 format ('  Command "angles":  acceptable input lines include:' /
     &  '  "angles [help]"           Display current problem units'
     &  ' and this message.' /
     &  '  "angles degrees"          Change angle units to degrees.' /
     &  '  "angles radians"          Change angle units to radians.' //
     &  '  "angles UNIT1 UNIT2 ANGLE1 [ANGLE2 ANGLE3 ...]"' /
     &  '     Convert from UNIT1 to UNIT2:  ANGLE1',
     &  ' [and ANGLE2, ANGLE3, ...].' /
     &  '     UNIT1 may be "degrees", "grads" or "radians".' /
     &  '     UNIT2 may be "degrees", "grads", "radians" or "dms".' //
     &  '  "angles UNIT1 UNIT2 ANGLE1 thru ANGLEN"' /
     &  '     Convert from UNIT1 to UNIT2:  all stored',
     &  ' variables' /
     &  '     with names in the range from ANGLE1 thru ANGLEN.' /
     &  '     UNIT1 may be "degrees", "grads" or "radians".' /
     &  '     UNIT2 may be "degrees", "grads", "radians" or "dms".' //
     &  '  "angles dms UNIT2 DEGREES MINUTES SECONDS"' /
     &  '     Convert from degrees, minutes, seconds to UNIT2.' /
     &  '     UNIT2 may be "degrees", "grads", "radians" or "dms".' //
     &  '  Synonyms:  [angles, angle, ang], [degrees, degree, deg],' /
     &  '   [grads, grad], [help, h], [radians, radian, rad].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c===============================================================================

c.... Input line:  angles [degrees,radians]

      if (nword .eq. 2) then          ! Specify angle unit.

c.... Assign angle units.

 8130 format ('  angles ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Second word not "degrees" or "radians":  ',a)

      if ((aword(2) .eq. 'degrees') .or.
     &    (aword(2) .eq. 'degree')  .or.
     &    (aword(2) .eq. 'deg'))    then
        angunit = 'degrees'
        if     (asys .eq. 'cyl') then
          acoordv = 'theta (deg)'
        elseif (asys .eq. 'sph') then
          acoordv = 'theta (deg)'
          acoordw = 'phi (deg)'
        endif
      elseif ((aword(2) .eq. 'radians') .or.
     &        (aword(2) .eq. 'radian')  .or.
     &        (aword(2) .eq. 'rad'))    then
        angunit = 'radians'
        if     (asys .eq. 'cyl') then
          acoordv = 'theta (rad)'
        elseif (asys .eq. 'sph') then
          acoordv = 'theta (rad)'
          acoordw = 'phi (rad)'
        endif
      else
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
        go to 999
      endif

c.... Inform user of angle units.

      write (   6, 8100) angunit
      write (iout, 8100) angunit
      write (   6, 8140) asys, acoordu, acoordv, acoordw
      write (iout, 8140) asys, acoordu, acoordv, acoordw

      go to 999

      endif                           ! Input line "angles [degrees, radians]".

c===============================================================================

c.... Test number of words in input line.

      nerr = 0

 8125 format ('  angles ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Wrong number of words on input line:  ',i3,'.')

      if (nword .eq. 3) then
        write (   6, 8125) ainput, ninput, nword
        write (iout, 8125) ainput, ninput, nword
        nerr = 1
      endif

c.... Test unit specifications.

 8120 format ('  angles ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Do not recognize unit ',a )
 8128 format ('  angles ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Units are the same:  ',a )

      if ((aword(2) .ne. 'degrees')  .and.
     &    (aword(2) .ne. 'deg'    )  .and.
     &    (aword(2) .ne. 'grads'  )  .and.
     &    (aword(2) .ne. 'grad'   )  .and.
     &    (aword(2) .ne. 'radians')  .and.
     &    (aword(2) .ne. 'rad'    )  .and.
     &    (aword(2) .ne. 'dms'    )) then
        write (   6, 8120) ainput, ninput, aword(2)
        write (iout, 8120) ainput, ninput, aword(2)
        nerr = 1
      endif

      if ((aword(3) .ne. 'degrees')  .and.
     &    (aword(3) .ne. 'deg'    )  .and.
     &    (aword(3) .ne. 'grads'  )  .and.
     &    (aword(3) .ne. 'grad'   )  .and.
     &    (aword(3) .ne. 'radians')  .and.
     &    (aword(3) .ne. 'rad'    )  .and.
     &    (aword(3) .ne. 'dms'    )) then
        write (   6, 8120) ainput, ninput, aword(3)
        write (iout, 8120) ainput, ninput, aword(3)
        nerr = 1
      endif

      if (aword(2) .eq. aword(3)) then
        write (   6, 8128) ainput, ninput, aword(3)
        write (iout, 8128) ainput, ninput, aword(3)
      endif

      if (nerr .ne. 0) go to 999

c===============================================================================

c.... Input line:  angles UNIT1 UNIT2 ANGLE1

      if ((nword .eq. 4) .and. (aword(2) .ne. 'dms')) then

        call varfind (4, 4)

 8135   format ('  angles ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word not integer or floating point value or variable:  ',a)

        if ((mtype(4) .ne. 2) .and. (mtype(4) .ne. 3)) then
          write (   6, 8135) ainput, ninput, aword(4)
          write (iout, 8135) ainput, ninput, aword(4)
          go to 999
        endif
cbugc***DEBUG begins.
cbug 9974 format ('DEBUG:  convert 1 angle.')
cbug      write (iout, 9974)
cbug      write (   6, 9974)
cbugc***DEBUG ends.
        call angout (aword(2), aword(3), fword(4), 0., 0.)
        go to 999
      endif                           ! Convert one value.

c===============================================================================

c.... Input line:  angles UNIT1 UNIT2 ANGLE1 thru ANGLEN

 8136 format ('  angles ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of object not from 1 to 24 characters:  ',a)
 8138 format ('  angles ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Min name exceeds max name:  ' / a / a)
 8139 format ('  angles ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word "thru" is duplicated.')

      if ((nword .eq. 6)          .and.
     &    (aword(2) .ne. 'dms')   .and.
     &    (aword(5) .eq. 'thru')) then

        if ((lword(4) .lt. 1) .or. (lword(4) .gt. 24)) then
          write (   6, 8136) ainput, ninput, aword(3)
          write (iout, 8136) ainput, ninput, aword(3)
          go to 999
        endif

        if ((lword(6) .lt. 1) .or. (lword(6) .gt. 24)) then
          write (   6, 8136) ainput, ninput, aword(5)
          write (iout, 8136) ainput, ninput, aword(5)
          go to 999
        endif

        call compsubs (aword(4), aword(6), nsmall, nerrc)
        if (nsmall .eq. 2) then
          write (   6, 8138) ainput, ninput, aword(3), aword(5)
          write (iout, 8138) ainput, ninput, aword(3), aword(5)
          go to 999
        endif

        if ((aword(4) .eq. 'thru')  .or.
     &      (aword(6) .eq. 'thru')) then
          write (   6, 8139) ainput, ninput
          write (iout, 8139) ainput, ninput
          go to 999
        endif

        do 132 nvar = 1, nvars
          if (avarbl(nvar) .eq. ' ') go to 132
          call compsubs (avarbl(nvar), aword(4), nsmall, nerrc)
          if (nsmall .eq. 1) go to 132
          call compsubs (avarbl(nvar), aword(6), nsmall, nerrc)
          if (nsmall .eq. 2) go to 132
cbugc***DEBUG begins.
cbug 9975 format ('DEBUG:  convert angles V1 thru VN.')
cbug      write (iout, 9975)
cbug      write (   6, 9975)
cbugc***DEBUG ends.
         call varout (nvar, nvar)
         call angout (aword(2), aword(3), fvar(nvar), 0., 0.)
  132  continue

       go to 999

      elseif (aword(2) .ne. 'dms') then

c....   Input line:  angles UNIT1 UNIT2 ANGLE1 [ANGLE2 ANGLE3 ...]

        call varfind (4, nword)

        do 140 nw = 4, nword
          if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
            write (   6, 8135) ainput, ninput, aword(nw)
            write (iout, 8135) ainput, ninput, aword(nw)
            go to 140
          endif
cbugc***DEBUG begins.
cbug 9976 format ('DEBUG:  convert >1 angle.')
cbug      write (iout, 9976)
cbug      write (   6, 9976)
cbugc***DEBUG ends.
        call angout (aword(2), aword(3), fword(nw), 0., 0.)
  140   continue
        go to 999

      endif                           ! Convert more than one value.

c===============================================================================

c.... Input line:  angles dms UNIT2 DEGREES MINUTES SECONDS

      if ((nword .eq. 6) .and. (aword(2) .eq. 'dms')) then

        call varfind (4, 6)

        do 220 nw = 4, 6
          if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
            write (   6, 8135) ainput, ninput, aword(nw)
            write (iout, 8135) ainput, ninput, aword(nw)
            go to 999
          endif
  220   continue
cbugc***DEBUG begins.
cbug 9977 format ('DEBUG:  convert dms.')
cbug      write (iout, 9977)
cbug      write (   6, 9977)
cbugc***DEBUG ends.
        call angout (aword(2), aword(3), fword(4), fword(5), fword(6))
        go to 999
      endif                           ! Converting from degrees, minutes, sec.

c===============================================================================

c.... Input line not recognized.

 8900 format ('  angles ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8900) ainput, ninput
      write (iout, 8900) ainput, ninput

c===============================================================================

  999 return

c.... End of subroutine angles.        (+1 line)
      end
