      subroutine axisym

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  axisym   Updated     2005 April 5 17:45.
c               axisym   Originated  1993 December 16 8:30.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display and create axisymmetric quadric surfaces, which may
c                 be planes, coincident planes, two parallel planes, spheres,
c                 circular ellipsoids, circular cylinders, circular cones,
c                 or circular hyperboloids of one or two sheets.
c
c     Logic:    Entry points: axisym.
c axisym        Called by:  helpall, main.
c
c axisym        Calls:  aptmopv, aptrois, aptrotv, aptsubr, apttran, apttris,
c               aptvdis, incrsubs, namechk, ptfind, qumove, quoper, quout,
c               vectfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      common /laxis/ rotm(3,3)        ! Principle axis rotation tensor.

      character*8 amount

c.... Input line:  axisym [help]

 8110 format (
     &  '  Command "axisym":  acceptable input lines include:' /
     &  '  "axisym [help]"  Display this message.' /
     &  '  "axisym all"     Display all axisymmetric quadrics.' /
     &  '  "axisym list [AXNAME1 AXNAME2 AXNAME3 ...]"' /
     &  '     Display all listed axisymmetric quadrics' /
     &  '     (default is all axisymmetric quadrics).' /
     &  '  "axisym AXNAME"   Display axisymmetric quadric surface',
     &  ' AXNAME.' //
     &  '  "axisym AXNAME fit PCEN VAXIS PNAME2 PNAME3"' /
     &  '     Create axisymmetric quadric AXNAME,',
     &  ' centered at point PCEN,' /
     &  '     with axis vector VAXIS, and through points PNAME2 and',
     &  ' PNAME3.' //
     &  '  "axisym AXNAME focus PFOC VAXIS PNAME2 PNAME3"' /
     &  '     Create axisymmetric quadric AXNAME,',
     &  ' with focus at point PFOC,' /
     &  '     with axis vector VAXIS, and through points PNAME2 and',
     &  ' PNAME3.' )

 8112 format (/
     &  '  "axisym AXQ02 move NUMAX INC AXQ01 VMOVE"' /
     &  '     Create a family of NUMAX axisymmetric quadrics AXQ02,',
     &  ' ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     and spaced at intervals of vector VMOVE,' /
     &  '     starting from axisymmetric quadric AXQ01.' //
     &  '  "axisym AXQ02 rotate NUMAX INC AXQ01 OPNAME PINV"' /
     &  '     Create a family of NUMAX axisymmetric quadrics AXQ02,',
     &  ' ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     by rotating the preceding axisymmetric quadric with',
     &  ' operator OPNAME' /
     &  '     and invariant point PINV, starting from axisymmetric',
     &  ' quadric AXQ01.' )

 8114 format (/
     &  '  "axisym AXQ02 scale NUMAX INC AXQ01 OPNAME PINV"' /
     &  '     Create a family of NUMAX quadrics AXQ02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     by scaling the preceding quadric with operator OPNAME' /
     &  '     and invariant point PINV, starting from axisymmetric',
     &  ' quadric AXQ01.' /
     &  '     WARNING:  a linear or radial scaling axis not parallel' /
     &  '     to that of AXQ01 will produce non-axisymmetric quadric',
     &  ' surfaces;' /
     &  '     a linear or radial scaling will changes spheres to',
     &  ' ellipsoids.' //
     &  '  Note:  use +/- for AXNAME to incr/decr the last' /
     &  '    axisymmetric quadric surface name:  ',a //
     &  '  Synonyms:  [focus, foc, focal], [help, h],' /
     &  '  [move, mv, translate, trans],',
     &  '  [rotate, rot], [scale, sc].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        write (   6, 8114) axisyms
        write (iout, 8114) axisyms
        go to 999
      endif

c=======================================================================********

c.... Input line:  axisym all
c.... Input line:  axisym list

 8120 format (/ 'Number of axisymmetric quadrics = ',i5)

      if ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'all')  .or.
     &    (aword(2) .eq. 'list'))) then
        naxisymx = 0
        do 120 n = 1, nquads
          if (asym(n) .ne. 'axisymmetric') go to 120
          if (aquad(n) .eq. ' ') go to 120
          call quout (n, n, 'min', nerrq)
          naxisymx = naxisymx + 1
  120   continue
        write (   6, 8120) naxisymx
        write (iout, 8120) naxisymx
        go to 999
      endif

c=======================================================================********

c.... Input line:  axisym list AXNAME1 AXNAME2 AXNAME3 ...

 8130 format ('  axisym ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named axisymmetric quadric surface does not exist:  ',a)

      if (aword(2) .eq. 'list') then
        do 128 nw = 3, nword
          nfind = 0
          do 125 n = 1, nquads
            if (asym(n) .ne. 'axisymmetric') go to 125
            call aptsubr (aquad(n), 1, 24, 80, astem, lstem,
     &                    asub, lsub, nerra)
            if ((aword(nw) .eq. aquad(n)) .or.
     &          (aword(nw) .eq. astem))    then
              nfind = nfind + 1
              call quout (n, n, 'min', nerrq)
            endif
  125     continue
          if (nfind .eq. 0) then
            write (   6, 8130) ainput, ninput, aword(nw)
            write (iout, 8130) ainput, ninput, aword(nw)
          endif
  128   continue
        go to 999
      endif

c=======================================================================********

c.... Input line:  axisym AXNAME

      if (nword .eq. 2) then
        do 130 n = 1, nquads
          if (asym(n) .ne. 'axisymmetric') go to 130
          if (aword(2) .eq. aquad(n)) then
            amount = 'maximum'
            call quout (n, n, amount, nerrq)
            go to 999
          endif
  130   continue
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
        go to 999
      endif

c=======================================================================********

c.... Test input for validity.

 8136 format ('  axisym ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of axisymmetric quadric not from 1 to 24 characters:  ',
     &  a)

      if ((lword(2) .lt. 1) .or. (lword(2) .gt. 24)) then
        write (   6, 8136) ainput, ninput, aword(2)
        write (iout, 8136) ainput, ninput, aword(2)
        go to 999
      endif

c=======================================================================********

c.... See if last axisymmetric surface name should be incremented and used.

 8128 format ('  axisym ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last axisymmetric surface can not be',
     &  ' incremented:  ',a)

      if (aword(2) .eq. '+') then
        aword(2) = axisyms
        lword(2) = laxisyms
        call incrsubs (1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8128) ainput, ninput, axisyms
          write (iout, 8128) ainput, ninput, axisyms
          go to 999
        endif
      endif

 8129 format ('  axisym ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last axisymmetric surface can not be',
     &  ' decremented:  ',a)

      if (aword(2) .eq. '-') then
        aword(2) = axisyms
        lword(2) = laxisyms
        call incrsubs (-1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8129) ainput, ninput, axisyms
          write (iout, 8129) ainput, ninput, axisyms
          go to 999
        endif
      endif

 8905 format ('  axisym ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of axisymmetric surface too dangerous to use:  ',a)

      call namechk (aword(2), lword(2), nerrchk)

      if (nerrchk .ne. 0) then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Input line:  axisym AXQ02 move   NUMAX INC AXQ01 VMOVE

      if ((aword(3) .eq. 'move')      .or.
     &    (aword(3) .eq. 'mv')        .or.
     &    (aword(3) .eq. 'translate') .or.
     &    (aword(3) .eq. 'trans'))    then

        call qumove

        go to 999

      endif                           ! Created translated axisym(s).

c===============================================================================

c.... Input line:  axisym AXQ02 rotate NUMAX INC AXQ01 OPNAME PINV
c.... Input line:  axisym AXQ02 scale  NUMAX INC AXQ01 OPNAME PINV

      if ((aword(3) .eq. 'scale')   .or.
     &    (aword(3) .eq. 'sc')      .or.
     &    (aword(3) .eq. 'rotate')  .or.
     &    (aword(3) .eq. 'rot'))    then

        call quoper

        go to 999

      endif                           ! Created rotated or scaled axisym(s).

c===============================================================================

c.... See if a quadric with the same name exists.

 8160 format ('  WARNING:  replacing existing quadric:  ',a)
 8162 format ('  axisym WARNING.  Reading ',a40,' line ',i5,'.' /
     &  '  Quadric being replaced is not axisymmetric.')

      nrep = 0
      do 136 n = 1, nquads
        if (aword(2) .eq. aquad(n)) then
          nrep = 1
          write (   6, 8160) aquad(n)
          write (iout, 8160) aquad(n)
          if (asym(n) .ne. 'axisymmetric') then
            write (   6, 8162) ainput, ninput
            write (iout, 8162) ainput, ninput
          endif
          call quout (n, n, 'min', nerrq)
          go to 138
        endif
  136 continue

      do 137 n = 1, nquads
        if (aquad(n) .eq. ' ') go to 138
  137 continue

      nquads = nquads + 1

 8165 format ('  axisym ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of quadrics exceeds limit ',i5)

      if (nquads .gt. nquadm) then
        nquads = nquadm
        write (   6, 8165) ainput, ninput, nquadm
        write (iout, 8165) ainput, ninput, nquadm
        go to 999
      endif

      n = nquads

  138 nerr = 0

c=======================================================================********

c.... Input line:  axisym AXNAME fit PCEN VAXIS PNAME2 PNAME3

      if (aword(3) .eq. 'fit') then

c.... Test for correct number of words on input line.

 8670 format ('  axisym ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 7 words on "axisym" input line:  ',i4)

      if (nword .ne. 7) then
        write (   6, 8670) ainput, ninput, nword
        write (iout, 8670) ainput, ninput, nword
        nerr = 1
      endif

c.... Find the point at the center of the axisymmetric quadric.

      call ptfind (4, np)

      if (np .eq. 0) then
        nerr = 1
        go to 690
      endif

      xpcen = x(np)
      ypcen = y(np)
      zpcen = z(np)

c.... Find the axis of the axisymmetric quadric.

  690 call vectfind (5, nv)

      if (nv .eq. 0) nerr = 1

c.... See if the axis vector has a real length.

 8750 format ('  axisym ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Axis vector has zero length:  ',a)

      if (vlen(nv) .eq. 0.0) then
        nerr = 1
        write (   6, 8750) ainput, ninput, aword(5)
        write (iout, 8750) ainput, ninput, aword(5)
      endif

c.... Find the two points on the axisymmetric quadric.

      call ptfind (6, np)

      if (np .eq. 0) then
        nerr = 1
        go to 770
      endif

      x1 = x(np)
      y1 = y(np)
      z1 = z(np)

  770 call ptfind (7, np)

      if (np .eq. 0) then
        nerr = 1
        go to 790
      endif

      x2 = x(np)
      y2 = y(np)
      z2 = z(np)

c.... See if the two surface points coincide.

 8790 format ('  axisym ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Points coincide.')

  790 call aptvdis (x1, y1, z1, x2, y2, z2, 1, tol, dx12, dy12, dz12,
     &              d12, nerra)

      if (d12 .eq. 0.0) then
        nerr = 1
        write (   6, 8790) ainput, ninput
        write (iout, 8790) ainput, ninput
      endif

      if (nerr .ne. 0) go to 830

c.... Find the operator to rotate the z axis to the axisymmetric quadric axis.

      call aptrotv (0., 0., 1., xcom(nv), ycom(nv), zcom(nv), tol,
     &              rotm, nerra)

c.... Rotate the points on the quadric with the inverse of the operator.

      call aptmopv (rotm, 1, xpcen, ypcen, zpcen, x1, y1, z1, 1, tol,
     &              nerra)

      call aptmopv (rotm, 1, xpcen, ypcen, zpcen, x2, y2, z2, 1, tol,
     &              nerra)

c.... Move the points to place the center of the quadric at the origin.

      call apttran (xpcen, ypcen, zpcen, x1, y1, z1, 1, tol, nerra)

      call apttran (xpcen, ypcen, zpcen, x2, y2, z2, 1, tol, nerra)

      rsq1 = x1**2 + y1**2
      rsq2 = x2**2 + y2**2

c.... See if the points are at different absolute z values.

 8820 format ('  axisym ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Type of quadric surface is indeterminate.')

      if ((z1**2 .eq. z2**2)  .and.
     &    (rsq2 .eq. rsq1))   then
        nerr = 1
        write (   6, 8820) ainput, ninput
        write (iout, 8820) ainput, ninput
      endif

  830 if (nerr .ne. 0) then
        if (nrep .eq. 0) then
          aquad(n) = ' '
          if (n .eq. nquads) nquads = nquads - 1
        endif
        go to 999
      endif

c.... Find the standard form of the equation of the axisymmetric quadric.

      aquads    = aword(2)
      laquads   = lword(2)
      axisyms   = aword(2)
      laxisyms  = lword(2)

      aquad(n)  = aword(2)
      laquad(n) = lword(2)
      qx(n)     =  0.0
      qy(n)     =  0.0
      qz(n)     =  0.0
      qxy(n)    =  0.0
      qyz(n)    =  0.0
      qzx(n)    =  0.0

      if (z2**2 .eq. z1**2) then         ! Two parallel planes.
        qc(n)  = -z1**2
        qxx(n) =  0.0
        qyy(n) =  0.0
        qzz(n) =  1.0
      else
        ratrz  = (rsq1 - rsq2) / (z2**2 - z1**2)
        qc(n)  =  -rsq1 - ratrz * z1**2
        qxx(n) =  1.0
        qyy(n) =  1.0
        qzz(n) =  ratrz
      endif                              ! Tested quadric surface type.

c.... Rotate the axisymmetric quadric to the specified axis.

      call aptrois (rotm, 0, 0., 0., 0., qc(n), qx(n), qy(n), qz(n),
     &              qxy(n), qyz(n), qzx(n), qxx(n), qyy(n), qzz(n),
     &              tol, nerra)

c.... Translate the center of the axisymmetric quadric to the specified point.

      call apttris (1, xpcen, ypcen, zpcen, qc(n),
     &              qx(n), qy(n), qz(n), qxy(n), qyz(n), qzx(n),
     &              qxx(n), qyy(n), qzz(n), tol, nerra)

      amount = 'maximum'
      call quout (n, n, amount, nerrq)

      go to 999

      endif                           ! Input line "axisym AXNAME fit ...".

c=======================================================================********

c.... Input line:  axisym AXNAME focus PFOC VAXIS PNAME2 PNAME3

      if ((aword(3) .eq. 'focus') .or.
     &    (aword(3) .eq. 'focal') .or.
     &    (aword(3) .eq. 'foc'))  then

c.... Test for correct number of words on input line.

      if (nword .ne. 7) then
        write (   6, 8670) ainput, ninput, nword
        write (iout, 8670) ainput, ninput, nword
        nerr = 1
      endif

c.... Find the point at the focus of the axisymmetric quadric.

      call ptfind (4, np)

      if (np .eq. 0) then
        nerr = 1
        go to 220
      endif

      xpfoc = x(np)
      ypfoc = y(np)
      zpfoc = z(np)

c.... Find the axis of the axisymmetric quadric.

  220 call vectfind (5, nv)

      if (nv .eq. 0) nerr = 1

c.... See if the axis vector has a real length.

      if (vlen(nv) .eq. 0.0) then
        write (   6, 8750) ainput, ninput, aword(5)
        write (iout, 8750) ainput, ninput, aword(5)
        nerr = 1
      endif

c.... Find the two points on the axisymmetric quadric.

      call ptfind (6, np)

      if (np .eq. 0) then
        nerr = 1
        go to 230
      endif

      x1 = x(np)
      y1 = y(np)
      z1 = z(np)

  230 call ptfind (7, np)

      if (np .eq. 0) then
        nerr = 1
        go to 240
      endif

      x2 = x(np)
      y2 = y(np)
      z2 = z(np)

c.... See if the two surface points coincide.

  240 call aptvdis (x1, y1, z1, x2, y2, z2, 1, tol, dx12, dy12, dz12,
     &              d12, nerra)

      if (d12 .eq. 0.0) then
        write (   6, 8790) ainput, ninput
        write (iout, 8790) ainput, ninput
        nerr = 1
      endif

      if (nerr .ne. 0) go to 250

c.... Find the operator to rotate the z axis to the axisymmetric quadric axis.

      call aptrotv (0., 0., 1., xcom(nv), ycom(nv), zcom(nv), tol,
     &              rotm, nerra)

c.... Rotate the points on the quadric with the inverse of the operator.

      call aptmopv (rotm, 1, xpfoc, ypfoc, zpfoc, x1, y1, z1, 1, tol,
     &              nerra)

      call aptmopv (rotm, 1, xpfoc, ypfoc, zpfoc, x2, y2, z2, 1, tol,
     &              nerra)

c.... Move the points to place the focus of the quadric at the origin.

      call apttran (xpfoc, ypfoc, zpfoc, x1, y1, z1, 1, tol, nerra)

      call apttran (xpfoc, ypfoc, zpfoc, x2, y2, z2, 1, tol, nerra)

c.... Find the spherical radii and zenith angles of the two points.

      rsph1 = sqrt (x1**2 + y1**2 + z1**2)
      rsph2 = sqrt (x2**2 + y2**2 + z2**2)

      if (rsph1 .eq. 0.0) then
        cosph1 = 0.0
      else
        cosph1 = z1 / rsph1
      endif

      if (rsph2 .eq. 0.0) then
        cosph2 = 0.0
      else
        cosph2 = z2 / rsph2
      endif

 8250 format ('  axisym ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Fit is impossible.')

      denom = rsph1 * z2 - rsph2 * z1

      if ((z2 - z1) * denom .lt. 0.0) then
        write (   6, 8250) ainput, ninput
        write (iout, 8250) ainput, ninput
        nerr = 1
      endif

  250 if (nerr .ne. 0) then
        if (nrep .eq. 0) then
          aquad(n) = ' '
          if (n .eq. nquads) nquads = nquads - 1
        endif
        go to 999
      endif

c.... Find the standard form of the equation of the axisymmetric quadric.

      aquads    = aword(2)
      laquads   = lword(2)
      axisyms   = aword(2)
      laxisyms  = lword(2)

      aquad(n)  = aword(2)
      laquad(n) = lword(2)
      qx(n)     =  0.0
      qy(n)     =  0.0
      qz(n)     =  0.0
      qxy(n)    =  0.0
      qyz(n)    =  0.0
      qzx(n)    =  0.0

      if (cosph1 .eq. cosph2) then    ! Circular cone.

        qc(n)  =  0.0
        qxx(n) =  cosph1**2
        qyy(n) =  cosph1**2
        qzz(n) =  -(1.0 - cosph1**2)
        qzzerr =  tol * (1.0 + cosph1**2)
        if (abs (qzz(n)) .le. qzzerr) qzz(n) = 0.0

      else                            ! Ellipsoid, paraboloid or hyperboloid.

        ratzr  =  (z2    - z1   ) / (rsph1 * z2 - rsph2 * z1)
        ratrr  =  (rsph1 - rsph2) / (rsph1 * z2 - rsph2 * z1)
        qc(n)  = -1.0
        qz(n)  =  2.0 * ratrr
        qxx(n) =  ratzr**2
        qyy(n) =  ratzr**2
        qzz(n) =  ratzr**2 - ratrr**2
        qzzerr =  tol * (ratzr**2 + ratrr**2)
        if (abs (qzz(n)) .le. qzzerr) qzz(n) = 0.0

      endif                              ! Tested quadric surface type.

c.... Rotate the axisymmetric quadric to the specified axis.

      call aptrois (rotm, 0, 0., 0., 0., qc(n), qx(n), qy(n), qz(n),
     &              qxy(n), qyz(n), qzx(n), qxx(n), qyy(n), qzz(n),
     &              tol, nerra)

c.... Translate the focus of the axisymmetric quadric to the specified point.

      call apttris (1, xpfoc, ypfoc, zpfoc, qc(n),
     &              qx(n), qy(n), qz(n), qxy(n), qyz(n), qzx(n),
     &              qxx(n), qyy(n), qzz(n), tol, nerra)

      amount = 'maximum'
      call quout (n, n, amount, nerrq)

      go to 999

      endif                           ! Input line "axisym AXNAME focus ...".

c=======================================================================********

 8880 format ('  axisym ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

  880 write (   6, 8880) ainput, ninput
      write (iout, 8880) ainput, ninput

c=======================================================================********

  999 return

c.... End of subroutine axisym.       (+1 line)
      end
