      subroutine bin

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  bin      Updated     2006 January 5 17:50.
c               bin      Originated  1993 November 2 10:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display and create probability bins.
c
c     Logic:    Entry point:  bin.
c bin           Called by:  helpall, main.
c
c bin           Calls:  aptsubr, binout, binstore, incrsubs, namechk, varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      character*8  amount

c.... Input line:  bin [help]

 8110 format ('  Command "bin":  acceptable input lines include:' /
     &  '  "bin [help]"       Display this message.' /
     &  '  "bin [all,list]"   Display all probability bins.' /
     &  '  "bin list BINAME1 BINAME2 BINAME3 ..."',
     &  ' Display all listed bins.' /
     &  '  "bin BINAME"       Display probability bin BINAME.' /
     &  '  "bin BINAME event [PRTOT|1]"  Create bin BINAME,' /
     &  '     for a discrete event with total probability PRTOT.' /
     &  '  "bin BINAME value VRAN [PRTOT|1]"  Create bin BINAME,' /
     &  '     for a discrete value VRAN with total probability PRTOT.' /
     &  '  "bin BINAME uniform VRANL VRANR [DPR|1]"' /
     &  '     Create bin BINAME, with uniform differential',
     &  ' probability DPR' /
     &  '     of values VRANL to VRANR > VRANL.' )

 8112 format (
     &  '  "bin BINAME linear VRANL VRANR',
     &  ' [DPRL|1] [DPRR|DPRL]"' /
     &  '     Create bin BINAME, with differential probabilities in',
     &  ' a linear distribution' /
     &  '     from DPRL for value VRANL, to DPRR for value',
     &  ' VRANR > VRANL.' /
     &  '  "bin BINAME power VRANL VRANR POW [PRTOT|1]"' /
     &  '     Create bin BINAME, with total probability PRTOT,',
     &  ' differential probabilities' /
     &  '     in a power-law distribution with power POW:' /
     &  '     POW = log (DPRR / DPRL) / log (VRANR / VRANL),' /
     &  '     PRTOT = (VRANR * DPRR - VRANL *',
     &  ' DPRL) / (POW + 1),' /
     &  '     or if POW = -1:  PRTOT = DPRL * VRANL *',
     &  ' log (DPRL / DPRR).' )

 8114 format (
     &  '  "bin BINAME exp VRANL VRANR SIGMA [PRTOT|1]" ' /
     &  '     Create bin BINAME, with total probability PRTOT,',
     &  ' differential probabilities' /
     &  '     in an exponential distribution with decay constant',
     &  ' SIGMA:' /
     &  '     SIGMA = log (DPRL / DPRR) / (VRANR - VRANL),' /
     &  '     PRTOT = (DPRL - DPRR) / SIGMA.' /
     &  '  "bin BINAME normal VMEAN DEV [PRTOT|1]"' /
     &  '     Create bin BINAME, with total probability PRTOT,',
     &  ' differential probabilities' /
     &  '     in a normal distribution with mean value VMEAN and',
     &  ' standard deviation DEV,' /
     &  '     for values from minus infinity to plus infinity.' )

 8116 format (
     &  '  "bin BINAME maxwell RATIO [PRTOT|1]"' /
     &  '     Create bin BINAME, with total probability PRTOT,',
     &  ' differential probabilities' /
     &  '     of beta = v / c in a  relativistic Maxwellian',
     &  ' distribution, for a ratio' /
     &  '     RATIO of avg temp to rest mass energy, for',
     &  ' 0 < beta < 1.' /
     &  '  "bin BINAME [planck, wien] BTEMP [PRTOT|1]"' /
     &  '     Create bin BINAME, with total probability PRTOT,',
     &  ' differential probabilities' /
     &  '     of frequency XNU in a [Planck, Wien] spectrum, for a',
     &  ' black-body temperature' /
     &  '     BTEMP, for values of XNU from 0 to infinity.' )
 8118 format (/
     &  '  Note:  all probabilities may be relative (unnormalized).' /
     &  '  Note:  use +/- for BINAME to incr/decr the last',
     &  ' bin name:  ',a /
     &  '  Note:  see commands "pdf" and "sample".' /
     &  '  Synonyms:  [help, h], [maxwell, Maxwell], [normal, norm],' /
     &  '    [planck, Planck], [power, pow], [wien, Wien]' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        write (   6, 8114)
        write (iout, 8114)
        write (   6, 8116)
        write (iout, 8116)
        write (   6, 8118) abins
        write (iout, 8118) abins
        go to 999
      endif

c===============================================================================

c.... Input line:  bin all
c.... Input line:  bin list

 8120 format (/ 'Number of probability bins  = ',i5)

      if ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'all')    .or.
     &    (aword(2) .eq. 'list'))) then

        nbinx = 0
        amount = 'minimum'
        call binout (1, nbins, amount)
        do 120 n = 1, nbins
          if (abin(n) .eq. ' ') go to 120
          nbinx = nbinx + 1
  120   continue
        write (   6, 8120) nbinx
        write (iout, 8120) nbinx
        go to 999
      endif

c===============================================================================

c.... Input line:  bin list BINAME1 BINAME2 BINAME3 ...

      if (aword(2) .eq. 'list') then

 8140 format ('  bin ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named bin does not exist:  ',a)

        amount = 'minimum'
        do 140 nw = 3, nword

          nfind = 0
          do 135 n = 1, nbins         ! Loop over bins.
            call aptsubr (abin(n), 1, 24, 80, astem, lstem,
     &                    asub, lsub, nerra)
            if ((aword(nw) .eq. abin(n)) .or.
     &          (aword(nw) .eq. astem))  then
              nfind = nfind + 1
              call binout (n, n, amount)
            endif
  135     continue                     ! End of loop over bins.
          if (nfind .eq. 0) then
            write (   6, 8140) ainput, ninput, aword(nw)
            write (iout, 8140) ainput, ninput, aword(nw)
          endif

  140   continue

        go to 999

      endif                           ! Word 2 is "list".

c===============================================================================

c.... Test for proper length and mode of bin name.

 8150 format ('  bin ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of bin not from 1 to 24 characters:  ',a)

      if ((lword(2) .lt. 1) .or. (lword(2) .gt. 24)) then
        write (   6, 8150) ainput, ninput, aword(2)
        write (iout, 8150) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Input line:  bin BINAME

c.... See if user wants one bin displayed.

      if (nword .eq. 2) then
        do 150 n = 1, nbins
          if (aword(2) .eq. abin(n)) then
            amount = 'maximum'
            call binout (n, n, amount)
            go to 999
          endif
  150   continue
        write (   6, 8140) ainput, ninput, aword(2)
        write (iout, 8140) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... See if last bin name should be incremented and used.

 8128 format ('  bin ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last bin can not be incremented:  ',a)

      if (aword(2) .eq. '+') then
        aword(2) = abins
        lword(2) = labins
        call incrsubs (1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8128) ainput, ninput, abins
          write (iout, 8128) ainput, ninput, abins
          go to 999
        endif
      endif

 8129 format ('  bin ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last bin can not be decremented:  ',a)

      if (aword(2) .eq. '-') then
        aword(2) = abins
        lword(2) = labins
        call incrsubs (-1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8129) ainput, ninput, abins
          write (iout, 8129) ainput, ninput, abins
          go to 999
        endif
      endif

 8905 format ('  bin ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of bin too dangerous to use:  ',a)

      call namechk (aword(2), lword(2), nerrchk)

      if (nerrchk .ne. 0) then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Find the index in the bin array at which to store this bin.
c....   Set nrep = 1 if replacing an existing bin.

      call binstore (nrep, n)

      if (n .eq. 0) go to 999

      nerr = 0

c===============================================================================

c.... Input line:  bin BINAME event [PRTOT|1]

c.... Discrete event bin.

      if (aword(3) .eq. 'event') then

        if (nword .eq. 3) then
          nword    = 4
          mtype(4) = 3
          aword(4) = '1.0'
          fword(4) = 1.0
          iword(4) = 1
        elseif (nword .ne. 4) then
          go to 998
        endif

 8160   format ('  bin ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word not integer or floating point value or variable:  ',a)

          call varfind (4, 4)

          if ((mtype(4) .ne. 2) .and. (mtype(4) .ne. 3)) then
            write (   6, 8160) ainput, ninput, aword(4)
            write (iout, 8160) ainput, ninput, aword(4)
            nerr = 1
          endif

 8165   format ('  bin ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Probability is not non-negative:  ',1pe20.12)

        if (fword(4) .lt. 0.0) then
          write (   6, 8165) ainput, ninput, fword(4)
          write (iout, 8165) ainput, ninput, fword(4)
          nerr = 1
        endif

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nbins) nbins = nbins - 1
          endif
          go to 999
        endif

        abin(n)   = aword(2)
        abins     = aword(2)
        labins    = lword(2)
        atypeb(n) = 'discrete event'
        probl(n)  = -1.e99
        probr(n)  = -1.e99
        vranl(n)  = -1.e99
        vranr(n)  = -1.e99
        prbin(n)  = fword(4)
        conbin(n) = -1.e99
        cosbin(n) = -1.e99
        vexbin(n) = -1.e99            ! Expected value.

c....   Print the data for the bin.

        amount = 'maximum'
        call binout (n, n, amount)

        go to 999

      endif                           ! Discrete event bin.

c===============================================================================

c.... Input line:  bin BINAME value VRAN [PRTOT|1]

c.... Discrete value bin.

      if (aword(3) .eq. 'value') then

        if (nword .eq. 4) then
          nword    = 5
          mtype(5) = 3
          aword(5) = '1.0'
          fword(5) = 1.0
          iword(5) = 1
        elseif (nword .ne. 5) then
          go to 998
        endif

        call varfind (4, 5)

        do 190 nw = 4, 5

          if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
            write (   6, 8160) ainput, ninput, aword(nw)
            write (iout, 8160) ainput, ninput, aword(nw)
            nerr = 1
          endif

  190   continue

        if (fword(5) .lt. 0.0) then
          write (   6, 8165) ainput, ninput, fword(5)
          write (iout, 8165) ainput, ninput, fword(5)
          nerr = 1
        endif

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nbins) nbins = nbins - 1
          endif
          go to 999
        endif

        abin(n)   = aword(2)
        abins     = aword(2)
        labins    = lword(2)
        atypeb(n) = 'discrete value'
        probl(n)  = -1.e99
        probr(n)  = -1.e99
        vranl(n)  = fword(4)
        vranr(n)  = -1.e99
        prbin(n)  = fword(5)
        conbin(n) = -1.e99
        cosbin(n) = -1.e99
        vexbin(n) = vranl(n)          ! Expected value.

c....   Print the data for the bin.

        amount = 'maximum'
        call binout (n, n, amount)

        go to 999

      endif                           ! Discrete value bin.

c===============================================================================

c.... Input line:  bin BINAME maxwell RATIO [PRTOT|1]

c.... Relativistic Maxwellian distribution bin.

      if ((aword(3) .eq. 'maxwell')  .or.
     &    (aword(3) .eq. 'Maxwell')) then

        if (nword .eq. 4) then
          nword    = 5
          mtype(5) = 3
          aword(5) = '1.0'
          fword(5) = 1.0
          iword(5) = 1
        elseif (nword .ne. 5) then
          go to 998
        endif

        call varfind (4, 5)

        do 340 nw = 4, 5
          if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
            write (   6, 8160) ainput, ninput, aword(nw)
            write (iout, 8160) ainput, ninput, aword(nw)
            nerr = 1
          endif
  340   continue

        if (fword(5) .lt. 0.0) then
          write (   6, 8165) ainput, ninput, fword(5)
          write (iout, 8165) ainput, ninput, fword(5)
          nerr = 1
        endif

 8340   format ('  bin ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Energy ratio is negative:  ',1pe20.12)

        if (fword(4) .lt. 0.0)  then
          write (   6, 8340) ainput, ninput, fword(4)
          write (iout, 8340) ainput, ninput, fword(4)
          nerr = 1
        endif

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nbins) nbins = nbins - 1
          endif
          go to 999
        endif

        abin(n)   = aword(2)
        abins     = aword(2)
        labins    = lword(2)
        atypeb(n) = 'rel Maxwellian'
        prbin(n)  = fword(5)
        probl(n)  = 0.0
        probr(n)  = 0.0
        vranl(n)  = 0.0
        vranr(n)  = 1.0
        conbin(n) = fword(4)          ! Ratio of temperature to rest mass.
        cosbin(n) = -1.e99
        vexbin(n) = -1.e99            ! Expected value.

c....   Print the data for the bin.

        amount = 'maximum'
        call binout (n, n, amount)

        go to 999

      endif                           ! Relativistic Maxwellian bin.

c===============================================================================

c.... Input line:  bin BINAME planck BTEMP [PRTOT|1]

c.... Planck spectrum bin.

      if ((aword(3) .eq. 'planck')  .or.
     &    (aword(3) .eq. 'Planck')) then

        if (nword .eq. 4) then
          nword    = 5
          mtype(5) = 3
          aword(5) = '1.0'
          fword(5) = 1.0
          iword(5) = 1
        elseif (nword .ne. 5) then
          go to 998
        endif

        call varfind (4, 5)

        do 360 nw = 4, 5
          if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
            write (   6, 8160) ainput, ninput, aword(nw)
            write (iout, 8160) ainput, ninput, aword(nw)
            nerr = 1
          endif
  360   continue

        if (fword(5) .lt. 0.0) then
          write (   6, 8165) ainput, ninput, fword(5)
          write (iout, 8165) ainput, ninput, fword(5)
          nerr = 1
        endif

 8360   format ('  bin ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Black-body temperature is negative:  ',1pe20.12)

        if (fword(4) .lt. 0.0)  then
          write (   6, 8360) ainput, ninput, fword(4)
          write (iout, 8360) ainput, ninput, fword(4)
          nerr = 1
        endif

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nbins) nbins = nbins - 1
          endif
          go to 999
        endif

        abin(n)   = aword(2)
        abins     = aword(2)
        labins    = lword(2)
        atypeb(n) = 'Planck'
        prbin(n)  = fword(5)
        probl(n)  = 0.0
        probr(n)  = 0.0
        vranl(n)  = 0.0
        vranr(n)  = 1.e99
        conbin(n) = fword(4)          ! Black-body temperature.
        cosbin(n) = -1.e99
        vexbin(n) = 3.83223 * fword(4)  ! Expected value.

c....   Print the data for the bin.

        amount = 'maximum'
        call binout (n, n, amount)

        go to 999

      endif                           ! Planck spectrum bin.

c===============================================================================

c.... Input line:  bin BINAME wien BTEMP [PRTOT|1]

c.... Wien spectrum bin.

      if ((aword(3) .eq. 'wien')  .or.
     &    (aword(3) .eq. 'Wien')) then

        if (nword .eq. 4) then
          nword    = 5
          mtype(5) = 3
          aword(5) = '1.0'
          fword(5) = 1.0
          iword(5) = 1
        elseif (nword .ne. 5) then
          go to 998
        endif

        call varfind (4, 5)

        do 380 nw = 4, 5
          if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
            write (   6, 8160) ainput, ninput, aword(nw)
            write (iout, 8160) ainput, ninput, aword(nw)
            nerr = 1
          endif
  380   continue

        if (fword(5) .lt. 0.0) then
          write (   6, 8165) ainput, ninput, fword(5)
          write (iout, 8165) ainput, ninput, fword(5)
          nerr = 1
        endif

 8380   format ('  bin ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Black-body temperature is negative:  ',1pe20.12)

        if (fword(4) .lt. 0.0)  then
          write (   6, 8380) ainput, ninput, fword(4)
          write (iout, 8380) ainput, ninput, fword(4)
          nerr = 1
        endif

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nbins) nbins = nbins - 1
          endif
          go to 999
        endif

        abin(n)   = aword(2)
        abins     = aword(2)
        labins    = lword(2)
        atypeb(n) = 'Wien'
        prbin(n)  = fword(5)
        probl(n)  = 0.0
        probr(n)  = 0.0
        vranl(n)  = 0.0
        vranr(n)  = 1.e99
        conbin(n) = fword(4)          ! Black-body temperature.
        cosbin(n) = -1.e99
        vexbin(n) = 4.0 * fword(4)    ! Expected value.

c....   Print the data for the bin.

        amount = 'maximum'
        call binout (n, n, amount)

        go to 999

      endif                           ! Wien spectrum bin.

c===============================================================================

c.... Input line:  bin BINAME uniform VRANL VRANR [DPR|1]

c.... Uniform distribution (histogram) bin.

      if (aword(3) .eq. 'uniform') then

        if (nword .eq. 5) then
          nword    = 6
          mtype(6) = 3
          aword(6) = '1.0'
          fword(6) = 1.0
          iword(6) = 1
        elseif (nword .ne. 6) then
          go to 998
        endif

        call varfind (4, 6)

        do 220 nw = 4, 6
          if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
            write (   6, 8160) ainput, ninput, aword(nw)
            write (iout, 8160) ainput, ninput, aword(nw)
            nerr = 1
          endif
  220   continue

        if (fword(6) .lt. 0.0) then
          write (   6, 8165) ainput, ninput, fword(6)
          write (iout, 8165) ainput, ninput, fword(6)
          nerr = 1
        endif

 8220   format ('  bin ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Width of interval is not positive:  ',1pe20.12)

        if (fword(5) .le. fword(4)) then
          dval = fword(5) - fword(4)
          write (   6, 8220) ainput, ninput, dval
          write (iout, 8220) ainput, ninput, dval
          nerr = 1
        endif

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nbins) nbins = nbins - 1
          endif
          go to 999
        endif

        abin(n)   = aword(2)
        abins     = aword(2)
        labins    = lword(2)
        atypeb(n) = 'uniform'
        probl(n)  = fword(6)
        probr(n)  = fword(6)
        vranl(n)  = fword(4)
        vranr(n)  = fword(5)
        prbin(n)  = fword(6) * (vranr(n) - vranl(n))
        conbin(n) = -1.e99
        cosbin(n) = -1.e99
        vexbin(n) = 0.5 * (vranl(n) + vranr(n))  ! Expected value.

c....   Print the data for the bin.

        amount = 'maximum'
        call binout (n, n, amount)

        go to 999

      endif                           ! Uniform (histogram) bin.

c===============================================================================

c.... Input line:  bin BINAME normal VMEAN DEV [PRTOT|1]

c.... Normal (Gaussian) distribution bin.

      if ((aword(3) .eq. 'normal') .or.
     &    (aword(3) .eq. 'norm'))  then

        if (nword .eq. 5) then
          nword    = 6
          mtype(6) = 3
          aword(6) = '1.0'
          fword(6) = 1.0
          iword(6) = 1
        elseif (nword .ne. 6) then
          go to 998
        endif

        call varfind (4, 6)

        do 320 nw = 4, 6
          if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
            write (   6, 8160) ainput, ninput, aword(nw)
            write (iout, 8160) ainput, ninput, aword(nw)
            nerr = 1
          endif
  320   continue

        if (fword(6) .lt. 0.0) then
          write (   6, 8165) ainput, ninput, fword(6)
          write (iout, 8165) ainput, ninput, fword(6)
          nerr = 1
        endif

 8320   format ('  bin ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Standard deviation is not positive:  ',1pe20.12)

        if (fword(5) .le. 0.0)  then
          write (   6, 8320) ainput, ninput, fword(5)
          write (iout, 8320) ainput, ninput, fword(5)
          nerr = 1
        endif

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nbins) nbins = nbins - 1
          endif
          go to 999
        endif

        abin(n)   = aword(2)
        abins     = aword(2)
        labins    = lword(2)
        atypeb(n) = 'normal'
        prbin(n)  = fword(6)
        probl(n)  = 0.0
        probr(n)  = 0.0
        vranl(n)  = -1.e99
        vranr(n)  =  1.e99
        conbin(n) = fword(4)          ! Mean value.  
        cosbin(n) = fword(5)          ! Standard deviation.
        vexbin(n) = conbin(n)         ! Expected value.

c....   Print the data for the bin.

        amount = 'maximum'
        call binout (n, n, amount)

        go to 999

      endif                           ! Normal (Gaussian) distribution bin.

c===============================================================================

c.... Input line:  bin BINAME linear VRANL VRANR [DPRL|1] [[DPRR|DPRL]]

c.... Linear distribution bin.

      if (aword(3) .eq. 'linear') then

        if (nword .eq. 5) then
          nword    = 6
          mtype(6) = 3
          aword(6) = '1.0'
          fword(6) = 1.0
          iword(6) = 1
        endif

        if (nword .eq. 6) then
          nword    = 7
          mtype(7) = 3
          aword(7) = aword(6)
          fword(7) = fword(6)
          iword(7) = iword(6)
        elseif (nword .ne. 7) then
          go to 998
        endif

        call varfind (4, 7)

        do 240 nw = 4, 7
          if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
            write (   6, 8160) ainput, ninput, aword(nw)
            write (iout, 8160) ainput, ninput, aword(nw)
            nerr = 1
          endif
  240   continue

        if (fword(5) .le. fword(4)) then
          dval = fword(5) - fword(4)
          write (   6, 8220) ainput, ninput, dval
          write (iout, 8220) ainput, ninput, dval
          nerr = 1
        endif

        if (fword(6) .lt. 0.0) then
          write (   6, 8165) ainput, ninput, fword(6)
          write (iout, 8165) ainput, ninput, fword(6)
          nerr = 1
        endif

        if (fword(7) .lt. 0.0) then
          write (   6, 8165) ainput, ninput, fword(7)
          write (iout, 8165) ainput, ninput, fword(7)
          nerr = 1
        endif

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nbins) nbins = nbins - 1
          endif
          go to 999
        endif

        abin(n)   = aword(2)
        abins     = aword(2)
        labins    = lword(2)
        atypeb(n) = 'linear'
        probl(n)  = fword(6)
        probr(n)  = fword(7)
        vranl(n)  = fword(4)
        vranr(n)  = fword(5)
        dval      = vranr(n) - vranl(n)
        prbin(n)  = 0.5 * (probl(n) + probr(n)) * dval
        conbin(n) = (probr(n) - probl(n)) / dval  ! Slope.
        cosbin(n) = -1.e99
        if (probl(n) .eq. probr(n)) then
          atypeb(n) = 'uniform'
          vexbin(n) = 0.5 * (vranl(n) + vranr(n))  ! Expected value.
        else
          fact      = 2.0 * (probl(n) + 2.0 * probr(n)) /
     &                (3.0 * (probl(n) + probr(n)))
          vexbin(n) = vranl(n) + 0.5 * fact * (vranr(n) - vranl(n))
        endif

c....   Print the data for the bin.

        amount = 'maximum'
        call binout (n, n, amount)

        go to 999

      endif                           ! Linear distribution bin.

c===============================================================================

c.... Input line:  bin BINAME exp VRANL VRANR SIGMA [PRTOT|1]

c.... Exponential distribution bin.

      if (aword(3) .eq. 'exp') then

        if (nword .eq. 6) then
          nword    = 7
          mtype(7) = 3
          aword(7) = '1.0'
          fword(7) = 1.0
          iword(7) = 1
        elseif (nword .ne. 7) then
          go to 998
        endif

        call varfind (4, 7)

        do 260 nw = 4, 7
          if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
            write (   6, 8160) ainput, ninput, aword(nw)
            write (iout, 8160) ainput, ninput, aword(nw)
            nerr = 1
          endif
  260   continue

        if (fword(7) .lt. 0.0) then
          write (   6, 8165) ainput, ninput, fword(7)
          write (iout, 8165) ainput, ninput, fword(7)
          nerr = 1
        endif

        if (fword(5) .le. fword(4)) then
          dval = fword(5) - fword(4)
          write (   6, 8220) ainput, ninput, dval
          write (iout, 8220) ainput, ninput, dval
          nerr = 1
        endif

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nbins) nbins = nbins - 1
          endif
          go to 999
        endif

        abin(n)   = aword(2)
        abins     = aword(2)
        labins    = lword(2)
        atypeb(n) = 'exponential'
        prbin(n)  = fword(7)
        vranl(n)  = fword(4)
        vranr(n)  = fword(5)
        dval      = vranr(n) - vranl(n)
        conbin(n) = fword(6)          ! Sigma.    
        cosbin(n) = -1.e99

        if (conbin(n) .eq. 0.0) then
          atypeb(n) = 'uniform'
          probl(n)  = prbin(n) / dval
          probr(n)  = prbin(n) / dval
          vexbin(n) = 0.5 * (vranl(n) + vranr(n))  ! Expected value.
        else
          arg       = conbin(n) * dval
          expfun    = exp (-arg)
          probl(n)  = conbin(n) * prbin(n) / (1.0 - expfun)
          probr(n)  = probl(n) * expfun
          fact      = 1.0 - arg * expfun / (1.0 - expfun)
          vexbin(n) = vranl(n) + fact / conbin(n)  ! Expected value.
        endif

c....   Print the data for the bin.

        amount = 'maximum'
        call binout (n, n, amount)

        go to 999

      endif                           ! Exponential distribution bin.

c===============================================================================

c.... Input line:  bin BINAME power VRANL VRANR POW [PRTOT|1]

c.... Power-law distribution bin.

      if ((aword(3) .eq. 'power') .or.
     &    (aword(3) .eq. 'pow'))  then

        if (nword .eq. 6) then
          nword    = 7
          mtype(7) = 3
          aword(7) = '1.0'
          fword(7) = 1.0
          iword(7) = 1
        elseif (nword .ne. 7) then
          go to 998
        endif

        call varfind (4, 7)

        do 280 nw = 4, 7
          if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
            write (   6, 8160) ainput, ninput, aword(nw)
            write (iout, 8160) ainput, ninput, aword(nw)
            nerr = 1
          endif
  280   continue

        if (fword(7) .lt. 0.0) then
          write (   6, 8165) ainput, ninput, fword(7)
          write (iout, 8165) ainput, ninput, fword(7)
          nerr = 1
        endif

 8290   format ('  bin ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  With nonzero POW, VRANL must not be negative:  ',
     &    1pe20.12)

        if ((fword(4) .lt. 0.0)  .and.
     &      (fword(6) .ne. 0.0)) then
          write (   6, 8290) ainput, ninput, fword(4)
          write (iout, 8290) ainput, ninput, fword(4)
          nerr = 1
        endif

 8295   format ('  bin ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  With a negative POW, VRANL must not be zero.')

        if ((fword(4) .eq. 0.0)  .and.
     &      (fword(6) .lt. 0.0)) then
          write (   6, 8295) ainput, ninput
          write (iout, 8295) ainput, ninput
          nerr = 1
        endif

        if (fword(5) .le. fword(4)) then
          dval = fword(5) - fword(4)
          write (   6, 8220) ainput, ninput, dval
          write (iout, 8220) ainput, ninput, dval
          nerr = 1
        endif

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nbins) nbins = nbins - 1
          endif
          go to 999
        endif

        abin(n)   = aword(2)
        abins     = aword(2)
        labins    = lword(2)
        atypeb(n) = 'power-law'
        prbin(n)  = fword(7)
        vranl(n)  = fword(4)
        vranr(n)  = fword(5)
        dval      = vranr(n) - vranl(n)
        conbin(n) = fword(6)          ! Power.       
        cosbin(n) = -1.e99

        if (abs (conbin(n) + 1.0) .le. tol) then
          fact      = prbin(n) / log (vranr(n) / vranl(n))
          probl(n)  = fact / vranl(n)
          probr(n)  = fact / vranr(n)
          vexbin(n) = dval / log (vranr(n) / vranl(n))  ! Expected value.
        elseif (conbin(n) .eq. 0.0) then
          atypeb(n) = 'uniform'
          probl(n)  = prbin(n) / dval
          probr(n)  = prbin(n) / dval
          vexbin(n) = 0.5 * (vranl(n) + vranr(n))  ! Expected value.
        else
          pow1      = conbin(n) + 1.0
          fact      = pow1 * prbin(n) /
     &                (vranr(n)**pow1 - vranl(n)**pow1)
          probl(n)  = fact * vranl(n)**conbin(n)
          probr(n)  = fact * vranr(n)**conbin(n)

          if (abs (conbin(n) + 2.0) .le. tol) then
            vexbin(n) = vranl(n) * vranr(n) *
     &                  log (vranr(n) / vranl(n)) / dval
          else
            pow2      = conbin(n) + 2.0
            factr     = pow1 / pow2
            vexbin(n) = factr * (vranr(n)**pow2 - vranl(n)**pow2) /
     &                          (vranr(n)**pow1 - vranl(n)**pow1)
          endif

        endif
 
c....   Print the data for the bin.

        amount = 'maximum'
        call binout (n, n, amount)

        go to 999

      endif                           ! Power-law distribution bin.

c===============================================================================

 8710 format ('  bin ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

  998 write (   6, 8710) ainput, ninput
      write (iout, 8710) ainput, ninput

c===============================================================================

  999 return

c.... End of subroutine bin.          (+1 line)
      end
