      subroutine brick

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  brick    Updated     2005 April 5 17:45.
c               brick    Originated  1993 November 29 10:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display and create bricks by specifying the minimum and
c               maximum values of the bounding coordinate surfaces.
c
c     Logic:    Entry points:  brick.
c brick         Called by:  helpall, main.
c
c brick         Calls:  aptsubr, brout, brstore, brtest, incrsubs, namechk,
c               varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      character*8  amount             ! Amount of output to write in brout.
      character*24 anamex             ! Temporary name of a brick.

c.... Input line:  brick [help]

 8110 format (
     &  '  Command "brick":  acceptable input lines include:' //
     &  '  "brick [help]"  Display this message.' /
     &  '  "brick all"     Display all bricks.' /
     &  '  "brick list [BRNAME1 BRNAME2 BRNAME3 ...]"' /
     &  '         Display all listed bricks (default is all',
     &  ' bricks).' /
     &  '  "brick BRNAME"  Display brick BRNAME.' //
     &  '  "brick BRNAME = UMIN UMAX VMIN VMAX WMIN WMAX"' /
     &  '     Create brick BRNAME, bounded by the coordinate' /
     &  '     values UMIN, UMAX, VMIN, VMAX, WMIN, WMAX, in the' /
     &  '     current coordinate system (U, V, W).' )

 8112 format (/
     &  '  "brick BR02 increment NUMBR INC BR01 [-] CDIR [FMULT|1]"' /
     &  '     Create a family of NUMBR bricks BR02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     each sharing a coordinate surface in the [negative]',
     &  ' coordinate direction' /
     &  '     CDIR (x, y, z, rcyl, theta, rsph, phi), with the',
     &  ' preceding brick,' /
     &  '     starting from brick BR01, and with a thickness in',
     &  ' direction CDIR' /
     &  '     equal to FMULT times the thickness of the preceding',
     &  ' brick.' )

 8114 format (/
     &  '  Note:  use +/- for BRNAME or BR02 to incr/decr the last',
     &  ' brick name:  ',a //
     &  '  Synonyms:  [brick, br], [help, h], [increment, incr].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        write (   6, 8114) abricks
        write (iout, 8114) abricks
        go to 999
      endif

c===============================================================================

c.... Input line:  brick all
c.... Input line:  brick list

 8120 format (/ 'Number of bricks = ',i5)

      if ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'all')    .or.
     &    (aword(2) .eq. 'list'))) then

        nbrickx = 0
        do 120 n = 1, nbricks
          if (abrick(n) .eq. ' ') go to 120
          nbrickx = nbrickx + 1
  120   continue
        if (nbrickx .eq. 1) then
          amount = 'maximum'
        else
          amount = 'minimum'
        endif
        call brout (1, nbricks, amount)
        write (   6, 8120) nbrickx
        write (iout, 8120) nbrickx
        go to 999
      endif

c===============================================================================

c.... Input line:  brick list BRNAME1 BRNAME2 BRNAME3 ...

 8130 format ('  brick ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named brick does not exist:  ',a)

      if (aword(2) .eq. 'list') then
        nfind = 0
        do 128 nw = 3, nword
          do 125 n = 1, nbricks
            call aptsubr (abrick(n), 1, 24, 80, astem, lstem,
     &                    asub, lsub, nerra)
            if ((aword(nw) .eq. abrick(n)) .or.
     &          (aword(nw) .eq. astem))    then
              nfind = nfind + 1
              call brout (n, n, 'min')
            endif
  125     continue
          if (nfind .eq. 0) then
            write (   6, 8130) ainput, ninput, aword(nw)
            write (iout, 8130) ainput, ninput, aword(nw)
          endif
  128   continue
        go to 999
      endif

c===============================================================================

c.... Input line:  brick BRNAME

c.... See if user wants one brick displayed.

       if (nword .eq. 2) then
        do 130 n = 1, nbricks
          if (aword(2) .eq. abrick(n)) then
            amount = 'maximum'
            call brout (n, n, amount)
            go to 999
          endif
  130   continue
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Test input for validity.

 8136 format ('  brick ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of brick not from 1 to 24 characters:  ',a)

      if ((lword(2) .lt. 1) .or. (lword(2) .gt. 24)) then
        write (   6, 8136) ainput, ninput, aword(2)
        write (iout, 8136) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... See if last brick name should be incremented and used.

 8128 format ('  brick ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last brick can not be incremented:  ',a)

      if (aword(2) .eq. '+') then
        aword(2) = abricks
        lword(2) = labricks
        call incrsubs (1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8128) ainput, ninput, abricks
          write (iout, 8128) ainput, ninput, abricks
          go to 999
        endif
      endif

 8129 format ('  brick ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last brick can not be decremented:  ',a)

      if (aword(2) .eq. '-') then
        aword(2) = abricks
        lword(2) = labricks
        call incrsubs (-1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8129) ainput, ninput, abricks
          write (iout, 8129) ainput, ninput, abricks
          go to 999
        endif
      endif

 8905 format ('  brick ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of brick too dangerous to use:  ',a)

      call namechk (aword(2), lword(2), nerrchk)

      if (nerrchk .ne. 0) then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Input line:  brick BR02 increment NUMBR INC BR01 [-] x [FMULT|1]
c.... Input line:  brick BR02 increment NUMBR INC BR01 [-] y [FMULT|1]
c.... Input line:  brick BR02 increment NUMBR INC BR01 [-] z [FMULT|1]
c.... Input line:  brick BR02 increment NUMBR INC BR01 [-] rcyl [FMULT|1]
c.... Input line:  brick BR02 increment NUMBR INC BR01 [-] rsph [FMULT|1]
c.... Input line:  brick BR02 increment NUMBR INC BR01 [-] phi [FMULT|1]
c.... Input line:  brick BR02 increment NUMBR INC BR01 [-] theta [FMULT|1]


      if ((aword(3) .eq. 'increment') .or.
     &    (aword(3) .eq. 'incr'))     then

        nerr = 0

 8505   format ('  brick ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Wrong number of words in input line:  ',i3)

        if (((aword(7) .eq. '-') .and.
     &      ((nword .lt. 8) .or. (nword .gt. 9)))  .or.
     &      ((aword(7) .ne. '-') .and.
     &      ((nword .lt. 7) .or. (nword .gt. 8)))) then
          write (   6, 8505) ainput, ninput, nword
          write (iout, 8505) ainput, ninput, nword
          nerr = 1
        endif

 8520   format ('  brick ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word NUMBR is not a positive integer:  ',a)
 8530   format ('  brick ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word NUMBR is not an integer or integer variable:  ',a)

        call varfind (4, 4)

        if (mtype(4) .eq. 2) then
          numbr = iword(4)
          if (numbr .lt. 1) then
            write (   6, 8520) ainput, ninput, aword(4)
            write (iout, 8520) ainput, ninput, aword(4)
            nerr = 1
          endif
        else
          write (   6, 8530) ainput, ninput, aword(4)
          write (iout, 8530) ainput, ninput, aword(4)
          nerr = 1
        endif

 8510   format ('  brick ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word INC is zero.')
 8515   format ('  brick ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word INC is not an integer or integer variable:  ',a)

        call varfind (5, 5)

        if (mtype(5) .eq. 2) then
          inc = iword(5)
          if (inc .eq. 0) then
            write (   6, 8510) ainput, ninput
            write (iout, 8510) ainput, ninput
            nerr = 1
          endif
        else
          write (   6, 8515) ainput, ninput, aword(5)
          write (iout, 8515) ainput, ninput, aword(5)
          nerr = 1
        endif

c....   Find the base brick.

 8540   format ('  brick ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Named brick does not exist:  ',a)

        do 540 nb = 1, nbricks
          if (aword(6) .eq. abrick(nb)) then
            nbase = nb
            call brout (nbase, nbase, 'min')
            go to 550
          endif
  540   continue
        write (   6, 8540) ainput, ninput, aword(6)
        write (iout, 8540) ainput, ninput, aword(6)
        nerr = 1
        nbase = 0

c....   Find the coordinate direction in which to create the bricks.

  550   if (aword(7) .eq. '-') then
          nwnext = 8
        else
          nwnext = 7
        endif

c....   Find the thickness multiplier.

 8560   format ('  brick ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  FMULT not integer or floating point value or',
     &    ' variable:  ',a)
 8565   format ('  brick ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  FMULT not positive:  ',a)

        nw = nwnext + 1
        if (nword .lt. nw) then
          fmult = 1.0
        else

          call varfind (nw, nw)

          if ((mtype(nw) .eq. 2) .or. (mtype(nw) .eq. 3)) then
            fmult = fword(nw)
          else
            write (   6, 8560) ainput, ninput, aword(nw)
            write (iout, 8560) ainput, ninput, aword(nw)
            nerr = 1
          endif
        endif

        if (fmult .le. 0.0) then
          write (   6, 8565) ainput, ninput, aword(nw)
          write (iout, 8565) ainput, ninput, aword(nw)
          nerr = 1
        endif

 8570   format ('  brick ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Coordinate direction does not match base brick:  ',a)

        fu  = 0.0
        fv  = 0.0
        fw  = 0.0
        fmu = 1.0
        fmv = 1.0
        fmw = 1.0

        if (nbase .gt. 0) then        ! Base brick exists.

        if     (((asysbr(nbase) .eq. 'rect')    .and.
     &           (aword(nwnext) .eq. 'x'))      .or.
     &          ((asysbr(nbase) .eq. 'cyl')     .and.
     &           (aword(nwnext) .eq. 'rcyl'))   .or.
     &          ((asysbr(nbase) .eq. 'sph')     .and.
     &           (aword(nwnext) .eq. 'rsph')))  then
          fu  = 1.0
          fmu = fmult
        elseif (((asysbr(nbase) .eq. 'rect')    .and.
     &           (aword(nwnext) .eq. 'y'))      .or.
     &          ((asysbr(nbase) .eq. 'cyl')     .and.
     &           (aword(nwnext) .eq. 'theta'))  .or.
     &          ((asysbr(nbase) .eq. 'sph')     .and.
     &           (aword(nwnext) .eq. 'theta'))) then
          fv  = 1.0
          fmv = fmult
        elseif (((asysbr(nbase) .eq. 'rect')    .and.
     &           (aword(nwnext) .eq. 'z'))      .or.
     &          ((asysbr(nbase) .eq. 'cyl')     .and.
     &           (aword(nwnext) .eq. 'z'))      .or.
     &          ((asysbr(nbase) .eq. 'sph')     .and.
     &           (aword(nwnext) .eq. 'phi')))   then
          fw  = 1.0
          fmw = fmult
        else
          write (   6, 8570) ainput, ninput, aword(nwnext)
          write (iout, 8570) ainput, ninput, aword(nwnext)
          nerr = 1
        endif

        endif                         ! Base brick exists.

        if (aword(7) .eq. '-') then
          fu = -fmult * fu
          fv = -fmult * fv
          fw = -fmult * fw
        endif

c....   See if the brick names can be generated uniquely.

 8580   format ('  brick ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Can not generate',i5,' brick names from:  ',a)

        anamex  = aword(2)
        lanamex = lword(2)
        namex   = inc * (numbr - 1)

        call incrsubs (namex, anamex, 1, lanamex, nerra)

        if (nerra .ne. 0) then
          nerr = 1
          write (   6, 8580) ainput, ninput, numbr, aword(2)
          write (iout, 8580) ainput, ninput, numbr, aword(2)
        endif

        call namechk (anamex, lanamex, nerrchk)

        if (nerrchk .ne. 0) then
          nerr = 1
          write (   6, 8905) ainput, ninput, anamex
          write (iout, 8905) ainput, ninput, anamex
        endif

        if (nerr .ne. 0) go to 999

c....   Create the family of bricks.

        anamex  = aword(2)             ! Initial brick name.
        lanamex = lword(2)             ! Initial brick name length.

        do 590 nbr = 1, numbr         ! Loop over the new bricks.

c....     Find the index in the brick list for the new brick.

          aword(2) = anamex
          lword(2) = lanamex

          call brstore (nrep, n)

          if (n .eq. 0) go to 999

          umin      = ubr(nbase) + fu * dubr(nbase)
          umax      = umin + fmu * dubr(nbase)
          vmin      = vbr(nbase) + fv * dvbr(nbase)
          vmax      = vmin + fmv * dvbr(nbase)
          wmin      = wbr(nbase) + fw * dwbr(nbase)
          wmax      = wmin + fmw * dwbr(nbase)

          call brtest (asysbr(nbase), angbr(nbase),
     &                 umin, umax, vmin, vmax, wmin, wmax, nerr)

          if (nerr .ne. 0) then
            go to 999
          endif

          abrick(n) = anamex
          abricks   = anamex
          labricks  = lanamex
          asysbr(n) = asysbr(nbase)
          angbr(n)  = angbr(nbase)

          ubr(n)    = umin
          dubr(n)   = fmu * dubr(nbase)
          vbr(n)    = vmin
          dvbr(n)   = fmv * dvbr(nbase)
          wbr(n)    = wmin
          dwbr(n)   = fmw * dwbr(nbase)

          call brout (n, n, 'min')

          nbase = n

c....     Find the next brick name, by incrementing by inc.

          call incrsubs (inc, anamex, 1, lanamex, nerra)

          call namechk (anamex, lanamex, nerrchk)

          if (nerrchk .ne. 0) then
            write (   6, 8905) ainput, ninput, anamex
            write (iout, 8905) ainput, ninput, anamex
            go to 999
          endif

  590   continue                      ! End of loop over the new bricks.

        go to 999

      endif                           ! Create a family of bricks.

c===============================================================================

c.... Find the index in the brick array at which to store this brick.
c....   Set nrep = 1 if replacing an existing brick.

      call brstore (nrep, n)

      if (n .eq. 0) go to 999

      nerr = 0

c===============================================================================

c.... Input line:  brick BRNAME = UMIN UMAX VMIN VMAX WMIN WMAX
c.... Input line:  brick BRNAME = XMIN XMAX YMIN YMAX ZMIN ZMAX
c.... Input line:  brick BRNAME = RCYLMIN RCYLMAX THMIN THMAX ZMIN ZMAX
c.... Input line:  brick BRNAME = RSPHMIN RSPHMAX THMIN THMAX PHIMIN PHIMAX

      if (aword(3) .eq. '=') then

 8135   format ('  brick ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of words on "brick BRNAME =" line not 9:  ',i3)

        if (nword .ne. 9) then
          write (   6, 8135) ainput, ninput, nword
          write (iout, 8135) ainput, ninput, nword
          go to 999
        endif

c....   Find the coordinate values.

 8140   format ('  brick ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word not integer or floating point value or',
     &    ' variable:  ',a)

        call varfind (4, 9)

        do 150 nw = 4, 9
          if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
            write (   6, 8140) ainput, ninput, aword(nw)
            write (iout, 8140) ainput, ninput, aword(nw)
            nerr = 1
          endif
  150   continue

c....   Test for unacceptable coordinate values.

        call brtest (asys, angunit, fword(4), fword(5),
     &               fword(6), fword(7), fword(8), fword(9), nerr)

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            abrick(n) = ' '
            if (n .eq. nbricks) nbricks = nbricks - 1
          endif
          go to 999
        endif

c....   Store and display the brick specifications.

        abrick(n) = aword(2)
        abricks   = aword(2)
        labricks  = lword(2)

        asysbr(n) = asys
        angbr(n)  = angunit
        ubr(n)    = fword(4)
        dubr(n)   = fword(5) - fword(4)
        vbr(n)    = fword(6)
        dvbr(n)   = fword(7) - fword(6)
        wbr(n)    = fword(8)
        dwbr(n)   = fword(9) - fword(8)
        volbr(n)  = 0.0               ! Calculate later in brout.

        amount = 'maximum'
        call brout (n, n, amount)

        go to 999

      endif                           ! Create a brick.

c===============================================================================

 8440 format ('  brick ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8440) ainput, ninput
      write (iout, 8440) ainput, ninput

c===============================================================================

  999 return

c.... End of subroutine brick.        (+1 line)
      end
