      subroutine brtest (asysx, angunitx, umin, umax, vmin, vmax,
     &                   wmin, wmax, nerrbr)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  brtest   Updated     1994 August 9 11:40.
c               brtest   Originated  1994 August 9 11:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To test bricks to see if their coordinates are good.
c               The brick is from umin to umax, vmin to vmax, wmin to wmax.
c
c     Logic:    Entry points:  brtest.
c brtest        Called by:  brick.
c
c     Biblio:   ?
c
c     Glossary:
c
c     asysx     Input    The coordinate system used to define the brick.
c
c     nerrbr    Output   Indicates an error in the coordinates, if not 0.
c
c     umax      Input    The maximum value of coordinate u.
c
c     umin      Input    The minimum value of coordinate u.
c
c     vmax      Input    The maximum value of coordinate v.
c
c     vmin      Input    The minimum value of coordinate v.
c
c     wmax      Input    The maximum value of coordinate w.
c
c     wmin      Input    The minimum value of coordinate w.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*8 asysx
      character*8 angunitx

c.... Test for unacceptable coordinates.

 8120 format ('  brtest ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Maximum U is less than minimum U',1p2e20.12)
 8130 format ('  brtest ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Maximum V is less than minimum V',1p2e20.12)
 8140 format ('  brtest ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Maximum W is less than minimum W',1p2e20.12)
 8150 format ('  brtest ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  RCYL or RSPH < 0:               ',1p2e20.12)
 8160 format ('  brtest ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  THETA increment > 360 degrees:  ',1pe20.12)
 8170 format ('  brtest ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  PHI < 0 or > 180 degrees:       ',1p2e20.12)

      if (umax .lt. umin) then
        write (   6, 8120) ainput, ninput, umin, umax
        write (iout, 8120) ainput, ninput, umin, umax
        nerrbr = 1
      endif

      if (vmax .lt. vmin) then
        write (   6, 8130) ainput, ninput, vmin, vmax
        write (iout, 8130) ainput, ninput, vmin, vmax
        nerrbr = 1
      endif

      if (wmax .lt. wmin) then
        write (   6, 8120) ainput, ninput, wmin, wmax
        write (iout, 8120) ainput, ninput, wmin, wmax
        nerrbr = 1
      endif

      if (asysx .ne. 'rect') then

        if (umin .lt. 0.0) then
          write (   6, 8150) ainput, ninput, umin, umax
          write (iout, 8150) ainput, ninput, umin, umax
          nerrbr = 1
        endif

        fact = 180.0 / pi
        if (angunitx .eq. 'degrees') fact = 1.0

        dv = vmax - vmin
        if (fact * abs (dv) .gt. 360.0) then
          dtheta = fact * dv
          write (   6, 8160) ainput, ninput, dtheta
          write (iout, 8160) ainput, ninput, dtheta
          nerrbr = 1
        endif

        phimin = fact * wmin
        phimax = fact * wmax

        if ((asysx .eq. 'sph')          .and.
     &     ((phimin .lt. 0.0)     .or.
     &      (phimax .gt. 180.0))) then
          write (   6, 8170) ainput, ninput, phimin, phimax
          write (iout, 8170) ainput, ninput, phimin, phimax
          nerrbr = 1
        endif

      endif

      return

c.... End of subroutine brtest.       (+1 line)
      end
