      subroutine compsubs (aword1, aword2, nsmall, nerrc)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  compsubs Updated     2007 January 23 18:00.
c               compsubs Originated  2003 April 24 16:30.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To compare two (possibly subscripted) character strings, and
c               find the lesser string, based on the stem and the subscripts.
c
c     Logic:    Entry points:  compsubs.
c compsubs      Called by:  angles, cluster, delete, pdf, plotx, sort, symbol,
c               variable.
c
c compsubs      Calls:  aptsubm.
c
c     Biblio:   ?
c
c     Glossary: See store.h.
c
c     aword1    input    First character string.
c
c     aword2    input    Second character string.
c
c     nsmall    output   0 if aword1 and aword2 are identical.
c                          1 if aword1 .lt. aword2.
c                          2 if aword1 .gt. aword2.
c
c     nerrc     output   Non-zero if either subscript is not a recognizable
c                          integer.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      character*24    aword1          ! First character string.
      character*24    aword2          ! Second character string.
      character*24    astem1          ! ASCII stem of aword1.
      character*24    astem2          ! ASCII stem of aword2.
      character*24    asub1(16)       ! Subscript of aword1.
      character*24    asub2(16)       ! Subscript of aword2.

      dimension       fsub1(16)       ! Floating point subscript of aword1.
      dimension       fsub2(16)       ! Floating point subscript of aword2.
      dimension       isub1(16)       ! Integer subscript of aword1.
      dimension       isub2(16)       ! Integer subscript of aword2.
      dimension       lsub1(16)       ! Length of subscript of aword1.
      dimension       lsub2(16)       ! Length of subscript of aword2.
      dimension       msub1(16)       ! Mode of subscript of aword1.
      dimension       msub2(16)       ! Mode of subscript of aword2.

c.... Initialize.
cbugc***DEBUG begins.
cbug 9100 format (// 'compsubs DEBUG 0.' /
cbug     &  '  aword1 = <',a,'>' /
cbug     &  '  aword2 = <',a,'>' )
cbug      write (   6, 9100) aword1, aword2
cbug      write (iout, 9100) aword1, aword2
cbugc***DEBUG ends.

      nerr1 = 0
      nerr2 = 0
      nerrc = 0

c.... Find makeup of first word.

      call aptsubm (aword1, 1, 24, 24, idmax, iemax, '(', ',', ')', 16,
     &         astem1, istem1, fstem1, mstem1, lstem1,
     &  nsub1, asub1,  isub1,  fsub1,  msub1,  lsub1, nerr1)
cbugc***DEBUG begins.
cbug 9101 format (/ 'compsubs DEBUG 1.' /
cbug     &  'aword1 = <',a16,'>' /
cbug     &  'astem1 = <',a16,'>' /
cbug     &  'istem1 = ',i12 /
cbug     &  'fstem1 = ',1pe20.12 /
cbug     &  'mstem1 = ',i2 /
cbug     &  'lstem1 = ',i2 )
cbug 9102 format (/ 'compsubs DEBUG 1.' /
cbug     &  'asub1  = <',a16,'>' /
cbug     &  'isub1  = ',i12 /
cbug     &  'fsub1  = ',1pe20.12 /
cbug     &  'msub1  = ',i2 /
cbug     &  'lsub1  = ',i2 )
cbug      write (   6, 9101) aword1, astem1, istem1, fstem1, mstem1, lstem1
cbug      write (iout, 9101) aword1, astem1, istem1, fstem1, mstem1, lstem1
cbug      write (   6, 9102) (asub1(n), isub1(n), fsub1(n), msub1(n),
cbug     &   lsub1(n), n = 1, nsub1)
cbug      write (iout, 9102) (asub1(n), isub1(n), fsub1(n), msub1(n),
cbug     &   lsub1(n), n = 1, nsub1)
cbugc***DEBUG ends.

c.... Find makeup of second word.

      call aptsubm (aword2, 1, 24, 24, idmax, iemax, '(', ',', ')', 16,
     &         astem2, istem2, fstem2, mstem2, lstem2,
     &  nsub2, asub2,  isub2,  fsub2,  msub2,  lsub2, nerr2)
cbugc***DEBUG begins.
cbug 9103 format (/ 'compsubs DEBUG 2.' /
cbug     &  'aword2 = <',a16,'>' /
cbug     &  'astem2 = <',a16,'>' /
cbug     &  'istem2 = ',i12 /
cbug     &  'fstem2 = ',1pe20.12 /
cbug     &  'mstem2 = ',i2 /
cbug     &  'lstem2 = ',i2 )
cbug 9104 format (/ 'compsubs DEBUG 2.' /
cbug     &  'asub2  = <',a16,'>' /
cbug     &  'isub2  = ',i12 /
cbug     &  'fsub2  = ',1pe20.12 /
cbug     &  'msub2  = ',i2 /
cbug     &  'lsub2  = ',i2 )
cbug      write (   6, 9103) aword2, astem2, istem2, fstem2, mstem2, lstem2
cbug      write (iout, 9103) aword2, astem2, istem2, fstem2, mstem2, lstem2
cbug      write (   6, 9104) (asub2(n), isub2(n), fsub2(n), msub2(n),
cbug     &   lsub2(n), n = 1, nsub2)
cbug      write (iout, 9104) (asub2(n), isub2(n), fsub2(n), msub2(n),
cbug     &   lsub2(n), n = 1, nsub2)
cbugc***DEBUG ends.

c.... Compare words as ASCII strings, if error.

      if ((nerr1 .ne. 0) .or. (nerr2 .ne. 0)) then
cbugc***DEBUG begins.
cbug 9905 format ('compsubs DEBUG.  Compare whole ASCII words.')
cbug       write (   3, 9905)
cbug       write (iout, 9905)
cbugc***DEBUG ends.

        if (aword1 .eq. aword2) then
          nsmall = 0
        elseif (aword1 .lt. aword2) then
          nsmall = 1
        else
          nsmall = 2
        endif
        go to 999
      endif                           ! Compared whole words.

c...  Compare stems as integers if both are integer, otherwise, ASCII.

      if ((mstem1 .eq. 2) .and. (mstem2 .eq. 2)) then

c....   Both stems are integers.
cbugc***DEBUG begins.
cbug 9903 format ('compsubs DEBUG.  Both stems are integers.')
cbug       write (   3, 9903)
cbug       write (iout, 9903)
cbugc***DEBUG ends.

        if (istem1 .eq. istem2) then
          nsmall = 0
        elseif (istem1 .lt. istem2) then
          nsmall = 1
        else
          nsmall = 2
        endif
      else
cbugc***DEBUG begins.
cbug 9704 format ('compsubs DEBUG.  Both stems are non-integer.')
cbug       write (   3, 9704)
cbug       write (iout, 9704)
cbugc***DEBUG ends.

        if (astem1 .eq. astem2) then
          nsmall = 0
        elseif (astem1 .lt. astem2) then
          nsmall = 1
        else
          nsmall = 2
        endif
      endif

c.... See if there are any subscripts.

      if ((nsub1 .eq. 0) .and. (nsub2 .eq. 0)) go to 999

c.... If stems are different, ignore subscripts.

      if (nsmall .ne. 0) go to 999

c.... Compare subscripts.

      nsubmax = max (nsub1, nsub2)

      do 120 n = 1, nsubmax
cbugc***DEBUG begins.
cbug 9721 format ('compsubs DEBUG.  Comparing subscript',i3)
cbug       write (   3, 9721) n
cbug       write (iout, 9721) n
cbugc***DEBUG ends.

        if (nsub1 .lt. n) then
          nsmall = 1
          go to 999
        elseif (nsub2 .lt. n) then
          nsmall = 2
          go to 999
        endif
cbugc***DEBUG begins.
cbug 9722 format ('compsubs DEBUG.  Both have subscript',i3)
cbug       write (   3, 9722) n
cbug       write (iout, 9722) n
cbugc***DEBUG ends.

        if ((msub1(n) .eq. 2) .and. (msub2(n) .eq. 2)) then
          if (isub1(n) .eq. isub2(n)) then
            nsmall = 0
          elseif (isub1(n) .lt. isub2(n)) then
            nsmall = 1
          else
            nsmall = 2
          endif
        else
          if (asub1(n) .eq. asub2(n)) then
            nsmall = 0
          elseif (asub1(n) .lt. asub2(n)) then
            nsmall = 1
          else
            nsmall = 2
          endif
        endif
cbugc***DEBUG begins.
cbug 9723 format ('compsubs DEBUG.  nsmall =',i3)
cbug       write (   3, 9723) nsmall
cbug       write (iout, 9723) nsmall
cbugc***DEBUG ends.

        if (nsmall .ne. 0) go to 999

  120 continue

  999 continue

      if ((nerr1 .ne. 0) .or. (nerr2 .ne. 0)) nerrc = 1
cbugc***DEBUG begins.
cbug 9907 format ('compsubs DEBUG.  nsmall = ',i2,'  nerrc = ',i3 )
cbug       write (   3, 9907) nsmall, nerrc
cbug       write (iout, 9907) nsmall, nerrc
cbugc***DEBUG ends.

      return

c.... End of subroutine compsubs.     (+1 line)
      end
