      subroutine cone

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  cone     Updated     2005 April 5 17:45.
c               cone     Originated  1993 December 16 11:50.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display and create circular cones
c               (but see option "scale").
c
c     Logic:    Entry points:  cone.
c cone          Called by:  helpall, main.
c
c cone          Calls:  aptlnlc, aptmopv, aptrois, aptrotv, aptsubr, apttran,
c               apttris, aptvang, aptvusz, incrsubs, lout, namechk, ptfind,
c               qumove, quoper, quout, qustore, varfind, vectfind.
c                 
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      common /lcone/ rotm(3,3)        ! Principle axis rotation tensor.

      character*8  amount             ! Amount of ouput data.
      character*24 anamex             ! Temporary name of a cone.
      character*8  ans                ! User's response.

c.... Input line:  cone [help]

 8110 format (
     &  '  Command "cone":  acceptable input lines include:' /
     &  '  "cone [help]"  Display this message.' /
     &  '  "cone all"     Display all circular cones.' /
     &  '  "cone list [CNNAME1 CNNAME2 CNNAME3 ...]"' /
     &  '     Display all listed circular cones (default is all',
     &  ' circular cones).' /
     &  '  "cone CNNAME"  Display circular cone CNNAME.' //
     &  '  "cone CNNAME line PAXIS VAXIS ALNAME"' /
     &  '     Create circular cone CNNAME with axis point PAXIS and',
     &  ' axis vector VAXIS,' /
     &  '     through the endpoints of line ALNAME.' /
     &  '  "cone CNNAME fit PVER VAXIS PNAME2"' /
     &  '     Create circular cone CNNAME with vertex point PVER and',
     &  ' axis vector VAXIS,' /
     &  '     through point PNAME2.' /
     &  '  "cone CNNAME angle PVER VAXIS ANGLE"' /
     &  '     Create circular cone CNNAME with vertex point PVER and',
     &  ' axis vector VAXIS,' /
     &  '     with half-angle ANGLE.' )

 8112 format (/
     &  '  "cone CNNAME nest NUMCONE INC PVER VAXIS ANG1 ANG2"' /
     &  '     Create a family of NUMCONE nested circular cones',
     &  ' CNNAME, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     with the same vertex point PVER and axis vector VAXIS,' /
     &  '     and with vertex half-angles equally spaced from ANG1',
     &  ' to ANG2.' //
     &  '  "cone CONE02 move NUMCONE INC CONE01 VMOVE"' /
     &  '     Create a family of NUMCONE circular cones CONE02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     and spaced at intervals of vector VMOVE,' /
     &  '     starting from circular cone CONE01.' )

 8114 format (/
     &  '  "cone CONE02 rotate NUMCONE INC CONE01 OPNAME PINV"' /
     &  '     Create a family of NUMCONE circular cones CONE02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     by rotating the preceding circular cone with operator',
     &  ' OPNAME' /
     &  '     and invariant point PINV, starting from circular cone',
     &  ' CONE01.' //
     &  '  "cone CONE02 scale NUMCONE INC CONE01 OPNAME PINV"' /
     &  '     Create a family of NUMCONE cones CONE02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     by scaling the preceding cone with operator OPNAME and' /
     &  '     invariant point PINV, starting from circular cone',
     &  ' CONE01.' /
     &  '     WARNING:  a linear or radial scaling axis not parallel' /
     &  '     to that of CONE01 will produce elliptic cones.' //
     &  '  Type <return> for more help.' )

 8115 format (a)

 8116 format (/
     &  '  Note:  use +/- for CNNAME or CONE02 to incr/decr the last' /
     &  '    circular cone name:  ',a //
     &  '  Synonyms:  [help, h], [line, ln, l]',
     &  ' [move, mv, translate, trans],' /
     &  '    [point, pnt, pt, p], [rotate, rot], [scale, sc].' )

      if (nword .eq. 0) go to 999

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        write (   6, 8114)
        write (iout, 8114)
        if (ainput .eq. 'tty') then
          read  (   5, 8115) ans
        endif
        write (   6, 8116) acones
        write (iout, 8116) acones
        go to 999
      endif

c=======================================================================********

c.... Input line:  cone all
c.... Input line:  cone list

 8120 format (/ 'Number of cones = ',i5)

      if ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'all')    .or.
     &    (aword(2) .eq. 'list'))) then

        nconex = 0
        do 120 n = 1, nquads
          if (aqtype(n) .ne. aqtypes(12)) go to 120
          if (aquad(n) .eq. ' ') go to 120
          call quout (n, n, 'min', nerrq)
          nconex = nconex + 1
  120   continue
        write (   6, 8120) nconex
        write (iout, 8120) nconex
        go to 999
      endif

c=======================================================================********

c.... Input line:  cone list CNNAME1 CNNAME2 CNNAME3 ...

 8130 format ('  cone ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named cone does not exist:  ',a)

      if (aword(2) .eq. 'list') then
        do 128 nw = 3, nword
          nfind = 0
          do 125 n = 1, nquads
            if (aqtype(n) .ne. aqtypes(12)) go to 125
            call aptsubr (aquad(n), 1, 24, 80, astem, lstem,
     &                    asub, lsub, nerra)
            if ((aword(nw) .eq. aquad(n)) .or.
     &          (aword(nw) .eq. astem))   then
              nfind = nfind + 1
              call quout (n, n, 'min', nerrq)
            endif
  125     continue
          if (nfind .eq.0) then
            write (   6, 8130) ainput, ninput, aword(nw)
            write (iout, 8130) ainput, ninput, aword(nw)
          endif
  128   continue
        go to 999
      endif

c=======================================================================********

c.... Input line:  cone CNNAME

      if (nword .eq. 2) then
        do 130 n = 1, nquads
          if (aqtype(n) .ne. aqtypes(12)) go to 130
          if (aword(2) .eq. aquad(n)) then
            amount = 'maximum'
            call quout (n, n, amount, nerrq)
            go to 999
          endif
  130   continue
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
        go to 999
      endif

c=======================================================================********

c.... Test input for validity.

 8136 format ('  cone ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of cone not from 1 to 24 characters:  ',a)

      if ((lword(2) .lt. 1) .or. (lword(2) .gt. 24)) then
        write (   6, 8136) ainput, ninput, aword(2)
        write (iout, 8136) ainput, ninput, aword(2)
        go to 999
      endif

c=======================================================================********

c.... See if last cone name should be incremented and used.

 8128 format ('  cone ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last cone can not be incremented:  ',a)

      if (aword(2) .eq. '+') then
        aword(2) = acones
        lword(2) = lacones
        call incrsubs (1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8128) ainput, ninput, acones
          write (iout, 8128) ainput, ninput, acones
          go to 999
        endif
      endif

 8129 format ('  cone ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last cone can not be decremented:  ',a)

      if (aword(2) .eq. '-') then
        aword(2) = acones
        lword(2) = lacones
        call incrsubs (-1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8129) ainput, ninput, acones
          write (iout, 8129) ainput, ninput, acones
          go to 999
        endif
      endif

 8905 format ('  cone ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of cone too dangerous to use:  ',a)

      call namechk (aword(2), lword(2), nerrchk)

      if (nerrchk .ne. 0) then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Input line:  cone CNNAME nest NUMCONE INC PVER VAXIS ANG1 ANG2

      if (aword(3) .eq. 'nest') then

        nerr = 0

 8505   format ('  cone ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Not 9 words in input line:  ',i3)

        if (nword .ne. 9) then
          write (   6, 8505) ainput, ninput, nword
          write (iout, 8505) ainput, ninput, nword
          nerr = 1
        endif

 8520   format ('  cone ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word NUMCONE is less than 2:  ',a)
 8525   format ('  cone ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word NUMCONE is not an integer or integer variable:  ',a)

        call varfind (4, 4)

        if (mtype(4) .eq. 2) then
          numcone = iword(4)
          if (numcone .lt. 2) then
            write (   6, 8520) ainput, ninput, aword(4)
            write (iout, 8520) ainput, ninput, aword(4)
            nerr = 1
          endif
        else
          write (   6, 8525) ainput, ninput, aword(4)
          write (iout, 8525) ainput, ninput, aword(4)
          nerr = 1
        endif

 8510   format ('  cone ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word INC is zero.')
 8515   format ('  cone ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word INC is not an integer or integer variable:  ',a)

        call varfind (5, 5)

        if (mtype(5) .eq. 2) then
          inc = iword(5)
          if (inc .eq. 0) then
            write (   6, 8510) ainput, ninput
            write (iout, 8510) ainput, ninput
            nerr = 1
          endif
        else
          write (   6, 8515) ainput, ninput, aword(5)
          write (iout, 8515) ainput, ninput, aword(5)
          nerr = 1
        endif

c....   Find the vertex point of the cone.

        call ptfind (6, np)

        if (np .eq. 0) nerr = 1

c....   Find the axis vector of the cone.

        call vectfind (7, nv)

        if (nv .eq. 0) nerr = 1

c.... See if the axis vector has a real length.

        if (vlen(nv) .eq. 0.0) then
          write (   6, 8290) ainput, ninput, aword(5)
          write (iout, 8290) ainput, ninput, aword(5)
          nerr = 1
        endif

c....   Find the initial and final vertex half-angles, and the angle increment.

        call varfind (8, 9)

        if ((mtype(8) .ne. 2) .and. (mtype(8) .ne. 3)) then
          write (   6, 8300) ainput, ninput, aword(8)
          write (iout, 8300) ainput, ninput, aword(8)
          nerr = 1
        endif

        if ((mtype(9) .ne. 2) .and. (mtype(9) .ne. 3)) then
          write (   6, 8300) ainput, ninput, aword(9)
          write (iout, 8300) ainput, ninput, aword(9)
          nerr = 1
        endif

        fact = 1.0
        if (angunit .eq. 'degrees') fact = pi / 180.0
        ang1 = fact * fword(8)
        ang2 = fact * fword(9)
        dang = (ang2 - ang1) / (numcone - 1)

c....   See if the cone names can be generated uniquely.

 8570 format ('  cone ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not generate',i5,' cone names from:  ',a)

        anamex  = aword(2)
        lanamex = lword(2)
        namex   = inc * (numcone - 1)

        call incrsubs (namex, anamex, 1, lanamex, nerra)

        if (nerra .ne. 0) then
          write (   6, 8570) ainput, ninput, numcone, aword(2)
          write (iout, 8570) ainput, ninput, numcone, aword(2)
          nerr = 1
        endif

        call namechk (anamex, lanamex, nerrchk)

        if (nerrchk .ne. 0) then
          nerr = 1
          write (   6, 8905) ainput, ninput, anamex
          write (iout, 8905) ainput, ninput, anamex
        endif

c....   Find the operator to rotate the axis of the cone onto the z axis.

        if (nv .ne. 0) then           ! Vector exists.

          call aptrotv (xcom(nv), ycom(nv), zcom(nv), 0., 0., 1., tol,
     &                  rotm, nerra)

        endif

c....   See if all half-angles are between 0 and 180 degrees.

 8575 format ('  cone ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  A half-angle is 0 or less, or 180 or more degrees.')

        angerr = tol * (abs (ang1) + abs (ang2))
        if (abs (ang1) .le. angerr)      ang1 = 0.0
        if (abs (ang1 - pi) .le. angerr) ang1 = pi
        if (abs (ang2) .le. angerr)      ang2 = 0.0
        if (abs (ang2 - pi) .le. angerr) ang2 = pi

        if ((ang1 .le. 0.0) .or.
     &      (ang1 .ge. pi)  .or.
     &      (ang2 .le. 0.0) .or.
     &      (ang2 .ge. pi)) then
          write (   6, 8575) ainput, ninput
          write (iout, 8575) ainput, ninput
          nerr = 1
        endif

        if (nerr .ne. 0) go to 999

c....   Create the nested cones.

        anamex  = aword(2)            ! Initial cone name.
        lanamex = lword(2)            ! Initial cone name length.

        do 580 ncn = 1, numcone       ! Loop over the new cones.

          angle  = ang1 + (ncn - 1) * dang

          vcos = cos (angle)

c....     Find the index in the quadric surface list for the new cone.

          aword(2) = anamex
          lword(2) = lanamex

          call qustore (12, nrep, n)

          if (n .eq. 0) go to 999

          acones    = anamex
          lacones   = lanamex
          aquads    = anamex
          laquads   = lanamex
          axisyms   = anamex
          laxisyms  = lanamex
    
          aquad(n)  = anamex
          laquad(n) = lanamex
          aqtype(n) = aqtypes(12)
          asym(n)   = 'axisymmetric'

          qc(n)     = 0.0
          qx(n)     = 0.0
          qy(n)     = 0.0
          qz(n)     = 0.0
          qxy(n)    = 0.0
          qyz(n)    = 0.0
          qzx(n)    = 0.0
          qxx(n)    = vcos**2
          qyy(n)    = vcos**2
          qzz(n)    = vcos**2 - 1.0

c....     See if cone is actually a plane.

 8580 format ('  WARNING:  cone ',a,' changed to a plane.')

          if (abs (vcos) .le. tol) then
            qz(n)  = -1.0
            qxx(n) = 0.0
            qyy(n) = 0.0
            qzz(n) = 0.0
            write (   6, 8580) anamex
            write (iout, 8580) anamex
          endif
    
          xcen(n) = x(np)
          ycen(n) = y(np)
          zcen(n) = z(np)

c....     Rotate the cone back to the base axis.

          call aptrois (rotm, 1, 0., 0., 0., qc(n), qx(n), qy(n), qz(n),
     &                  qxy(n), qyz(n), qzx(n), qxx(n), qyy(n), qzz(n),
     &                  tol, nerra)

c....     Translate the cone back to the vertex point.

          call apttris (1, xcen(n), ycen(n), zcen(n),
     &                  qc(n), qx(n), qy(n), qz(n),
     &                  qxy(n), qyz(n), qzx(n),
     &                  qxx(n), qyy(n), qzz(n), tol, nerra)

          call quout (n, n, 'min', nerrq)

c....     Find the next cone name, by incrementing by inc.

          call incrsubs (inc, anamex, 1, lanamex, nerra)

          call namechk (anamex, lanamex, nerrchk)

          if (nerrchk .ne. 0) then
            write (   6, 8905) ainput, ninput, anamex
            write (iout, 8905) ainput, ninput, anamex
            go to 999
          endif

  580   continue                      ! End of loop over the new cones.

        go to 999

      endif                           ! Created nested cone(s).

c===============================================================================

c.... Input line:  cone CONE02 move NUMCONE INC CONE01 VMOVE

      if ((aword(3) .eq. 'move')      .or.
     &    (aword(3) .eq. 'mv')        .or.
     &    (aword(3) .eq. 'translate') .or.
     &    (aword(3) .eq. 'trans'))    then

        call qumove

        go to 999

      endif                           ! Created translated cone(s).

c===============================================================================

c.... Input line:  cone CONE02 rotate NUMCONE INC CONE01 OPNAME PINV
c.... Input line:  cone CONE02 scale  NUMCONE INC CONE01 OPNAME PINV

      if ((aword(3) .eq. 'scale')   .or.
     &    (aword(3) .eq. 'sc')      .or.
     &    (aword(3) .eq. 'rotate')  .or.
     &    (aword(3) .eq. 'rot'))    then

        call quoper

        go to 999

      endif                           ! Created rotated or scaled cone(s).

c===============================================================================

c.... Find the index in the quadric surface array at which to store this cone.
c....   Set nrep = 1 if replacing an existing quadric surface.

      call qustore (12, nrep, n)

      if (n .eq. 0) go to 999

      nerr = 0

c=======================================================================********

c.... Input line:  cone CNNAME angle PVER  VAXIS ANGLE
c.... Input line:  cone CNNAME point PVER  VAXIS PNAME2
c.... Input line:  cone CNNAME line  PAXIS VAXIS ALNAME

 8250 format ('  cone ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 6 words on "cone" input line',i4)

      if (nword .ne. 6) then
        write (   6, 8250) ainput, ninput, nword
        write (iout, 8250) ainput, ninput, nword
        nerr = 1
      endif

c.... Find the point on the axis of the cone.

      call ptfind (4, np)

      if (np .eq. 0) then
        nerr = 1
        go to  270
      endif

      xpa = x(np)
      ypa = y(np)
      zpa = z(np)

c.... Find the axis vector of the cone.

  270 call vectfind (5, nv)

      if (nv .eq. 0) nerr = 1

c.... See if the axis vector has a real length.

 8290 format ('  cone ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Axis vector has zero length:  ',a)

      if (vlen(nv) .eq. 0.0) then
        nerr = 1
        write (   6, 8290) ainput, ninput, aword(5)
        write (iout, 8290) ainput, ninput, aword(5)
      endif

c-----------------------------------------------------------------------********

      if (aword(3) .eq. 'angle') then ! Vertex and angle specified.

c.... Input line:  cone CNNAME angle PVER VAXIS ANGLE

c.... Find the angle of the cone.

 8300 format ('  cone ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer or floating point value or variable:  ',a)

      call varfind (6, 6)

      if ((mtype(6) .ne. 2) .and. (mtype(6) .ne. 3)) then
        write (   6, 8300) ainput, ninput, aword(6)
        write (iout, 8300) ainput, ninput, aword(6)
        nerr = 1
      endif

  310 fact = 1.0
      if (angunit .eq. 'degrees') fact = pi / 180.0

c.... See if the half-angle is greater than 0 and less than 180 degrees.

      angerr = tol * fact * abs (fword(6))
      angle  = fact * fword(6)
      if (abs (angle) .le. angerr) angle = 0.0
      if (abs (angle - pi) .le. angerr) angle = pi

 8310 format ('  cone ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Angle is 0 or less, or 180 or more degrees.')

      if ((angle .le. 0.0) .or.
     &    (angle .ge. pi)) then
        write (   6, 8310) ainput, ninput
        write (iout, 8310) ainput, ninput
        nerr = 1
      endif

      vcos = cos (angle)

      if (nerr .ne. 0) then
        if (nrep .eq. 0) then
          aquad(n) = ' '
          if (n .eq. nquads) nquads = nquads - 1
        endif
        go to 999
      endif

c.... Find the coefficients of the quadric equation for the cone, in
c....   standard form.

      acones    = aword(2)
      lacones   = lword(2)
      aquads    = aword(2)
      laquads   = lword(2)
      axisyms   = aword(2)
      laxisyms  = lword(2)

      aquad(n)  = aword(2)
      laquad(n) = lword(2)
      qc(n)     = 0.0
      qx(n)     = 0.0
      qy(n)     = 0.0
      qz(n)     = 0.0
      qxy(n)    = 0.0
      qyz(n)    = 0.0
      qzx(n)    = 0.0
      qxx(n)    = vcos**2
      qyy(n)    = vcos**2
      qzz(n)    = vcos**2 - 1.0

c.... See if cone is actually a plane.

 8320 format ('  WARNING:  cone ',a,' changed to a plane.')

      if (abs (vcos) .le. tol) then
        qz(n)  = -1.0
        qxx(n) = 0.0
        qyy(n) = 0.0
        qzz(n) = 0.0
        write (   6, 8320) aword(2)
        write (iout, 8320) aword(2)
      endif

      xvercn = xpa
      yvercn = ypa
      zvercn = zpa

      go to 360

      endif                           ! Vertex and angle specified.

c-----------------------------------------------------------------------********

      if (aword(3) .eq. 'fit') then

c.... Input line:  cone CNNAME fit PVER VAXIS PNAME2

c.... Find the point on the surface of the cone.

      call ptfind (6, np)

      if (np .eq. 0) then
        nerr = 1
        go to 330
      endif

      x1 = x(np)
      y1 = y(np)
      z1 = z(np)

c.... Find the cosine and sine of the half-angle at the vertex.

  330 dsx = x1 - xpa
      dsy = y1 - ypa
      dsz = z1 - zpa

      call aptvusz (dsx, dsy, dsz, tol, ux, uy, uz, vlength)

 8330 format ('  cone ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Surface point coincides with vertex point.')

      if (vlength .eq. 0.0) then
        nerr = 1
        write (   6, 8330) ainput, ninput
        write (iout, 8330) ainput, ninput
      endif

      call aptvang (xcom(nv), ycom(nv), zcom(nv), dsx, dsy, dsz,
     &              1, tol, vcos, nerra)

      if (nerr .ne. 0) then
        if (nrep .eq. 0) then
          aquad(n) = ' '
          if (n .eq. nquads) nquads = nquads - 1
        endif
        go to 999
      endif

c.... Find the coefficients of the quadric equation for the cone, in
c....   standard form.

      acones    = aword(2)
      lacones   = lword(2)
      axisyms   = aword(2)
      laxisyms  = lword(2)
      aquads    = aword(2)
      laquads   = lword(2)

      aquad(n)  = aword(2)
      laquad(n) = lword(2)
      qc(n)     = 0.0
      qx(n)     = 0.0
      qy(n)     = 0.0
      qz(n)     = 0.0
      qxy(n)    = 0.0
      qyz(n)    = 0.0
      qzx(n)    = 0.0
      qxx(n)    = vcos**2
      qyy(n)    = vcos**2
      qzz(n)    = vcos**2 - 1.0

      xvercn = xpa
      yvercn = ypa
      zvercn = zpa

      go to 360

      endif                           ! Vertex and angle specified.

c-----------------------------------------------------------------------********

      if ((aword(3) .eq. 'line') .or.
     &    (aword(3) .eq. 'ln')   .or.
     &    (aword(3) .eq. 'l'))   then

c.... Input line:  cone CNNAME line PAXIS VAXIS ALNAME

c.... Find the line on the surface of the cone, and its endpoints.

 8340 format ('  cone ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named line does not exist:  ',a)
 8335 format ('  cone ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named point does not exist:  ',a)

      do 340 nn = 1, nlines
        if (aword(6) .eq. aline(nn)) then

          call lout (nn, nn)

          do 332 np = 1, npoints
            if (alipt1(nn) .eq. apoint(np)) then
              x1 = x(np)
              y1 = y(np)
              z1 = z(np)
              go to 334
            endif
  332     continue

          nerr = 1
          write (   6, 8335) ainput, ninput, alipt1(nn)
          write (iout, 8335) ainput, ninput, alipt1(nn)

  334     do 336 np = 1, npoints
            if (alipt2(nn) .eq. apoint(np)) then
              x2 = x(np)
              y2 = y(np)
              z2 = z(np)
              go to 350
            endif
  336     continue

          nerr = 1
          write (   6, 8335) ainput, ninput, alipt2(nn)
          write (iout, 8335) ainput, ninput, alipt2(nn)

          go to 350
        endif
  340 continue
      nerr = 1
      write (   6, 8340) ainput, ninput, aword(6)
      write (iout, 8340) ainput, ninput, aword(6)

  350 dsx = x2 - x1
      dsy = y2 - y1
      dsz = z2 - z1

      call aptvusz (dsx, dsy, dsz, tol, ux, uy, uz, vlength)

 8350 format ('  cone ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Line in surface has zero length.')

      if (vlength .eq. 0.0) then
        nerr = 1
        write (   6, 8350) ainput, ninput
        write (iout, 8350) ainput, ninput
      endif

c.... Find the operator to rotate the cone axis to the z axis.

      call aptrotv (xcom(nv), ycom(nv), zcom(nv), 0., 0., 1., tol,
     &              rotm, nerra)

c.... Rotate and translate the endpoints of the line to the coordinate system
c....   with the cone axis on the z axis.

      call aptmopv (rotm, 0, xpa, ypa, zpa, x1, y1, z1,
     &              1, tol, nerra)

      call aptmopv (rotm, 0, xpa, ypa, zpa, x2, y2, z2,
     &              1, tol, nerra)

      call apttran (xpa, ypa, zpa, x1, y1, z1, 1, tol,
     &              nerra)

      call apttran (xpa, ypa, zpa, x2, y2, z2, 1, tol,
     &              nerra)

c.... Find the vertex point, the intersection of the line with the axis.

      rcyl1 = sqrt (x1**2 + y1**2)
      rcyl2 = sqrt (x2**2 + y2**2)

      call aptlnlc (0., 0., 0., 1., rcyl1, z1, rcyl2, z2,
     &              1, tol, dpmin, fracab, fraccd, xvercn, zvercn, ipar,
     &              nerra)

      yvercn = 0.0

 8355 format ('  cone ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Line in surface does not define a cone.')

      if (ipar .ne. 0) then
        nerr = 1
        write (   6, 8355) ainput, ninput
        write (iout, 8355) ainput, ninput
      endif

      if (nerr .ne. 0) then
        if (nrep .eq. 0) then
          aquad(n) = ' '
          if (n .eq. nquads) nquads = nquads - 1
        endif
        go to 999
      endif

c.... Find the coefficients of the quadric equation for the cone, in
c....   standard form.


      acones    = aword(2)
      lacones   = lword(2)
      aquads    = aword(2)
      laquads   = lword(2)
      axisyms   = aword(2)
      laxisyms  = lword(2)

      aquad(n)  = aword(2)
      laquad(n) = lword(2)
      qc(n)     = 0.0
      qx(n)     = 0.0
      qy(n)     = 0.0
      qz(n)     = 0.0
      qxy(n)    = 0.0
      qyz(n)    = 0.0
      qzx(n)    = 0.0
      qxx(n)    = (z2 - z1)**2
      qyy(n)    = (z2 - z1)**2
      qzz(n)    = -(rcyl2 - rcyl1)**2

c.... Translate and rotate the vertex back to the original coordinate system.

      xtran = -xpa
      ytran = -ypa
      ztran = -zpa

      call apttran (xtran, ytran, ztran, xvercn, yvercn, zvercn, 1, tol,
     &              nerra)

      call aptmopv (rotm, 1, xpa, ypa, zpa, xvercn, yvercn, zvercn,
     &              1, tol, nerra)

      go to 360

      endif                           ! Vertex and angle specified.

c-----------------------------------------------------------------------********

c.... Find the operator to rotate the z axis to the axis of the cone.

  360 call aptrotv (0., 0., 1., xcom(nv), ycom(nv), zcom(nv), tol,
     &              rotm, nerra)

c.... Rotate the cone to the specified axis.

      call aptrois (rotm, 0, 0., 0., 0., qc(n), qx(n), qy(n), qz(n),
     &              qxy(n), qyz(n), qzx(n), qxx(n), qyy(n), qzz(n),
     &              tol, nerra)

c.... Translate the vertex of the cone to the specified point.

      call apttris (1, xvercn, yvercn, zvercn, qc(n),
     &              qx(n), qy(n), qz(n), qxy(n), qyz(n), qzx(n),
     &              qxx(n), qyy(n), qzz(n), tol, nerra)

      amount = 'maximum'
      call quout (n, n, amount, nerrq)

c=======================================================================********

  999 return

c.... End of subroutine cone.         (+1 line)
      end
