      subroutine cuts

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  cuts     Updated     1999 September 11 14:00.
c               cuts     Originated  1999 August 25 16:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c
c     Purpose:  For a triangle with specified edge lengths, to find each
c               straight line cut that cuts the triangle into two sections
c               with equal perimeters and equal areas (there may be 0, 1 or 2
c               such cuts), and for each such cut, to find the fractional
c               and absolute distances along the cut edges from their shared
c               vertex to the end points of the cut, and the length of the
c               cutting line.
c
c     Logic:    Entry points:  cuts.
c cuts          Called by:  helpall, main.
c
c cuts          Calls:  aptrabc, aptrich, varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      dimension    dcut1(3)           ! Distance of endpoint 1 from vertex.
      dimension    dcut2(3)           ! Distance of endpoint 2 from vertex.
      dimension    dcut3(3)           ! Length of cutting line.
      dimension    fcut1(3)           ! Fractional distance of endpoint 1.
      dimension    fcut2(3)           ! Fractional distance of endpoint 2.
cbugc***DEBUG begins.
cbug      dimension    fa(2)
cbug      dimension    fb(2)
cbug      dimension    aa(2)
cbug      dimension    bb(2)
cbug      dimension    cc(2)
cbugc***DEBUG ends.

      character*1  acut1(3)           ! Name of vertex or edge of endpoint 1.
      character*1  acut2(3)           ! Name of vertex or edge of endpoint 2.

c=======================================================================********

c.... Input line:  cuts [help]

 8110 format ('  Command "cuts":  acceptable input lines include:' /
     &  '  "cuts [help]"           Display this message.' //
     &  '  "cuts VA VB VC"' /
     &  '   Find the number and location of any straight line cuts',
     &  ' of the triangle' /
     &  '     with edges VA, VB and VC, that partitions the triangle',
     &  ' into sections' /
     &  '     with equal perimeters and areas.' //
     &  '  Synonyms:  [help, h].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c=======================================================================********

c.... Input line:  cuts VA VB VC

 8010 format ('  cuts ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 4 words on "cuts" line.' )

      nerr = 0

      if (nword .ne. 4) then
        write (   6, 8010) ainput, ninput
        write (iout, 8010) ainput, ninput
        nerr = 1
      endif

c.... Find lengths of triangle edges.

      call varfind (2, nword)

c.... Verify that all are integer or floating point.

 8130 format ('  cuts ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer or floating point value or variable:  ',a)

      do 110 nw = 2, nword
        if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
          write (   6, 8130) ainput, ninput, aword(nw)
          write (iout, 8130) ainput, ninput, aword(nw)
          nerr = 1
        endif
  110 continue

      a = fword(2)
      b = fword(3)
      c = fword(4)

 8128 format ('  Edge lengths abc',1p3e20.12 )

      write (   6, 8128) a, b, c
      write (iout, 8128) a, b, c

c.... Test for a good triangle.

      call aptrich (a, b, c, tol, nerra)

 8115 format ('Not a triangle.  Edge(s) too short or negative.')
 8116 format ('Not a triangle.  Linear or edge(s) too long.')

      if (nerra .eq. 1) then
        write (   6, 8115)
        write (iout, 8115)
        nerr = 1
      endif

      if (nerra .eq. 2) then
        write (   6, 8116)
        write (iout, 8116)
        nerr = 2
      endif

      if (nerr .ne. 0) go to 999

c.... Find all cuts.

      call aptrabc (a, b, c, tol, ncut, acut1, fcut1, dcut1,
     &              acut2, fcut2, dcut2, dcut3, nerr)

 8210     format (/ 'Lines that cut triangle (with edges labeled',
     &     ' a, b, and c, and opposite' /
     &     '  vertices labeled A, B and C) into figures with equal',
     &     ' perimeters and areas:' )

          write (   6, 8210)
          write (iout, 8210)

 8220     format (/ 'Cut from ',a1,' to ',a1,'.')
 8230     format ('  Frac dist 1,2 = ',1p2e20.12)
 8240     format ('  Dist 1, 2, cut= ',1p3e20.12)

          do 420 nn = 1, ncut

            write (   6, 8220) acut1(nn), acut2(nn)
            write (iout, 8220) acut1(nn), acut2(nn)

            write (   6, 8230) fcut1(nn), fcut2(nn)
            write (iout, 8230) fcut1(nn), fcut2(nn)

            write (   6, 8240) dcut1(nn), dcut2(nn), dcut3(nn)
            write (iout, 8240) dcut1(nn), dcut2(nn), dcut3(nn)

  420     continue

c=======================================================================********

cbugc***DEBUG begins.
cbugc.... Test subroutine aptrcab.
cbug 
cbug      call aptrcab (a, b, c, tol, ncut, fa, aa, fb, bb, cc, nerr)
cbug
cbug 9918 format ('ncut =',i2 /
cbug     &        '  fa, fb       =  ',1p2e20.12 /
cbug     &        '  aa,bb,cc     =  ',1p3e20.12 )
cbug
cbug      if (ncut .gt. 0) then
cbug        do 440 n = 1, ncut
cbug          write (   6, 9918) n, fa(n), fb(n), aa(n), bb(n), cc(n)
cbug          write (iout, 9918) n, fa(n), fb(n), aa(n), bb(n), cc(n)
cbug  440   continue
cbug      endif
cbug 
cbug      call aptrcab (b, c, a, tol, ncut, fa, aa, fb, bb, cc, nerr)
cbug
cbug      if (ncut .gt. 0) then
cbug        do 450 n = 1, ncut
cbug          write (   6, 9918) n, fa(n), fb(n), aa(n), bb(n), cc(n)
cbug          write (iout, 9918) n, fa(n), fb(n), aa(n), bb(n), cc(n)
cbug  450   continue
cbug      endif
cbug 
cbug      call aptrcab (c, a, b, tol, ncut, fa, aa, fb, bb, cc, nerr)
cbug
cbug      if (ncut .gt. 0) then
cbug        do 460 n = 1, ncut
cbug          write (   6, 9918) n, fa(n), fb(n), aa(n), bb(n), cc(n)
cbug          write (iout, 9918) n, fa(n), fb(n), aa(n), bb(n), cc(n)
cbug  460   continue
cbug      endif
cbugc***DEBUG ends.

c=======================================================================********

  999 return

c.... End of subroutine cuts.       (+1 line)
      end
