      subroutine delim

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  delim    Updated     2004 November 23 13:30.
c               delim    Originated  1994 September 1 10:20.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To specify a new delimiter for interpreting input lines.
c
c     Logic:    Entry points:  delim.
c delim         Called by:  helpall, main.
c
c delim         Calls:  aptchfp, aptchmv, aptchsq.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*1  afds               ! Old delimiter.

      character*64 asrce              ! Characters $, %, @, A-Z, a-z.

c=======================================================================********

c.... Initialize.

      afds = afd

c=======================================================================********

c.... Input line:  delimiter

      if (nword .le. 1) then
        afd = ' '
        go to 140
      endif

c=======================================================================********

c.... Input line:  delimiter help     (When the current field delimiter is " ".)
c.... Input line:  delimiterChelp     (When the current field delimiter is "C".)

 8110 format ('  Command "delimiter":  acceptable input lines include:' //
     &  '  "delimiter help"  Display this message.' //
     &  '  "delimiter"' /
     &  '     Restore the field delimiter to a blank character.' //
     &  '  "delimiter',a1,'C"' /
     &  '     Change the field delimiter to the character "C".' /
     &  '     which may be any of the following characters:' /
     &  '     (blank) ,  :  <  >  [  \  ]  ^  _  `  {  |  }' //
     &  '  Synonyms:  [delimiter, delimit, delim], [help, h].' )

      if ((nword .eq. 2)          .and.
     &   ((aword(2) .eq. 'help')  .or.
     &    (aword(2) .eq. 'h')))    then
        write (   6, 8110) afd
        write (iout, 8110) afd
        go to 999
      endif

c=======================================================================********

c.... Test input for validity.

 8120 format ('  delim ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Too many words on "delimiter" line:  ',i3)

      do 120 nw = 3, nword
        if (aword(nw) .ne. ' ') then
          write (   6, 8120) ainput, ninput, nword
          write (iout, 8120) ainput, ninput, nword
          go to 999
        endif
  120 continue

c=======================================================================********

c.... Input line:  delimiter D        (When the current field delimiter is " ".)
c.... Input line:  delimiterCD        (When the current field delimiter is "C".)

c.... Find the new field delimiter.

      call aptchsq (aword(2), 1, 80, ' ', nout, nerra)

 8130 format ('  delim ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  More than one character in field delimiter:  ',a)

      if (nout .gt. 1) then
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
        go to 999
      endif

c.... Test for acceptability.

 8128 format ('  delim ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Delimiter may only be blank or (),:<>[\]^_`{|}:  ',a)

      asrce = ' ,:<>[\]^_`{|}'

      call aptchfp (asrce, 1, 24, aword(2), 1, ipat, nerr)

      if (ipat .eq. 0) then
        write (   6, 8128) ainput, ninput, aword(2)
        write (iout, 8128) ainput, ninput, aword(2)
        go to 999
      endif

c.... Store in afd.

      afd  = ' '

      call aptchmv (aword(2), 1, 1, afd, 1, nerra)

c=======================================================================********

c.... Print the old and new delimiter.

 8135 format ('  The field delimiter was    a blank character.')
 8140 format ('  The field delimiter was    "',a1,'".')
 8145 format ('  The field delimiter is now a blank character.')
 8150 format ('  The field delimiter is now "',a1,'".')
 8160 format ('  Type "delim" to restore it to a blank character.' /
     &  '  Type "delim',a1,'help" for display of a help message.' )

  140 if (afd .ne. afds) then
        if (afds .eq. ' ') then
          write (   6, 8135)
          write (iout, 8135)
        else
          write (   6, 8140) afds
          write (iout, 8140) afds
        endif
      endif

      if (afd .eq. ' ') then
        write (   6, 8145)
        write (iout, 8145)
      else
        write (   6, 8150) afd
        write (iout, 8150) afd
      endif

      if (afd .ne. ' ') then
        write (   6, 8160) afd
        write (iout, 8160) afd
      endif

c=======================================================================********

  999 return

c.... End of subroutine delim.        (+1 line)
      end
