      subroutine distance

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  distance Updated     2004 November 23 13:30.
c               distance Originated  1993 November 3 13:50.
c
c     Author:   Arthur L. Edwards, L-298, x-21903.
c
c     Purpose:  To find the distance between:
c               a point and another point or a cluster of points;
c               a cluster of points and another cluster of points,
c               a point or cluster of points and a line, a triangle, a plane,
c                 sphere, cylinder, cone, hyperbolic paraboloid, ellipsoid,
c                 axisymmetric quadric surface, or general quadric surface;
c               a point and the bounding surfaces of a brick, a tetrahedron
c                 or a zone;
c
c               or to find the distances and/or intersections between:
c               a line and another line, a triangle or a plane;
c               a triangle and another triangle or a plane;
c               a plane and another plane, a sphere or a circular cylinder;
c               a plane and a general quadric surface;
c               a sphere and another sphere or a circular cylinder;
c               a circular cylinder and another circular cylinder.
c
c     Logic:    Entry points:  distance.
c distance      Called by:  helpall, main.
c
c distance      Calls:  aptlnln, aptlnpl, aptplpl, aptptln, aptptpl, aptripl,
c               aptvdis, aptvpln, aptvunb, aptvusz, brout, clout, coord,
c               intcyl, intplcy, intplqu, intpls, intquad, intspcy, intsph,
c               lout, plotpt, plout, pout, ptbrick, ptdist, ptfind, pttet,
c               ptzone, quout, tetout, trout, zout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      common /ccdist/ asysx           ! Coordinate system.
      character*8     asysx           ! Coordinate system.
      common /ccdist/ amount          ! Amount of output data.
      character*8     amount          ! Amount of output data.
      common /ccdist/ ageom1          ! First object type.
      character*24    ageom1          ! First object type.
      common /ccdist/ ageom2          ! Second object type.
      character*24    ageom2          ! Second object type.
      common /ccdist/ dside(nzquadm)  ! Value of F(x,y,z) at point.

c.... Input line:  distance [help]

 8110 format ('  Command "distance":  acceptable input lines include:' /
     &  '  "distance [help]"  Display this message.' //
     &  '  "distance point PNAME1 point PNAME2"' /
     &  '     Find the distance from point PNAME1 to point PNAME2.' /
     &  '  "distance point PNAME cluster CLNAME"' /
     &  '     Find the distances from point PNAME1 to the points',
     &  ' in cluster CLNAME.' /
     &  '  "distance cluster CLNAME1 cluster CLNAME2"' /
     &  '     Find the distances from the points in cluster CLNAME1' /
     &  '     to the points in cluster CLNAME2.' //
     &  '  "distance point PNAME line ALNAME"' /
     &  '     Find the distance from point PNAME to line ALNAME.' /
     &  '  "distance cluster CLNAME line ALNAME"' /
     &  '     Find the distance from the points in cluster CLNAME',
     &  ' to line ALNAME.' //
     &  '  "distance point PNAME triangle TRNAME"' /
     &  '     Find the distance from point PNAME to triangle',
     &  ' TRNAME.' )

 8111 format (
     &  '  "distance cluster CLNAME triangle TRNAME"' /
     &  '     Find the distance from the points in cluster CLNAME',
     &  ' to triangle TRNAME.' //
     &  '  "distance point PNAME QTYPE QNAME"' /
     &  '     Find the distance from point PNAME to quadric type',
     &  ' QTYPE with name QNAME,' /
     &  '     where QTYPE is plane, sphere, cylinder, cone, hyperb,',
     &  ' ellipsoid,' /
     &  '     axisym, quadric.' /
     &  '  "distance cluster CLNAME QTYPE QNAME"' /
     &  '     Find the distance from the points in cluster CLNAME',
     &  ' to quadric type QTYPE' /
     &  '     with name QNAME,' /
     &  '     where QTYPE is plane, sphere, cylinder',
     &  ' cone, ellipsoid, axisym, quadric.' )

 8112 format (/
     &  '  "distance point PNAME brick BRNAME"' /
     &  '     Find the distances from point PNAME to the six surfaces' /
     &  '     bounding brick BRNAME.' //
     &  '  "distance point PNAME tetrahedron TETNAME"' /
     &  '     Find the distances from point PNAME to the four planes' /
     &  '     bounding tetrahedron TETNAME.' //
     &  '  "distance point PNAME zone ZNAME"' /
     &  '     Find the distances from point PNAME to the quadric',
     &  ' surfaces' /
     &  '     bounding zone ZNAME.' //
     &  '  Type <return> for more help.' )

 8113 format (a)

 8114 format (/
     &  '  "distance line ALNAME1 line ALNAME2"' /
     &  '     Find the distance from line ALNAME1 to line ALNAME2.' /
     &  '  "distance line ALNAME triangle TRNAME"' /
     &  '     Find the intersection of line ALNAME with triangle',
     &  ' TRNAME.' /
     &  '  "distance line ALNAME plane PLNAME"' /
     &  '     Find the intersection of line ALNAME with plane',
     &  ' PLNAME.' //
     &  '  "distance triangle TRNAME1 triangle TRNAME2"' /
     &  '     Find the intersection of triangle TRNAME1 with triangle',
     &  ' TRNAME2.' /
     &  '  "distance triangle TRNAME plane PLNAME"' /
     &  '     Find the intersection of triangle TRNAME with plane',
     &  ' PLNAME.' //
     &  '  "distance plane PLNAME1 plane PLNAME2"' /
     &  '     Find distance/intersection of plane PLNAME1 and plane',
     &  ' PLNAME2.' )

 8115 format (
     &  '  "distance plane PLNAME QTYPE QNAME"' /
     &  '     Find distance/intersection of plane PLNAME and quadric',
     &  ' type QTYPE' /
     &  '     with name QNAME, where QTYPE is plane, sphere, cylinder,',
     &  ' cone,' /
     &  '     ellipsoid, axisym or quadric.' //
     &  '  "distance sphere SPHNAME1 sphere SPHNAME2"' /
     &  '     Find distance/intersection of spheres SPHNAME1 and',
     &  ' SPHNAME2.' /
     &  '  "distance sphere SPHNAME cylinder CYLNAME"' /
     &  '     Find distance/intersection of sphere SPHNAME and',
     &  ' circular cylinder CYLNAME.' /
     &  '  "distance cylinder CYLNAME1 cylinder CYLNAME2"' /
     &  '     Find distance/intersection of circular cylinders',
     &  ' CYLNAME1 and CYLNAME2.' //
     &  '  "distance quadric QNAME1 quadric QNAME2"' /
     &  '     Find distance/intersection of quadric surfaces',
     &  ' QNAME1 and QNAME2.' )

 8116 format (/
     &  '  Synonyms:  [brick, br], [cluster, cl], [cylinder, cyl],' /
     &  '    [distance, dist, intersect, inter, int],' /
     &  '    [ellipsoid, ellipse, ell, el], [help, h],',
     &  ' [hyperb, saddle], [line, ln, l],' /
     &  '    [plane, pl], [point, pnt, pt, p], [quadric, quad, q]',
     &  ' [sphere, sph].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8111)
        write (iout, 8111)
        write (   6, 8112)
        write (iout, 8112)
        if (ainput .eq. 'tty') then
          read  (   5, 8113) ans
        endif
        write (   6, 8114)
        write (iout, 8114)
        write (   6, 8115)
        write (iout, 8115)
        write (   6, 8116)
        write (iout, 8116)
        go to 999
      endif

c===============================================================================

c.... Test input for validity.

 8118 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on input line not 5:  ',i3)
      if (nword .ne. 5) then
        write (   6, 8118) ainput, ninput, nword
        write (iout, 8118) ainput, ninput, nword
        go to 999
      endif

c===============================================================================

c.... Initialize.

      nerr   = 0
      nerra  = 0
      nerrb  = 0
      nerrc  = 0

      proxptx = -1.e99
      proxpty = -1.e99
      proxptz = -1.e99

      proxvx = -1.e99
      proxvy = -1.e99
      proxvz = -1.e99

c===============================================================================

c.... Find the geometric objects.

      ageom1 = aword(2)
      ageom2 = aword(4)

c===============================================================================

c.... See if the first geometric object is a point.

      if ((aword(2) .eq. 'point') .or.
     &    (aword(2) .eq. 'pnt')   .or.
     &    (aword(2) .eq. 'pt')    .or.
     &    (aword(2) .eq. 'p'))    then  ! Find the index of the point.

        call ptfind (3, np1)

        if (np1 .eq. 0) then
          nerr = 1
          go to 180
        endif

        ageom1 = 'point'

      endif                           ! Tested first object type for point.

c===============================================================================

c.... See if the first geometric object is a cluster.

 8120   format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  There are no clusters.')

 8125   format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Named cluster point does not exist:  ',a)

 8130   format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Named cluster does not exist:  ',a)

      if ((aword(2) .eq. 'cluster') .or.
     &    (aword(2) .eq. 'cl'))     then

        ageom1 = 'cluster'

        if (nclusts .eq. 0) then
          write (   6, 8120) ainput, ninput
          write (iout, 8120) ainput, ninput
          go to 999
        endif

        do 120 ncl = 1, nclusts       ! Loop over clusters.
          if (aword(3) .eq. aclust(ncl)) then

            call clout (ncl, ncl, 'min')

            ncl1 = ncl

            do 118 npr = 1, npairs    ! Loop over cluster pairs.
              if (apaircl(npr) .eq. aclust(ncl)) then

                do 116 np = 1, npoints  ! Loop over points.
                  if (apoint(np) .eq. apairpt(npr)) then

cbug                  call pout (np, np, 'min')

                    go to 118         ! Find next point in cluster.
                  endif
  116           continue              ! End of loop over points.

                write (   6, 8125) ainput, ninput, apairpt(npr)
                write (iout, 8125) ainput, ninput, apairpt(npr)
                go to 999

              endif

  118       continue                  ! End of loop over cluster pairs.

            go to 180                 ! Cluster ok, all cluster points ok.

          endif

  120   continue                      ! End of loop over clusters.

        write (   6, 8130) ainput, ninput, aword(3)
        write (iout, 8130) ainput, ninput, aword(3)
        go to 999

      endif                           ! Tested object type for cluster.

c===============================================================================

c.... See if the first geometric object is a line.

 8135 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named line does not exist:  ',a)
 8140 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  End point of line does not exist:  ',a)

      if ((aword(2) .eq. 'line') .or.
     &    (aword(2) .eq. 'ln')   .or.
     &    (aword(2) .eq. 'l'))   then ! Find the index of the line.

        ageom1 = 'line'
        do 128 n = 1, nlines
          if (aword(3) .eq. aline(n)) then
            nl1 = n
            call lout (nl1, nl1)

            do 122 np = 1, npoints
              if (alipt1(nl1) .eq. apoint(np)) then
                xline11 = x(np)
                yline11 = y(np)
                zline11 = z(np)
                go to 124
              endif
  122       continue
            nerr = 1
            write (   6, 8140) ainput, ninput, alipt1(nl1)
            write (iout, 8140) ainput, ninput, alipt1(nl1)

  124       do 126 np = 1, npoints
              if (alipt2(nl1) .eq. apoint(np)) then
                xline12 = x(np)
                yline12 = y(np)
                zline12 = z(np)
                go to 180
              endif
  126       continue
            nerr = 1
            write (   6, 8140) ainput, ninput, alipt2(nl1)
            write (iout, 8140) ainput, ninput, alipt2(nl1)

            go to 180
          endif
  128   continue
        nerr = 1
        write (   6, 8135) ainput, ninput, aword(3)
        write (iout, 8135) ainput, ninput, aword(3)
        go to 180
      endif

c===============================================================================

c.... See if the first geometric object is a triangle.

 8150 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named triangle does not exist:  ',a)
 8160 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Vertex point does not exist:  ',a)

      if ((aword(2) .eq. 'triangle') .or.
     &    (aword(2) .eq. 'tri'))  then  ! Find the index of the triangle.

        ageom1 = 'triangle'
        do 160 n = 1, ntrigs
          if (aword(3) .eq. atrig(n)) then
            ntr1 = n

            do 130 np = 1, npoints
              if (apoint(np) .eq. atript1(ntr1)) then
                xtra = x(np)
                ytra = y(np)
                ztra = z(np)
                go to 135
              endif
  130       continue
            nerr = 1
            write (   6, 8160) ainput, ninput, atript1(ntr1)
            write (iout, 8160) ainput, ninput, atript1(ntr1)

  135       do 140 np = 1, npoints
              if (apoint(np) .eq. atript2(ntr1)) then
                xtrb = x(np)
                ytrb = y(np)
                ztrb = z(np)
                go to 145
              endif
  140       continue
            nerr = 1
            write (   6, 8160) ainput, ninput, atript2(ntr1)
            write (iout, 8160) ainput, ninput, atript2(ntr1)

  145       do 150 np = 1, npoints
              if (apoint(np) .eq. atript3(ntr1)) then
                xtrc = x(np)
                ytrc = y(np)
                ztrc = z(np)
                go to 155
              endif
  150       continue
            nerr = 1
            write (   6, 8160) ainput, ninput, atript3(ntr1)
            write (iout, 8160) ainput, ninput, atript3(ntr1)

  155       amount = 'maximum'
            call trout (ntr1, ntr1, amount)
            go to 180
          endif
  160   continue
        nerr = 1
        write (   6, 8150) ainput, ninput, aword(3)
        write (iout, 8150) ainput, ninput, aword(3)
        go to 180
      endif

c===============================================================================

c.... See if the first geometric object is a plane.

 8170 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named plane does not exist:  ',a)

      if ((aword(2) .eq. 'plane') .or.
     &    (aword(2) .eq. 'pl'))   then  ! Find the index of the plane.

        ageom1 = 'plane'
        do 170 n = 1, nquads
          if (aqtype(n) .ne. aqtypes(0)) go to 170
          if (aword(3) .eq. aquad(n)) then
            npl1 = n
            call plout (npl1, npl1)
            go to 180
          endif
  170   continue
        nerr = 1
        write (   6, 8170) ainput, ninput, aword(3)
        write (iout, 8170) ainput, ninput, aword(3)
        go to 180
      endif

c===============================================================================

c.... See if the first geometric object is a hyperbolic paraboloid.

 8172 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named hyperbolic paraboloid does not exist:  ',a)

      if ((aword(2) .eq. 'hyperb')  .or.
     &    (aword(2) .eq. 'saddle')) then  ! Find the index of the saddle.

        ageom1 = 'hyperb'
        do 172 n = 1, nquads
          if (aqtype(n) .ne. aqtypes(8)) go to 172
          if (aword(3) .eq. aquad(n)) then
            nq2 = n
            call quout (nq2, nq2, 'min', nerrq)
            go to 180
          endif
  172   continue
        nerr = 1
        write (   6, 8172) ainput, ninput, aword(3)
        write (iout, 8172) ainput, ninput, aword(3)
        go to 180
      endif

c===============================================================================

c.... See if the first geometric object is a sphere.

 8175 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named sphere does not exist:  ',a)

      if ((aword(2) .eq. 'sphere') .or.
     &    (aword(2) .eq. 'sph'))   then  ! Find the index of the sphere.

        ageom1 = 'sphere'

        do 175 n = 1, nquads
          if (aqtype(n) .ne. aqtypes(16)) go to 175
          if (aword(3) .eq. aquad(n)) then
            nq1 = n
            call quout (nq1, nq1, 'min', nerrq)
            go to 180
          endif
  175   continue
        nerr = 1
        write (   6, 8175) ainput, ninput, aword(3)
        write (iout, 8175) ainput, ninput, aword(3)
        go to 180

      endif

c===============================================================================

c.... See if the first geometric object is a circular cylinder.

 8178 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named cylinder does not exist:  ',a)

      if ((aword(2) .eq. 'cylinder') .or.
     &    (aword(2) .eq. 'cyl'))     then  ! Find the index of the cylinder.

        ageom1 = 'cylinder'
        do 178 n = 1, nquads
          if (aqtype(n) .ne. aqtypes(7)) go to 178
          if (aword(3) .eq. aquad(n)) then
            nq1 = n
            call quout (nq1, nq1, 'min', nerrq)
            go to 180
          endif
  178   continue
        nerr = 1
        write (   6, 8178) ainput, ninput, aword(3)
        write (iout, 8178) ainput, ninput, aword(3)
        go to 180
      endif

c===============================================================================

c.... See if the first geometric object is a general quadric surface.

 8179 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named quadric surface does not exist:  ',a)

      if ((aword(2) .eq. 'quadric') .or.
     &    (aword(2) .eq. 'quad')    .or.
     &    (aword(2) .eq. 'q'))      then  ! Find the index of the quadric.

        ageom1 = 'quadric'
        do 179 n = 1, nquads
          if (aword(3) .eq. aquad(n)) then
            nq1  = n
            npl1 = n
            call quout (nq1, nq1, 'min', nerrq)
            if (aqtype(n) .eq. aqtypes(0)) ageom1 = 'plane'
            if (aqtype(n) .eq. aqtypes(7)) ageom1 = 'cylinder'
            if (aqtype(n) .eq. aqtypes(16)) ageom1 = 'sphere'
            go to 180
          endif
  179   continue
        nerr = 1
        write (   6, 8179) ainput, ninput, aword(3)
        write (iout, 8179) ainput, ninput, aword(3)
        go to 180
      endif

c===============================================================================

c.... Find the type of the second geometric object.

  180 continue

c===============================================================================

c.... See if the second geometric object is a point.

      if ((aword(4) .eq. 'point') .or.
     &    (aword(4) .eq. 'pnt')   .or.
     &    (aword(4) .eq. 'pt')    .or.
     &    (aword(4) .eq. 'p'))    then

        call ptfind (5, np2)

        if (np2 .eq. 0) then
          nerr = 1
          go to 410
        endif

        ageom2 = 'point'

      endif

c===============================================================================

c.... See if the second geometric object is a cluster.

      if ((aword(4) .eq. 'cluster') .or.
     &    (aword(4) .eq. 'cl'))     then

        ageom2 = 'cluster'

        if (nclusts .eq. 0) then
          write (   6, 8120) ainput, ninput
          write (iout, 8120) ainput, ninput
          go to 999
        endif

        do 210 ncl = 1, nclusts       ! Loop over clusters.
          if (aword(5) .eq. aclust(ncl)) then

            call clout (ncl, ncl, 'min')

            ncl2 = ncl

            do 200 npr = 1, npairs    ! Loop over cluster pairs.
              if (apaircl(npr) .eq. aclust(ncl)) then

                do 190 np = 1, npoints  ! Loop over points.
                  if (apoint(np) .eq. apairpt(npr)) then

cbug                  call pout (np, np, 'min')

                    go to 200         ! Find next point in cluster.
                  endif
  190           continue              ! End of loop over points.

                write (   6, 8125) ainput, ninput, apairpt(npr)
                write (iout, 8125) ainput, ninput, apairpt(npr)
                go to 999

              endif

  200       continue                  ! End of loop over cluster pairs.

            go to 410                 ! Cluster ok, all cluster points ok.

          endif

  210   continue                      ! End of loop over clusters.

        write (   6, 8130) ainput, ninput, aword(5)
        write (iout, 8130) ainput, ninput, aword(5)
        go to 999

      endif                           ! Tested object type for cluster.

c===============================================================================

c.... See if the second geometric object is a line.

      if ((aword(4) .eq. 'line') .or.
     &    (aword(4) .eq. 'ln')   .or.
     &    (aword(4) .eq. 'l'))   then ! Find the index of the line.

        ageom2 = 'line'
        do 250 n = 1, nlines
          if (aword(5) .eq. aline(n)) then
            nl2 = n
            call lout (nl2, nl2)

            do 220 np = 1, npoints
              if (alipt1(nl2) .eq. apoint(np)) then
                xline21 = x(np)
                yline21 = y(np)
                zline21 = z(np)
                go to 230
              endif
  220       continue
            nerr = 1
            write (   6, 8140) ainput, ninput, alipt1(nl2)
            write (iout, 8140) ainput, ninput, alipt1(nl2)

  230       do 240 np = 1, npoints
              if (alipt2(nl2) .eq. apoint(np)) then
                xline22 = x(np)
                yline22 = y(np)
                zline22 = z(np)
                go to 410
              endif
  240       continue
            nerr = 1
            write (   6, 8140) ainput, ninput, alipt2(nl2)
            write (iout, 8140) ainput, ninput, alipt2(nl2)

            go to 410
          endif
  250   continue
        nerr = 1
        write (   6, 8135) ainput, ninput, aword(5)
        write (iout, 8135) ainput, ninput, aword(5)
        go to 410
      endif

c===============================================================================

c.... See if the second geometric object is a triangle.

      if ((aword(4) .eq. 'triangle') .or.
     &    (aword(4) .eq. 'tri'))     then  ! Find the index of the triangle.

        ageom2 = 'triangle'
        do 320 n = 1, ntrigs
          if (aword(5) .eq. atrig(n)) then
            ntr2 = n

            do 260 np = 1, npoints
              if (apoint(np) .eq. atript1(ntr2)) then
                xtrd = x(np)
                ytrd = y(np)
                ztrd = z(np)
                go to 270
              endif
  260       continue
            nerr = 1
            write (   6, 8160) ainput, ninput, atript1(ntr2)
            write (iout, 8160) ainput, ninput, atript1(ntr2)

  270       do 280 np = 1, npoints
              if (apoint(np) .eq. atript2(ntr2)) then
                xtre = x(np)
                ytre = y(np)
                ztre = z(np)
                go to 290
              endif
  280       continue
            nerr = 1
            write (   6, 8160) ainput, ninput, atript2(ntr2)
            write (iout, 8160) ainput, ninput, atript2(ntr2)

  290       do 300 np = 1, npoints
              if (apoint(np) .eq. atript3(ntr2)) then
                xtrf = x(np)
                ytrf = y(np)
                ztrf = z(np)
                go to 310
              endif
  300       continue
            nerr = 1
            write (   6, 8160) ainput, ninput, atript3(ntr2)
            write (iout, 8160) ainput, ninput, atript3(ntr2)

  310       amount = 'maximum'
            call trout (ntr2, ntr2, amount)
            go to 410
          endif
  320   continue
        nerr = 1
        write (   6, 8150) ainput, ninput, aword(5)
        write (iout, 8150) ainput, ninput, aword(5)
        go to 410
      endif

c===============================================================================

c.... See if the second geometric object is a plane.

      if ((aword(4) .eq. 'plane') .or.
     &    (aword(4) .eq. 'pl'))   then  ! Find the index of the plane.

        ageom2 = 'plane'
        do 330 n = 1, nquads
          if (aqtype(n) .ne. aqtypes(0)) go to 330
          if (aword(5) .eq. aquad(n)) then
            npl2 = n
            call plout (npl2, npl2)
            go to 410
          endif
  330   continue
        nerr = 1
        write (   6, 8170) ainput, ninput, aword(5)
        write (iout, 8170) ainput, ninput, aword(5)
        go to 410
      endif

c===============================================================================

c.... See if the second geometric object is a sphere.

      if ((aword(4) .eq. 'sphere') .or.
     &    (aword(4) .eq. 'sph'))   then  ! Find the index of the sphere.

        ageom2 = 'sphere'
        do 340 n = 1, nquads
          if (aqtype(n) .ne. aqtypes(16)) go to 340
          if (aword(5) .eq. aquad(n)) then
            nq2 = n
            call quout (nq2, nq2, 'min', nerrq)
            go to 410
          endif
  340   continue
        nerr = 1
        write (   6, 8175) ainput, ninput, aword(5)
        write (iout, 8175) ainput, ninput, aword(5)
        go to 410
      endif

c===============================================================================

c.... See if the second geometric object is a circular cylinder.

      if ((aword(4) .eq. 'cylinder') .or.
     &    (aword(4) .eq. 'cyl'))     then  ! Find the index of the cylinder.

        ageom2 = 'cylinder'
        do 350 n = 1, nquads
          if (aqtype(n) .ne. aqtypes(7)) go to 350
          if (aword(5) .eq. aquad(n)) then
            nq2 = n
            call quout (nq2, nq2, 'min', nerrq)
            go to 410
          endif
  350   continue
        nerr = 1
        write (   6, 8178) ainput, ninput, aword(5)
        write (iout, 8178) ainput, ninput, aword(5)
        go to 410
      endif

c===============================================================================

c.... See if the second geometric object is a circular cone.

 8360 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named cone does not exist:  ',a)

      if (aword(4) .eq. 'cone') then  ! Find the index of the cone.

        ageom2 = 'cone'
        do 360 n = 1, nquads
          if (aqtype(n) .ne. aqtypes(12)) go to 360
          if (aword(5) .eq. aquad(n)) then
            nq2 = n
            call quout (nq2, nq2, 'min', nerrq)
            go to 410
          endif
  360   continue
        nerr = 1
        write (   6, 8360) ainput, ninput, aword(5)
        write (iout, 8360) ainput, ninput, aword(5)
        go to 410
      endif

c===============================================================================

c.... See if the second geometric object is a hyperbolic paraboloid.

 8362 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named hyperbolic paraboloid does not exist:  ',a)

      if ((aword(4) .eq. 'hyperb')  .or.
     &    (aword(4) .eq. 'saddle')) then  ! Find the index of the saddle.

        ageom2 = 'hyperb'
        do 362 n = 1, nquads
          if (aqtype(n) .ne. aqtypes(8)) go to 362
          if (aword(5) .eq. aquad(n)) then
            nq2 = n
            call quout (nq2, nq2, 'min', nerrq)
            go to 410
          endif
  362   continue
        nerr = 1
        write (   6, 8362) ainput, ninput, aword(5)
        write (iout, 8362) ainput, ninput, aword(5)
        go to 410
      endif

c===============================================================================

c.... See if the second geometric object is an ellipsoid.

 8370 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named ellipsoid does not exist:  ',a)

      if ((aword(4) .eq. 'ellipsoid') .or.
     &    (aword(4) .eq. 'ellipse')   .or.
     &    (aword(4) .eq. 'ell')       .or.
     &    (aword(4) .eq. 'el'))       then  ! Find the index of the cone.

        ageom2 = 'ellipsoid'
        do 370 n = 1, nquads
          if (aqtype(n) .ne. aqtypes(15)) go to 370
          if (aword(5) .eq. aquad(n)) then
            nq2 = n
            call quout (nq2, nq2, 'min', nerrq)
            go to 410
          endif
  370   continue
        nerr = 1
        write (   6, 8370) ainput, ninput, aword(5)
        write (iout, 8370) ainput, ninput, aword(5)
        go to 410
      endif

c===============================================================================

c.... See if the second geometric object is an axisymmetric quadric surface.

 8380 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named axisymmetric quadric surface does not exist:  ',a)

      if (aword(4) .eq. 'axisym') then  ! Find the index of the axisym.

        ageom2 = 'axisym'
        do 380 n = 1, nquads
          if (asym(n) .ne. 'axisymmetric') go to 380
          if (aword(5) .eq. aquad(n)) then
            nq2  = n
            npl2 = n
            call quout (nq2, nq2, 'min', nerrq)
            if (aqtype(n) .eq. aqtypes(0)) ageom2 = 'plane'
            if (aqtype(n) .eq. aqtypes(7)) ageom2 = 'cylinder'
            if (aqtype(n) .eq. aqtypes(16)) ageom2 = 'sphere'
            go to 410
          endif
  380   continue
        nerr = 1
        write (   6, 8380) ainput, ninput, aword(5)
        write (iout, 8380) ainput, ninput, aword(5)
        go to 410
      endif

c===============================================================================

c.... See if the second geometric object is a general quadric surface.

 8390 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named quadric surface does not exist:  ',a)

      if ((aword(4) .eq. 'quadric') .or.
     &    (aword(4) .eq. 'quad')    .or.
     &    (aword(4) .eq. 'q'))      then  ! Find the index of the quadric.

        ageom2 = 'quadric'
        do 390 n = 1, nquads
          if (aword(5) .eq. aquad(n)) then
            nq2  = n
            npl2 = n
            call quout (nq2, nq2, 'min', nerrq)
            if (aqtype(n) .eq. aqtypes(0)) ageom2 = 'plane'
            if (aqtype(n) .eq. aqtypes(7)) ageom2 = 'cylinder'
            if (aqtype(n) .eq. aqtypes(16)) ageom2 = 'sphere'
            go to 410
          endif
  390   continue
        nerr = 1
        write (   6, 8390) ainput, ninput, aword(5)
        write (iout, 8390) ainput, ninput, aword(5)
        go to 410
      endif

c===============================================================================

c.... See if the second geometric object is a brick.

 8392 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named brick does not exist:  ',a)

      if ((aword(4) .eq. 'brick') .or.
     &    (aword(4) .eq. 'br'))   then  ! Find the brick index .

        ageom2 = 'brick'
        do 392 n = 1, nbricks
          if (aword(5) .eq. abrick(n)) then
            nb2 = n
            call brout (nb2, nb2, 'min')
            go to 410
          endif
  392   continue
        nerr = 1
        write (   6, 8392) ainput, ninput, aword(5)
        write (iout, 8392) ainput, ninput, aword(5)
        go to 410
      endif

c===============================================================================

c.... See if the second geometric object is a tetrahedron.

 8394 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named tetrahedron does not exist:  ',a)

      if ((aword(4) .eq. 'tetrahedron') .or.
     &    (aword(4) .eq. 'tetra')       .or.
     &    (aword(4) .eq. 'tet'))        then  ! Find the tetrahedron index .

        ageom2 = 'tetrahedron'
        do 394 n = 1, ntets
          if (aword(5) .eq. atetra(n)) then
            nt2 = n
            call tetout (nt2, nt2, 'min')
            go to 410
          endif
  394   continue
        nerr = 1
        write (   6, 8394) ainput, ninput, aword(5)
        write (iout, 8394) ainput, ninput, aword(5)
        go to 410
      endif

c===============================================================================

c.... See if the second geometric object is a zone.

 8395 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named zone does not exist:  ',a)

      if ((aword(4) .eq. 'zone') .or.
     &    (aword(4) .eq. 'zn')   .or.
     &    (aword(4) .eq. 'z'))   then  ! Find the index of the zone.

        ageom2 = 'zone'
        do 395 n = 1, nzones
          if (aword(5) .eq. azone(n)) then
            nz2 = n
            call zout (nz2, nz2, 'min')
            go to 410
          endif
  395   continue
        nerr = 1
        write (   6, 8395) ainput, ninput, aword(5)
        write (iout, 8395) ainput, ninput, aword(5)
        go to 410
      endif

c===============================================================================

c.... Found the first and second geometric object types.

  410 continue

c===============================================================================

      if (nerr .ne. 0) go to 999

c.... Switch to cartesian coordinates, if necessary.

      asysx = asys
      if (asysx .ne. 'rect') then
        nword = 2
        aword(2) = 'rect'
        call coord
      endif

c===============================================================================

c.... Find the distance between the two geometric objects.

c===============================================================================

c.... Input line:  distance point PNAME1 point PNAME2

 8420 format ('Points coincide:  ',a,' and ',a)

      if ((ageom1 .eq. 'point')  .and.
     &    (ageom2 .eq. 'point')) then

        call aptvdis (x(np1), y(np1), z(np1), x(np2), y(np2), z(np2),
     &                1, tol, dx, dy, dz, pdist, nerra)

        if (pdist .eq. 0.0) then

          write (   6, 8420) apoint(np1), apoint(np2)
          write (iout, 8420) apoint(np1), apoint(np2)

          proxptx = x(np1)
          proxpty = y(np1)
          proxptz = z(np1)

          proxvx = 0.0
          proxvy = 0.0
          proxvz = 0.0

          go to 740
        endif

 8720   format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Non-zero nerr returned by apt subroutine:  ',3i3)

        if (nerra .ne. 0) then
          write (   6, 8720) ainput, ninput, nerra, nerrb, nerrc
          write (iout, 8720) ainput, ninput, nerra, nerrb, nerrc
          go to 740
        endif

c....   Print the distance components and the distance.

 8730   format ('Proximal dx,dy,dz ',1p3e20.12 /
     &          '  Exact distance  ',1pe20.12)
        write (   6, 8730) dx, dy, dz, pdist
        write (iout, 8730) dx, dy, dz, pdist

        proxptx = x(np2)
        proxpty = y(np2)
        proxptz = z(np2)

        proxvx = dx
        proxvy = dy
        proxvz = dz

        go to 740

      endif                           ! Point to point.

c===============================================================================

c.... Input line:  distance point PNAME cluster CLNAME

      if ((ageom1 .eq. 'point')    .and.
     &    (ageom2 .eq. 'cluster')) then

c....  Initialize.

       npmin = 0
       npmax = 0
       nptot = 0
       davg  = 0.0
       dmin  =  1.e99
       dmax  = -1.e99

c....  Loop over the points in the cluster.

        do 440 npr = 1, npairs        ! Loop over cluster pairs.
          if (apaircl(npr) .eq. aclust(ncl2)) then

            do 430 np = 1, npoints    ! Loop over points.
              if (apoint(np) .eq. apairpt(npr)) then

                if (np .eq. np1) go to 440

                call pout (np1, np1, 'min')
                call pout (np, np, 'min')

                call aptvdis (x(np1), y(np1), z(np1),
     &                        x(np), y(np), z(np),
     &                        1, tol, dx, dy, dz, pdist, nerra)

                if (pdist .eq. 0.0) then

                  write (   6, 8420) apoint(np1), apoint(ncl2)
                  write (iout, 8420) apoint(np1), apoint(ncl2)

                  proxptx = x(np1)
                  proxpty = y(np1)
                  proxptz = z(np1)
        
                  proxvx = 0.0
                  proxvy = 0.0
                  proxvz = 0.0

                endif

                if (nerra .ne. 0) then
                  write (   6, 8720) ainput, ninput, nerra, nerrb, nerrc
                  write (iout, 8720) ainput, ninput, nerra, nerrb, nerrc
                  go to 740
                endif

c....           Print the distance components and the distance.

                write (   6, 8730) dx, dy, dz, pdist
                write (iout, 8730) dx, dy, dz, pdist

                proxptx = x(np)
                proxpty = y(np)
                proxptz = z(np)

                proxvx = dx
                proxvy = dy
                proxvz = dz

                nptot = nptot + 1
                davg  = davg + pdist
                if (pdist .lt. dmin) then
                  dmin  = pdist
                  npmin = np
                endif
                if (pdist .gt. dmax) then
                  dmax  = pdist
                  npmax = np
                endif

                go to 440             ! Find next point in cluster.
              endif
  430       continue                  ! End of loop over points.

          endif

  440   continue                      ! End of loop over cluster pairs.

 8450   format ('Min dist',1pe20.12,' pt ',a,' to pt ',a /
     &          'Max dist',1pe20.12,' pt ',a,' to pt ',a )
 8452   format ('Avg dist',1pe20.12,' for',i4,' points.')

        if (npmin .ne. npmax) then
          write (   6, 8450) dmin, apoint(np1), apoint(npmin),
     &                       dmax, apoint(np1), apoint(npmax)
          write (iout, 8450) dmin, apoint(np1), apoint(npmin),
     &                       dmax, apoint(np1), apoint(npmax)
        endif

        if (nptot .gt. 1) then
          davg = davg / nptot
          write (   6, 8452) davg, nptot
          write (iout, 8452) davg, nptot
        endif

        go to 740

      endif                           ! Point to cluster.

c===============================================================================

c.... Input line:  distance cluster CLNAME1 cluster CLNAME2

      if ((ageom1 .eq. 'cluster')  .and.
     &    (ageom2 .eq. 'cluster')) then

c....  Loop over the points in the first cluster.

c....  Initialize.

       npmin1 = 0
       npmax1 = 0
       npmin2 = 0
       npmax2 = 0
       nptot  = 0
       davg   = 0.0
       dmin   =  1.e99
       dmax   = -1.e99

        do 490 npr1 = 1, npairs       ! Loop over cluster pairs.
          if (apaircl(npr1) .eq. aclust(ncl1)) then

            do 480 np1 = 1, npoints   ! Loop over points.
              if (apoint(np1) .eq. apairpt(npr1)) then

c....           Loop over the points in the second cluster.

                nprbeg = 1
                if (ncl1 .eq. ncl2) nprbeg = npr1 + 1

                do 470 npr2 = nprbeg, npairs  ! Loop over cluster pairs.
                  if (apaircl(npr2) .eq. aclust(ncl2)) then

                    do 460 np2 = 1, npoints  ! Loop over points.
                      if (apoint(np2) .eq. apairpt(npr2)) then

                        if (np1 .eq. np2) go to 470

                        call pout (np1, np1, 'min')
                        call pout (np2, np2, 'min')

                        call aptvdis (x(np1), y(np1), z(np1),
     &                                x(np2), y(np2), z(np2),
     &                                1, tol, dx, dy, dz, pdist, nerra)

                        if (pdist .eq. 0.0) then

                          write (   6, 8420) apoint(np1), apoint(np2)
                          write (iout, 8420) apoint(np1), apoint(np2)

                          proxptx = x(np1)
                          proxpty = y(np1)
                          proxptz = z(np1)
                
                          proxvx = 0.0
                          proxvy = 0.0
                          proxvz = 0.0

                        endif

                        if (nerra .ne. 0) then
                          write (   6, 8720) ainput, ninput, nerra, nerrb, nerrc
                          write (iout, 8720) ainput, ninput, nerra, nerrb, nerrc
                          go to 740
                        endif

c....                   Print the distance components and the distance.

                        write (   6, 8730) dx, dy, dz, pdist
                        write (iout, 8730) dx, dy, dz, pdist

                        proxptx = x(np2)
                        proxpty = y(np2)
                        proxptz = z(np2)

                        proxvx = dx
                        proxvy = dy
                        proxvz = dz

                        nptot = nptot + 1
                        davg  = davg + pdist
                        if (pdist .lt. dmin) then
                          dmin   = pdist
                          npmin1 = np1
                          npmin2 = np2
                        endif
                        if (pdist .gt. dmax) then
                          dmax   = pdist
                          npmax1 = np1
                          npmax2 = np2
                        endif

                        go to 470     ! Find next point in cluster.
                      endif
  460               continue          ! End of loop over points.

                  endif

  470           continue              ! End of loop over cluster pairs.

                go to 490             ! Find next point in cluster.
              endif
  480       continue                  ! End of loop over points.

          endif

  490   continue                      ! End of loop over cluster pairs.

        if ((npmin1 .ne. npmax1)  .or.
     &      (npmin2 .ne. npmax2)) then
          write (   6, 8450) dmin, apoint(npmin1), apoint(npmin2),
     &                       dmax, apoint(npmax1), apoint(npmax2)
          write (iout, 8450) dmin, apoint(npmin1), apoint(npmin2),
     &                       dmax, apoint(npmax1), apoint(npmax2)
        endif

        if (nptot .gt. 1) then
          davg = davg / nptot
          write (   6, 8452) davg, nptot
          write (iout, 8452) davg, nptot
        endif

        go to 740

      endif                           ! Cluster to cluster.

c===============================================================================

c.... Input line:  distance point PNAME line ALNAME

 8530 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not find distance from line of zero length.')
 8532 format ('Point is on line  ',1p3e20.12)
 8534 format ('Proximal x,y,z    ',1p3e20.12)

      if ((ageom1 .eq. 'point') .and.
     &    (ageom2 .eq. 'line')) then

        call aptptln (x(np1), y(np1), z(np1),
     &                xline21, yline21, zline21,
     &                xline22, yline22, zline22,
     &                1, tol, 0, dpmin, fdmin, cx, cy, cz,
     &                nlim, itrun, nerra)

        call aptvdis (x(np1), y(np1), z(np1), cx, cy, cz, 1, tol, 
     &                dx, dy, dz, pdist, nerrb)

        if (itrun .eq. -1) then
          nerr = 1
          write (   6, 8530) ainput, ninput
          write (iout, 8530) ainput, ninput
        endif

        if (pdist .eq. 0.0) then
          write (   6, 8532) x(np1), y(np1), z(np1)
          write (iout, 8532) x(np1), y(np1), z(np1)

          proxptx = x(np1)
          proxpty = y(np1)
          proxptz = z(np1)

          proxvx = 0.0
          proxvy = 0.0
          proxvz = 0.0

          go to 740
        else
          write (   6, 8534) cx, cy, cz
          write (iout, 8534) cx, cy, cz

          proxptx = cx
          proxpty = cy
          proxptz = cz

        endif

        if ((nerra .ne. 0)  .or.
     &      (nerrb .ne. 0)) then
          write (   6, 8720) ainput, ninput, nerra, nerrb, nerrc
          write (iout, 8720) ainput, ninput, nerra, nerrb, nerrc
          go to 740
        endif

c....   Print the distance components and the distance.

        write (   6, 8730) dx, dy, dz, pdist
        write (iout, 8730) dx, dy, dz, pdist

        proxvx = dx
        proxvy = dy
        proxvz = dz

        go to 740

      endif                           ! Point to line.

c===============================================================================

c.... Input line:  distance cluster CLNAME line ALNAME

      if ((ageom1 .eq. 'cluster') .and.
     &    (ageom2 .eq. 'line'))   then

c....  Initialize.

       npmin = 0
       npmax = 0
       nptot = 0
       davg  = 0.0
       dmin  =  1.e99
       dmax  = -1.e99

c....  Loop over the points in the cluster.

        do 550 npr = 1, npairs        ! Loop over cluster pairs.
          if (apaircl(npr) .eq. aclust(ncl1)) then

            do 540 np = 1, npoints    ! Loop over points.
              if (apoint(np) .eq. apairpt(npr)) then

                call pout (np, np, 'min')

                call aptptln (x(np), y(np), z(np),
     &                        xline21, yline21, zline21,
     &                        xline22, yline22, zline22,
     &                        1, tol, 0, dpmin, fdmin, cx, cy, cz,
     &                        nlim, itrun, nerra)

                call aptvdis (x(np), y(np), z(np), cx, cy, cz, 1, tol, 
     &                        dx, dy, dz, pdist, nerrb)

                if (itrun .eq. -1) then  ! Line end points coincide.
                  write (   6, 8530) ainput, ninput
                  write (iout, 8530) ainput, ninput
                  go to 740
                endif

                if ((nerra .ne. 0)  .or.
     &              (nerrb .ne. 0)) then
                  write (   6, 8720) ainput, ninput, nerra, nerrb, nerrc
                  write (iout, 8720) ainput, ninput, nerra, nerrb, nerrc
                  go to 540
                endif

                if (pdist .eq. 0.0) then

                  write (   6, 8532) x(np), y(np), z(np)
                  write (iout, 8532) x(np), y(np), z(np)

                  proxptx = x(np)
                  proxpty = y(np)
                  proxptz = z(np)

                else
                  write (   6, 8534) cx, cy, cz
                  write (iout, 8534) cx, cy, cz

                  proxptx = cx
                  proxpty = cy
                  proxptz = cz
                endif

c....           Print the distance components and the distance.

                write (   6, 8730) dx, dy, dz, pdist
                write (iout, 8730) dx, dy, dz, pdist

                proxvx = dx
                proxvy = dy
                proxvz = dz

                nptot = nptot + 1
                davg  = davg + pdist
                if (pdist .lt. dmin) then
                  dmin  = pdist
                  npmin = np
                endif
                if (pdist .gt. dmax) then
                  dmax  = pdist
                  npmax = np
                endif

                go to 550             ! Find next point in cluster.
              endif
  540       continue                  ! End of loop over points.

          endif

  550   continue                      ! End of loop over cluster pairs.

 8550   format ('Min dist',1pe20.12,' from pt ',a,' to line ',a /
     &          'Max dist',1pe20.12,' from pt ',a,' to line ',a )

        if (npmin .ne. npmax) then
          write (   6, 8550) dmin, apoint(npmin), aline(nl2),
     &                       dmax, apoint(npmax), aline(nl2)
          write (iout, 8550) dmin, apoint(npmin), aline(nl2),
     &                       dmax, apoint(npmax), aline(nl2)
        endif

        if (nptot .gt. 1) then
          davg = davg / nptot
          write (   6, 8452) davg, nptot
          write (iout, 8452) davg, nptot
        endif

        go to 740

      endif                           ! Cluster to line.

c===============================================================================

c.... Input line:  distance line ALNAME1 line ALNAME2

 8560 format ('Lines are parallel:  ',a,' and ',a)
 8562 format ('Lines intersect at',1p3e20.12)

      if ((ageom1 .eq. 'line')  .and.
     &    (ageom2 .eq. 'line')) then

        call aptlnln (xline11, yline11, zline11,
     &                xline12, yline12, zline12,
     &                xline21, yline21, zline21,
     &                xline22, yline22, zline22,
     &                1, tol, dpmin, fracab, fraccd,
     &                ex, ey, ez, fx, fy, fz, itrun, ipar, nerra)

        call aptvdis (ex, ey, ez, fx, fy, fz, 1, tol,
     &                dx, dy, dz, pdist, nerrb)

        if ((nerra .ne. 0)  .or.
     &      (nerrb .ne. 0)) then
          write (   6, 8720) ainput, ninput, nerra, nerrb, nerrc
          write (iout, 8720) ainput, ninput, nerra, nerrb, nerrc
          go to 740
        endif

        if (ipar .eq. 1) then         ! Lines are parallel.
          write (   6, 8560) aline(nl1), aline(nl2)
          write (iout, 8560) aline(nl1), aline(nl2)
        elseif (ipar .gt. 1) then
          nerr = 1
          write (   6, 8530) ainput, ninput
          write (iout, 8530) ainput, ninput
        endif

        if (pdist .eq. 0.0) then

          write (   6, 8562) ex, ey, ez
          write (iout, 8562) ex, ey, ez

          proxptx = ex
          proxpty = ey
          proxptz = ez

          proxvx = 0.0
          proxvy = 0.0
          proxvz = 0.0

          go to 740
        else
          write (   6, 8534) ex, ey, ez
          write (iout, 8534) ex, ey, ez
          write (   6, 8534) fx, fy, fz
          write (iout, 8534) fx, fy, fz

          proxptx = fx
          proxpty = fy
          proxptz = fz
        endif

c....   Print the distance components and the distance.

        write (   6, 8730) dx, dy, dz, pdist
        write (iout, 8730) dx, dy, dz, pdist

        proxvx = dx
        proxvy = dy
        proxvz = dz

        go to 740

      endif                           ! Line to line.

c===============================================================================

c.... Input line:  distance point PNAME triangle TRNAME

 8570 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Triangle is indeterminate.')
 8572 format ('Point is coplanar ',1p3e20.12)
 8574 format ('Proximal x,y,z    ',1p3e20.12)
 8576 format ('Vertex weights    ',1p3e20.12)

      if ((ageom1 .eq. 'point')     .and.
     &    (ageom2 .eq. 'triangle')) then

c....   Find the distance from the point to plane of the triangle, the proximal
c....     point in the plane of the triangle, and the local coordinates of the
c....     proximal point.

        call aptripl (x(np1), y(np1), z(np1), xtrd, ytrd, ztrd,
     &                xtre, ytre, ztre, xtrf, ytrf, ztrf, 1, tol,
     &                dpmin, cx, cy, cz, wd, we, wf, nerra)

        call aptvdis (x(np1), y(np1), z(np1), cx, cy, cz, 1, tol,
     &                dx, dy, dz, pdist, nerrb)

        if ((nerra .ne. 0)  .or.
     &      (nerrb .ne. 0)) then
          write (   6, 8720) ainput, ninput, nerra, nerrb, nerrc
          write (iout, 8720) ainput, ninput, nerra, nerrb, nerrc
          go to 740
        endif

        if ((abs (wd) .ge. bignum)  .or.
     &      (abs (we) .ge. bignum)  .or.
     &      (abs (wf) .ge. bignum)) then
          nerr = 1
          write (   6, 8570) ainput, ninput
          write (iout, 8570) ainput, ninput
          go to 740
        endif

        if (pdist .eq. 0.0) then

          write (   6, 8572) x(np1), y(np1), z(np1)
          write (iout, 8572) x(np1), y(np1), z(np1)
          write (   6, 8576) wd, we, wf
          write (iout, 8576) wd, we, wf

          proxptx = x(np1)
          proxpty = y(np1)
          proxptz = z(np1)

          proxvx = 0.0
          proxvy = 0.0
          proxvz = 0.0

          go to 740

        else

          write (   6, 8574) cx, cy, cz
          write (iout, 8574) cx, cy, cz

          proxptx = cx
          proxpty = cy
          proxptz = cz

        endif

        write (   6, 8576) wd, we, wf
        write (iout, 8576) wd, we, wf

c.... Print the distance components and the distance.

        write (   6, 8730) dx, dy, dz, pdist
        write (iout, 8730) dx, dy, dz, pdist

        proxvx = dx
        proxvy = dy
        proxvz = dz

        go to 740

      endif                           ! Point to triangle.

c===============================================================================

c.... Input line:  distance cluster CLNAME triangle TRNAME

c.... Find the distance from each point to the plane of the triangle, the
c       proximal point in the plane of the triangle, and the local coordinates
c       of the proximal point.

      if ((ageom1 .eq. 'cluster')   .and.
     &    (ageom2 .eq. 'triangle')) then

c....  Initialize.

       npmin = 0
       npmax = 0
       nptot = 0
       davg  = 0.0
       dmin  =  1.e99
       dmax  = -1.e99

c....  Loop over the points in the cluster.

        do 590 npr = 1, npairs        ! Loop over cluster pairs.
          if (apaircl(npr) .eq. aclust(ncl1)) then

            do 580 np = 1, npoints    ! Loop over points.
              if (apoint(np) .eq. apairpt(npr)) then

                call pout (np, np, 'min')

                call aptripl (x(np), y(np), z(np), xtrd, ytrd, ztrd,
     &                        xtre, ytre, ztre, xtrf, ytrf, ztrf,
     &                        1, tol,
     &                        dpmin, cx, cy, cz, wd, we, wf, nerra)

                call aptvdis (x(np), y(np), z(np), cx, cy, cz, 1, tol,
     &                        dx, dy, dz, pdist, nerrb)

                if ((nerra .ne. 0)  .or.
     &              (nerrb .ne. 0)) then
                  write (   6, 8720) ainput, ninput, nerra, nerrb, nerrc
                  write (iout, 8720) ainput, ninput, nerra, nerrb, nerrc
                  go to 580
                endif

                if ((abs (wd) .ge. bignum)  .or.
     &              (abs (we) .ge. bignum)  .or.
     &              (abs (wf) .ge. bignum)) then
                  write (   6, 8570) ainput, ninput
c
                  write (iout, 8570) ainput, ninput
                  go to 740
                endif

                if (pdist .eq. 0.0) then

                  write (   6, 8572) x(np), y(np), z(np)
                  write (iout, 8572) x(np), y(np), z(np)
                  write (   6, 8576) wd, we, wf
                  write (iout, 8576) wd, we, wf

                  proxptx = x(np)
                  proxpty = y(np)
                  proxptz = z(np)
        
                else

                  write (   6, 8574) cx, cy, cz
                  write (iout, 8574) cx, cy, cz

                  proxptx = cx
                  proxpty = cy
                  proxptz = cz

                endif

                write (   6, 8576) wd, we, wf
                write (iout, 8576) wd, we, wf

c....           Print the distance components and the distance.

                write (   6, 8730) dx, dy, dz, pdist
                write (iout, 8730) dx, dy, dz, pdist

                proxvx = dx
                proxvy = dy
                proxvz = dz

                nptot = nptot + 1
                davg  = davg + pdist
                if (pdist .lt. dmin) then
                  dmin  = pdist
                  npmin = np
                endif
                if (pdist .gt. dmax) then
                  dmax  = pdist
                  npmax = np
                endif

                go to 590             ! Find next point in cluster.
              endif
  580       continue                  ! End of loop over points.

          endif

  590   continue                      ! End of loop over cluster pairs.

 8590   format ('Min dist',1pe20.12,' pt ',a,' to triangle ',a /
     &          'Max dist',1pe20.12,' pt ',a,' to triangle ',a )

        if (npmin .ne. npmax) then
          write (   6, 8590) dmin, apoint(npmin), atrig(ntr2),
     &                       dmax, apoint(npmax), atrig(ntr2)
          write (iout, 8590) dmin, apoint(npmin), atrig(ntr2),
     &                       dmax, apoint(npmax), atrig(ntr2)
        endif

        if (nptot .gt. 1) then
          davg = davg / nptot
          write (   6, 8452) davg, nptot
          write (iout, 8452) davg, nptot
        endif

        go to 740

      endif                           ! Cluster to triangle.

c===============================================================================

c.... Input line:  distance point PNAME plane PLNAME

 8610 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Plane is indeterminate.')
 8612 format ('Point is in plane ',1p3e20.12)
 8614 format ('Proximal x,y,z    ',1p3e20.12)

      if ((ageom1 .eq. 'point')  .and.
     &    (ageom2 .eq. 'plane')) then

        call aptptpl (x(np1), y(np1), z(np1),
     &                xcen(npl2), ycen(npl2), zcen(npl2),
     &                qx(npl2), qy(npl2), qz(npl2), 1, tol,
     &                dpmin, cx, cy, cz,
     &                itrun, nerra)

        call aptvdis (x(np1), y(np1), z(np1), cx, cy, cz, 1, tol,
     &                dx, dy, dz, pdist, nerrb)

        pdist = -dpmin

        if (itrun .gt. 1) then
          nerr = 1
          write (   6, 8610) ainput, ninput
          write (iout, 8610) ainput, ninput
        endif

        if (pdist .eq. 0.0) then

          write (   6, 8612) x(np1), y(np1), z(np1)
          write (iout, 8612) x(np1), y(np1), z(np1)

          proxptx = x(np1)
          proxpty = y(np1)
          proxptz = z(np1)

          proxvx = 0.0
          proxvy = 0.0
          proxvz = 0.0

          go to 740

        else

          write (   6, 8614) cx, cy, cz
          write (iout, 8614) cx, cy, cz

          proxptx = cx
          proxpty = cy
          proxptz = cz

        endif

        if ((nerra .ne. 0)  .or.
     &      (nerrb .ne. 0)) then
          write (   6, 8720) ainput, ninput, nerra, nerrb, nerrc
          write (iout, 8720) ainput, ninput, nerra, nerrb, nerrc
          go to 740
        endif

c....   Print the distance components and the distance.

        write (   6, 8730) dx, dy, dz, pdist
        write (iout, 8730) dx, dy, dz, pdist

        proxvx = dx
        proxvy = dy
        proxvz = dz

        go to 740
 
      endif                           ! Point to plane.

c===============================================================================

c.... Input line:  distance point PNAME sphere    SPHNAME
c.... Input line:  distance point PNAME cylinder  CYLNAME
c.... Input line:  distance point PNAME cone      CNNAME
c.... Input line:  distance point PNAME hyperb    HPNAME
c.... Input line:  distance point PNAME ellipsoid ELNAME
c.... Input line:  distance point PNAME axisym    AXNAME
c.... Input line:  distance point PNAME quadric   QNAME

      if ((ageom1 .eq. 'point')      .and.
     &   ((ageom2 .eq. 'sphere')     .or.
     &    (ageom2 .eq. 'cylinder')   .or.
     &    (ageom2 .eq. 'cone')       .or.
     &    (ageom2 .eq. 'hyperb')     .or.
     &    (ageom2 .eq. 'ellipsoid')  .or.
     &    (ageom2 .eq. 'axisym')     .or.
     &    (ageom2 .eq. 'quadric')))  then

        call ptdist (np1, nq2)

        go to 740

      endif                           ! Point to quadric surface.

c===============================================================================

c.... Input line:  distance cluster CLNAME plane PLNAME

      if ((ageom1 .eq. 'cluster') .and.
     &    (ageom2 .eq. 'plane'))  then

c....  Initialize.

       npmin = 0
       npmax = 0
       nptot = 0
       davg  = 0.0
       dmin  =  1.e99
       dmax  = -1.e99

c....  Loop over the points in the cluster.

        do 630 npr = 1, npairs        ! Loop over cluster pairs.
          if (apaircl(npr) .eq. aclust(ncl1)) then

            do 620 np = 1, npoints    ! Loop over points.
              if (apoint(np) .eq. apairpt(npr)) then

                call pout (np, np, 'min')

                call aptptpl (x(np), y(np), z(np),
     &                        xcen(npl2), ycen(npl2), zcen(npl2),
     &                        qx(npl2), qy(npl2), qz(npl2), 1, tol,
     &                        dpmin, cx, cy, cz, itrun, nerra)

                call aptvdis (x(np), y(np), z(np), cx, cy, cz, 1, tol,
     &                        dx, dy, dz, pdist, nerrb)

                if ((nerra .ne. 0)  .or.
     &              (nerrb .ne. 0)) then
                  write (   6, 8720) ainput, ninput, nerra, nerrb, nerrc
                  write (iout, 8720) ainput, ninput, nerra, nerrb, nerrc
                  go to 620
                endif

                if (itrun .gt. 1) then
                  write (   6, 8610) ainput, ninput
                  write (iout, 8610) ainput, ninput
                  go to 740
                endif

                if (pdist .eq. 0.0) then

                  write (   6, 8612) x(np), y(np), z(np)
                  write (iout, 8612) x(np), y(np), z(np)

                  proxptx = x(np)
                  proxpty = y(np)
                  proxptz = z(np)

                else

                  write (   6, 8614) cx, cy, cz
                  write (iout, 8614) cx, cy, cz

                  proxptx = cx
                  proxpty = cy
                  proxptz = cz

                endif

c....           Print the distance components and the distance.

                write (   6, 8730) dx, dy, dz, pdist
                write (iout, 8730) dx, dy, dz, pdist

                proxvx = dx
                proxvy = dy
                proxvz = dz

                nptot = nptot + 1
                davg  = davg + pdist
                if (pdist .lt. dmin) then
                  dmin  = pdist
                  npmin = np
                endif
                if (pdist .gt. dmax) then
                  dmax  = pdist
                  npmax = np
                endif

                go to 630             ! Find next point in cluster.
              endif
  620       continue                  ! End of loop over points.

          endif

  630   continue                      ! End of loop over cluster pairs.

 8630   format ('Min dist',1pe20.12,' pt ',a,' to plane ',a /
     &          'Max dist',1pe20.12,' pt ',a,' to plane ',a )

        if (npmin .ne. npmax) then
          write (   6, 8630) dmin, apoint(npmin), aquad(npl2),
     &                       dmax, apoint(npmax), aquad(npl2)
          write (iout, 8630) dmin, apoint(npmin), aquad(npl2),
     &                       dmax, apoint(npmax), aquad(npl2)
        endif

        if (nptot .gt. 1) then
          davg = davg / nptot
          write (   6, 8452) davg, nptot
          write (iout, 8452) davg, nptot
        endif

        go to 740

      endif                           ! Cluster to plane.

c===============================================================================

c.... Input line:  distance cluster CLNAME sphere    SPHNAME
c.... Input line:  distance cluster CLNAME cylinder  CYLNAME
c.... Input line:  distance cluster CLNAME cone      CNNAME
c.... Input line:  distance cluster CLNAME hyperb    HPNAME
c.... Input line:  distance cluster CLNAME ellipsoid ELNAME
c.... Input line:  distance cluster CLNAME axisym    AXNAME
c.... Input line:  distance cluster CLNAME quadric   QNAME

      if ((ageom1 .eq. 'cluster')    .and.
     &   ((ageom2 .eq. 'sphere')     .or.
     &    (ageom2 .eq. 'cylinder')   .or.
     &    (ageom2 .eq. 'cone')       .or.
     &    (ageom2 .eq. 'hyperb')     .or.
     &    (ageom2 .eq. 'ellipsoid')  .or.
     &    (ageom2 .eq. 'axisym')     .or.
     &    (ageom2 .eq. 'quadric')))  then

c....  Initialize.

       npmin = 0
       npmax = 0
       nptot = 0
       davg  = 0.0
       dmin  =  1.e99
       dmax  = -1.e99

c....  Loop over the points in the cluster.

        do 640 npr = 1, npairs        ! Loop over cluster pairs.
          if (apaircl(npr) .eq. aclust(ncl1)) then

            do 635 np = 1, npoints    ! Loop over points.
              if (apoint(np) .eq. apairpt(npr)) then

                call pout (np, np, 'min')

                call ptdist (np, nq2)

                nptot = nptot + 1
                davg  = davg + pdist
                if (pdist .lt. dmin) then
                  dmin  = pdist
                  npmin = np
                endif
                if (pdist .gt. dmax) then
                  dmax  = pdist
                  npmax = np
                endif

                go to 640             ! Find next point in cluster.
              endif
  635       continue                  ! End of loop over points.

          endif

  640   continue                      ! End of loop over cluster pairs.

        if (npmin .ne. npmax) then
          write (   6, 8630) dmin, apoint(npmin), aquad(nq2),
     &                       dmax, apoint(npmax), aquad(nq2)
          write (iout, 8630) dmin, apoint(npmin), aquad(nq2),
     &                       dmax, apoint(npmax), aquad(nq2)
        endif

        if (nptot .gt. 1) then
          davg = davg / nptot
          write (   6, 8452) davg, nptot
          write (iout, 8452) davg, nptot
        endif

        go to 740

      endif                           ! Cluster to quadric surface..

c===============================================================================

c.... Input line:  distance point PNAME brick BRNAME

      if ((ageom1 .eq. 'point')  .and.
     &    (ageom2 .eq. 'brick')) then

        call ptbrick (1, np1, nb2, nside, fracu, fracv, fracw)

        if (nerra .ne. 0) go to 999

 8226   format ('point ',a,' is in      brick       ',a)
 8227   format ('  Fractional wts  ',1p3e20.12 )
 8228   format ('point ',a,' is not in  brick       ',a)

        if (nside .eq. 1) then
          write (   6, 8226) apoint(np1), abrick(nb2)
          write (iout, 8226) apoint(np1), abrick(nb2)
          write (   6, 8227) fracu, fracv, fracw
          write (iout, 8227) fracu, fracv, fracw
        else
          write (   6, 8228) apoint(np1), abrick(nb2)
          write (iout, 8228) apoint(np1), abrick(nb2)
        endif

        go to 740

      endif

c===============================================================================

c.... Input line:  distance point PNAME tetrahedron TETNAME

      if ((ageom1 .eq. 'point')        .and.
     &    (ageom2 .eq. 'tetrahedron')) then

        call pttet (1, np1, nt2, nside, wa, wb, wc, wd, nerra)

        if (nerra .ne. 0) go to 999

 8256   format ('point ',a,' is in      tetrahedron ',a)
 8257   format ('Vertex wts w1, w2 ',1p2e20.12 /
     &          'Vertex wts w3, w4 ',1p2e20.12 )
 8258   format ('point ',a,' is not in  tetrahedron ',a)

        if (nside .eq. 1) then
          write (   6, 8256) apoint(np1), atetra(nt2)
          write (iout, 8256) apoint(np1), atetra(nt2)
          write (   6, 8257) wa, wb, wc, wd
          write (iout, 8257) wa, wb, wc, wd
        else
          write (   6, 8258) apoint(np1), atetra(nt2)
          write (iout, 8258) apoint(np1), atetra(nt2)
        endif

        go to 740

      endif

c===============================================================================

c.... Input line:  distance point PNAME zone ZNAME

      if ((ageom1 .eq. 'point') .and.
     &    (ageom2 .eq. 'zone')) then

        amount = 'maximum'

        call ptzone (1, np1, nz2, amount, dside, nside, nerra)

 8286   format ('point ',a,' is in      zone        ',a)
 8287   format ('  dside ',a,2x,1pe20.12)

        if (nerra .ne. 0) go to 999

        if (nside .eq. 1) then
          write (   6, 8286) apoint(np1), azone(nz2)
          write (iout, 8286) apoint(np1), azone(nz2)
        endif

        go to 740

      endif

c===============================================================================

c.... Input line:  distance line ALNAME triangle TRNAME

 8650 format ('Parallel distance ',1pe20.12)
 8652 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Triangle or line is indeterminate.')
 8654 format ('Intersection point',1p3e20.12)

      if ((ageom1 .eq. 'line')      .and.
     &    (ageom2 .eq. 'triangle')) then

c....   Find the normal vector to the triangle.

        call aptvpln (xtrd, ytrd, ztrd, xtre, ytre, ztre,
     &                xtrf, ytrf, ztrf, 1, tol,
     &                xn, yn, zn, vlength, nerra)

c....   Find the intersection (or distance) of the line and the triangle.

        call aptlnpl (xline11, yline11, zline11,
     &                xline12, yline12, zline12,
     &                xtrd, ytrd, ztrd, xn, yn, zn, 1, tol,
     &                dpmin, dint, fracps, cx, cy, cz, ipar, nerrb)

        if ((nerra .ne. 0)  .or.
     &      (nerrb .ne. 0)) then
          write (   6, 8720) ainput, ninput, nerra, nerrb, nerrc
          write (iout, 8720) ainput, ninput, nerra, nerrb, nerrc
          go to 740
        endif

        if (ipar .eq. 0) then
          pdist = 0.0
          write (   6, 8654) cx, cy, cz
          write (iout, 8654) cx, cy, cz
          call plotpt (cx, cy, cz, nerrg)

          proxptx = cx
          proxpty = cy
          proxptz = cz

          proxvx = 0.0
          proxvy = 0.0
          proxvz = 0.0

        elseif (ipar .eq. 1) then

          pdist = dpmin
          write (   6, 8650) dpmin
          write (iout, 8650) dpmin

          call aptptpl (xline11, yline11, zline11,
     &                  xtrd, ytrd, ztrd, xn, yn, zn, 1, tol,
     &                  dpmin, cx, cy, cz, itrun, nerr)

          proxptx = cx
          proxpty = cy
          proxptz = cz

          proxvx = cx - xline11
          proxvy = cy - yline11
          proxvz = cz - zline11

        elseif (ipar .gt. 1) then
          nerr = 1
          write (   6, 8652) ainput, ninput
          write (iout, 8652) ainput, ninput
        endif

        go to 740

      endif                           ! Line to triangle.

c===============================================================================

c.... Input line:  distance line ALNAME plane PLNAME

 8660 format ('Parallel distance ',1pe20.12)
 8662 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Plane or line is indeterminate.')
 8664 format ('Intersection point',1p3e20.12)

      if ((ageom1 .eq. 'line')   .and.
     &    (ageom2 .eq. 'plane')) then

c....   Find the intersection (or distance) of the line and the plane.

        call aptlnpl (xline11, yline11, zline11,
     &                xline12, yline12, zline12,
     &                xcen(npl2), ycen(npl2), zcen(npl2),
     &                qx(npl2), qy(npl2), qz(npl2), 1, tol,
     &                dpmin, dint, fracps, cx, cy, cz, ipar, nerra)

        if (nerra .ne. 0) then
          write (   6, 8720) ainput, ninput, nerra, nerrb, nerrc
          write (iout, 8720) ainput, ninput, nerra, nerrb, nerrc
          go to 740
        endif

        if (ipar .eq. 0) then
          pdist = 0.0
          write (   6, 8664) cx, cy, cz
          write (iout, 8664) cx, cy, cz
          call plotpt (cx, cy, cz, nerrg)

          proxptx = cx
          proxpty = cy
          proxptz = cz

          proxvx = 0.0
          proxvy = 0.0
          proxvz = 0.0

        elseif (ipar .eq. 1) then

          pdist = dpmin
          write (   6, 8660) dpmin
          write (iout, 8660) dpmin

          proxptx = xcen(npl2)
          proxpty = ycen(npl2)
          proxptz = zcen(npl2)

          call aptvunb (qx(npl2), qy(npl2), qz(npl2), 1, tol,
     &                  ux, uy, uz, vlength, nerra)

          proxvx = ux * dpmin
          proxvy = uy * dpmin
          proxvz = uz * dpmin

          if ((xcen(npl2) - xline11) * proxvx .lt. 0.0) then
            proxvx = -proxvx
            proxvy = -proxvy
            proxvz = -proxvz
          endif
          

        elseif (ipar .gt. 1) then
          nerr = 1
          write (   6, 8662) ainput, ninput
          write (iout, 8662) ainput, ninput
        endif

        go to 740

      endif                           ! Line to triangle.

c===============================================================================

c.... Input line:  distance triangle TRNAME1 triangle TRNAME2

 8670 format ('Intersection point',1p3e20.12 /
     &        'Intersection point',1p3e20.12 /
     &        'Vector            ',1p3e20.12)
 8672 format ('Triangles are coplanar:  ',a,' and ',a)
 8674 format ('Parallel distance ',1pe20.12)
 8676 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Triangle is indeterminate:  ',a,' or ',a)

      if ((ageom1 .eq. 'triangle')  .and.
     &    (ageom2 .eq. 'triangle')) then

c....   Find the normal vector to the first triangle.

        call aptvpln (xtra, ytra, ztra, xtrb, ytrb, ztrb,
     &                xtrc, ytrc, ztrc, 1, tol,
     &                xna, yna, zna, vlength, nerra)

c....   Find the normal vector to the second triangle.

        call aptvpln (xtrd, ytrd, ztrd, xtre, ytre, ztre,
     &                xtrf, ytrf, ztrf, 1, tol,
     &                xnb, ynb, znb, vlength, nerrb)

c....   Find the intersection or distance between the triangles.

        call aptplpl (xtra, ytra, ztra, xna, yna, zna,
     &                xtrd, ytrd, ztrd, xnb, ynb, znb, 1, tol,
     &                ex, ey, ez, fx, fy, fz, uparx, upary, uparz,
     &                ipar, dpmin, itrun, nerrc)

        if ((nerra .ne. 0)  .or.
     &      (nerrb .ne. 0)  .or.
     &      (nerrc .ne. 0)) then
          write (   6, 8720) ainput, ninput, nerra, nerrb, nerrc
          write (iout, 8720) ainput, ninput, nerra, nerrb, nerrc
          go to 740
        endif

        if (ipar .eq. 0) then         ! Triangles intersect.

          pdist = 0.0
          write (   6, 8670) ex, ey, ez, fx, fy, fz, uparx, upary, uparz
          write (iout, 8670) ex, ey, ez, fx, fy, fz, uparx, upary, uparz
          call plotpt (ex, ey, ez, nerrg)
          call plotpt (fx, fy, fz, nerrg)

          proxptx = ex
          proxpty = ey
          proxptz = ez

          proxvx = 0.0
          proxvy = 0.0
          proxvz = 0.0

        elseif (ipar .eq. 1) then     ! Triangles are parallel.

          pdist = dpmin

          if (itrun .eq. 1) then      ! Triangles are coplanar.

            write (   6, 8672) atrig(ntr1), atrig(ntr2)
            write (iout, 8672) atrig(ntr1), atrig(ntr2)

          else                        ! Triangles are not coplanar.

            write (   6, 8674) dpmin
            write (iout, 8674) dpmin

          endif

          proxptx = fx
          proxpty = fy
          proxptz = fz

          call aptvusz (xna, yna, zna, tol, ux, uy, uz)

          proxvx = ux * dpmin
          proxvy = uy * dpmin
          proxvz = uz * dpmin

          if ((xtrd - xtra) * proxvx .lt. 0.0) then
            proxvx = -proxvx
            proxvy = -proxvy
            proxvz = -proxvz
          endif

        elseif (ipar .gt. 1) then     ! Triangles are indeterminate.

          nerr = 1
          write (   6, 8676) ainput, ninput, atrig(ntr1), atrig(ntr2)
          write (iout, 8676) ainput, ninput, atrig(ntr1), atrig(ntr2)

        endif                         ! Tested for relation between triangles.

        go to 740

      endif                           ! Triangle to triangle.

c===============================================================================

c.... Input line:  distance triangle TRNAME plane PLNAME

 8680 format ('Intersection point',1p3e20.12 /
     &        'Intersection point',1p3e20.12 /
     &        'Vector            ',1p3e20.12)
 8682 format ('Triangle and plane are coplanar:  ',a,' and ',a)
 8684 format ('Parallel distance ',1pe20.12)
 8686 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Plane is indeterminate:  ',a,' or ',a)

      if ((ageom1 .eq. 'triangle') .and.
     &    (ageom2 .eq. 'plane'))   then

c....   Find the normal vector to the triangle.

        call aptvpln (xtra, ytra, ztra, xtrb, ytrb, ztrb,
     &                xtrc, ytrc, ztrc, 1, tol,
     &                xna, yna, zna, vlength, nerra)

c....   Find the intersection or distance between the triangle and the plane.

        call aptplpl (xtra, ytra, ztra, xna, yna, zna,
     &                xcen(npl2), ycen(npl2), zcen(npl2),
     &                qx(npl2), qy(npl2), qz(npl2), 1, tol,
     &                ex, ey, ez, fx, fy, fz, uparx, upary, uparz,
     &                ipar, dpmin, itrun, nerrb)

        if ((nerra .ne. 0)  .or.
     &      (nerrb .ne. 0)) then
          write (   6, 8720) ainput, ninput, nerra, nerrb, nerrc
          write (iout, 8720) ainput, ninput, nerra, nerrb, nerrc
          go to 740
        endif

        if (ipar .eq. 0) then         ! Triangle and plane intersect.

          pdist = 0.0
          write (   6, 8680) ex, ey, ez, fx, fy, fz, uparx, upary, uparz
          write (iout, 8680) ex, ey, ez, fx, fy, fz, uparx, upary, uparz
          call plotpt (ex, ey, ez, nerrg)
          call plotpt (fx, fy, fz, nerrg)

          proxptx = ex
          proxpty = ey
          proxptz = ez

          proxvx = 0.0
          proxvy = 0.0
          proxvz = 0.0

        elseif (ipar .eq. 1) then     ! Triangle and plane are parallel.

          pdist = dpmin

          if (itrun .eq. 1) then      ! Triangle and plane are coplanar.

            write (   6, 8682) atrig(ntr1), aquad(npl2)
            write (iout, 8682) atrig(ntr1), aquad(npl2)

          else                        ! Triangle and plane are parallel.

            write (   6, 8684) dpmin
            write (iout, 8684) dpmin

          endif

          proxptx = xcen(npl2)
          proxpty = ycen(npl2)
          proxptz = zcen(npl2)

          call aptvunb (xna, yna, zna, 1, tol,
     &                  ux, uy, uz, vlength, nerra)

          proxvx = ux * dpmin
          proxvy = uy * dpmin
          proxvz = uz * dpmin

          if ((xcen(npl2) - xtra) * proxvx .lt. 0.0) then
            proxvx = -proxvx
            proxvy = -proxvy
            proxvz = -proxvz
          endif

        elseif (ipar .gt. 1) then     ! Triangle or plane is indeterminate.

          nerr = 1
          write (   6, 8686) ainput, ninput, atrig(ntr1), aquad(npl2)
          write (iout, 8686) ainput, ninput, atrig(ntr1), aquad(npl2)

        endif                         ! Tested for relation between planes.

        go to 740

      endif                           ! Triangle to plane.

c===============================================================================

c.... Input line:  distance plane PLNAME1 plane PLNAME2

 8690 format ('Intersection point',1p3e20.12 /
     &        'Intersection point',1p3e20.12 /
     &        'Vector            ',1p3e20.12)
 8692 format ('Planes are coincident:  ',a,' and ',a)
 8694 format ('Parallel distance ',1pe20.12)
 8696 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Plane is indeterminate:  ',a,' or ',a)

      if ((ageom1 .eq. 'plane')  .and.
     &    (ageom2 .eq. 'plane')) then

c....   Find the intersection or distance between the planes.

        call aptplpl (xcen(npl1), ycen(npl1), zcen(npl1),
     &                qx(npl1), qy(npl1), qz(npl1),
     &                xcen(npl2), ycen(npl2), zcen(npl2),
     &                qx(npl2), qy(npl2), qz(npl2), 1, tol,
     &                ex, ey, ez, fx, fy, fz, uparx, upary, uparz,
     &                ipar, dpmin, itrun, nerra)

        if (nerra .ne. 0) then
          write (   6, 8720) ainput, ninput, nerra, nerrb, nerrc
          write (iout, 8720) ainput, ninput, nerra, nerrb, nerrc
          go to 740
        endif

        if (ipar .eq. 0) then         ! Planes intersect.

          pdist = 0.0
          write (   6, 8690) ex, ey, ez, fx, fy, fz, uparx, upary, uparz
          write (iout, 8690) ex, ey, ez, fx, fy, fz, uparx, upary, uparz
          call plotpt (ex, ey, ez, nerrg)
          call plotpt (fx, fy, fz, nerrg)

          proxptx = ex
          proxpty = ey
          proxptz = ez

          proxvx = 0.0
          proxvy = 0.0
          proxvz = 0.0

        elseif (ipar .eq. 1) then     ! Planes are coincident or parallel.

          pdist = dpmin

          if (itrun .eq. 1) then      ! Planes are coincident.

            write (   6, 8692) aquad(npl1), aquad(npl2)
            write (iout, 8692) aquad(npl1), aquad(npl2)

          else                        ! Planes are parallel.

            write (   6, 8694) dpmin
            write (iout, 8694) dpmin

          endif

          proxptx = xcen(npl2)
          proxpty = ycen(npl2)
          proxptz = zcen(npl2)

          call aptvunb (xcen(npl1), ycen(npl1), zcen(npl1), 1, tol,
     &                  ux, uy, uz, vlength, nerra)

          proxvx = ux * dpmin
          proxvy = uy * dpmin
          proxvz = uz * dpmin

          if ((xcen(npl2) - xcen(npl1)) * proxvx .lt. 0.0) then
            proxvx = -proxvx
            proxvy = -proxvy
            proxvz = -proxvz
          endif

        elseif (ipar .gt. 1) then     ! Planes are indeterminate.

          nerr = 1
          write (   6, 8696) ainput, ninput, aquad(npl1), aquad(npl2)
          write (iout, 8696) ainput, ninput, aquad(npl1), aquad(npl2)

        endif                         ! Tested for relation between planes.

        go to 740

      endif                           ! Plane to plane.

c===============================================================================

c.... Input line:  distance plane PLNAME sphere SPHNAME

      if ((ageom1 .eq. 'plane')   .and.
     &    (ageom2 .eq. 'sphere')) then

        call intpls (npl1, nq2)

        call intplqu (npl1, nq2)

        go to 740

      endif                           ! Plane to sphere.

c===============================================================================

c.... Input line:  distance plane PLNAME cylinder CYLNAME

      if ((ageom1 .eq. 'plane')     .and.
     &    (ageom2 .eq. 'cylinder')) then

        call intplcy (npl1, nq2)

        go to 740

      endif                           ! Plane to cylinder.

c===============================================================================

c.... Input line:  distance plane PLNAME quadric QNAME

      if ((ageom1 .eq. 'plane')     .and.
     &   ((ageom2 .eq. 'cone')      .or.
     &    (ageom2 .eq. 'hyperb')    .or.
     &    (ageom2 .eq. 'ellipsoid') .or.
     &    (ageom2 .eq. 'axisym')    .or.
     &    (ageom2 .eq. 'quadric'))) then

        call intplqu (npl1, nq2)

        go to 740

      endif                           ! Plane to cylinder.

c===============================================================================

c.... Input line:  distance sphere SPHNAME plane PLNAME

      if ((ageom2 .eq. 'plane')   .and.
     &    (ageom1 .eq. 'sphere')) then

        call intpls (npl2, nq1)

        call intplqu (npl1, nq2)

        go to 740

      endif                           ! Sphere to plane.

c===============================================================================

c.... Input line:  distance sphere SPHNAME1 sphere SPHNAME2

      if ((ageom1 .eq. 'sphere')  .and.
     &    (ageom2 .eq. 'sphere')) then

        call intsph (nq1, nq2)

        go to 740

      endif                           ! Sphere to sphere.

c===============================================================================

c.... Input line:  distance sphere SPHNAME cylinder CYLNAME

      if ((ageom1 .eq. 'sphere')    .and.
     &    (ageom2 .eq. 'cylinder')) then

        call intspcy (nq1, nq2)

        go to 740

      endif                           ! Sphere to cylinder.

c===============================================================================

c.... Input line:  distance cylinder CYLNAME plane PLNAME

      if ((ageom2 .eq. 'plane')   .and.
     &    (ageom1 .eq. 'cylinder')) then

        call intplcy (npl2, nq1)

        go to 740

      endif                           ! Cylinder to plane.

c===============================================================================

c.... Input line:  distance cylinder CYLNAME sphere SPHNAME

      if ((ageom2 .eq. 'sphere')    .and.
     &    (ageom1 .eq. 'cylinder')) then

        call intspcy (nq2, nq1)

        go to 740

      endif                           ! Cylinder to sphere.

c===============================================================================

c.... Input line:  distance cylinder CYLNAME1 cylinder CYLNAME2

      if ((ageom1 .eq. 'cylinder')   .and.
     &    (ageom2 .eq. 'cylinder')) then

        call intcyl (nq1, nq2)

        go to 740

      endif                           ! Cylinder to cylinder.

c===============================================================================

c.... Input line:  distance quadric QNAME plane PLNAME

      if ((ageom2 .eq. 'plane')     .and.
     &   ((ageom1 .eq. 'sphere')    .or.
     &    (ageom1 .eq. 'cone')      .or.
     &    (ageom1 .eq. 'hyperb')    .or.
     &    (ageom1 .eq. 'ellipsoid') .or.
     &    (ageom1 .eq. 'axisym')    .or.
     &    (ageom1 .eq. 'quadric'))) then

        call intplqu (npl2, nq1)

        go to 740

      endif                           ! Cylinder to plane.

c===============================================================================

c.... Input line:  distance quadric QNAME1 quadric QNAME2

      if (((ageom1 .eq. 'plane')     .or.
     &     (ageom1 .eq. 'sphere')    .or.
     &     (ageom1 .eq. 'cone')      .or.
     &     (ageom1 .eq. 'hyperb')    .or.
     &     (ageom1 .eq. 'ellipsoid') .or.
     &     (ageom1 .eq. 'axisym')    .or.
     &     (ageom1 .eq. 'quadric'))   .and.
     &    ((ageom2 .eq. 'cone')      .or.
     &     (ageom2 .eq. 'hyperb')    .or.
     &     (ageom2 .eq. 'sphere')    .or.
     &     (ageom2 .eq. 'ellipsoid') .or.
     &     (ageom2 .eq. 'axisym')    .or.
     &     (ageom2 .eq. 'quadric'))) then

        call intquad (nq1, nq2)

        go to 740

      endif                           ! Cylinder to plane.

c===============================================================================

c.... Input line:  unrecognized

 8710 format ('  distance ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Method is not recognized:  object ',a,'  and object ',a)

      nerr = 1
      write (   6, 8710) ainput, ninput, ageom1, ageom2
      write (iout, 8710) ainput, ninput, ageom1, ageom2

c===============================================================================

c.... Switch back to the original coordinate system, if necessary.

  740 if (asysx .ne. 'rect') then
        nword = 2
        aword(2) = asysx
        call coord
      endif

  999 return


c.... End of subroutine distance.     (+1 line)
      end
