      subroutine doenddo

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  doenddo  Updated     2005 February 1 18:00.
c               doenddo  Originated  2004 October 19 14:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To control the execution of blocks of commands beginning with
c               a "do" command and ending with an "enddo" command, and to
c               display the do loop control data.
c               Such "do loop" blocks may be nested.
c
c     Logic:    Entry points:  doenddo.
c doenddo       Called by:  helpall, main.
c
c doenddo       Calls:  input, varfind, variable.
c
c     Biblio:   ?
c
c     Glossary:
c
c     avarl(24)          Loop variable name.
c
c     ilbeg(16)          First line of loop in command file.
c
c     lvarl(16)          Number of characters in name avarl.
c
c     ncmds              Number of lines in command file geom.cmd.
c
c     ninput             Number of current command line.
c
c     nlbeg(16)          Initial value of loop variable.
c
c     nlend(16)          Final value of loop variable.
c
c     nlinc(16)          Increment in value of loop variable.
c
c     nlnow(16)          Current value of loop variable.
c
c     nloop              Index of current do loop.  Max 16.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c=======================================================================********

c.... Input line:  do   [help]
c.... Input line:  enddo help

 8110 format (
     &  '  Command "do":  acceptable input lines include:' //
     &  '  "do [help]"    Display this message.' /
     &  '  "enddo help"   Display this message.' //
     &  '  "do VARNAME [=] N1 N2 [INC|1]"' /
     &  '     Execute any commands following this command,' /
     &  '     through the corresponding command "enddo",' /
     &  '     for each value N of variable VARNAME in the range from' /
     &  '     N1 to N2, with increment INC (default 1).' //
     &  '  "enddo"' /
     &  '     Repeat the current "do-enddo" block for N = N + INC,' /
     &  '     unless N + INC is not in the range from N1 to N2.' //
     &  '  Note:  "do-enddo" blocks of commands may be nested.' //
     &  '  Synonyms:  [help, h].' )

 8112   format (/ '  Current              do loop level',i3,'.' /
     &    '  Loop variable ',a,1x,i8,' (',i8,') ',i8,' =',i8,'.'  /
     &    '  Input file ',a24', line',i8,'.  ncmds =',i8,'.')

      if (((nword .eq. 1)         .and.
     &     (aword(1) .eq. 'do'))  .or.
     &    ((nword .eq. 2)         .and.
     &    ((aword(2) .eq. 'help') .or.
     &     (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        if (nloop .gt. 0) then
          write (   6, 8112) nloop, avarl(nloop),
     &      nlbeg(nloop), nlinc(nloop), nlend(nloop), nlnow(nloop),
     &      ainput, ilbeg(nloop), ncmds
          write (iout, 8112) nloop, avarl(nloop),
     &      nlbeg(nloop), nlinc(nloop), nlend(nloop), nlnow(nloop),
     &      ainput, ilbeg(nloop), ncmds
        endif
cbugc***DEBUG begins.
cbug 8113 format ('doenddo DEBUG.  ninput=',i8,'  ncmds=',i8,'.')
cbug      write (   6, 8113) ninput, ncmds
cbug      write (iout, 8113) ninput, ncmds
cbugc***DEBUG ends.
        go to 999
      endif

c=======================================================================********

c.... Input line:  do VARNAME N1 N2 [INC|1]

 8125 format ('  doenddo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on "do" line not 4 or 5:  ',i3)

 8128 format ('  doenddo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Loop variable is not unique in nested loops:  ',a)

 8130 format ('  doenddo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Argument is not an integer or integer variable:  ',a)

 8135 format ('  doenddo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Loop variable increment may not be zero:  ',a)

 8140 format ('  doenddo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Variable name is not acceptable:  ',a)

 8141 format ('  doenddo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Incompatible N1, N2, INC.',3i8)

      if (aword(1) .eq. 'do') then

        nwordx = nword

c....   Add missing "=".

        if ((nword .ge. 4) .and. (aword(3) .ne. '=')) then
          nword = nword + 1
          do nw = nword, 4, -1
            aword(nw) = aword(nw-1)
            iword(nw) = iword(nw-1)
            fword(nw) = fword(nw-1)
            mtype(nw) = mtype(nw-1)
          enddo
          aword(3) = '='
          iword(3) = 0
          fword(3) = 0.0
          mtype(3) = 1
        endif

c....   Test for correct number of arguments.

        if ((nword .lt. 5) .or. (nword .gt. 6)) then
          write (   6, 8125) ainput, ninput, nwordx
          write (iout, 8125) ainput, ninput, nwordx
          go to 999
        endif

c....   Add increment of 1, if not specified.

        if (nword .eq. 5) then
          nword    = 6
          aword(6) = '1'
          iword(6) = 1
          fword(6) = 1.0
          lword(6) = 1
          mtype(6) = 2
        endif

c....   See if loop variable name is unique, if loops are nested.

        if (nloop .ge. 1) then
          do nl = 1, nloop
cbugc***DEBUG begins.
cbug 9765 format ('nloop=',i3,'  nl=',i3,'  aword(2)= ',a8,
cbug     &  '  avarl(nl)= ',a8)
cbug      write (   6, 9765) nloop, nl, aword(2), avarl(nl)
cbug      write (iout, 9765) nloop, nl, aword(2), avarl(nl)
cbugc***DEBUG ends.
            if (aword(2) .eq. avarl(nl)) then
              write (   6, 8128) ainput, ninput, aword(2)
              write (iout, 8128) ainput, ninput, aword(2)
              go to 999
            endif
          enddo
        endif


c....   Find values of arguments 4, 5 and 6, and see if integers.

        call varfind (4, 6)

        do nw = 4, 6
          if (mtype(nw) .ne. 2) then
            write (   6, 8130) ainput, ninput, aword(nw)
            write (iout, 8130) ainput, ninput, aword(nw)
            go to 999
          endif
        enddo

        if (iword(6) .eq. 0) then
          write (   6, 8135) ainput, ninput, aword(6)
          write (iout, 8135) ainput, ninput, aword(6)
          go to 999
        endif

c....   See if increment has right sign.

        if (iword(6) * (iword(5) - iword(4)) .lt. 0) then
          write (   6, 8141) ainput, ninput, iword(4), iword(5),
     &                       iword(6)
          write (iout, 8141) ainput, ninput, iword(4), iword(5),
     &                       iword(6)
          go to 999
        endif

c....   Store loop parameters.

        nloop = nloop + 1
cbugc***DEBUG begins.
cbug 9777 format ('doenddo DEBUG.  nbug =',i3,'  nloop =',i3,'.')
cbug      nbug = 1
cbug      write (   6, 9777) nbug, nloop
cbug      write (iout, 9777) nbug, nloop
cbugc***DEBUG ends.

 8145   format ('  doenddo ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Number of nested loops exceeds maximum:  ',i5)

        if (nloop .gt. nloopm) then
          nloop = nloop - 1
          write (   6, 8145) ainput, ninput, nloop
          write (iout, 8145) ainput, ninput, nloop
          go to 999
        endif

        if (nloop .eq. 1) then
          nlnest = 1
        else
          nlnest = max (nloop, nlnest)
        endif

        nlbeg(nloop) = iword(4)
        nlend(nloop) = iword(5)
        nlinc(nloop) = iword(6)

c....   Initialize loop index variable.

        nword    = 3

        mtype(1) = 1
        aword(1) = 'variable'
        iword(1) = 0
        fword(1) = 0.0
        lword(1) = 8

        avarl(nloop) = aword(2)
        lvarl(nloop) = lword(2)

        mtype(3) = mtype(4)
        aword(3) = aword(4)
        iword(3) = iword(4)
        fword(3) = fword(4)
        lword(3) = lword(4)

 8142 format ('<var    ',a,' = ',i20)

        write (   6, 8142) avarl(nloop), iword(4)
        write (iout, 8142) avarl(nloop), iword(4)

        call variable

c....   Verify loop index variable.

        do n = 1, nvars
          if (avarbl(n) .eq. avarl(nloop)) then
            if ((mvar(n) .eq. 2) .and. (ivar(n) .eq. nlbeg(nloop))) then
              go to 150
            endif
          endif
        enddo

c....   Loop index variable not initialized.

        write (   6, 8140) ainput, ninput, avarl(nloop)
        write (iout, 8140) ainput, ninput, avarl(nloop)
        nloop = nloop - 1
cbugc***DEBUG begins.
cbug      nbug = 2
cbug      write (   6, 9777) nbug, nloop
cbug      write (iout, 9777) nbug, nloop
cbugc***DEBUG ends.
        go to 999

c....   Initialize for this do loop.

  150   nlnow(nloop) = nlbeg(nloop)
cbugc***DEBUG begins.
cbug 7771 format ('doenddo DEBUG.  nloop =',i3,'  ilbeg(nloop) =',i4 /
cbug     &  '  ninput =',i4,'  ncmds =',i4 /
cbug     &  '  ainput =',a)
cbug      write (   6, 7771) nloop, ilbeg(nloop), ninput, ncmds, ainput
cbug      write (iout, 7771) nloop, ilbeg(nloop), ninput, ncmds, ainput
cbugc***DEBUG ends.

       if (ainput .eq. 'geom.cmd') then
         ilbeg(nloop) = ninput + 1
       else
         ilbeg(nloop) = ncmds + 1
       endif

cbugc***DEBUG begins.
cbug      write (   6, 7771) nloop, ilbeg(nloop), ninput, ncmds, ainput
cbug      write (iout, 7771) nloop, ilbeg(nloop), ninput, ncmds, ainput
cbugc***DEBUG ends.

cbugc***DEBUG begins.
cbug 8150   format (/ '  Executing "do"    of do loop level',i3,'.' /
cbug     &    '  Loop variable ',a,1x,i8,' (',i8,') ',i8,' =',i8,'.'  /
cbug     &    '  Input file ',a24', line',i8,'.  ncmds =',i8,'.')
cbug
cbug        write (   6, 8150) nloop, avarl(nloop),
cbug     &    nlbeg(nloop), nlinc(nloop), nlend(nloop), nlnow(nloop),
cbug     &    ainput, ilbeg(nloop), ncmds
cbug        write (iout, 8150) nloop, avarl(nloop),
cbug     &    nlbeg(nloop), nlinc(nloop), nlend(nloop), nlnow(nloop),
cbug     &    ainput, ilbeg(nloop), ncmds
cbug      write (   6, 8113) ninput, ncmds
cbug      write (iout, 8113) ninput, ncmds
cbug 9120 format ('Loop',i3,2x,a,2x,5i6)
cbug      write (   6, 9120) (n, avarl(n), nlbeg(n), nlinc(n), nlend(n),
cbug     &  nlnow(n), ilbeg(n), n = 1, nloop)
cbug      write (iout, 9120) (n, avarl(n), nlbeg(n), nlinc(n), nlend(n),
cbug     &  nlnow(n), ilbeg(n), n = 1, nloop)
cbugc***DEBUG ends.

        go to 999

      endif                           ! Input line "do ...'.

c=======================================================================********

c.... Input line:  enddo

c-------------------------------------------------------------------------------

c.... Test for errors.

      if (aword(1) .ne. 'enddo') go to 310
cbugc***DEBUG begins.
cbug 8255 format (/ '  Executing "enddo" of do loop level',i3,'.' /
cbug     &  '  Loop variable ',a,1x,i8,' (',i8,') ',i8,' =',i8,'.' /
cbug     &  '  Input file ',a24', line',i8,'.  ncmds =',i8,'.')
cbug
cbug      write (   6, 8255) nloop, avarl(nloop),
cbug     &  nlbeg(nloop), nlinc(nloop), nlend(nloop), nlnow(nloop),
cbug     &  ainput, ilbeg(nloop), ncmds
cbug      write (iout, 8255) nloop, avarl(nloop),
cbug     &  nlbeg(nloop), nlinc(nloop), nlend(nloop), nlnow(nloop),
cbug     &  ainput, ilbeg(nloop), ncmds
cbug      write (   6, 8113) ninput, ncmds
cbug      write (iout, 8113) ninput, ncmds
cbug      write (   6, 9120) (n, avarl(n), nlbeg(n), nlinc(n), nlend(n),
cbug     &  nlnow(n), ilbeg(n), n = 1, nloop)
cbug      write (iout, 9120) (n, avarl(n), nlbeg(n), nlinc(n), nlend(n),
cbug     &  nlnow(n), ilbeg(n), n = 1, nloop)
cbugc***DEBUG ends.

c.... Test for the correct number of arguments.

 8225 format ('  doenddo ERROR.  Reading ',a40,' line ',i5,'.' /
     &'  Number of words on "enddo" line not 1:  ',i3)

      if (nword .ne. 1) then
        write (   6, 8225) ainput, ninput, nword
        write (iout, 8225) ainput, ninput, nword
        go to 999
      endif

c.... See if currently in a do loop.

 8230 format ('  doenddo ERROR.  Reading ',a40,' line ',i5,'.' /
     & '  Can not use command "enddo" unless in a do loop.')

      if (nloop .eq. 0) then
        write (   6, 8230) ainput, ninput
        write (iout, 8230) ainput, ninput
        go to 999
      endif

c.... See if the loop variable still exists, and has the correct value.

 8240 format ('  doenddo WARNING.  Reading ',a40,' line ',i5,'.' /
     &  '  Loop variable not an integer:  ',a,' =',1pe20.12,'.')

 8242 format ('  doenddo WARNING.  Reading ',a40,' line ',i5,'.' /
     &  '  Loop variable value changed within loop:  ',a,' =',i8,'.')

      do n = 1, nvars                 ! Loop over variables.
        if (avarbl(n) .eq. avarl(nloop)) then
          if (mvar(n) .ne. 2) then
            write (   6, 8240) ainput, ninput, avarl(nloop), fvar(n)
            write (iout, 8240) ainput, ninput, avarl(nloop), fvar(n)
            go to 999
          endif
          if (ivar(n) .ne. nlnow(nloop)) then
            write (   6, 8242) ainput, ninput, avarl(nloop), ivar(n)
            write (iout, 8242) ainput, ninput, avarl(nloop), ivar(n)
            nlnow(nloop) = ivar(n)
          endif
          go to 250
        endif
      enddo                           ! End of loop over variables.

 8250 format ('  doenddo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Loop variable no longer exists:  ',a)

      write (   6, 8250) ainput, ninput, avarl(nloop)
      write (iout, 8250) ainput, ninput, avarl(nloop)
      go to 999

  250 continue                        ! Loop variable is correct.

c-------------------------------------------------------------------------------

c.... Increment the loop variable.

      nlnext = nlnow(nloop) + nlinc(nloop)

c.... See if the loop variable is outside the specified range.

      if (((nlinc(nloop) .lt. 0) .and.
     &     (nlnext .lt. nlend(nloop))) .or.
     &    ((nlinc(nloop) .gt. 0) .and.
     &     (nlnext .gt. nlend(nloop)))) go to 255

c-------------------------------------------------------------------------------

c.... Repeat the present loop with the new loop variable.

c.... Store the new loop variable.

      nlnow(nloop) = nlnext

      write (   6, 8142) avarl(nloop), nlnow(nloop)
      write (iout, 8142) avarl(nloop), nlnow(nloop)

      nword    = 3

      mtype(1) = 1
      aword(1) = 'variable'

      mtype(2) = 1
      aword(2) = avarl(nloop)
      lword(2) = lvarl(nloop)

      mtype(3) = 2
      aword(3) = 'nlnow(nloop)'
      iword(3) = nlnow(nloop)
      fword(3) = nlnow(nloop)

      call variable

c-------------------------------------------------------------------------------

c.... Repeat the loop commands saved in geom.cmd.

cbugc***DEBUG begins.
cbug 8270 format (/ '  Will repeat          do loop level',i3,'.' /
cbug     &  '  Loop variable ',a,1x,i8,' (',i8,') ',i8,' =',i8,'.' /
cbug     &  '  Input file ',a24', line',i8,'.  ncmds =',i8,'.')
cbug
cbug      write (   6, 8270) nloop, avarl(nloop),
cbug     &  nlbeg(nloop), nlinc(nloop), nlend(nloop), nlnow(nloop),
cbug     &  ainput, ilbeg(nloop), ncmds
cbug      write (iout, 8270) nloop, avarl(nloop),
cbug     &  nlbeg(nloop), nlinc(nloop), nlend(nloop), nlnow(nloop),
cbug     &  ainput, ilbeg(nloop), ncmds
cbug      write (   6, 8113) ninput, ncmds
cbug      write (iout, 8113) ninput, ncmds
cbug      write (   6, 9120) (n, avarl(n), nlbeg(n), nlinc(n), nlend(n),
cbug     &  nlnow(n), ilbeg(n), n = 1, nloop)
cbug      write (iout, 9120) (n, avarl(n), nlbeg(n), nlinc(n), nlend(n),
cbug     &  nlnow(n), ilbeg(n), n = 1, nloop)
cbugc***DEBUG ends.
 8252 format ('<input geom.cmd',i8,' $')

      write (   6, 8252) ilbeg(nloop)
      write (iout, 8252) ilbeg(nloop)

      nword = 4

      mtype(1) = 1
      aword(1) = 'input'

      mtype(2) = 1
      aword(2) = 'geom.cmd'

      mtype(3) = 2
      aword(3) = 'ilbeg(nloop)'
      iword(3) = ilbeg(nloop)

      mtype(4) = 1
      aword(4) = '$'

      call input

      go to 999

c-------------------------------------------------------------------------------

c.... Completed this do loop.  See if any outer loops remain.

  255 continue

cbugc***DEBUG begins.
cbug 8260 format (/ '  Completed            do loop level',i3,'.' /
cbug     &  '  Loop variable ',a,1x,i8,' (',i8,') ',i8,' =',i8,'.'  /
cbug     &  '  Input file ',a24', line',i8,'.  ncmds =',i8,'.')
cbug
cbug      write (   6, 8260) nloop, avarl(nloop),
cbug     &  nlbeg(nloop), nlinc(nloop), nlend(nloop), nlnow(nloop),
cbug     &  ainput, ilbeg(nloop), ncmds
cbug      write (iout, 8260) nloop, avarl(nloop),
cbug     &  nlbeg(nloop), nlinc(nloop), nlend(nloop), nlnow(nloop),
cbug     &  ainput, ilbeg(nloop), ncmds
cbug      write (   6, 8113) ninput, ncmds
cbug      write (iout, 8113) ninput, ncmds
cbug      write (   6, 9120) (n, avarl(n), nlbeg(n), nlinc(n), nlend(n),
cbug     &  nlnow(n), ilbeg(n), n = 1, nloop)
cbug      write (iout, 9120) (n, avarl(n), nlbeg(n), nlinc(n), nlend(n),
cbug     &  nlnow(n), ilbeg(n), n = 1, nloop)
cbugc***DEBUG ends.

      if (nloop .eq. 1) then          ! End of outer do loop.
        nloop    = 0
cbugc***DEBUG begins.
cbug      nbug = 3
cbug      write (   6, 9777) nbug, nloop
cbug      write (iout, 9777) nbug, nloop
cbugc***DEBUG ends.
        nlnest   = 0
        go to 270
      endif                           ! End of outer do loop.

c-------------------------------------------------------------------------------

c.... One or more outer loops remain.  Continue the next one.

      nloop = nloop - 1
cbugc***DEBUG begins.
cbug      nbug = 4
cbug      write (   6, 9777) nbug, nloop
cbug      write (iout, 9777) nbug, nloop
cbugc***DEBUG ends.
cbugc***DEBUG begins.
cbug 8265 format (/ '  Will continue        do loop level',i3,'.' /
cbug     &  '  Loop variable ',a,1x,i8,' (',i8,') ',i8,' =',i8,'.'  /
cbug     &  '  Input file ',a24', line',i8,'.  ncmds =',i8,'.')
cbug
cbug      write (   6, 8265) nloop, avarl(nloop),
cbug     &  nlbeg(nloop), nlinc(nloop), nlend(nloop), nlnow(nloop),
cbug     &  ainput, ilbeg(nloop), ncmds
cbug      write (iout, 8265) nloop, avarl(nloop),
cbug     &  nlbeg(nloop), nlinc(nloop), nlend(nloop), nlnow(nloop),
cbug     &  ainput, ilbeg(nloop), ncmds
cbug      write (   6, 8113) ninput, ncmds
cbug      write (iout, 8113) ninput, ncmds
cbug      write (   6, 9120) (n, avarl(n), nlbeg(n), nlinc(n), nlend(n),
cbug     &  nlnow(n), ilbeg(n), n = 1, nloop)
cbug      write (iout, 9120) (n, avarl(n), nlbeg(n), nlinc(n), nlend(n),
cbug     &  nlnow(n), ilbeg(n), n = 1, nloop)
cbugc***DEBUG ends.

c.... If on first pass through this and outer loops, read from input file.

      do nl = nloop, nlnest
        if (nlnow(nl) .ne. nlbeg(nl)) go to 999
      enddo

c-------------------------------------------------------------------------------

c.... Still on the first pass through outer nested loops,
c....   or done with all loops.
c....   Continue reading the input file.

 8256 format ('<return')

  270 if (ainput .eq. 'geom.cmd') then  ! Return to input file.             

        write (   6, 8256)
        write (iout, 8256)

        nword        = 1
        mtype(1)     = 1
        aword(1)     = 'return'

        call input

      endif                             ! Return to input file.

      go to 999

c===============================================================================

 8310 format ('  doenddo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

  310 write (   6, 8140) ainput, ninput
      write (iout, 8140) ainput, ninput

c=======================================================================********

  999 return

c.... End of subroutine doenddo.      (+1 line)
      end
