      subroutine ellipsoid

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  ellipsoid  Updated     2005 April 5 17:45.
c               ellipsoid  Originated  1993 December 16 8:30.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display and create ellipsoids.
c
c     Logic:    Entry points: ellipsoid.
c ellipsoid     Called by:  helpall, main.
c
c ellipsoid     Calls:  aptmopv, aptrois, aptrotv, aptsubr, apttran, apttris,
c               aptvdis, incrsubs, namechk, ptfind, qumove, quoper, quout,
c               qustore, varfind, vectfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      common /laxis/ rotm(3,3)        ! Principle axis rotation tensor.

      character*8 amount

c.... Input line:  ellipsoid [help]

 8110 format (
     &  '  Command "ellipsoid":  acceptable input lines include:' /
     &  '  "ellipsoid [help]"  Display this message.' /
     &  '  "ellipsoid all"     Display all ellipsoids.' /
     &  '  "ellipsoid list [ELNAME1 ELNAME2 ELNAME3 ...]"' /
     &  '     Display all listed ellipsoids',
     &  ' (default is all ellipsoids).' /
     &  '  "ellipsoid ELNAME"  Display ellipsoid ELNAME.' //
     &  '  "ellipsoid ELNAME = PCEN SAX SAY SAZ"' /
     &  '     Create ellipsoid ELNAME, centered',
     &  ' at point PCEN,' /
     &  '     with x, y and z semi-axes SAX, SAY and SAZ.' /
     &  '  "ellipsoid ELNAME fit PCEN VAXIS PNAME2 PNAME3"' /
     &  '     Create axisymmetric ellipsoid ELNAME, centered at point',
     &  ' PCEN,' /
     &  '     with axis vector VAXIS, and through points PNAME1 and',
     &  ' PNAME2.' )

 8112 format (/
     &  '  "ellipsoid ELL02 NUMELL INC move ELL01 VMOVE"' /
     &  '     Create a family of NUMELL ellipsoids ELL02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     spaced at intervals of vector VMOVE,' /
     &  '     starting from ellipsoid ELL01.' //
     &  '  "ellipsoid ELL02 rotate NUMELL INC ELL01 OPNAME PINV"' /
     &  '     Create a family of NUMELL ellipsoids ELL02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     by rotating the preceding ellipsoid with operator',
     &  ' OPNAME' /
     &  '     and invariant point PINV, starting from ellipsoid',
     &  ' ELL01.' //
     &  '  "ellipsoid ELL02 scale NUMELL INC ELL01 OPNAME PINV"' /
     &  '     Create a family of NUMELL ellipsoids ELL02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     by scaling the preceding ellipsoid with operator OPNAME' /
     &  '     and invariant point PINV, starting from ellipsoid',
     &  ' ELL01.' )

 8114 format (/
     &  '  Note:  use +/- for ELNAME or ELL02 to incr/decr the last',
     &  ' ellipsoid name:' / 2x,a //
     &  '  Synonyms:  [ellipsoid, ellipse, ell, el], [help, h],' /
     &  '    [move, mv, translate, trans], [rotate, rot],',
     &  ' [scale, sc].' )

c.... Initialize.

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        write (   6, 8114) aellips
        write (iout, 8114) aellips
        go to 999
      endif

c=======================================================================********

c.... Input line:  ellipsoid all
c.... Input line:  ellipsoid list

 8120 format (/ 'Number of ellipsoids = ',i5)

      if ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'all')    .or.
     &    (aword(2) .eq. 'list'))) then

        nellipx = 0
        do 120 n = 1, nquads
          if (aqtype(n) .ne. aqtypes(15)) go to 120
          if (aquad(n) .eq. ' ') go to 120
          call quout (n, n, 'min', nerrq)
          nellipx = nellipx + 1
  120   continue
        write (   6, 8120) nellipx
        write (iout, 8120) nellipx
        go to 999
      endif

c=======================================================================********

c.... Input line:  ellipsoid list ELNAME1 ELNAME2 ELNAME3 ...

 8130 format ('  ellipsoid ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named ellipsoid does not exist:  ',a)

      if (aword(2) .eq. 'list') then
        do 128 nw = 3, nword
          nfind = 0
          do 125 n = 1, nquads
            if (aqtype(n) .ne. aqtypes(15)) go to 125
            call aptsubr (aquad(n), 1, 24, 80, astem, lstem,
     &                    asub, lsub, nerra)
            if ((aword(nw) .eq. aquad(n)) .or.
     &          (aword(nw) .eq. astem))   then
              nfind = nfind + 1
              call quout (n, n, 'min', nerrq)
            endif
  125     continue
          if (nfind .eq. 0) then
            write (   6, 8130) ainput, ninput, aword(nw)
            write (iout, 8130) ainput, ninput, aword(nw)
          endif
  128   continue
        go to 999
      endif

c=======================================================================********

c.... Input line:  ellipsoid ELNAME

      if (nword .eq. 2) then
        do 130 n = 1, nquads
          if (aqtype(n) .ne. aqtypes(15)) go to 130
          if (aword(2) .eq. aquad(n)) then
            amount = 'maximum'
            call quout (n, n, amount, nerrq)
            go to 999
          endif
  130   continue
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
        go to 999
      endif

c=======================================================================********

c.... Test input for validity.

 8136 format ('  ellipsoid ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of ellipsoid not from 1 to 24 characters:  ',a)

      if ((lword(2) .lt. 1) .or. (lword(2) .gt. 24)) then
        write (   6, 8136) ainput, ninput, aword(2)
        write (iout, 8136) ainput, ninput, aword(2)
        go to 999
      endif

c=======================================================================********

c.... See if last ellipsoid name should be incremented and used.

 8128 format ('  ellipsoid ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last ellipsoid can not be incremented:  ',a)

      if (aword(2) .eq. '+') then
        aword(2) = aellips
        lword(2) = laellips
        call incrsubs (1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8128) ainput, ninput, aellips
          write (iout, 8128) ainput, ninput, aellips
          go to 999
        endif
      endif

 8129 format ('  ellipsoid ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last ellipsoid can not be decremented:  ',a)

      if (aword(2) .eq. '-') then
        aword(2) = aellips
        lword(2) = laellips
        call incrsubs (-1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8129) ainput, ninput, aellips
          write (iout, 8129) ainput, ninput, aellips
          go to 999
        endif
      endif

 8905 format ('  ellipsoid ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of ellipsoid too dangerous to use:  ',a)

      call namechk (aword(2), lword(2), nerrchk)

      if (nerrchk .ne. 0) then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Input line:  ellipsoid ELL02 move NUMELL INC ELL01 VMOVE

      if ((aword(3) .eq. 'move')      .or.
     &    (aword(3) .eq. 'mv')        .or.
     &    (aword(3) .eq. 'translate') .or.
     &    (aword(3) .eq. 'trans'))    then

        call qumove

        go to 999

      endif                           ! Created translated ellipsoid(s).

c===============================================================================

c.... Input line:  ellipsoid ELL02 rotate NUMELL INC ELL01 OPNAME PINV
c.... Input line:  ellipsoid ELL02 scale  NUMELL INC ELL01 OPNAME PINV

      if ((aword(3) .eq. 'scale')   .or.
     &    (aword(3) .eq. 'sc')      .or.
     &    (aword(3) .eq. 'rotate')  .or.
     &    (aword(3) .eq. 'rot'))    then

        call quoper

        go to 999

      endif                           ! Created rotated or scaled ellipsoid(s).

c===============================================================================

c.... Find the index in the quadric surface array at which to store this
c....   ellipsoid.  Set nrep = 1 if replacing an existing quadric surface.

      call qustore (15, nrep, n)

      if (n .eq. 0) go to 999

      nerr = 0

c=======================================================================********

c.... Input line:  ellipsoid ELNAME = PCEN SAX SAY SAZ

      if ((nword .eq. 7)       .and.
     &    (aword(3) .eq. '=')) then

c....   Find the point at the center of the ellipsoid.

        call ptfind (4, np)

        if (np .eq. 0) then
          nerr = 1
        endif

        xpcen = x(np)
        ypcen = y(np)
        zpcen = z(np)

c....   Find values of semi-axes.

 8700   format ('  ellipsoid ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word not integer or floating point value or variable:  ',a)

        call varfind (5, 7)

        do 710 nw = 5, 7
          if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
            write (   6, 8700) ainput, ninput, aword(nw)
            write (iout, 8700) ainput, ninput, aword(nw)
            nerr = 1
          endif
  710   continue

c....   See if semi-axes are all sufficiently positive.

 8720   format ('  ellipsoid ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Semi-axis not sufficiently positive',1pe20.12)

        errsq = tol * (xpcen**2 + ypcen**2 + zpcen**2)

        do 720 nw = 5, 7
          if ((fword(nw)**2 .le. errsq) .or.
     &        (fword(nw)    .le. 0.0))  then
            write (   6, 8720) ainput, ninput, fword(nw)
            write (iout, 8720) ainput, ninput, fword(nw)
            nerr = 1
          endif
  720   continue

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            aquad(n) = ' '
            if (n .eq. nquads) nquads = nquads - 1
          endif
          go to 999
        endif

c....   Find the coefficients of the quadric equation.


        aellips   = aword(2)
        laellips  = lword(2)
        aquads    = aword(2)
        laquads   = lword(2)

        aquad(n)  = aword(2)
        laquad(n) = lword(2)
        qc(n)     = -1.0
        qx(n)     = 0.0
        qy(n)     = 0.0
        qz(n)     = 0.0
        qxy(n)    = 0.0
        qyz(n)    = 0.0
        qzx(n)    = 0.0
        qxx(n)    = 1.0 / fword(5)**2
        qyy(n)    = 1.0 / fword(6)**2
        qzz(n)    = 1.0 / fword(7)**2

        if ((qxx(n) .eq. qyy(n))  .or.
     &      (qyy(n) .eq. qzz(n))  .or.
     &      (qzz(n) .eq. qxx(n))) then
          axisyms  = aword(2)
          laxisyms = lword(2)
        endif

c....   Translate the center of the ellipsoid to the specified point.

        call apttris (1, xpcen, ypcen, zpcen, qc(n),
     &                qx(n), qy(n), qz(n), qxy(n), qyz(n), qzx(n),
     &                qxx(n), qyy(n), qzz(n), tol, nerra)

        amount = 'maximum'
        call quout (n, n, amount, nerrq)

        go to 999

      endif                           ! Ellipsoid, center, semiaxes.

c-----------------------------------------------------------------------********

c.... Input line:  ellipsoid ELNAME fit PCEN VAXIS PNAME2 PNAME3

      if (aword(3) .eq. 'fit') then

 8745   format ('  ellipsoid ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  "ellipsoid ... fit" line does not have 7 words:  ',i3)
 
        if (nword .ne. 7) then
          nerr = 1
          write (   6, 8745) ainput, ninput, nword
          write (iout, 8745) ainput, ninput, nword
        endif

c....   Find the point at the center of the ellipsoid.

        call ptfind (4, np)

        if (np .eq. 0) then
          nerr = 1
        endif

        xpcen = x(np)
        ypcen = y(np)
        zpcen = z(np)

c....   Find the axis of the ellipsoid.

  730   call vectfind (5, nv)

        if (nv .eq. 0) then
          nerr = 1
          go to 760
        endif

c....   See if the axis vector has a real length.

 8750   format ('  ellipsoid ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Axis vector has zero length:  ',a)

        if (vlen(nv) .eq. 0.0) then
          nerr = 1
          write (   6, 8750) ainput, ninput, aword(5)
          write (iout, 8750) ainput, ninput, aword(5)
        endif

c....   Find the two points on the ellipsoid.

  760   call ptfind (6, np)

        if (np .eq. 0) then
          nerr = 1
          go to 770
        endif

        x1 = x(np)
        y1 = y(np)
        z1 = z(np)

  770   call ptfind (7, np)

        if (np .eq. 0) then
          nerr = 1
          go to 790
        endif

        x2 = x(np)
        y2 = y(np)
        z2 = z(np)

c....   See if the points coincide.

 8790   format ('  ellipsoid ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Points coincide.')

  790   call aptvdis (x1, y1, z1, x2, y2, z2, 1, tol, dx12, dy12, dz12,
     &                d12, nerra)

        if (d12 .eq. 0.0) then
          nerr = 1
          write (   6, 8790) ainput, ninput
          write (iout, 8790) ainput, ninput
        endif

        if (nerr .ne. 0) go to 830

c....   Find the operator to rotate the z axis to the ellipsoid axis.

        call aptrotv (0., 0., 1., xcom(nv), ycom(nv), zcom(nv), tol,
     &                rotm, nerra)

c....   Rotate the points on the ellipsoid with the inverse of the operator.

        call aptmopv (rotm, 1, xpcen, ypcen, zpcen, x1, y1, z1, 1, tol,
     &                nerra)

        call aptmopv (rotm, 1, xpcen, ypcen, zpcen, x2, y2, z2, 1, tol,
     &                nerra)

c....   Move the points to place the center of the ellipsoid at the origin.

        call apttran (xpcen, ypcen, zpcen, x1, y1, z1, 1, tol, nerra)

        call apttran (xpcen, ypcen, zpcen, x2, y2, z2, 1, tol, nerra)

        rsq1 = x1**2 + y1**2
        rsq2 = x2**2 + y2**2

c....   See if the points are at different absolute z values.

 8810   format ('  ellipsoid ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Points do not define an ellipsoid.')

        if ((z1**2 .eq. z2**2) .or.
     &      (rsq2 .ge. rsq1))  then
          write (   6, 8810) ainput, ninput
          write (iout, 8810) ainput, ninput
          nerr = 1
        endif

 8820   format ('  ellipsoid ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Type of quadric surface is indeterminate.')

        if ((z1**2 .eq. z2**2) .and.
     &      (rsq2 .eq. rsq1))  then
          nerr = 1
          write (   6, 8820) ainput, ninput
          write (iout, 8820) ainput, ninput
        endif

  830   if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            aquad(n) = ' '
            if (n .eq. nquads) nquads = nquads - 1
          endif
          go to 999
        endif

c....   Find the standard form of the equation of the ellipsoid.

        aquads    = aword(2)
        laquads   = lword(2)
        aellips   = aword(2)
        laellips  = lword(2)
        axisyms   = aword(2)
        laxisyms  = lword(2)

        aquad(n)  = aword(2)
        laquad(n) = lword(2)

        qzznew    = (rsq1 - rsq2) / (z2**2 - z1**2)

        qc(n)     =  -rsq1 - qzznew * z1**2
        qx(n)     =  0.0
        qy(n)     =  0.0
        qz(n)     =  0.0
        qxy(n)    =  0.0
        qyz(n)    =  0.0
        qzx(n)    =  0.0
        qxx(n)    =  1.0
        qyy(n)    =  1.0
        qzz(n)    =  qzznew

c....   Rotate the ellipsoid to the specified axis.

        call aptrois (rotm, 0, 0., 0., 0., qc(n), qx(n), qy(n), qz(n),
     &                qxy(n), qyz(n), qzx(n), qxx(n), qyy(n), qzz(n),
     &                tol, nerra)

c....   Translate the center of the ellipsoid to the specified point.

        call apttris (1, xpcen, ypcen, zpcen, qc(n),
     &                qx(n), qy(n), qz(n), qxy(n), qyz(n), qzx(n),
     &                qxx(n), qyy(n), qzz(n), tol, nerra)

        amount = 'maximum'
        call quout (n, n, amount, nerrq)

        go to 999

      endif                           ! Ellipsoid:  center, axis, fit 2 points.

c=======================================================================********

 8900 format ('  ellipsoid ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8900) ainput, ninput
      write (iout, 8900) ainput, ninput

      if (nrep .eq. 0) then
        aquad(n) = ' '
        if (n .eq. nquads) nquads = nquads - 1
      endif

c=======================================================================********

  999 return

c.... End of subroutine ellipsoid.    (+1 line)
      end
