      subroutine extrema

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  extrema  Updated     1999 November 3 12:10.
c               extrema  Originated  1994 April 27 18:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find the extrema of one or all quadric surfaces in a
c               specified direction.
c
c     Logic:    Entry points:  extrema.
c extrema       Called by:  helpall, main.
c
c extrema       Calls:  aptcris, aptqexv, aptvxun, plotpt, quout, vectfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      common /lextrv/ ex(2)           ! Coordinate x of an extreme point.
      common /lextrv/ ey(2)           ! Coordinate y of an extreme point.
      common /lextrv/ ez(2)           ! Coordinate z of an extreme point.
      common /lextrv/ adir(2)         ! Type of extreme point.
      character*8 adir                ! Type of extreme point.
      dimension rcurve(4)             ! Radius of curvature.
      dimension cx(4)                 ! Coordinate x of center of curvature.
      dimension cy(4)                 ! Coordinate y of center of curvature.
      dimension cz(4)                 ! Coordinate z of center of curvature.
      dimension ux(4)                 ! Component x of unit direction vector.
      dimension uy(4)                 ! Component y of unit direction vector.
      dimension uz(4)                 ! Component z of unit direction vector.

c.... Input line:  extrema [help]

 8110 format (
     &  '  Command "extrema":  acceptable input lines include:' /
     &  '  "extrema [help]"       Display this message.' //
     &  '  "extrema QNAME VNAME"  Find the extrema of quadric',
     &  ' surface QNAME' /
     &  '     in the direction of vector VNAME, and the' /
     &  '     nature of the surface at any such extrema.' //
     &  '  "extrema all VNAME"    Find the extrema of all quadric',
     &  ' surfaces' /
     &  '     in the direction of vector VNAME, and the' /
     &  '     nature of the surface at any such extrema.' //
     &  '  Note:  the types of extrema include minima, maxima,',
     &  ' simple planes,' /
     &  '    the intersections of planes, saddle points, and',
     &  ' degenerate points.' //
     &  '  Synonyms:  [extrema, extr], [help, h].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c=======================================================================********

c.... Input line:  extrema QNAME VNAME
c.... Input line:  extrema all   VNAME

c.... Test the input line for validity.

 8120 format ('  extrema ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words in input line not 3:  ',i3)

      if (nword .ne. 3) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        go to 999
      endif

c.... Find the vector.

      call vectfind (3, nv)

      if (nv .eq. 0) go to 999

 8140 format ('  extrema ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Direction vector has zero length:  ',a)

      if (vlen(nv) .eq. 0.0) then
        write (   6, 8140) ainput, ninput, aword(3)
        write (iout, 8140) ainput, ninput, aword(3)
        go to 999
      endif

      dx = xcom(nv)
      dy = ycom(nv)
      dz = zcom(nv)

c.... Find the extrema of the specified quadric surface(s) in direction VNAME.

      nquadx = 0

      do 160 nq = 1, nquads
        if (aquad(nq) .eq. ' ') go to 160

        if ((aword(2) .eq. aquad(nq)) .or.
     &      (aword(2) .eq. 'all'))    then

          nquadx = nquadx + 1

          call quout (nq, nq, 'min', nerrq)

          call aptqexv (dx, dy, dz, qc(nq), qx(nq), qy(nq), qz(nq),
     &                  qxy(nq), qyz(nq), qzx(nq),
     &                  qxx(nq), qyy(nq), qzz(nq), tol,
     &                  nextr, ex, ey, ez, adir, nerr)

 8150     format ('No extrema exist.')
 8160     format (/ 'Type of extremum and surface properties:' //
     &            a,'     (xyz)',1p3e20.12 )

          if (nextr .eq. 0) then
            write (   6, 8150)
            write (iout, 8150)
          else
            do 250 n = 1, nextr
                write (   6, 8160) adir(n), ex(n), ey(n), ez(n)
                write (iout, 8160) adir(n), ex(n), ey(n), ez(n)
                call plotpt (ex(n), ey(n), ez(n), nerrg)

c....       Find the curves with zero and extreme curvature, and their
c....         radii, centers of curvature, and direction vectors.

            call aptcris (ex(n), ey(n), ez(n), qc(nq), qx(nq), qy(nq),
     &                    qz(nq),
     &                    qxy(nq), qyz(nq), qzx(nq),
     &                    qxx(nq), qyy(nq), qzz(nq), tol, bx, by, bz,
     &                    nrad, rcurve, cx, cy, cz, ux, uy, uz, nerr)

 8210       format (/ 'Normal vector is zero.',
     &        '  Curvatures are indeterminate.')
 8220       format (/ 'Normal       (xyz)',1p3e20.12)
 8230       format (/ 'Number of curves with zero or extreme',
     &                ' curvature = ',i3,'.' //
     &                'Curvature, radius and center of curvature, unit',
     &        ' direction vector along curve,' /
     &               '  and normal vector of plane containing center',
     &        ' and curve:' )
 8240       format (/ 'Curvature         ',1p3e20.12)    
 8250       format (  'Radius            ',1p3e20.12)    
 8260       format (  'Center       (xyz)',1p3e20.12)    
 8270       format (  'Direction    (xyz)',1p3e20.12)    
 8280       format (  'Plane normal (xyz)',1p3e20.12)    

            if (nrad .eq. 0) then     ! No curves.  
              write (   6, 8210)
              write (iout, 8210)
              go to 250
            else                      ! Curves
              write (   6, 8220) bx, by, bz
              write (iout, 8220) bx, by, bz
              write (   6, 8230) nrad
              write (iout, 8230) nrad
              do 240 nn = 1, nrad     ! Loop over curves with extreme curvature.
                curve = 1.0 / rcurve(nn)
                if (abs (curve) .le. tol) curve = 0.0
                write (   6, 8240) curve
                write (iout, 8240) curve
                write (   6, 8250) rcurve(nn)
                write (iout, 8250) rcurve(nn)
                write (   6, 8260) cx(nn), cy(nn), cz(nn)
                write (iout, 8260) cx(nn), cy(nn), cz(nn)
                write (   6, 8270) ux(nn), uy(nn), uz(nn)
                write (iout, 8270) ux(nn), uy(nn), uz(nn)

                call aptvxun (bx, by, bz, ux(nn), uy(nn), uz(nn), 1,
     &                        tol,
     &                        pnx, pny, pnz, pnlen, nerra)

                write (   6, 8280) pnx, pny, pnz
                write (iout, 8280) pnx, pny, pnz

  240         continue                ! End of loop over curves.
            endif                     ! Tested number of curves.

  250       continue                  ! End of loop over extrema.
          endif

        endif
  160 continue

 8170 format ('  extrema ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named quadric surface does not exist:  ',a)

      if (nquadx .eq. 0) then
        write (   6, 8170) ainput, ninput, aword(2)
        write (iout, 8170) ainput, ninput, aword(2)
      endif

c=======================================================================********

  999 return

c.... End of subroutine extrema.      (+1 line)
      end
