      subroutine factor

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  factor   Updated     2006 March 29 17:00.
c               factor   Originated  2003 September 2 18:20.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find all of the prime factors of one or more integers.
c
c     Logic:    Entry points:  factor
c factor        Called by:  helpall, main.
c
c factor        Calls:  aptpfac, bigfind, varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      integer ifact(18)               ! Prime factor of M.
      integer ipow (18)               ! Exponent of ifact.
      integer icof (18)               ! Cofactor of ifact.
      character*8 amount              ! Determines big integer display.

c===============================================================================

c.... Input line:  factor [help]

 8110 format ('  Command "factor":  acceptable input lines include:' //
     &  '  "factor [help]"   Display this message.' //
     &  '  "factor M1 M2 [thru] M3 ..."' /
     &  '     For each integer M = M1, M2 [, through] M3, ...,' /
     &  '     find all of the prime factors,' /
     &  '     and find how many smaller integers are relatively',
     &  ' prime to M,' /
     &  '     and how many have common factors with M.' //
     &  '  Synonyms:  [factor, totient], [help, h].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c===============================================================================

c.... Input line:  factor M1 M2 M3 ...

      do 200 nw = 2, nword            ! Loop over integers.

 8120   format (' ')

        write (   6, 8120)
        write (iout, 8120)

        if (aword(nw) .eq. 'thru') go to 200

        call varfind (nw, nw)
        call bigfind (nw, nbig)

 8130   format ('  factor ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word is not an integer:  ',a)

        if (nbig .eq. 0) then
          if (mtype(nw) .ne. 2) then
            write (   6, 8130) ainput, ninput, aword(nw)
            write (iout, 8130) ainput, ninput, aword(nw)
            go to 200
          endif
        endif

 8140   format ('  factor WARNING.  Reading ',a40,' line ',i5,'.' /
     &    '  Changed negative integer to positive:  ',i20 )
 8142   format ('  factor WARNING.  Reading ',a40,' line ',i5,'.' /
     &    '  Cannot factor zero:  ',i20 )

        if (nbig .eq. 0) then
          if (iword(nw) .lt. 0) then
            write (   6, 8140) ainput, ninput, iword(nw)
            write (iout, 8140) ainput, ninput, iword(nw)
            iword(nw) = -iword(nw)
          elseif (iword(nw) .eq. 0) then
            write (   6, 8142) ainput, ninput, iword(nw)
            write (iout, 8142) ainput, ninput, iword(nw)
            go to 200
          endif
        else
          if ((ndig(nbig) .eq. 1) .and. (idig(1,nbig) .eq.  0)) then
            write (   6, 8142) ainput, ninput, iword(nw)
            write (iout, 8142) ainput, ninput, iword(nw)
            go to 200
          endif
        endif

c....   If a big integer, see if it can be convertied to a decimal word.

        if (nbig .ne. 0) then
          amount = 'maximum'
          call bigout (nbig, nbig, amount)

          call aptbtod (10, idig(1,nbig), ndig(nbig), idmax, iemax,
     &                  ideca, fdeca, floga, nerra)

 8143   format ('  factor WARNING.  Reading ',a40,' line ',i5,'.' /
     &    '  Big integer too big to convert to a decimal word.' )

          if (ideca .ge. 0) then
            iword(nw) = ideca
            nbig = 0
          else
            write (   6, 8143) ainput, ninput
            write (iout, 8143) ainput, ninput
          endif

        endif                         ! Input was a big integer.

        if (nbig .eq. 0) then         ! Integer is not a big integer.

c....     Find the prime factors and Euler's totient function of iword(nw).

          inum = iword(nw)

 8145     format ('  Factoring    M =',i20 )

          write (   6, 8145) inum
          write (iout, 8145) inum

          call aptpfac (inum, 18, ifact, ipow, nfact, itot, inumr,
     &                  nerra)

 8150     format ('  factor ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Integer can not be factored:  ',i20 /
     &      '  Residue =',i20,'  nerra =',i2)

          if(nerra .ne. 0) then
            write (   6, 8150) ainput, ninput, inum, inumr, nerra
            write (iout, 8150) ainput, ninput, inum, inumr, nerra
            go to 200
          endif

 8160     format ('  PRIME NUMBER.  Can not be factored.')

          if ((nfact .eq. 1) .and. (ipow(1) .eq. 1)) then
            WRITE (   6, 8160)
            write (iout, 8160)
            go to 180
          endif

 8161     format ('  Factor ',i3,'     =',i20,
     &      '      Cofactor =',i20)
 8162     format ('  Factor ',i3,'     =',i20,'^',i1,
     &      '    Cofactor =',i20)
 8170     format ('  Factor ',i3,'     =',i20,'^',i2,
     &      '   Cofactor =',i20)

          do n = 1, nfact
            icof(n) = inum / ifact(n)**ipow(n)
            if (ipow(n) .eq. 1) then
              write (   6, 8161) n, ifact(n), icof(n)
              write (iout, 8161) n, ifact(n), icof(n)
            elseif (ipow(n) .lt. 10) then
              write (   6, 8162) n, ifact(n), ipow(n), icof(n)
              write (iout, 8162) n, ifact(n), ipow(n), icof(n)
            else
              write (   6, 8170) n, ifact(n), ipow(n), icof(n)
              write (iout, 8170) n, ifact(n), ipow(n), icof(n)
            endif
          enddo

c....     Display the Euler totient function of inum.

  180     idiff = inum - itot - 1

 8190     format ('  Totient function',i20,
     &            ' <M, relatively prime to M.' /
     &            '  <M not rel prime',i20 )

          write (   6, 8190) itot, idiff
          write (iout, 8190) itot, idiff

c===============================================================================

        else                          ! Integer is a big integer.

c....     Find the prime factors and Euler's totient function of abig(nbig).

 8245     format ('  Factoring a big integer.')

          write (   6, 8245)
          write (iout, 8245)

          amount = 'minimum'
          call bigout (nbig, nbig, amount)

c???????????????????????????????????????????????????????????????????????????????

c....     Need an APT subroutine to factor a big integer.

c???          call aptprib (idig(1,nbig), ndig(nbig),
c???     &                  ifact?, ipow?, nfact?, itot?, inumr?, nerra)

c???????????????????????????????????????????????????????????????????????????????

        endif                         ! Tested type of integer.

  200 continue                        ! End of loop over integers.

c===============================================================================

c.... Input line:  factor ... M1 thru M2 ...

 8230   format ('  factor ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word "thru" in wrong position.')

      do 300 nw = 2, nword            ! Look for "thru" option.

        if (aword(nw) .eq. 'thru') then

          write (   6, 8120)
          write (iout, 8120)

c....     Test for errors.

          if ((nw .lt. 3) .or. (nw .eq. nword)) then
            write (   6, 8230) ainput, ninput
            write (iout, 8230) ainput, ninput
            go to 300
          endif

          if (mtype(nw-1) .ne. 2) then
            write (   6, 8130) ainput, ninput, aword(nw-1)
            write (iout, 8130) ainput, ninput
            go to 300
          endif

          if (mtype(nw+1) .ne. 2) then
            write (   6, 8130) ainput, ninput, aword(nw+1)
            write (iout, 8130) ainput, ninput
            go to 300
          endif

c....     Loop over range of numbers specified by "thru" option..

          inum1 = iword(nw-1)
          inum2 = iword(nw+1)
          ndir  = 1
          if (inum2 .lt. inum1) ndir = -1

          do 290 inum = inum1, inum2, ndir  ! Loop over integers.

            write (   6, 8120)
            write (iout, 8120)

            write (   6, 8145) inum
            write (iout, 8145) inum

            call aptpfac (inum, 18, ifact, ipow, nfact, itot, inumr,
     &                    nerra)

            if(nerra .ne. 0) then
              write (   6, 8150) ainput, ninput, inum, inumr, nerra
              write (iout, 8150) ainput, ninput, inum, inumr, nerra
              go to 300
            endif

            if ((nfact .eq. 1) .and. (ipow(1) .eq. 1)) then
              write (   6, 8160)
              write (iout, 8160)
              go to 280
            endif

            do n = 1, nfact
              icof(n) = inum / ifact(n)**ipow(n)
              if (ipow(n) .eq. 1) then
                write (   6, 8161) n, ifact(n), icof(n)
                write (iout, 8161) n, ifact(n), icof(n)
              elseif (ipow(n) .lt. 10) then
                write (   6, 8162) n, ifact(n), ipow(n), icof(n)
                write (iout, 8162) n, ifact(n), ipow(n), icof(n)
              else
                write (   6, 8170) n, ifact(n), ipow(n), icof(n)
                write (iout, 8170) n, ifact(n), ipow(n), icof(n)
              endif
            enddo

c....       Display the Euler totient function of inum.

  280       idiff = inum - itot - 1

            write (   6, 8190) itot, idiff
            write (iout, 8190) itot, idiff

  290     continue

        endif                         ! Found word "thru" in command.

  300 continue                        ! Look for "thru" option.

c===============================================================================

  999 return

c.... End of subroutine factor.       (+1 line)
      end
