      subroutine hyperb

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  hyperb   Updated     2005 April 5 17:45.
c               hyperb   Originated  1997 November 7 13:30.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display and create hyperbolic paraboloids.
c               A quadrangle in 3-D space, including its four vertices, its
c               four edges, and its center point, may be fitted by a
c               hyperbolic paraboloid.
c
c     Logic:    Entry points:  hyperb.
c hyperb        Called by:  helpall, main.
c
c hyperb        Calls:  aptqhyp, aptsubr, incrsubs, namechk, ptfind, quout,
c               qustore.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      common /lhyperb/ abc(9,9)       ! Coefficients of equations (row, column).
      common /lhyperb/ cba(9,9)       ! Inverse of matrix abc     (row, column).
      common /lhyperb/ d(9)           ! Right-hand side constants.
      common /lhyperb/ dm(9)          ! Right-hand side constants check.
      common /lhyperb/ dr(9)          ! Right-hand side constants residuals.
      common /lhyperb/ work(9,10)     ! Working matrix.
      common /lhyperb/ xyz(9)         ! Solution.
      common /lhyperb/ xyzm(9)        ! Solution check.
      common /lhyperb/ xyzr(9)        ! Solution residuals.

      common /lhyperb/ alabq(9)       ! Names of unknowns.
      character*8      alabq          ! Names of unknowns.

      data alabq / 'qx',  'qy',  'qz',  'qxy', 'qyz', 'qzx',
     &             'qxx', 'qyy', 'qzz' /

c=======================================================================********

c.... Input line:  hyperb [help]

 8110 format ('  Command "hyperb":  acceptable input lines include:' /
     &  '  "hyperb [help]"  Display this message.' //
     &  '  "hyperb all"     Display all hyperbolic paraboloids.' /
     &  '  "hyperb list [HPNAME1 HPNAME2 HPNAME3 ...]"' /
     &  '     Display all listed hyperbolic paraboloids', /
     &  '     (default is all hyperbolic paraboloids).' /
     &  '  "hyperb HPNAME"  Display hyperbolic paraboloid HPNAME.' //
     &  '  "hyperb HPNAME fit PNAME1 PNAME2 PNAME3 PNAME4"' /
     &  '     Create quadric surface HPNAME, a plane or hyperbolic',
     &  ' paraboloid,' /
     &  '     that contains on its surface the quadrangle with',
     &  ' vertex points' /
     &  '     PNAME1, PNAME2, PNAME3 and PNAME4, including its edges',
     &  ' and center point.' //
     &  '  Note:  use +/- for HPNAME to incr/decr the last hyperbolic',
     &  ' paraboloid' /
     &  '     name:  ',a //
     &  '  Synonyms:  [help, h], [hyperb, saddle].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c=======================================================================********

c.... Input line:  hyperb all
c.... Input line:  hyperb list

 8120 format (/ 'Number of hyperbolic paraboloids    = ',i5)

      if ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'all')  .or.
     &    (aword(2) .eq. 'list'))) then
        nhyperbx  = 0
        nhyperb1x = 0
        do 120 n = 1, nquads
          if (aqtype(n) .ne. aqtypes(8)) go to 120
          if (aquad(n) .eq. ' ') go to 120
          call quout (n, n, 'min', nerrq)
          nhyperbx = nhyperbx + 1
  120   continue
        write (   6, 8120) nhyperbx
        write (iout, 8120) nhyperbx
        go to 999
      endif

c=======================================================================********

c.... Input line:  hyperb list HPNAME1 HPNAME2 HPNAME3 ...

 8135 format ('  hyperb ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named hyperbolic paraboloid does not exist:  ',a)

      if (aword(2) .eq. 'list') then
        do 128 nw = 3, nword
          nfind = 0
          do 125 n = 1, nquads
            if ((aqtype(n) .ne. aqtypes(8))   .and.
     &          (aqtype(n) .ne. aqtypes(13))) go to 125
            call aptsubr (aquad(n), 1, 16, 80, astem, lstem,
     &                    asub, lsub, nerra)
            if ((aword(nw) .eq. aquad(n)) .or.
     &          (aword(nw) .eq. astem))   then
              nfind = nfind + 1
              call quout (n, n, 'min', nerrq)
            endif
  125     continue
          if (nfind .eq. 0) then
            write (   6, 8135) ainput, ninput, aword(nw)
            write (iout, 8135) ainput, ninput, aword(nw)
          endif
  128   continue
        go to 999
      endif

c=======================================================================********

c.... Input line:  hyperb HPNAME

      if (nword .eq. 2) then
        do 130 n = 1, nquads
          if (aqtype(n) .ne. aqtypes(8)) go to 130
          if (aword(2) .eq. aquad(n)) then
            amount = 'maximum'
            call quout (n, n, amount, nerrq)
            go to 999
          endif
  130   continue
        write (   6, 8135) ainput, ninput, aword(2)
        write (iout, 8135) ainput, ninput, aword(2)
        go to 999
      endif

c=======================================================================********

c.... See if the last quadric name should be incremented and used.

 8160 format ('  hyperb ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last hyperbolic paraboloid can not be',
     &  ' incremented:  ',a)

      if (aword(2) .eq. '+') then
        aword(2) = ahyperbs
        lword(2) = lahyperbs
        call incrsubs (1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8160) ainput, ninput, aquads
          write (iout, 8160) ainput, ninput, aquads
          go to 999
        endif
      endif

 8161 format ('  hyperb ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last hyperbolic paraboloid can not be',
     &  ' decremented:  ',a)

      if (aword(2) .eq. '-') then
        aword(2) = ahyperbs
        lword(2) = lahyperbs
        call incrsubs (-1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8161) ainput, ninput, aquads
          write (iout, 8161) ainput, ninput, aquads
          go to 999
        endif
      endif

 8905 format ('  hyperb ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of hyperbolic paraboloid too dangerous to use:  ',a)

      call namechk (aword(2), lword(2), nerrchk)

      if (nerrchk .ne. 0) then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

c=======================================================================********

c.... Input line:  hyperb HPNAME fit PNAME1 PNAME2 PNAME3 PNAME4

 8125 format ('  hyperb ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word 3 not "fit":  ',a)

      if (aword(3) .ne. 'fit') then
        write (   6, 8125) ainput, ninput, aword(3)
        write (iout, 8125) ainput, ninput, aword(3)
        go to 999
      endif

c.... Test for the wrong number of words on "hyperb" line.

 8130 format ('  hyperb ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 7 words on "hyperb" line:  ',i3)

      if (nword .ne. 7) then
        write (   6, 8130) ainput, ninput, nword
        write (iout, 8130) ainput, ninput, nword
        go to 999
      endif

      nerr = 0

c.... Find the four points.

      call ptfind (4, np1)

      if (np1 .eq. 0) then
        nerr = 1
      endif

      call ptfind (5, np2)

      if (np2 .eq. 0) then
        nerr = 1
      endif

      call ptfind (6, np3)

      if (np3 .eq. 0) then
        nerr = 1
      endif

      call ptfind (7, np4)

      if (np4 .eq. 0) then
        nerr = 1
      endif

      if (nerr .ne. 0) go to 999

c.... Find the hyperbolic paraboloid that fits the four points.
cxxxx subroutine aptqhyp (ax, ay, az, bx, by, bz,
cxxxx     &               cx, cy, cz, dx, dy, dz,
cxxxx     &               tol, qc, qx, qy, qz, qxy, qyz, qzx,
cxxxx     &               qxx, qyy, qzz, nerr)


      call aptqhyp (x(np1), y(np1), z(np1), x(np2), y(np2), z(np2),
     &              x(np3), y(np3), z(np3), x(np4), y(np4), z(np4),
     &              tol, ac, ax, ay, az, axy, ayz, azx,
     &              axx, ayy, azz, nerra)

 8140 format ('  hyperb ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Singular matrix.  No solution in 10 tries.' / 
     &  '  Three or more points may be coincident.' )

      if (nerra .eq. 1) then
        write (   6, 8140) ainput, ninput
        write (iout, 8140) ainput, ninput
        go to 999
      endif

 8145 format ('  hyperb ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Two or more points are coincident.' )

      if (nerra .eq. 4) then
        write (   6, 8145) ainput, ninput
        write (iout, 8145) ainput, ninput
        go to 999
      endif

 8150 format ('  hyperb WARNING.  Reading ',a40,' line ',i5,'.' /
     &  '  Inaccurate solution, but may be usable.')

      if ((nerra .eq. 2) .or. (nerra .eq. 3)) then
        write (   6, 8150) ainput, ninput
        write (iout, 8150) ainput, ninput
      endif

c=======================================================================********

c.... Find the index in the quadric surface array at which to store this
c....   hyperbolic paraboloid.
c....   Set nrep = 1 if replacing an existing quadric surface.

      call qustore (-1, nrep, n)

      if (n .eq. 0) go to 999

c.... Store data for the new quadric surface.


      ahyperbs  = aword(2)
      lahyperbs = lword(2)
      aquads    = aword(2)
      laquads   = lword(2)

      aquad(n)  = aword(2)
      laquad(n) = lword(2)
      qc(n)     = ac
      qx(n)     = ax
      qy(n)     = ay
      qz(n)     = az
      qxy(n)    = axy
      qyz(n)    = ayz
      qzx(n)    = azx
      qxx(n)    = axx
      qyy(n)    = ayy
      qzz(n)    = azz

      amount = 'maximum'
      call quout (n, n, amount, nerrq)

      go to 999

c=======================================================================********

  999 return

c.... End of subroutine hyperb.       (+1 line)
      end
