      subroutine ifendif

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  ifendif  Updated     2005 January 3 16:45.
c               ifendif  Originated  1995 March 10 14:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To test the condition for executing a block of commands
c               beginning with an "if" command and ending with an "endif"
c               command, and to display the results of that test and the
c               end of the block of commands.
c
c     Logic:    Entry points:  ifendif.
c ifendif       Called by:  helpall, main.
c
c ifendif       Calls:  varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     nifnest            Number of outer and nested if-endif blocks not to be
c                          executed.
c
c     niftots            Total number of nested if-endif blocks.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c=======================================================================********

c.... Input line:  if   [help]
c.... Input line:  endif help

 8110 format (
     &  '  Command "if":  acceptable input lines include:' /
     &  '  "if [help]"   Display this message.' /
     &  '  "endif help"  Display this message.' //
     &  '  "if VNAME1 [<,<=,=,/=,>=,>] VNAME2 [then]"' /
     &  '     Execute any commands following this command,' /
     &  '     through the corresponding command "endif",' /
     &  '     if integer or floating point values or variables',
     &  ' VNAME1 and VNAME2' /
     &  '     satisfy the condition VNAME1 [less than, less than or'
     &  ' equal to, equal to,' /
     &  '     not equal to, greater than or equal to,',
     &  ' greater than] VNAME2.' //
     &  '  "if VNAME1 null [then]"' /
     &  '     Execute the commands following this command,' /
     &  '     through the corresponding "endif" command,' /
     &  '     if value VNAME1 has not been specified.' //
     &  '  "endif"         End the current "if-endif" block' /
     &  '     (optional if the "if" condition was satisfied).' //
     &  '  Note:  "if-endif" blocks of commands may be nested.' )

 8111 format (/
     &  '  Synonyms:  [/=, =/, .ne.], [<, .lt.], [<=, =<, .le.],',
     &  ' [=, .eq.],' /
     &  '    [=>, >=, .ge.], [>, .gt.], [help, h]/' )
 8112 format (/
     &  '  Currently in unexecuted "if-endif" block, nest level',i5,'.')

      if (((nword .eq. 1)         .and.
     &     (aword(1) .eq. 'if'))  .or.
     &    ((nword .eq. 2)         .and.
     &    ((aword(2) .eq. 'help') .or.
     &     (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8111)
        write (iout, 8111)
        if (nifnest .gt. 0) then
          write (   6, 8112) nifnest
          write (iout, 8112) nifnest
        endif
        go to 999
      endif

c=======================================================================********

c.... Input line:  if VNAME1 <  VNAME2 [then]
c.... Input line:  if VNAME1 <= VNAME2 [then]
c.... Input line:  if VNAME1 =  VNAME2 [then]
c.... Input line:  if VNAME1 >  VNAME2 [then]
c.... Input line:  if VNAME1 >= VNAME2 [then]
c.... Input line:  if VNAME1 null [then]
c.... Input line:  if VNAME1 \= VNAME2 [then]
c.... Input line:  endif

c.... See if already inside failed "if-endif" block.
c....   If so, look for nested "if-endif" blocks, or end of current block.

 8114 format ('  Begins nested "if-endif" block',i3,
     &  ', not to be executed.')
 8115 format ('  Ends outer    "if-endif" block',i3,
     &  ' that was not executed.')
 8116 format ('  Ends nested   "if-endif" block',i3,
     &  ' that was not executed.')

      if (nifnest .gt. 0) then        ! Inside an "if-endif" block.

        if (aword(1) .eq. 'if') then

          niftots = niftots + 1
          nifnest = nifnest + 1
          write (   6, 8114) nifnest
          write (iout, 8114) nifnest
        endif

        if (aword(1) .eq. 'endif') then
          niftots = niftots - 1

 8122   format ('  ifendif ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Can not use command "endif" without command "if".')

          if (niftots .lt. 0) then
            niftots = 0
            write (   6, 8122) ainput, ninput
            write (iout, 8122) ainput, ninput
            go to 999
          endif

          nifnest = nifnest - 1

          if (nifnest .eq. 0) then    ! End of failed "if-endif" block.
            write (   6, 8115) nifnest
            write (iout, 8115) nifnest
          else
            write (   6, 8116) nifnest
            write (iout, 8116) nifnest
          endif

        endif

        go to 999
      endif                           ! Tested for inside an "if-endif" block.

c.... Test for "endif" at end of an executed "if-endif" block.

 8117 format ('  Ends "if-endif" block',i3,' that was executed.')

      if ((nword .eq. 1) .and.
     &    (aword(1) .eq. 'endif')) then
          niftots = niftots - 1

          if (niftots .lt. 0) then
            niftots = 0
            write (   6, 8122) ainput, ninput
            write (iout, 8122) ainput, ninput
            go to 999
          endif

          write (   6, 8117) nifnest
          write (iout, 8117) nifnest
        go to 999
      endif

c=======================================================================********

c.... Input line:  if ...

      niftots = niftots + 1

c=======================================================================********

c.... Input line:  if VNAME1 null [then]

      nerr = 0

      if (aword(3) .eq. 'null') then

      if ((nword .eq. 4)          .and.
     &    (aword(4) .eq. 'then')) then
        nword = 3
      endif

 8118 format ('  ifendif ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on "if VNAME1 null" line not 3:  ',i3)

      if (nword .ne. 3) then
        write (   6, 8118) ainput, ninput, nword
        write (iout, 8118) ainput, ninput, nword
        nifnest = 1
        write (   6, 8180)
        write (iout, 8180)
        go to 999
      endif

c.... Convert any variables to their numerical values.

      call varfind (2, 2)

      if ((mtype(2) .eq. 2) .or. (mtype(2) .eq. 3)) then
        nifnest = 1
      endif

      go to 200

      endif                           ! Input line:  if VNAME1 null [then]

c=======================================================================********

c.... Input line:  if VNAME1 <  VNAME2 [then]
c.... Input line:  if VNAME1 <= VNAME2 [then]
c.... Input line:  if VNAME1 =  VNAME2 [then]
c.... Input line:  if VNAME1 \= VNAME2 [then]
c.... Input line:  if VNAME1 >= VNAME2 [then]
c.... Input line:  if VNAME1 >  VNAME2 [then]
c.... Input line:  if VNAME1 null [then]

c.... Test input line for errors.

      nerr = 0

c.... Remove optional argument "then".

      if ((nword .eq. 5)          .and.
     &    (aword(5) .eq. 'then')) then
        nword = 4
      endif

 8120 format ('  ifendif ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on "if" line not 4:  ',i3)

      if (nword .ne. 4) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        nerr = 1
      endif

c.... Convert any variables to their numerical values.

      call varfind (2, 2)

 8130 format ('  ifendif ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word 2 not integer or floating point value or variable:  ',a)

      if ((mtype(2) .ne. 2) .and. (mtype(2) .ne. 3)) then
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
        nerr = 1
      endif

      call varfind (4, 4)

 8140 format ('  ifendif ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word 4 not integer or floating point value or variable:  ',a)

      if ((mtype(4) .ne. 2) .and. (mtype(4) .ne. 3)) then
        write (   6, 8140) ainput, ninput, aword(4)
        write (iout, 8140) ainput, ninput, aword(4)
        nerr = 1
      endif

      if (nerr .ne. 0) then
        nifnest = 1
        go to 999
      endif

c.... See if the condition is satisfied.  If not, skip commands in block.

 8150 format ('  ifendif ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word 3 is not a recognized relation:  ',a)

      if     ((aword(3) .eq. '<')     .or.
     &        (aword(3) .eq. '.lt.')) then
        if (fword(2) .ge. fword(4)) nifnest = 1
      elseif ((aword(3) .eq. '<=')    .or.
     &        (aword(3) .eq. '=<')    .or.
     &        (aword(3) .eq. '.le.')) then
        if (fword(2) .gt. fword(4)) nifnest = 1
      elseif ((aword(3) .eq. '=')     .or.
     &        (aword(3) .eq. '.eq.')) then
        if (fword(2) .ne. fword(4)) nifnest = 1
      elseif ((aword(3) .eq. '/=')     .or.
     &        (aword(3) .eq. '=/')    .or.
     &        (aword(3) .eq. '.ne.')) then
        if (fword(2) .eq. fword(4)) nifnest = 1
      elseif ((aword(3) .eq. '>=')    .or.
     &        (aword(3) .eq. '=>')    .or.
     &        (aword(3) .eq. '.ge.')) then
        if (fword(2) .lt. fword(4)) nifnest = 1
      elseif ((aword(3) .eq. '>')     .or.
     &        (aword(3) .eq. '.gt.')) then
        if (fword(2) .le. fword(4)) nifnest = 1
      else
        write (   6, 8150) ainput, ninput, aword(3)
        write (iout, 8150) ainput, ninput, aword(3)
        nifnest = 1
        nerr = 1
      endif

 8170 format ('  Condition was     satisfied.  Execute this "if-endif"',
     &  ' block of commands.')
 8180 format ('  Condition was not satisfied.  Skip this "if-endif"',
     &  ' block',i3,' of commands.')

  200 if (nifnest .eq. 0) then
        write (   6, 8170)
        write (iout, 8170)
      else
        write (   6, 8180) nifnest
        write (iout, 8180) nifnest
      endif

c=======================================================================********

  999 return

c.... End of subroutine ifendif.      (+1 line)
      END
