      subroutine intpls (npl, nsph)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  intpls   Updated     1996 March 12 18:00.
c               intpls   Originated  1994 April 5 11:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find any intersection between plane npl and sphere nsph,
c               and the axis and distance from the plane to the sphere center,
c               and the surface separation.
c
c     Logic:    Entry points:  intpls.
c intpls        Called by:  distance.
c
c intpls        Calls:  aptplsp, plotpt.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c=======================================================================********

c.... Initialize.

      pdist = -1.e99

c.... Find the radius of the sphere.

      fact  = (qx(nsph)**2 + qy(nsph)**2 + qz(nsph)**2) / qxx(nsph)**2
      rsph  = sqrt (0.25 * fact - qc(nsph) / qxx(nsph))

c=======================================================================********

c.... Find any intersection of the plane and the sphere.

      call aptplsp (rsph, xcen(nsph), ycen(nsph), zcen(nsph),
     &              xcen(npl), ycen(npl), zcen(npl),
     &              qx(npl), qy(npl), qz(npl), 1, tol,
     &              d12, rcirc, ucirc, vcirc, wcirc, itrun, nerra)

 8160 format ('Dist to center    ',1pe20.12 )

      write (   6, 8160) d12
      write (iout, 8160) d12

      distex  = d12  - rsph
      distin  = d12  + rsph
      disterr = tol * (abs (d12) + rsph)

      if (abs (distex) .le. disterr) distex = 0.0
      if (abs (distin) .le. disterr) distin = 0.0

      dpmin = distex
      if (abs (distin) .lt. abs (distex)) dpmin = distin

 8170 format ('Separation dists  ',1p2e20.12 )

      write (   6, 8170) distex, distin 
      write (iout, 8170) distex, distin 

 8200 format ('No intersection.')
 8210 format ('  Tangent at point',1p3e20.12 )
 8220 format ('  Radius of circle',1pe20.12 )
 8230 format ('  Center of circle',1p3e20.12 )
 8240 format ('  Axis   of circle',1p3e20.12 )

      if (rcirc .lt. 0.0) then         ! No intersection.
        pdist = dpmin
        write (   6, 8200)
        write (iout, 8200)
      elseif (rcirc .eq. 0.0) then     ! Tangent.
        pdist = 0.0
        write (   6, 8210) ucirc, vcirc, wcirc
        write (iout, 8210) ucirc, vcirc, wcirc
        call plotpt (ucirc, vcirc, wcirc, nerrg)
      elseif (rcirc .gt. 0.0) then     ! Intersect in circle.
        pdist = dpmin
        write (   6, 8220) rcirc
        write (iout, 8220) rcirc
        write (   6, 8230) ucirc, vcirc, wcirc
        write (iout, 8230) ucirc, vcirc, wcirc
        call plotpt (ucirc, vcirc, wcirc, nerrg)
        write (   6, 8240) qx(npl), qy(npl), qz(npl)
        write (iout, 8240) qx(npl), qy(npl), qz(npl)
      endif

  999 return

c.... End of subroutine intpls.       (+1 line)
      end
