      subroutine intspcy (nsph, ncyl)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  intspcy  Updated     1996 December 9 17:40.
c               intspcy  Originated  1995 January 5 16:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find the distance of separation or overlap between a sphere
c               and a circular cylinder.
c
c     Logic:    Entry points:  intspcy.
c intspcy       Called by:  distance.
c
c intspcy       Calls:  aptspcy, plotpt.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Find the distance between the sphere and the circular cylinder.
 
      call aptspcy (qc(nsph), qx(nsph), qy(nsph), qz(nsph),
     &              qxy(nsph), qyz(nsph), qzx(nsph), 
     &              qxx(nsph), qyy(nsph), qzz(nsph),
     &              qc(ncyl), qx(ncyl), qy(ncyl), qz(ncyl),
     &              qxy(ncyl), qyz(ncyl), qzx(ncyl),
     &              qxx(ncyl), qyy(ncyl), qzz(ncyl), tol,
     &              rsph, rcyl, ax1, ay1, az1, ax2, ay2, az2,
     &              bx, by, bz, daxis,
     &              cx1, cy1, cz1, cx2, cy2, cz2,
     &              dx, dy, dz, dsurf, nerr)
 
      pdist = dsurf

 8160 format ('Center on axis at ',1p3e20.12)
 8200 format ('Sphere prox x,y,z ',1p3e20.12)
 8210 format ('Cyl    prox x,y,z ',1p3e20.12)
 8230 format ('Proximal dx,dy,dz ',1p3e20.12)
 8235 format ('  Min dist   (ext)',1pe20.12,
     &  '  daxis - rsph - rcyl, 1| |2' /
     &        '  Separation (ext)',1pe20.12,
     &  '  daxis + rsph + rcyl, |1 2|' /
     &        '  Separation (int)',1pe20.12,
     &  '  daxis - rsph + rcyl, |1 |2' /
     &        '  Separation (int)',1pe20.12,
     &  '  daxis + rsph - rcyl, 1| 2|' )

      if (daxis .eq. 0.0) then        ! Center of sphere is on cylinder axis.

        dmm = -rsph - rcyl
        dpp = rsph + rcyl
        errd = tol * dpp
        dmp = -rsph + rcyl
        if (abs (dmp) .le. errd) dmp = 0.0
        dpm = rsph - rcyl
        if (abs (dpm) .le. errd) dpm = 0.0

        write (   6, 8160) ax1, ay1, az1
        write (iout, 8160) ax1, ay1, az1
        call plotpt (ax1, ay1, az1, nerrg)

        write (   6, 8200) cx1, cy1, cz1
        write (iout, 8200) cx1, cy1, cz1
        write (   6, 8210) cx2, cy2, cz2
        write (iout, 8210) cx2, cy2, cz2

        call plotpt (cx1, cy1, cz1, nerrg)
        call plotpt (cx2, cy2, cz2, nerrg)

        proxptx = cx2
        proxpty = cy2
        proxptz = cz2

        write (   6, 8230) dx, dy, dz
        write (iout, 8230) dx, dy, dz

        proxvx = dx
        proxvy = dy
        proxvz = dz

        write (   6, 8235) dmm, dpp, dmp, dpm
        write (iout, 8235) dmm, dpp, dmp, dpm
        go to 999

      else                            ! Center of sphere not on cylinder axis.

 8182   format ('Sphere cent x,y,z ',1p3e20.12 /
     &          'Axis prox   x,y,z ',1p3e20.12 )
 8184   format ('Axis prox dx,dy,dz',1p3e20.12 /
     &          'Center-axis dist  ',1pe20.12 )
 8190   format ('Tangent at x,y,z  ',1p3e20.12 )

c....   Display the sphere center and axis proximal point and their distance.

        write (   6, 8182) ax1, ay1, az1, ax2, ay2, az2
        write (iout, 8182) ax1, ay1, az1, ax2, ay2, az2

        call plotpt (ax1, ay1, az1, nerrg)
        call plotpt (ax2, ay2, az2, nerrg)

        write (   6, 8184) bx, by, bz, daxis
        write (iout, 8184) bx, by, bz, daxis

c....   Find all of the separation distances.

        dmm = dsurf
        dpp = daxis + rsph + rcyl
        dmp = daxis - rsph + rcyl
        errd = tol * dpp
        if (abs (dmp) .le. errd) dmp = 0.0
        dpm = daxis + rsph - rcyl
        if (abs (dpm) .le. errd) dpm = 0.0

c....   Display the surface proximal points and their distance.

        if (dsurf .eq. 0.0) then      ! The sphere and cylinder are tangent.
          write (   6, 8190) cx1, cy1, cz1
          write (iout, 8190) cx1, cy1, cz1

          proxptx = cx1
          proxpty = cy1
          proxptz = cz1

          proxvx = 0.0
          proxvy = 0.0
          proxvz = 0.0

          call plotpt (cx1, cy1, cz1, nerrg)

          write (   6, 8235) dmm, dpp, dmp, dpm
          write (iout, 8235) dmm, dpp, dmp, dpm
          go to 999
        endif

        write (   6, 8200) cx1, cy1, cz1
        write (iout, 8200) cx1, cy1, cz1
        write (   6, 8210) cx2, cy2, cz2
        write (iout, 8210) cx2, cy2, cz2

        call plotpt (cx1, cy1, cz1, nerrg)
        call plotpt (cx2, cy2, cz2, nerrg)

        proxptx = cx2
        proxpty = cy2
        proxptz = cz2

        write (   6, 8230) dx, dy, dz
        write (iout, 8230) dx, dy, dz

        proxvx = dx
        proxvy = dy
        proxvz = dz

        write (   6, 8235) dmm, dpp, dmp, dpm
        write (iout, 8235) dmm, dpp, dmp, dpm

      endif                           ! Tested axis intersection or not.

  999 return

c.... End of subroutine intspcy.      (+1 line)
      end
