      subroutine intsph (nq1, nq2)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  intsph   Updated     1996 March 12 18:15.
c               intsph   Originated  1994 February 11 11:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find any intersection between sphere nq1 and sphere nq2,
c               the axis and distance between their centers, and the separation
c               or overlap between their surfaces.
c
c     Logic:    Entry points:  intsph.
c intsph        Called by:  distance.
c
c intsph        Calls:  aptspsp, aptvadd, aptvdis, aptvunz, plotpt.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c=======================================================================********

c.... Input line:  distance sphere SPHNAME1 sphere SPHNAME2

c.... Initialize.

      pdist = -1.e99

c.... Find the radius of the first sphere.

      fact  = (qx(nq1)**2 + qy(nq1)**2 + qz(nq1)**2) / qxx(nq1)**2
      rsph1 = sqrt (0.25 * fact - qc(nq1) / qxx(nq1))

c.... Find the radius of the second sphere.

      fact  = (qx(nq2)**2 + qy(nq2)**2 + qz(nq2)**2) / qxx(nq2)**2
      rsph2  = sqrt (0.25 * fact - qc(nq2) / qxx(nq2))

c=======================================================================********

c.... Find axis, distance between centers, overlap or separation.

      call aptvdis (xcen(nq1), ycen(nq1), zcen(nq1),
     &              xcen(nq2), ycen(nq2), zcen(nq2),
     &              1, tol, dx, dy, dz, d12, nerr)

c.... Find any intersection of the two spheres.

      call aptspsp (rsph1, xcen(nq1), ycen(nq1), zcen(nq1),
     &              rsph2, xcen(nq2), ycen(nq2), zcen(nq2), 1, tol,
     &              radc, xcirc, ycirc, zcirc,
     &                    xnorm, ynorm, znorm, nerr)

 8160 format ('Axis, sph 1 to 2  ',1p3e20.12 /
     &        '  Center distance ',1pe20.12 )

      write (   6, 8160) dx, dy, dz, d12
      write (iout, 8160) dx, dy, dz, d12

      distex1 = d12 - rsph1 - rsph2
      distex2 = d12 + rsph1 + rsph2
      distin1 = d12 - rsph1 + rsph2
      distin2 = d12 + rsph1 - rsph2
      disterr = tol * (abs (d12) + rsph1 + rsph2)

 8170 format ('  Min dist   (ext)',1pe20.12,'  d12 - r1 - r2, 1| |2' /
     &        '  Separation (ext)',1pe20.12,'  d12 + r1 + r2, |1 2|' /
     &        '  Separation (int)',1pe20.12,'  d12 - r1 + r2, |1 |2' /
     &        '  Separation (int)',1pe20.12,'  d12 + r1 - r2, 1| 2|' )

      if (abs (distex1) .lt. disterr) distex1 = 0.0
      if (abs (distex2) .lt. disterr) distex2 = 0.0
      if (abs (distin1) .lt. disterr) distin1 = 0.0
      if (abs (distin2) .lt. disterr) distin2 = 0.0

      write (   6, 8170) distex1, distex2, distin1, distin2
      write (iout, 8170) distex1, distex2, distin1, distin2

 8200 format ('  No intersection.' )
 8210 format ('  Tangent at point',1p3e20.12 )
 8220 format ('  Radius of circle',1pe20.12 )
 8230 format ('  Center of circle',1p3e20.12 )
 8240 format ('  Axis   of circle',1p3e20.12 )
 8250 format ('  Spheres coincide.' )

      if (radc .lt. 0) then           ! No intersection.
        pdist = distex1
        write (   6, 8200)
        write (iout, 8200)
      elseif (radc .eq. 0.0) then     ! Tangent.
        pdist = 0.0
        write (   6, 8210) xcirc, ycirc, zcirc
        write (iout, 8210) xcirc, ycirc, zcirc
        call plotpt (xcirc, ycirc, zcirc, nerrg)
      elseif (radc .gt. 0) then       ! Intersect in circle, or coincide.

        pdist = distex1

        call aptvunz (xnorm, ynorm, znorm, tol, vlength)

        if (vlength .gt. 0.0) then    ! Intersect in a circle.
          write (   6, 8220) radc
          write (iout, 8220) radc
          write (   6, 8230) xcirc, ycirc, zcirc
          write (iout, 8230) xcirc, ycirc, zcirc
          call plotpt (xcirc, ycirc, zcirc, nerrg)
          write (   6, 8240) xnorm, ynorm, znorm
          write (iout, 8240) xnorm, ynorm, znorm
        else                          ! Spheres coincide.
          write (   6, 8250)
          write (iout, 8250)
        endif
      endif

c.... Find the proximal point on the first sphere.

      call aptvadd (xcen(nq1), ycen(nq1), zcen(nq1),  1.0, rsph1,
     &              xnorm, ynorm, znorm, 1, tol,
     &              ex, ey, ez, vlength, nerra)

c.... Find the proximal point on the second sphere.

      call aptvadd (xcen(nq2), ycen(nq2), zcen(nq2), -1.0, rsph2,
     &              xnorm, ynorm, znorm, 1, tol,
     &              fx, fy, fz, vlength, nerra)

c.... Find the vector between the proximal points.

      call aptvdis (ex, ey, ez, fx, fy, fz, 1, tol,
     &              dx, dy, dz, vlength, nerr)

c.... Display the proximal points and proximal vector.

 8260 format ('  Sph 1 prox (xyz)',1p3e20.12 /
     &        '  Sph 2 prox (xyz)',1p3e20.12 /
     &        '  Vector 1 to 2   ',1p3e20.12 /
     &        '  Proximal dist   ',1pe20.12  )

      write (   6, 8260) ex, ey, ez, fx, fy, fz, dx, dy, dz, distex1
      write (iout, 8260) ex, ey, ez, fx, fy, fz, dx, dy, dz, distex1

      call plotpt (ex, ey, ez, nerrg)
      call plotpt (fx, fy, fz, nerrg)

      proxptx = fx
      proxpty = fy
      proxptz = fz

      proxvx = dx
      proxvy = dy
      proxvz = dz

  999 return

c.... End of subroutine intsph.       (+1 line)
      end
