      subroutine mean

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  mean     Updated     2006 July 14 13:45.
c               mean     Originated  2006 July 7 14:10.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find mean value and standard deviation of sets of variables.
c
c     Logic:    Entry points:  mean
c mean          Called by:  helpall, main.
c
c mean          Calls:  aptchmv, aptsubr, compsubs, varout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*24 anamemn            ! Minimum 24-character name to average.
      character*24 anamemx            ! Maximum 24-character name to average.
cbugc***DEBUG begins.
cbug      character*64 abug
cbug 9330 format ('bigint DEBUG.  ',a)
cbugc***DEBUG ends.

c.... Input line:  mean [help]

 8110 format ('  Command "mean":  acceptable input lines include:' /
     &  '  "mean [help]"  Display this message.' //
     &  '  "mean all"' /
     &  '     Average all variables.' //
     &  '  "mean VARNAME1 thru VARNAME2"' /
     &  '     Average any variables with names from',
     &  ' VARNAME1 to VARNAME2,' //
     &  '     where the order is ASCII, except for subscripts,' /
     &  '     which are in numerical order.' /
     &  '  "mean VARNAME all"' /
     &  '     Average all variables with the names',
     &  ' VARNAME(NSUB),' /
     &  '     where NSUB is any subscript, or null.' //
     &  '  Synonyms:  [help, h].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c===============================================================================

c.... Test for the wrong number of words on "mean" line.

 8120 format ('  mean ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Wrong number of words on "mean" line:  ', i3)

      if ((nword .lt. 2) .or. (nword .gt. 4)) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        go to 999
      endif

      if ((nword .eq. 4) .and. (aword(3) .ne. 'thru')) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        go to 999
      endif

      if ((nword .eq. 3)            .and.
     &    (aword(3) .ne. 'all'))    then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        go to 999
      endif

c===============================================================================

c.... Find the maximum lengths of the variable names.

      lenmax = 24

      if ((aword(2) .ne. 'all')    .and.
     &   ((lword(2) .lt. 1)        .or.
     &    (lword(2) .gt. lenmax))) then
        write (   6, 8136) ainput, ninput, lenmax, aword(3)
        write (iout, 8136) ainput, ninput, lenmax, aword(3)
        go to 999
      endif

c===============================================================================

c.... Input line:  mean VARNAME

      if ((nword .eq. 2) .and. (aword(2) .ne. 'all')) then
        nword = 4
        aword(3) = 'thru'
        lword(4) = lword(2)
        aword(4) = aword(2)
        numtarg  = 1
      endif

c===============================================================================

c.... Input line:  mean VARNAME1 thru VARNAME2

c.... See if option "thru" is used.

 8136 format ('  mean ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of object not from 1 to ',i3,' characters:  ',a)
 8138 format ('  mean ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Min name > max name:  'a24,2x,a24)
 8139 format ('  mean ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word "thru" is duplicated.')

      anamemn  = '               !'
      anamemx  = '~~~~~~~~~~~~~~~~'
cbugc****DEBUG begins.
cbug      abug = 'initial mn, mx.  Line 119'
cbug      write (   6,  9330) abug
cbug      write (   6,  8145) anamemn, anamemx
cbug      write (iout,  8145) anamemn, anamemx
cbugc****DEBUG ends.

      if ((nword .eq. 4) .and.
     &    (aword(3) .eq. 'thru')) then

        if ((lword(2) .lt. 1) .or. (lword(2) .gt. lenmax)) then
          write (   6, 8136) ainput, ninput, lenmax, aword(2)
          write (iout, 8136) ainput, ninput, lenmax, aword(2)
          go to 999
        endif

        if ((lword(4) .lt. 1) .or. (lword(4) .gt. lenmax)) then
          write (   6, 8136) ainput, ninput, lenmax, aword(4)
          write (iout, 8136) ainput, ninput, lenmax, aword(4)
          go to 999
        endif

        call compsubs (aword(2), aword(4), nsmall, nerrc)

        if (nsmall .eq. 2) then
          write (   6, 8138) ainput, ninput, aword(2), aword(4)
          write (iout, 8138) ainput, ninput, aword(2), aword(4)
          go to 999
        endif

        if ((aword(2) .eq. 'thru')  .or.
     &      (aword(4) .eq. 'thru')) then
          write (   6, 8139) ainput, ninput
          write (iout, 8139) ainput, ninput
          go to 999
        endif

        anamemn = aword(2)
        anamemx = aword(4)
cbugc****DEBUG begins.
cbug 8145 format ('mean DEBUG.' /
cbug     &  'anamemn = ',a /
cbug     &  'anamemx = ',a )
cbug      abug = 'mn, mx.  Line 160'
cbug      write (   6,  9330) abug
cbug      write (   6,  8145) anamemn, anamemx
cbug      write (iout,  8145) anamemn, anamemx
cbugc****DEBUG ends.

        nword    = 2
        aword(2) = 'all'
        numtarg  = nvars

      endif                           ! Word 3 was "thru".

c===============================================================================

c.... Input line:  mean VARNAME all

      if ((nword .eq. 3) .and. (aword(3) .eq. 'all')) then

c....   Average all variables with name VARNAME(ISUB).

        call aptsubr (aword(2), 1, 24, 80, astem, lstem,
     &                asub, lsub, nerra)

        if (lsub(1) .eq. 0) then        ! No subscript.
          if (lstem .le. 14) then

            call aptchmv (astem, 1, lstem, anamemn, 1, nerr)
            call aptchmv (astem, 1, lstem, anamemx, 1, nerr)
cbugc****DEBUG begins.
cbug      abug = 'mn, mx.  Line 188'
cbug      write (   6,  9330) abug
cbug      write (   6,  8145) anamemn, anamemx
cbug      write (iout,  8145) anamemn, anamemx
cbugc****DEBUG ends.

            lnext = lstem + 1

            call aptchmv ('(', 1, 1, anamemn, lnext, nerr)
            call aptchmv ('(', 1, 1, anamemx, lnext, nerr)
cbugc****DEBUG begins.
cbug      write (   6,  8145) anamemn, anamemx
cbug      write (iout,  8145) anamemn, anamemx
cbugc****DEBUG ends.

            nword    = 2
            aword(2) = 'all'
            numtarg  = nvars

          endif
        endif                         ! Unsubscripted name.
      endif                           ! Average same name with any subscript.

c===============================================================================

c.... Input line:  mean VARNAME
c.... Input line:  mean VARNAME1 thru VARNAME2
c.... Input line:  mean all

 8140 format ('  mean ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named variable does not exist:  ',a)
cbugc****DEBUG begins.
cbug      write (   6,  8145) anamemn, anamemx
cbug      write (iout,  8145) anamemn, anamemx
cbugc****DEBUG ends.

        nval  = 0
        fvarmean = 0.0
        fvarstdd = 0.0
        fvarmin =  1.e308
        fvarmax = -1.e308
cbugc***DEBUG begins.
cbug 8142 format ('mean DEBUG.  fvarmean =',1pe20.12,'  fvarstdd =',1pe20.12)
cbug      abug = 'Initialize.'
cbug      write (   6,  9330) abug
cbug      write (   6,  8142) fvarmean, fvarstdd
cbug      write (iout,  8142) fvarmean, fvarstdd
cbugc***DEBUG ends.

 8250 format ('')

        write (   6, 8250)
        write (iout, 8250)

        do 130  n = 1, nvars
          if (avarbl(n) .eq. ' ') go to 130
          call compsubs (avarbl(n), anamemn, nsmall, nerrc)
          if (nsmall .eq. 1) go to 130
          call compsubs (avarbl(n), anamemx, nsmall, nerrc)
          if (nsmall .eq. 2) go to 130
          call varout (n, n)
          nval  = nval + 1
          fvarmin  = min (fvarmin, fvar(n))
          fvarmax  = max (fvarmax, fvar(n))
          fvarmean = fvarmean + fvar(n)
          fvarstdd = fvarstdd + fvar(n)**2
cbugc***DEBUG begins.
cbug      abug = 'In loop 130.'
cbug      write (   6,  9330) abug
cbug      write (iout,  9330) abug
cbug      write (   6,  8142) fvarmean, fvarstdd
cbug      write (iout,  8142) fvarmean, fvarstdd
cbugc***DEBUG ends.
          if (numtarg .eq. 1) go to 140
  130   continue

 8255 format (/ '  Total           ',1pe20.12,'  Samples =',i9 )

  140   write (   6, 8255) fvarmean, nval
        write (iout, 8255) fvarmean, nval
cbugc***DEBUG begins.
cbug      abug = 'Did loop 130.'
cbug      write (   6,  9330) abug
cbug      write (iout,  9330) abug
cbug      write (   6,  8142) fvarmean, fvarstdd
cbug      write (iout,  8142) fvarmean, fvarstdd
cbugc***DEBUG ends.

 8265 format (/ '  Mean, Std Dev   ',1p2e20.12 )

      if (nval .ge. 1) then
        fvarmean = fvarmean / nval
        fvarstdd = fvarstdd / nval
cbugc***DEBUG begins.
cbug      abug = 'Divide by nval'
cbug      write (   6,  9330) abug
cbug      write (iout,  9330) abug
cbug      write (   6,  8142) fvarmean, fvarstdd
cbug      write (iout,  8142) fvarmean, fvarstdd
cbugc***DEBUG ends.
        fvarstdd  = sqrt (fvarstdd - fvarmean**2)
        write (   6, 8265) fvarmean, fvarstdd
        write (iout, 8265) fvarmean, fvarstdd
      endif

 8275 format (  '  Minimum, Maximum',1p2e20.12 )

      if (nval .ge. 1) then
        write (   6, 8275) fvarmin, fvarmax
        write (iout, 8275) fvarmin, fvarmax
      endif

  999 return

c.... End of subroutine mean.         (+1 line)
      end
