      subroutine meshblk (ik, il, im)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  meshblk  Updated     2005 March 14 16:30.
c               meshblk  Originated  1995 August 14 16:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To assign the mesh indices of the current mesh block to
c               the points anamex, ..., with name increments ik, il and im
c               in the k, l and m directions of the mesh block.
c
c     Logic:    Entry points:  meshblk.
c
c meshblk       Calls:  incrsubs, meshsub, pout, ptfind.
c               Called by:  mesh.
c
c     Biblio:   ?
c
c     Glossary:
c
c     aword(2)  store.h  Name of first point being assigned indices.
c
c     ik        Input    Increment in the point name in the k direction.
c
c     il        Input    Increment in the point name in the l direction.
c
c     im        Input    Increment in the point name in the m direction.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      common /ablk/ anamex            ! Temporary name of a point.
      character*24  anamex            ! Temporary name of a point.

c.... Test for name duplication.

      nk    = kbl2 - kbl1 + 1
      nl    = lbl2 - lbl1 + 1
      nm    = mbl2 - mbl1 + 1
      ikabs = iabs (ik)
      ilabs = iabs (il)
      imabs = iabs (im)
      ikmax = (nk - 1) * ikabs
      ilmax = (nl - 1) * ilabs
      immax = (nm - 1) * imabs
      if (ikabs .eq. 0) ikabs = 1
      if (ilabs .eq. 0) ilabs = 1
      if (imabs .eq. 0) imabs = 1

 8220 format ('  meshblk ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Duplicate point names are specified.')

      do 240 incm = -immax, immax, imabs
        do 230 incl = -ilmax, ilmax, ilabs
          do 220 inck = -ikmax, ikmax, ikabs
            if ((inck .eq. 0)  .and.
     &          (incl .eq. 0)  .and.
     &          (incm .eq. 0)) go to 220

            inctot = inck + incl + incm
            if (inctot .eq. 0) then
              write (   6, 8220) ainput, ninput
              write (iout, 8220) ainput, ninput
              go to 999
            endif

  220     continue
  230   continue
  240 continue

c.... Assign the mesh block indices to the points.

      anamex  = aword(3)
      lanamex = lword(3)
      incold  = 0
      nprel   = 0
      npassg  = 0

 8305 format ('  mesh point ',a,2x,3i6,7x,'! RELEASED')
 8310 format ('  mesh point ',a,2x,3i6,7x,'! ASSIGNED')

      do 340 m = mbl1, mbl2           ! Loop over m direction.
        incm  = (m - mbl1) * im

        do 330 l = lbl1, lbl2         ! Loop over l direction.
          incl  = (l - lbl1) * il

          do 320 k = kbl1, kbl2       ! Loop over k direction.
            inck   = (k - kbl1) * ik

            inctot = inck + incl + incm  ! Increment from first name.
            incdel = inctot - incold     ! Increment from previous name.
            incold = inctot              ! Increment from first name.

c....       Find the next point name and storage index.

            call incrsubs (incdel, anamex, 1, lanamex, nerra)

            aword(2)  = anamex
            lword(2)  = lanamex

            call ptfind (2, np)

            call pout (np, np, 'min')

            if (np .eq. 0) go to 320  ! Named point does not exist.

c....       Release any current mesh indices.

            if ((kpoint(np) .ne. 0)  .and.
     &          (lpoint(np) .ne. 0)  .and.
     &          (mpoint(np) .ne. 0)) then
              nprel      = nprel + 1
              kpoint(np) = 0
              lpoint(np) = 0
              mpoint(np) = 0
              klm        = 0
              write (   6, 8305) apoint(np), kpoint(np), lpoint(np),
     &          mpoint(np)
              write (iout, 8305) apoint(np), kpoint(np), lpoint(np),
     &          mpoint(np)
            endif

c....       Assign new mesh indices to the point, and display.

            npassg     = npassg + 1
            kpoint(np) = k
            lpoint(np) = l
            mpoint(np) = m

            call meshsub (np, nrep, nerr)

 8230 format ('  meshblk ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not make point subscripts match mesh indices.')

            if (nerr .ne. 0) then
              write (   6, 8230) ainput, ninput
              write (iout, 8230) ainput, ninput
              go to 999
            endif

            klm        = kpoint(np) + (lpoint(np) - 1) * kmax +
     &                   (mpoint(np) - 1) * kmax * lmax

            write (   6, 8310) apoint(np), kpoint(np), lpoint(np),
     &        mpoint(np)
            write (iout, 8310) apoint(np), kpoint(np), lpoint(np),
     &        mpoint(np)

  320     continue                    ! End of loop over k direction.
  330   continue                      ! End of loop over l direction.
  340 continue                        ! End of loop over m direction.

 8410 format ('  Number of mesh points released =',i7)

      if (nprel .gt. 0) then
        write (   6, 8410) nprel
        write (iout, 8410) nprel
      endif

 8420 format ('  Number of mesh points assigned =',i7)

      if (npassg .gt. 0) then
        write (   6, 8420) npassg
        write (iout, 8420) npassg
      endif

  999 return

c.... End of subroutine meshblk.      (+1 line)
      end
