      subroutine meshtest

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  meshtest Updated     2005 March 9 14:40.
c               meshtest Originated  1995 December 5 16:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To see if any mesh indices in the mesh block have not been
c                 assigned to mesh points, and if any line, surface or
c                 volume elements in the mesh block violate any of the
c                 geometric restrictions:
c                 line elements should not have angles > 90 degrees;
c                 surface elements should not have diagonal fold angles
c                 > 90 degrees;
c                 volume elements should not have inverted vertices.
c
c     Logic:    Entry points:  meshtest.
c meshtest      Called by:  mesh.
c
c meshtest      Calls:  aptrips, aptvaxb, aptvdot, meshfind, pout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c-----------------------------------------------------------------------********

c.... Initialize for mesh point assignment test.

      nerr = 0

c-----------------------------------------------------------------------********

c.... See if all mesh indices in the mesh block have been assigned.

 8110 format ('  meshtest ERROR.  Reading ',a40,' line ',i5,'.' /
     &  'Mesh indices           ',3i6,7x,'! UNASSIGNED')

      do 130 m = mbl1, mbl2           ! Loop over m.
        do 120 l = lbl1, lbl2         ! Loop over l.
          do 110 k = kbl1, kbl2       ! Loop over k.

          call meshfind (k, l, m, np)

          if (np .le. 0) then
            nerr = 1
            klm  = k + (l - 1) * kmax + (m - 1) * kmax * lmax
            write (   6, 8110) ainput, ninput, k, l, m
            write (iout, 8110) ainput, ninput, k, l, m
          endif

  110     continue                    ! End of loop over k.
  120   continue                      ! End of loop over l.
  130 continue                        ! End of loop over m.

      if (nerr .ne. 0) go to 999

c-----------------------------------------------------------------------********

c.... See if any geometric tests are possible.

 8120 format ('Mesh block is a single point:')
 8125 format ('Mesh block is a single k-direction line element,',
     &  ' with end points:')
 8126 format ('Mesh block is a single l-direction line element,',
     &  ' with end points:')
 8128 format ('Mesh block is a single m-direction line element,',
     &  ' with end points:')
 8130 format ('Mesh block has no line element angles.' /
     &        'Mesh block has no surface elements.' /
     &        'Mesh block has no volume elements.')

      if (kbl2 .eq. kbl1) then        ! Mesh block is a k-layer.
        if (lbl2 .eq. lbl1) then      ! Mesh block is 1-D in m direction.
          if (mbl2 .eq. mbl1) then    ! Mesh block is a single mesh point.

            write (   6, 8120)
            write (iout, 8120)

            call meshfind (kbl1, lbl1, mbl1, np)
            call pout (np, np, 'minimum')

            write (   6, 8130)
            write (iout, 8130)

            go to 999

          elseif (mbl2 .eq. (mbl1 + 1)) then ! Mesh block is one line element.

            write (   6, 8128)
            write (iout, 8128)

            call meshfind (kbl1, lbl1, mbl1, np1)
            call meshfind (kbl1, lbl1, mbl2, np2)

            call pout (np1, np1, 'minimum')
            call pout (np2, np2, 'minimum')

            write (   6, 8130)
            write (iout, 8130)

            go to 999

          endif                       ! Mesh block is a single mesh point.

        elseif (mbl2 .eq. mbl1) then  ! Mesh block is 1-D in l direction.
          if (lbl2 .eq. (lbl1 + 1)) then     ! Mesh block is one line element.

            write (   6, 8126)
            write (iout, 8126)

            call meshfind (kbl1, lbl1, mbl1, np1)
            call meshfind (kbl1, lbl2, mbl1, np2)

            call pout (np1, np1, 'minimum')
            call pout (np2, np2, 'minimum')

            write (   6, 8130)
            write (iout, 8130)

            go to 999

          endif                       ! Mesh block is 1-D in l direction.
        endif                         ! Mesh block is a k layer.

      elseif (lbl2 .eq. lbl1) then    ! Mesh block is an l-layer.
        if (mbl2 .eq. mbl1) then      ! Mesh block is 1-D in k direction.
          if (kbl2 .eq. (kbl1 + 1)) then     ! Mesh block is one line element.

            write (   6, 8125)
            write (iout, 8125)

            call meshfind (kbl1, lbl1, mbl1, np1)
            call meshfind (kbl2, lbl1, mbl1, np2)

            call pout (np1, np1, 'minimum')
            call pout (np2, np2, 'minimum')

            write (   6, 8130)
            write (iout, 8130)

            go to 999

          endif                       ! Mesh block is 1-D in k direction.
        endif                         ! Mesh block is an l-layer.
      endif                           ! Tested dimensionality of a mesh block.

c-----------------------------------------------------------------------********

c.... Test any pairs of sequential line elements in the k direction.

 8140 format ('Mesh block has no k-direction line element angles.')

 8150 format ('  meshtest WARNING.  Reading ',a40,' line ',i5,'.' /
     &  '  Angle between two k-direction line elements exceeds',
     &  ' 90 degrees.' /
     &  '  Three consecutive points:' )

      klines = 0
      kbends = 0

      if (kbl2 .lt. (kbl1 + 2)) then  ! No test of line elements in k direction.

        write (   6, 8140)
        write (iout, 8140)

      else                            ! Test line elements in k direction.

        do 150 m = mbl1, mbl2         ! Loop over m (line elements).
          do 145 l = lbl1, lbl2       ! Loop over l (line elements).
            do 140 k = kbl1, kbl2 - 2 ! Loop over k (line elements).

              klines = klines + 1

c....         Find the end points of two adjacent line elements.

              k1 = k
              call meshfind (k1, l, m, np1)
              k2 = k + 1
              call meshfind (k2, l, m, np2)
              k3 = k + 2
              call meshfind (k3, l, m, np3)

c....         Find the vectors of the two adjacent line elements.

              v12x = x(np2) - x(np1)
              v12y = y(np2) - y(np1)
              v12z = z(np2) - z(np1)

              v23x = x(np3) - x(np2)
              v23y = y(np3) - y(np2)
              v23z = z(np3) - z(np2)

c....         See if the angle between the line elements exceeds 90 degrees.

              call aptvdot (v12x, v12y, v12z, v23x, v23y, v23z, 1, tol,
     &                      vdot123, nerra)

              if (vdot123 .lt. 0.0) then  ! Bend is > 90 degrees.

                kbends = kbends + 1
                write (   6, 8150) ainput, ninput
                write (iout, 8150) ainput, ninput

                call pout (np1, np1, 'minimum')
                call pout (np2, np2, 'minimum')
                call pout (np3, np3, 'minimum')

              endif                   ! Bend is > 90 degrees.

  140       continue                  ! End of loop over k (line elements).
  145     continue                    ! End of loop over l (line elements).
  150   continue                      ! End of loop over m (line elements).

      endif                           ! Tested line elements in k direction.

 8160 format ('Mesh block has no k-direction line element angles',
     &  ' > 90 degrees.')

      if ((klines .ne. 0) .and. (kbends .eq. 0)) then  ! No bends > 90 degrees.
        write (   6, 8160)
        write (iout, 8160)
      endif                           ! No bends > 90 degrees.

c-----------------------------------------------------------------------********

c.... Test any pairs of sequential line elements in the l direction.

 8170 format ('Mesh block has no l-direction line element angles.')

 8175 format ('  meshtest WARNING.  Reading ',a40,' line ',i5,'.' /
     &  '  Angle between two l-direction line elements exceeds',
     &  ' 90 degrees.' /
     &  '  Three consecutive points:' )

      llines = 0
      lbends = 0

      if (lbl2 .lt. (lbl1 + 2)) then  ! No test of line elements in l direction.

        write (   6, 8170)
        write (iout, 8170)

      else                            ! Test line elements in l direction.

        do 190 k = kbl1, kbl2         ! Loop over k (line elements).
          do 185 m = mbl1, mbl2       ! Loop over m (line elements).
            do 180 l = lbl1, lbl2 - 2 ! Loop over l (line elements).

              llines = llines + 1

c....         Find the end points of two adjacent line elements.

              l1 = l
              call meshfind (k, l1, m, np1)
              l2 = l + 1
              call meshfind (k, l2, m, np2)
              l3 = l + 2
              call meshfind (k, l3, m, np3)

c....         Find the vectors of the two adjacent line elements.

              v12x = x(np2) - x(np1)
              v12y = y(np2) - y(np1)
              v12z = z(np2) - z(np1)

              v23x = x(np3) - x(np2)
              v23y = y(np3) - y(np2)
              v23z = z(np3) - z(np2)

c....         See if the angle between the line elements exceeds 90 degrees.

              call aptvdot (v12x, v12y, v12z, v23x, v23y, v23z, 1, tol,
     &                      vdot123, nerra)

              if (vdot123 .lt. 0.0) then  ! Bend is > 90 degrees.

                lbends = lbends + 1
                write (   6, 8175) ainput, ninput
                write (iout, 8175) ainput, ninput

                call pout (np1, np1, 'minimum')
                call pout (np2, np2, 'minimum')
                call pout (np3, np3, 'minimum')

              endif                   ! Bend is > 90 degrees.

  180       continue                  ! End of loop over l (line elements).
  185     continue                    ! End of loop over m (line elements).
  190   continue                      ! End of loop over k (line elements).

      endif                           ! Finished line elements in l direction

 8195 format ('Mesh block has no l-direction line element angles',
     &  ' > 90 degrees.')

      if ((llines .ne. 0) .and. (lbends .eq. 0)) then  ! No bends > 90 degrees.
        write (   6, 8195)
        write (iout, 8195)
      endif                           ! No bends > 90 degrees.

c-----------------------------------------------------------------------********

c.... Test any pairs of sequential line elements in the m direction.

 8200 format ('Mesh block has no m-direction line element angles.')

 8210 format ('  meshtest WARNING.  Reading ',a40,' line ',i5,'.' /
     &  '  Angle between two m-direction line elements exceeds',
     &  ' 90 degrees.' /
     &  '  Three consecutive points:' )

      mlines = 0
      mbends = 0

      if (mbl2 .lt. (mbl1 + 2)) then  ! No test of line elements in m direction.

        write (   6, 8200)
        write (iout, 8200)

      else                            ! Test line elements in m direction.

        do 210 l = lbl1, lbl2         ! Loop over l (line elements).
          do 205 k = kbl1, kbl2       ! Loop over k (line elements).
            do 200 m = mbl1, mbl2 - 2 ! Loop over m (line elements).

              mlines = mlines + 1

c....         Find the end points of two adjacent line elements.

              m1 = m
              call meshfind (k, l, m1, np1)
              m2 = m + 1
              call meshfind (k, l, m2, np2)
              m3 = m + 2
              call meshfind (k, l, m3, np3)

c....         Find the vectors of the two adjacent line elements.

              v12x = x(np2) - x(np1)
              v12y = y(np2) - y(np1)
              v12z = z(np2) - z(np1)

              v23x = x(np3) - x(np2)
              v23y = y(np3) - y(np2)
              v23z = z(np3) - z(np2)

c....         See if the angle between the line elements exceeds 90 degrees.

              call aptvdot (v12x, v12y, v12z, v23x, v23y, v23z, 1, tol,
     &                      vdot123, nerra)

              if (vdot123 .lt. 0.0) then  ! Bend > 90 degrees.

                mbends = mbends + 1
                write (   6, 8210) ainput, ninput
                write (iout, 8210) ainput, ninput

                call pout (np1, np1, 'minimum')
                call pout (np2, np2, 'minimum')
                call pout (np3, np3, 'minimum')

              endif                   ! Bend > 90 degrees.

  200       continue                  ! End of loop over m (line elements).
  205     continue                    ! End of loop over k (line elements).
  210   continue                      ! End of loop over l (line elements).

      endif                           ! Finished line elements in m direction

 8215 format ('Mesh block has no m-direction line element angles',
     &  ' > 90 degrees.')

      if ((mlines .ne. 0) .and. (mbends .eq. 0)) then  ! No bends > 90 degrees.
        write (   6, 8215)
        write (iout, 8215)
      endif                           ! No bends > 90 degrees.

c-----------------------------------------------------------------------********

c.... Test any surface elements in each k-layer.

 8220 format ('Mesh block has no k-layer surface elements.')

 8230 format ('  meshtest WARNING.  Reading ',a40,' line ',i5,'.' /
     &  '  Diagonal fold angle of a  k-layer surface element exceeds',
     &  ' 90 degrees.' /
     &  '  This may be a boomerang or a bow-tie.  Four vertex points:' )

      ksurfs = 0
      kfolds = 0

      if ((lbl2 .eq. lbl1) .or. (mbl2 .eq. mbl1)) then  ! No k-layer surfaces.

        write (   6, 8220)
        write (iout, 8220)

      else                            ! Test k-layer surface elements.

        do 230 k = kbl1, kbl2         ! Loop over k (surface elements).
          do 225 m = mbl1, mbl2 - 1   ! Loop over m (surface elements).
            m1 = m
            m2 = m + 1
            do 220 l = lbl1, lbl2 - 1 ! Loop over l (surface elements).
              l1 = l
              l2 = l + 1

              ksurfs = ksurfs + 1

c....         Find the four vertex points of the surface element.

              call meshfind (k, l1, m1, np11)
              call meshfind (k, l1, m2, np12)
              call meshfind (k, l2, m1, np21)
              call meshfind (k, l2, m2, np22)

c....         Find the four edge vectors.

              ax = x(np21) - x(np11)
              ay = y(np21) - y(np11)
              az = z(np21) - z(np11)

              bx = x(np22) - x(np21)
              by = y(np22) - y(np21)
              bz = z(np22) - z(np21)

              cx = x(np12) - x(np22)
              cy = y(np12) - y(np22)
              cz = z(np12) - z(np22)

              dx = x(np11) - x(np12)
              dy = y(np11) - y(np12)
              dz = z(np11) - z(np12)

c....         Find the four normal vectors at the vertices.

              call aptvaxb (ax, ay, az, bx, by, bz, 1, tol,
     &                      abx, aby, abz, vlena, nerra)

              call aptvaxb (bx, by, bz, cx, cy, cz, 1, tol,
     &                      bcx, bcy, bcz, vlena, nerra)

              call aptvaxb (cx, cy, cz, dx, dy, dz, 1, tol,
     &                      cdx, cdy, cdz, vlena, nerra)

              call aptvaxb (dx, dy, dz, ax, ay, az, 1, tol,
     &                      dax, day, daz, vlena, nerra)

c....         See if either diagonal fold exceeds 90 degrees.

              call aptvdot (abx, aby, abz, cdx, cdy, cdz, 1, tol,
     &                      abcddot, nerra)

              call aptvdot (bcx, bcy, bcz, dax, day, daz, 1, tol,
     &                      bcdadot, nerra)

              if ((abcddot .lt. 0.0) .or. (bcdadot .lt. 0.0)) then

                kfolds = kfolds + 1
                write (   6, 8230) ainput, ninput
                write (iout, 8230) ainput, ninput

                call pout (np11, np11, 'minimum')
                call pout (np21, np21, 'minimum')
                call pout (np22, np22, 'minimum')
                call pout (np12, np12, 'minimum')

              endif

  220       continue                  ! End of loop over l (surface elements).
  225     continue                    ! End of loop over m (surface elements).
  230   continue                      ! End of loop over k (surface elements).

      endif                           ! Finished k-layer surface elements.

 8235 format ('Mesh block has no k-layer surface element diagonal',
     &  ' folds > 90 degrees.')

      if ((ksurfs .ne. 0) .and. (kfolds .eq. 0)) then ! No folds > 90 degrees.
        write (   6, 8235)
        write (iout, 8235)
      endif                           ! No folds > 90 degrees.

c-----------------------------------------------------------------------********

c.... Test any surface elements in each l-layer.

 8250 format ('Mesh block has no l-layer surface elements.')
 8255 format ('  meshtest WARNING.  Reading ',a40,' line ',i5,'.' /
     &  '  Diagonal fold angle of an l-layer surface element exceeds',
     &  ' 90 degrees.' /
     &  '  This may be a boomerang or a bow-tie.  Four vertex points:' )

      lsurfs = 0
      lfolds = 0

      if ((kbl2 .eq. kbl1) .or. (mbl2 .eq. mbl1)) then  ! No l-layer surfaces.

        write (   6, 8250)
        write (iout, 8250)

      else                            ! Test l-layer surface elements.

        do 250 l = lbl1, lbl2         ! Loop over l (surface elements).
          do 245 k = kbl1, kbl2 - 1   ! Loop over k (surface elements).
            k1 = k
            k2 = k + 1
            do 240 m = mbl1, mbl2 - 1 ! Loop over m (surface elements).
              m1 = m
              m2 = m + 1

              lsurfs = lsurfs + 1

c....         Find the four vertex points of the surface element.

              call meshfind (k1, l, m1, np11)
              call meshfind (k1, l, m2, np12)
              call meshfind (k2, l, m1, np21)
              call meshfind (k2, l, m2, np22)

c....         Find the four edge vectors.

              ax = x(np21) - x(np11)
              ay = y(np21) - y(np11)
              az = z(np21) - z(np11)

              bx = x(np22) - x(np21)
              by = y(np22) - y(np21)
              bz = z(np22) - z(np21)

              cx = x(np12) - x(np22)
              cy = y(np12) - y(np22)
              cz = z(np12) - z(np22)

              dx = x(np11) - x(np12)
              dy = y(np11) - y(np12)
              dz = z(np11) - z(np12)

c....         Find the four normal vectors at the vertices.

              call aptvaxb (ax, ay, az, bx, by, bz, 1, tol,
     &                      abx, aby, abz, vlena, nerra)

              call aptvaxb (bx, by, bz, cx, cy, cz, 1, tol,
     &                      bcx, bcy, bcz, vlena, nerra)

              call aptvaxb (cx, cy, cz, dx, dy, dz, 1, tol,
     &                      cdx, cdy, cdz, vlena, nerra)

              call aptvaxb (dx, dy, dz, ax, ay, az, 1, tol,
     &                      dax, day, daz, vlena, nerra)

c....         See if either diagonal fold exceeds 90 degrees.

              call aptvdot (abx, aby, abz, cdx, cdy, cdz, 1, tol,
     &                      abcddot, nerra)

              call aptvdot (bcx, bcy, bcz, dax, day, daz, 1, tol,
     &                      bcdadot, nerra)

              if ((abcddot .lt. 0.0) .or. (bcdadot .lt. 0.0)) then

                lfolds = lfolds + 1
                write (   6, 8255) ainput, ninput
                write (iout, 8255) ainput, ninput

                call pout (np11, np11, 'minimum')
                call pout (np21, np21, 'minimum')
                call pout (np22, np22, 'minimum')
                call pout (np12, np12, 'minimum')

              endif

  240       continue                  ! End of loop over m (surface elements).
  245     continue                    ! End of loop over k (surface elements).
  250   continue                      ! End of loop over l (surface elements).

      endif                           ! Finished l-layer surface elements.

 8260 format ('Mesh block has no l-layer surface element diagonal',
     &  ' folds > 90 degrees.')

      if ((lsurfs .ne. 0) .and. (lfolds .eq. 0)) then ! No folds > 90 degrees.
        write (   6, 8260)
        write (iout, 8260)
      endif                           ! No folds > 90 degrees.

c-----------------------------------------------------------------------********

c.... Test any surface elements in each m-layer.

 8270 format ('Mesh block has no m-layer surface elements.')
 8275 format ('  meshtest WARNING.  Reading ',a40,' line ',i5,'.' /
     &  '  Diagonal fold angle of an m-layer surface element exceeds',
     &  ' 90 degrees.' /
     &  '  This may be a boomerang or a bow-tie.  Four vertex points:' )

      msurfs = 0
      mfolds = 0

      if ((kbl2 .eq. kbl1) .or. (lbl2 .eq. lbl1)) then  ! No m-layer surfaces.

        write (   6, 8270)
        write (iout, 8270)

      else                            ! Test m-layer surface elements.

        do 270 m = mbl1, mbl2         ! Loop over m (surface elements).
          do 265 l = lbl1, lbl2 - 1   ! Loop over l (surface elements).
            l1 = l
            l2 = l + 1
            do 260 k = kbl1, kbl2 - 1 ! Loop over k (surface elements).
              k1 = k
              k2 = k + 1

              msurfs = msurfs + 1

c....         Find the four vertex points of the surface element.

              call meshfind (k1, l1, m, np11)
              call meshfind (k1, l2, m, np12)
              call meshfind (k2, l1, m, np21)
              call meshfind (k2, l2, m, np22)

c....         Find the four edge vectors.

              ax = x(np21) - x(np11)
              ay = y(np21) - y(np11)
              az = z(np21) - z(np11)

              bx = x(np22) - x(np21)
              by = y(np22) - y(np21)
              bz = z(np22) - z(np21)

              cx = x(np12) - x(np22)
              cy = y(np12) - y(np22)
              cz = z(np12) - z(np22)

              dx = x(np11) - x(np12)
              dy = y(np11) - y(np12)
              dz = z(np11) - z(np12)

c....         Find the four normal vectors at the vertices.

              call aptvaxb (ax, ay, az, bx, by, bz, 1, tol,
     &                      abx, aby, abz, vlena, nerra)

              call aptvaxb (bx, by, bz, cx, cy, cz, 1, tol,
     &                      bcx, bcy, bcz, vlena, nerra)

              call aptvaxb (cx, cy, cz, dx, dy, dz, 1, tol,
     &                      cdx, cdy, cdz, vlena, nerra)

              call aptvaxb (dx, dy, dz, ax, ay, az, 1, tol,
     &                      dax, day, daz, vlena, nerra)

c....         See if either diagonal fold exceeds 90 degrees.

              call aptvdot (abx, aby, abz, cdx, cdy, cdz, 1, tol,
     &                      abcddot, nerra)

              call aptvdot (bcx, bcy, bcz, dax, day, daz, 1, tol,
     &                      bcdadot, nerra)

              if ((abcddot .lt. 0.0) .or. (bcdadot .lt. 0.0)) then

                mfolds = mfolds + 1
                write (   6, 8275) ainput, ninput
                write (iout, 8275) ainput, ninput

                call pout (np11, np11, 'minimum')
                call pout (np21, np21, 'minimum')
                call pout (np22, np22, 'minimum')
                call pout (np12, np12, 'minimum')

              endif

  260       continue                  ! End of loop over k (surface elements).
  265     continue                    ! End of loop over l (surface elements).
  270   continue                      ! End of loop over m (surface elements).

      endif                           ! Finished m-layer surface elements.

 8280 format ('Mesh block has no m-layer surface element diagonal',
     &  ' folds > 90 degrees.')

      if ((msurfs .ne. 0) .and. (mfolds .eq. 0)) then ! No folds > 90 degrees.
        write (   6, 8280)
        write (iout, 8280)
      endif                           ! No folds > 90 degrees.

c-----------------------------------------------------------------------********

c.... Test any volume elements.

 8310 format ('Mesh block has no volume elements.')

 8320 format ('  meshtest WARNING.  Reading ',a40,' line ',i5,'.' /
     &  '  Vertex of a volume element is inverted.' /
     &  '  Vertex point and opposite end points of three',
     &  ' adjacent edges:')

      nvol    = 0
      ninvert = 0

      if ((kbl2 .eq. kbl1)  .or.
     &    (lbl2 .eq. lbl1)  .or.
     &    (mbl2 .eq. mbl1)) then

        write (   6, 8310)
        write (iout, 8310)

      else                            ! Test all volume elements.

        do 330 m = mbl1, mbl2 - 1     ! Loop over m (volume elements).
          m1 = m
          m2 = m + 1

          do 320 l = lbl1, lbl2 - 1   ! Loop over l (volume elements).
            l1 = l
            l2 = l + 1

            do 310 k = kbl1, kbl2 - 1 ! Loop over k (volume elements).
              k1 = k
              k2 = k + 1

              nvol = nvol + 1

c....         Find the 8 vertex points of the volume element.

              call meshfind (k1, l1, m1, np111)
              call meshfind (k2, l1, m1, np211)
              call meshfind (k1, l2, m1, np121)
              call meshfind (k2, l2, m1, np221)
              call meshfind (k1, l1, m2, np112)
              call meshfind (k2, l1, m2, np212)
              call meshfind (k1, l2, m2, np122)
              call meshfind (k2, l2, m2, np222)

c....         Find the 12 edge vectors of the volume element.

              vk11x = x(np211) - x(np111)
              vk11y = y(np211) - y(np111)
              vk11z = z(np211) - z(np111)

              vk21x = x(np221) - x(np121)
              vk21y = y(np221) - y(np121)
              vk21z = z(np221) - z(np121)

              vk12x = x(np212) - x(np112)
              vk12y = y(np212) - y(np112)
              vk12z = z(np212) - z(np112)

              vk22x = x(np222) - x(np122)
              vk22y = y(np222) - y(np122)
              vk22z = z(np222) - z(np122)

              vl11x = x(np121) - x(np111)
              vl11y = y(np121) - y(np111)
              vl11z = z(np121) - z(np111)

              vl21x = x(np221) - x(np211)
              vl21y = y(np221) - y(np211)
              vl21z = z(np221) - z(np211)

              vl12x = x(np122) - x(np112)
              vl12y = y(np122) - y(np112)
              vl12z = z(np122) - z(np112)

              vl22x = x(np222) - x(np212)
              vl22y = y(np222) - y(np212)
              vl22z = z(np222) - z(np212)

              vm11x = x(np112) - x(np111)
              vm11y = y(np112) - y(np111)
              vm11z = z(np112) - z(np111)

              vm21x = x(np212) - x(np211)
              vm21y = y(np212) - y(np211)
              vm21z = z(np212) - z(np211)

              vm12x = x(np122) - x(np121)
              vm12y = y(np122) - y(np121)
              vm12z = z(np122) - z(np121)

              vm22x = x(np222) - x(np221)
              vm22y = y(np222) - y(np221)
              vm22z = z(np222) - z(np221)

c....         Find the 8 scalar triple products at the 8 vertices.

              call aptrips (vk11x, vk11y, vk11z,
     &                      vl11x, vl11y, vl11z,
     &                      vm11x, vm11y, vm11z,
     &                      1, tol, strip111, nerra)

              call aptrips (vk11x, vk11y, vk11z,
     &                      vl21x, vl21y, vl21z,
     &                      vm21x, vm21y, vm21z,
     &                      1, tol, strip211, nerra)

              call aptrips (vk21x, vk21y, vk21z,
     &                      vl11x, vl11y, vl11z,
     &                      vm12x, vm12y, vm12z,
     &                      1, tol, strip121, nerra)

              call aptrips (vk21x, vk21y, vk21z,
     &                      vl21x, vl21y, vl21z,
     &                      vm22x, vm22y, vm22z,
     &                      1, tol, strip221, nerra)

              call aptrips (vk12x, vk12y, vk12z,
     &                      vl12x, vl12y, vl12z,
     &                      vm11x, vm11y, vm11z,
     &                      1, tol, strip112, nerra)

              call aptrips (vk12x, vk12y, vk12z,
     &                      vl22x, vl22y, vl22z,
     &                      vm21x, vm21y, vm21z,
     &                      1, tol, strip212, nerra)

              call aptrips (vk22x, vk22y, vk22z,
     &                      vl12x, vl12y, vl12z,
     &                      vm12x, vm12y, vm12z,
     &                      1, tol, strip122, nerra)

              call aptrips (vk22x, vk22y, vk22z,
     &                      vl22x, vl22y, vl22z,
     &                      vm22x, vm22y, vm22z,
     &                      1, tol, strip222, nerra)

c....         See if any vertices are inverted.

              if (strip111 .lt. 0.0) then  ! Inverted vertex 111.

                ninvert = ninvert + 1
                write (   6, 8320) ainput, ninput
                write (iout, 8320) ainput, ninput

                call pout (np111, np111, 'minimum')
                call pout (np211, np211, 'minimum')
                call pout (np121, np121, 'minimum')
                call pout (np112, np112, 'minimum')

              endif

              if (strip211 .lt. 0.0) then  ! Inverted vertex 211.

                ninvert = ninvert + 1
                write (   6, 8320) ainput, ninput
                write (iout, 8320) ainput, ninput

                call pout (np211, np211, 'minimum')
                call pout (np111, np111, 'minimum')
                call pout (np221, np221, 'minimum')
                call pout (np212, np212, 'minimum')

              endif

              if (strip121 .lt. 0.0) then  ! Inverted vertex 121.

                ninvert = ninvert + 1
                write (   6, 8320) ainput, ninput
                write (iout, 8320) ainput, ninput

                call pout (np121, np121, 'minimum')
                call pout (np221, np221, 'minimum')
                call pout (np111, np111, 'minimum')
                call pout (np122, np122, 'minimum')

              endif

              if (strip221 .lt. 0.0) then  ! Inverted vertex 221.

                ninvert = ninvert + 1
                write (   6, 8320) ainput, ninput
                write (iout, 8320) ainput, ninput

                call pout (np221, np221, 'minimum')
                call pout (np121, np121, 'minimum')
                call pout (np211, np211, 'minimum')
                call pout (np222, np222, 'minimum')

              endif

              if (strip112 .lt. 0.0) then  ! Inverted vertex 112.

                ninvert = ninvert + 1
                write (   6, 8320) ainput, ninput
                write (iout, 8320) ainput, ninput

                call pout (np112, np112, 'minimum')
                call pout (np212, np212, 'minimum')
                call pout (np122, np122, 'minimum')
                call pout (np111, np111, 'minimum')

              endif

              if (strip212 .lt. 0.0) then  ! Inverted vertex 212.

                ninvert = ninvert + 1
                write (   6, 8320) ainput, ninput
                write (iout, 8320) ainput, ninput

                call pout (np212, np212, 'minimum')
                call pout (np112, np112, 'minimum')
                call pout (np222, np222, 'minimum')
                call pout (np211, np211, 'minimum')

              endif

              if (strip122 .lt. 0.0) then  ! Inverted vertex 122.

                ninvert = ninvert + 1
                write (   6, 8320) ainput, ninput
                write (iout, 8320) ainput, ninput

                call pout (np122, np122, 'minimum')
                call pout (np222, np222, 'minimum')
                call pout (np112, np112, 'minimum')
                call pout (np121, np121, 'minimum')

              endif

              if (strip222 .lt. 0.0) then  ! Inverted vertex 222.

                ninvert = ninvert + 1
                write (   6, 8320) ainput, ninput
                write (iout, 8320) ainput, ninput

                call pout (np222, np222, 'minimum')
                call pout (np122, np122, 'minimum')
                call pout (np212, np212, 'minimum')
                call pout (np221, np221, 'minimum')

              endif

  310       continue                  ! End of loop over k (volume elements).
  320     continue                    ! End of loop over l (volume elements).
  330   continue                      ! End of loop over m (volume elements).

      endif                           ! Finished volume elements.

 8330 format ('Mesh block has no volume elements with inverted',
     &  ' vertices.')

      if ((nvol .ne. 0) .and. (ninvert .eq. 0)) then  ! No inverted vertices.
        write (   6, 8330)
        write (iout, 8330)
      endif                           ! No inverted vertices.

c-----------------------------------------------------------------------********

  999 return

c.... End of subroutine meshtest.    (+1 line)
      end
