      subroutine movex

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  movex    Updated     2004 November 23 13:30.
c               movex    Originated  1993 November 4 16:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To interpret data on "move" input lines, to move points,
c               clusters, vectors, planes, spheres, circular cylinders,
c               circular cones, hyperbolic paraboloids, ellipsoids, axisymmetric
c               quadric surfaces and general quadric surfaces in 3-D space.
c
c     Logic:    Entry points:  movex.
c movex         Called by:  helpall, main, translate* .
c
c movex         Calls:  apttris, clout, pout, ptfind, quout, qutype, varfind,
c               vectfind, vout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      character*8 amount              ! Amount of output desired.

c.... Input line:  move [help]

 8110 format ('  Command "move":  acceptable input lines include:' /
     &  '  "move [help]"  to get this message.' //
     &  '  "move point   PNAME  VMOVE  [VMULT|1]"' /
     &  '     Move point   PNAME  by vector VMOVE  multiplied by',
     &  ' VMULT.' /
     &  '  "move cluster CLNAME VMOVE  [VMULT|1]"' /
     &  '     Move cluster CLNAME by vector VMOVE  multiplied by',
     &  ' VMULT.' /
     &  '  "move point  all     VMOVE  [VMULT|1]"' /
     &  '     Move all points     by vector VMOVE  multiplied by',
     &  ' VMULT.' /
     &  '  "move vector VNAME   VMOVE [VMULT|1]"' /
     &  '     Move vector VNAME   by vector VMOVE multiplied by',
     &  ' VMULT.' )
 8112 format (
     &  '  "move vector all     VMOVE [VMULT|1]"' /
     &  '     Move all vectors    by vector VMOVE multiplied by',
     &  ' VMULT.' /
     &  '  "move QTYPE  QNAME   VMOVE  [VMULT|1]"' /
     &  '     Move the quadric of type QTYPE with name QNAME,',
     &  ' by vector VMOVE' /
     &  '     multiplied by VMULT.' /
     &  '  "move QTYPE  all     VMOVE  [VMULT|1]"' /
     &  '     Move all quadrics of type QTYPE,',
     &  ' by vector VMOVE' /
     &  '     multiplied by VMULT.' //
     &  '  Note:  if VMULT is not specified, it defaults to 1.' //
     &  '  Options:   QTYPE must be plane, sphere, cylinder, cone,',
     &  ' hyperb,' /
     &  '     axisym or quadric.' )
 8114 format (/
     &  '  Synonyms:  [cylinder, cyl], [ellipsoid, ellipse, ell, el],' /
     &  '    [help, h], [hyperb, saddle],',
     &  ' [move, mv, translate, trans],' /
     &  ' [plane, pl],' /
     &  '    [point, pnt, pt, p], [quadric, quad, q], [sphere, sph],',
     &  ' [vector, vect, v].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        write (   6, 8114)
        write (iout, 8114)
        go to 999
      endif

c===============================================================================

c.... Test for wrong number of words on "move" line.

 8120 format ('  movex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 4 or 5 words on "move" line:  ',i3)
      if ((nword .ne. 4) .and. (nword .ne. 5)) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        go to 999
      endif

c===============================================================================

c.... Test for the correct mode of VMULT.

 8130 format ('  movex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word VMULT not integer or floating point value or',
     &  ' variable:  ',a)

      if (nword .eq. 4) then
        nword    = 5
        fword(5) = 1.0
        mtype(5) = 3
      endif

      call varfind (5, 5)

      if ((mtype(5) .ne. 2) .and. (mtype(5) .ne. 3)) then
        write (   6, 8130) ainput, ninput, aword(5)
        write (iout, 8130) ainput, ninput, aword(5)
        go to 999
      endif

      vmult = fword(5)

c===============================================================================

c.... Find the components of the motion.

      call vectfind (4, n)

      if (n .eq. 0) go to 999

 8132 format ('  dx, dy, dz      ',1p3e20.12)

      dx = xcom(n) * vmult
      dy = ycom(n) * vmult
      dz = zcom(n) * vmult

      write (   6, 8132) dx, dy, dz
      write (iout, 8132) dx, dy, dz

c===============================================================================

c.... Input line:  move point PNAME VMOVE [VMULT|1]
c.... Input line:  move point all   VMOVE [VMULT|1]

      if ((aword(2) .eq. 'point') .or.
     &    (aword(2) .eq. 'pnt')   .or.
     &    (aword(2) .eq. 'pt')    .or.
     &    (aword(2) .eq. 'p'))    then

 8160 format ('  movex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  There are no points.')

      if (npoints .eq. 0) then
        write (   6, 8160) ainput, ninput
        write (iout, 8160) ainput, ninput
        go to 999
      endif

c===============================================================================

c.... See if user wants all points moved.

      npointx = 0
      if (npoints .eq. 0) go to 190

      if (aword(3) .eq. 'all') then   ! Move all points.

        do 170 n = 1, npoints
          if (apoint(n) .eq. ' ') go to 170

          npointx = npointx + 1

          amount = 'maximum'
          call pout (n, n, amount)

          x(n)  = x(n) + dx
          y(n)  = y(n) + dy
          z(n)  = z(n) + dz

          tolxx = tol * (abs (x(n)) + abs (dx))
          tolyy = tol * (abs (y(n)) + abs (dy))
          tolzz = tol * (abs (z(n)) + abs (dz))
          if (abs (x(n)) .le. tolxx) x(n) = 0.0
          if (abs (y(n)) .le. tolyy) y(n) = 0.0
          if (abs (z(n)) .le. tolzz) z(n) = 0.0

          amount = 'maximum'
          call pout (n, n, amount)

  170   continue

        go to 200

      endif                           ! Moved all points

c===============================================================================

c.... See if user wants one point moved.

      call ptfind (3, n)

      if (n .eq. 0) go to 999

      npointx = npointx + 1

      x(n)  = x(n) + dx
      y(n)  = y(n) + dy
      z(n)  = z(n) + dz

      tolxx = tol * (abs (x(n)) + abs (dx))
      tolyy = tol * (abs (y(n)) + abs (dy))
      tolzz = tol * (abs (z(n)) + abs (dz))
      if (abs (x(n)) .le. tolxx) x(n) = 0.0
      if (abs (y(n)) .le. tolyy) y(n) = 0.0
      if (abs (z(n)) .le. tolzz) z(n) = 0.0

      amount = 'maximum'
      call pout (n, n, amount)

      go to 200

 8180 format ('  movex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named point does not exist:  ',a)

  190 write (   6, 8180) ainput, ninput, aword(3)
      write (iout, 8180) ainput, ninput, aword(3)
      go to 999

 8200 format (/ 'Number of points moved = ',i5)

  200 write (   6, 8200) npointx
      write (iout, 8200) npointx

      go to 999

      endif                           ! Moved one or all points.

c===============================================================================

c.... Input line:  move cluster CLNAME VMOVE [VMULT|1]

      if ((aword(2) .eq. 'cluster') .or.
     &    (aword(2) .eq. 'cl'))     then

 8300 format ('  movex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  There are no clusters.')

      if (nclusts .eq. 0) then
        write (   6, 8300) ainput, ninput
        write (iout, 8300) ainput, ninput
        go to 999
      endif

      do 330 n = 1, nclusts           ! Loop over clusters.
        if (aword(3) .ne. aclust(n)) go to 330

        call clout (n, n, 'min')

        if (aclust(n) .eq. ' ') go to 340

        do 320 npr = 1, npairs        ! Loop over cluster pairs.
          if (apaircl(npr) .ne. aclust(n)) go to 320

          do 310 np = 1, npoints      ! Loop over points.
            if (apoint(np) .ne. apairpt(npr)) go to 310

            amount = 'maximum'
            call pout (np, np, amount)

            tolxx = tol * (abs (x(np)) + abs (dx))
            tolyy = tol * (abs (y(np)) + abs (dy))
            tolzz = tol * (abs (z(np)) + abs (dz))
            x(np)  = x(np) + dx
            y(np)  = y(np) + dy
            z(np)  = z(np) + dz
            if (abs (x(np)) .lt. tolxx) x(np) = 0.0
            if (abs (y(np)) .lt. tolyy) y(np) = 0.0
            if (abs (z(np)) .lt. tolzz) z(np) = 0.0

            amount = 'maximum'
            call pout (np, np, amount)

            go to 320                 ! Find next point in cluster.
  310     continue                    ! End of loop over points.

 8310     format ('  movex ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Named cluster point does not exist:  ',a)

          write (   6, 8310) ainput, ninput, apairpt(npr)
          write (iout, 8310) ainput, ninput, apairpt(npr)
          go to 999

  320   continue                      ! End of loop over cluster pairs.

        go to 999
  330 continue                        ! End of loop over clusters.

 8340 format ('  movex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named cluster does not exist:  ',a)

  340 write (   6, 8340) ainput, ninput, aword(3)
      write (iout, 8340) ainput, ninput, aword(3)
      go to 999

      endif                           ! Moved one or all clusters.

c===============================================================================

c.... Input line:  move vector VNAME VMOVE [VMULT|1]
c.... Input line:  move vector all   VMOVE [VMULT|1]

      if ((aword(2) .eq. 'vector') .or.
     &    (aword(2) .eq. 'vect')   .or.
     &    (aword(2) .eq. 'v'))     then

c.... See if user wants all vectors moved.

      nvectx = 0
      if (nvectors .eq. 0) go to 230

      if (aword(3) .eq. 'all') then   ! Move all vectors.

        do 210 n = 1, nvectors
          if (avector(n) .eq. ' ') go to 210

          nvectx = nvectx + 1

          call vout (n, n)


          tolxx   = tol * (abs (xbnd(n)) + abs (dx))
          tolyy   = tol * (abs (ybnd(n)) + abs (dy))
          tolzz   = tol * (abs (zbnd(n)) + abs (dz))
          xbnd(n) = xbnd(n) + dx
          ybnd(n) = ybnd(n) + dy
          zbnd(n) = zbnd(n) + dz
          if (abs (xbnd(n)) .lt. tolxx) xbnd(n) = 0.0
          if (abs (ybnd(n)) .lt. tolyy) ybnd(n) = 0.0
          if (abs (zbnd(n)) .lt. tolzz) zbnd(n) = 0.0

          call vout (n, n)

  210   continue

        go to 240

      endif                           ! Moved all vectors

c.... See if user wants one vector moved.

      call vectfind (3, n)

      if (n .eq. 0) go to 230

      nvectx = nvectx + 1

      tolxx   = tol * (abs (xbnd(n)) + abs (dx))
      tolyy   = tol * (abs (ybnd(n)) + abs (dy))
      tolzz   = tol * (abs (zbnd(n)) + abs (dz))
      xbnd(n) = xbnd(n) + dx
      ybnd(n) = ybnd(n) + dy
      zbnd(n) = zbnd(n) + dz
      if (abs (xbnd(n)) .lt. tolxx) xbnd(n) = 0.0
      if (abs (ybnd(n)) .lt. tolyy) ybnd(n) = 0.0
      if (abs (zbnd(n)) .lt. tolzz) zbnd(n) = 0.0

      call vout (n, n)

      go to 240

 8220 format ('  movex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named vector does not exist:  ',a)

  230 write (   6, 8220) ainput, ninput, aword(3)
      write (iout, 8220) ainput, ninput, aword(3)
      go to 999

 8240 format (/ 'Number of vectors moved = ',i5)

  240 write (   6, 8240) nvectx
      write (iout, 8240) nvectx

      go to 999

      endif                           ! Moved one or all vectors.

c===============================================================================

c.... Input line:  move plane     PLNAME  VMOVE [VMULT|1]
c.... Input line:  move plane     all     VMOVE [VMULT|1]
c.... Input line:  move sphere    SPHNAME VMOVE [VMULT|1]
c.... Input line:  move sphere    all     VMOVE [VMULT|1]
c.... Input line:  move cylinder  CYLNAME VMOVE [VMULT|1]
c.... Input line:  move cylinder  all     VMOVE [VMULT|1]
c.... Input line:  move cone      CNNAME  VMOVE [VMULT|1]
c.... Input line:  move cone      all     VMOVE [VMULT|1]
c.... Input line:  move hyperb    HPNAME  VMOVE [VMULT|1]
c.... Input line:  move hyperb    all     VMOVE [VMULT|1]
c.... Input line:  move ellipsoid ELNAME  VMOVE [VMULT|1]
c.... Input line:  move ellipsoid all     VMOVE [VMULT|1]
c.... Input line:  move axisym    AXNAME  VMOVE [VMULT|1]
c.... Input line:  move axisym    all     VMOVE [VMULT|1]
c.... Input line:  move quadric   QNAME   VMOVE [VMULT|1]
c.... Input line:  move quadric   all     VMOVE [VMULT|1]

      if ((aword(2) .eq. 'quadric')   .or.
     &    (aword(2) .eq. 'quad')      .or.
     &    (aword(2) .eq. 'q')         .or.
     &    (aword(2) .eq. 'plane')     .or.
     &    (aword(2) .eq. 'pl')        .or.
     &    (aword(2) .eq. 'sphere')    .or.
     &    (aword(2) .eq. 'sph')       .or.
     &    (aword(2) .eq. 'cylinder')  .or.
     &    (aword(2) .eq. 'cyl')       .or.
     &    (aword(2) .eq. 'cone')      .or.
     &    (aword(2) .eq. 'hyperb')    .or.
     &    (aword(2) .eq. 'saddle')    .or.
     &    (aword(2) .eq. 'ellipsoid') .or.
     &    (aword(2) .eq. 'ellipse')   .or.
     &    (aword(2) .eq. 'ell')       .or.
     &    (aword(2) .eq. 'el')        .or.
     &    (aword(2) .eq. 'axisym'))   then

c.... See if user wants all quadric surfaces of the specified type moved.

 8260 format ('  movex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  There are no such quadric surfaces:  ',a)

      nquadx = 0
      if (nquads .eq. 0) go to 280

      if (aword(3) .eq. 'all') then

        nfind = 0

        do 260 n = 1, nquads
          if (aquad(n) .eq. ' ') go to 260

          call qutype (aword(2), n, 0, nerrq)

          if (nerrq .eq. 1) go to 260

          nfind = 1

          nquadx = nquadx + 1

          call quout (n, n, 'min', nerrq)

          call apttris (1, dx, dy, dz, qc(n), qx(n), qy(n), qz(n),
     &                  qxy(n), qyz(n), qzx(n),
     &                  qxx(n), qyy(n), qzz(n), tol, nerr)

          call quout (n, n, 'min', nerrq)


  260   continue

        if (nfind .eq. 0) then
          write (   6, 8260) ainput, ninput, aword(2)
          write (iout, 8260) ainput, ninput, aword(2)
          go to 999
        endif

        go to 290

      endif                           ! Moved all quadric surfaces of type.

c.... See if user wants one quadric surface moved.

      do 270 n = 1, nquads

        if (aword(3) .eq. aquad(n)) then

          call quout (aword(2), n, 1, nerrq)

          if (nerrq .eq. 1) go to 999

          nquadx = nquadx + 1

          call quout (n, n, 'min', nerrq)

          call apttris (1, dx,  dy,  dz, qc(n), qx(n), qy(n), qz(n),
     &                  qxy(n), qyz(n), qzx(n),
     &                  qxx(n), qyy(n), qzz(n), tol, nerr)

          call quout (n, n, 'min', nerrq)

          go to 290
        endif
  270 continue

 8270 format ('  movex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named quadric surface does not exist:  ',a)

  280 write (   6, 8270) ainput, ninput, aword(3)
      write (iout, 8270) ainput, ninput, aword(3)
      go to 999

 8290 format (/ 'Number of ',a24,' moved = ',i5)

  290 write (   6, 8290) aword(2), nquadx
      write (iout, 8290) aword(2), nquadx

      go to 999

      endif                           ! Moved one or all planes.

c===============================================================================

 8410 format ('  movex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named object or option not recognized:  ' / a / a)

      write (   6, 8410) ainput, ninput, aword(2), aword(3)
      write (iout, 8410) ainput, ninput, aword(2), aword(3)

  999 return

c.... End of subroutine movex.        (+1 line)
      end
