      subroutine pgout (n1, n2, amount)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  pgout    Updated     2004 August 18 17:00.
c               pgout    Originated  1994 March 4 16:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display data for regular polygons n1 to n2.
c
c     Logic:    Entry points:  pgout.
c pgout         Called by:  copy, delete, point, polygon, renamea, renamex,
c               search.
c
c pgout         Calls:  aptcsys, aptcsyv, aptpoly, aptvpln, plotpt, pout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*8 amount              ! Amount of output to write.

      common /ccpgout/ uver(nverpgm)  ! U coord of vertices of regular polygon.
      common /ccpgout/ vver(nverpgm)  ! V coord of vertices of regular polygon.
      common /ccpgout/ wver(nverpgm)  ! W coord of vertices of regular polygon.

c.... Initialize.

      nn1 = n1
      nn2 = n2
      if (nn1 .lt. 1) nn1 = 1
      if (nn2 .gt. npolygs) nn2 = npolygs

      iunit = 0
      if (angunit .eq. 'radians') iunit = 1

      do 150 n = nn1, nn2

        if (apolyg(n) .eq. ' ') go to 150

        nverpgs = nverpg(n)

 8125   format ('polygon ',a,' fit ',i6 /
     &    2x,a,2x,a,2x,a)

        write (   6, 8125) apolyg(n), nverpg(n), acenpg(n),
     &    averpg(n), acoppg(n)
        write (iout, 8125) apolyg(n), nverpg(n), acenpg(n),
     &    averpg(n), acoppg(n)

        if ((amount .eq. 'max')      .or.
     &      (amount .eq. 'maximum')) then

 8135     format ('coord  ',a,5x,a,4x,a,4x,a)

          write (   6, 8135) asys, acoordu, acoordv, acoordw
          write (iout, 8135) asys, acoordu, acoordv, acoordw

 8141     format ('  pgout ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Named point does not exist:  ',a)

          do 141 np = 1, npoints
            if (apoint(np) .eq. acenpg(n)) then
              xcenp = x(np)
              ycenp = y(np)
              zcenp = z(np)
              call pout (np, np, 'min')
              go to 142
            endif
  141     continue
          write (   6, 8141) ainput, ninput, acenpg(n)
          write (iout, 8141) ainput, ninput, acenpg(n)
          go to 150

  142     do 143 np = 1, npoints
            if (apoint(np) .eq. averpg(n)) then
              xverp = x(np)
              yverp = y(np)
              zverp = z(np)
              call pout (np, np, 'min')
              go to 144
            endif
  143     continue
          write (   6, 8141) ainput, ninput, averpg(n)
          write (iout, 8141) ainput, ninput, averpg(n)
          go to 150

  144     do 145 np = 1, npoints
            if (apoint(np) .eq. acoppg(n)) then
              xcopp = x(np)
              ycopp = y(np)
              zcopp = z(np)
              call pout (np, np, 'min')
              go to 146
            endif
  145     continue
          write (   6, 8141) ainput, ninput, acoppg(n)
          write (iout, 8141) ainput, ninput, acoppg(n)
          go to 150

c....     Find the data for the polygon.

  146     call aptpoly (xcenp, ycenp, zcenp,
     &                  xverp, yverp, zverp,
     &                  xcopp, ycopp, zcopp, nverpg(n), tol,
     &                  xver, yver, zver, sedge, rin, rout, area, nerra)

 8142     format ('  pgout ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Polygon is bad.')

          if ((nerra .ne. 0) .or. (area .eq. 0.0)) then
            write (   6, 8142) ainput, ninput
            write (iout, 8142) ainput, ninput
            go to 150
          endif

          call aptvpln (xcenp, ycenp, zcenp,
     &                  xverp, yverp, zverp,
     &                  xcopp, ycopp, zcopp, 1, tol,
     &                  unorm, vnorm, wnorm, vlength, nerra)

c....     Convert units of vertices.

          do 147 nn = 1, nverpg(n)
            uver(nn) = xver(nn)
            vver(nn) = yver(nn)
            wver(nn) = zver(nn)
            call plotpt (xver(nn), yver(nn), zver(nn), nerrg)
  147     continue

          if (asys .eq. 'cyl') then   ! Convert to cylindrical coordinates.
            call aptcsys (0, 1, iunit, uver, vver, wver,
     &                    nverpg(n), tol, nerra)
            call aptcsyv (0, 1, iunit, xcenp, ycenp, zcenp,
     &                    unorm, vnorm, wnorm, 1, tol, nerra)
          elseif (asys .eq. 'sph') then   ! Convert to spherical coordinates.
            call aptcsys (0, 2, iunit, uver, vver, wver,
     &                    nverpg(n), tol, nerra)
            call aptcsyv (0, 2, iunit, xcenp, ycenp, zcenp,
     &                    unorm, vnorm, wnorm, 1, tol, nerra)
          endif

 8148     format ('  Vertex',i5,5x,1p3e20.12)

          do 148 nn = 1, nverpg(n)
            write (   6, 8148) nn, uver(nn), vver(nn), wver(nn)
            write (iout, 8148) nn, uver(nn), vver(nn), wver(nn)
  148     continue

          circum = sedge * nverpg(n)
          angver = 180.0 - 360.0 / nverpg(n)
          if (angunit .eq. 'radians') angver = angver * pi / 180.0

 8150     format ('  Vertex angle    ',1pe20.12,2x,a /
     &        '  Edge length     ',1pe20.12 /
     &        '  Circumference   ',1pe20.12 /
     &        '  Area            ',1pe20.12 /
     &        '  Inscribed radius',1pe20.12 /
     &        '  Circumscr radius',1pe20.12 /
     &        '  Normal vector   ',1p3e20.12 )

          write (   6, 8150) angver, angunit, sedge, circum, area,
     &      rin, rout, unorm, vnorm, wnorm
          write (iout, 8150) angver, angunit, sedge, circum, area,
     &      rin, rout, unorm, vnorm, wnorm

        endif

  150 continue

      return

c.... End of subroutine pgout.        (+1 line)
      end
