      subroutine phout (n1, n2, amount)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  phout    Updated     1996 March 12 16:40.
c               phout    Originated  1994 March 15 11:50.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display data for regular polyhedrons n1 to n2.
c
c     Logic:    Entry points:  phout.
c phout         Called by:  copy, delete, point, polyhed, renamea, renamex,
c               search.
c
c phout         Calls:  aptcsys, aptcsyv, aptcube, aptdode, apticos, aptmopv,
c               aptocta, aptrott, apttetr, apttran, aptvdis, plotpt, pout.
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*8 amount              ! Amount of output to write.

      common /ccphout/ uver(20)       ! U coord of vertices of reg polyhedron.
      common /ccphout/ vver(20)       ! V coord of vertices of reg polyhedron.
      common /ccphout/ wver(20)       ! W coord of vertices of reg polyhedron.
      common /ccphout/ rotm(3,3)      ! Rotation tensor.

c.... Initialize.

      nn1 = n1
      nn2 = n2
      if (nn1 .lt. 1) nn1 = 1
      if (nn2 .gt. npolyhs) nn2 = npolyhs

      iunit = 0
      if (angunit .eq. 'radians') iunit = 1

      do 270 n = nn1, nn2             ! Loop over regular polyhedrons.

        if (apolyh(n) .eq. ' ') go to 270

 8110   format ('polyhedron ',a,' fit ',i3 /
     &    2x,a,2x,a,2x,a,' ! ',a16)

        write (   6, 8110) apolyh(n), nverph(n),
     &    acenph(n), averph(n), acopph(n), atypeh(n)
        write (iout, 8110) apolyh(n), nverph(n),
     &    acenph(n), averph(n), acopph(n), atypeh(n)

        if ((amount .eq. 'max')      .or.
     &      (amount .eq. 'maximum')) then

 8120     format ('coord  ',a,5x,a,4x,a,4x,a)

          write (   6, 8120) asys, acoordu, acoordv, acoordw
          write (iout, 8120) asys, acoordu, acoordv, acoordw

 8130     format ('  phout ERROR.  Reading ',a40,' line ',i5,'.' /
     &      '  Named point does not exist:  ',a)

          do 130 np = 1, npoints
            if (apoint(np) .eq. acenph(n)) then
              xcenp = x(np)
              ycenp = y(np)
              zcenp = z(np)
              xmove = -xcenp
              ymove = -ycenp
              zmove = -zcenp
              call pout (np, np, 'min')
              go to 140
            endif
  130     continue
          write (   6, 8130) ainput, ninput, acenph(n)
          write (iout, 8130) ainput, ninput, acenph(n)
          go to 270

  140     do 150 np = 1, npoints
            if (apoint(np) .eq. averph(n)) then
              xverp = x(np)
              yverp = y(np)
              zverp = z(np)
              call pout (np, np, 'min')
              go to 160
            endif
  150     continue
          write (   6, 8130) ainput, ninput, averph(n)
          write (iout, 8130) ainput, ninput, averph(n)
          go to 270

  160     do 170 np = 1, npoints
            if (apoint(np) .eq. acopph(n)) then
              xcopp = x(np)
              ycopp = y(np)
              zcopp = z(np)
              call pout (np, np, 'min')
              go to 180
            endif
  170     continue
          write (   6, 8130) ainput, ninput, acopph(n)
          write (iout, 8130) ainput, ninput, acopph(n)
          go to 270

c....     Find the data for the polyhedron.

  180     if     (nverph(n) .eq.  4) then  ! Tetrahedron.

            call apttetr (xverh, yverh, zverh, nver1, nver2)

            nverphs = 4
            nedgphs = 6
            edge    = 2.0 * sqrt (6.0) / 3.0
            edgetot = 6.0 * edge
            area    = 2.0 / sqrt (3.0)
            areatot = 4.0 * area
            volume  = 8.0 * sqrt (3.0) / 27.0
            radinsc = 1.0 / 3.0
            angdih  = acos (1.0 / 3.0)
            angcent = acos (-1.0 / 3.0)

          elseif (nverph(n) .eq.  6) then  ! Octahedron.

            call aptocta (xverh, yverh, zverh, nver1, nver2)

            nverphs = 6
            nedgphs = 12
            edge    = sqrt (2.0)
            edgetot = 12.0 * edge
            area    = 0.5 * sqrt (3.0)
            areatot = 8.0 * area
            volume  = 4.0 / 3.0
            radinsc = 1.0 / sqrt (3.0)
            angdih  = acos (-1.0 / 3.0)
            angcent = acos (0.0)

          elseif (nverph(n) .eq.  8) then  ! Cube.

            call aptcube (xverh, yverh, zverh, nver1, nver2)

            nverphs = 8
            nedgphs = 12
            edge    = 2.0 / sqrt (3.0)
            edgetot = 12.0 * edge
            area    = 4.0 / 3.0
            areatot = 8.0
            volume  = 8.0 * sqrt (3.0) / 9.0
            radinsc = 1.0 / sqrt (3.0)
            angdih  = acos (0.0)
            angcent = acos (1.0 / 3.0)

          elseif (nverph(n) .eq. 12) then  ! Icosahedron.

            call apticos (xverh, yverh, zverh, nver1, nver2)

            nverphs = 12
            nedgphs = 30
            edge    = 4.0 / sqrt (10.0 + 2.0 * sqrt (5.0))
            edgetot = 30.0 * edge
            area    = 0.25 * sqrt (3.0) * edge**2
            areatot = 20 * area
            volume  = (5.0 / 12.0) * (3.0 + sqrt (5.0)) * edge**3
            radinsc = sqrt ((5.0 + 2.0 * sqrt (5.0)) / 15.0)
            angdih  = acos (-sqrt (5.0) / 3.0)
            angcent = acos (1.0 / sqrt (5.0))

          elseif (nverph(n) .eq. 20) then  ! Dodecahedron.

            call aptdode (xverh, yverh, zverh, nver1, nver2)

            nverphs = 20
            nedgphs = 30
            edge    = 4.0 / (sqrt (3.0) + sqrt (15.0))
            edgetot = 30.0 * edge
            area    = 0.25 * sqrt (25.0 + 10.0 * sqrt (5.0)) * edge**2
            areatot = 12.0 * area
            volume  = 0.25 * (15.0 + 7.0 * sqrt (5.0)) * edge**3
            radinsc = sqrt ((5.0 + 2.0 * sqrt (5.0)) / 15.0)
            angdih  = acos (-1.0 / sqrt (5.0))
            angcent = acos (sqrt (5.0) / 3.0)

          endif

c....     Find the vectors from pcen to pver and pcop.

          call aptvdis (xcenp, ycenp, zcenp, xverp, yverp, zverp,
     &                  1, tol, dx1, dy1, dz1, distcp, nerra)
cbugc___DEBUG begins.
cbug 9901 format ('phout DEBUG 1 dx,dy,dz1,distcp=',1p4e12.4)
cbug      write (   6, 9901) dx1, dy1, dz1, distcp
cbug      write (iout, 9901) dx1, dy1, dz1, distcp
cbugc___DEBUG ends.

          call aptvdis (xcenp, ycenp, zcenp, xcopp, ycopp, zcopp,
     &                  1, tol, dx2, dy2, dz2, distcc, nerra)
cbugc___DEBUG begins.
cbug 9902 format ('phout DEBUG 2 dx,dy,dz2,distcc=',1p4e12.4)
cbug      write (   6, 9902) dx2, dy2, dz2, distcc
cbug      write (iout, 9902) dx2, dy2, dz2, distcc
cbugc___DEBUG ends.

          call aptvdis (0., 0., 0., xverh(1), yverh(1), zverh(1),
     &                  1, tol, du1, dv1, dw1, distcv1, nerra)
cbugc___DEBUG begins.
cbug 9903 format ('phout DEBUG 3 du,dv,dw1,distcv1=',1p4e12.4)
cbug      write (   6, 9903) du1, dv1, dw1, distcv1
cbug      write (iout, 9903) du1, dv1, dw1, distcv1
cbugc___DEBUG ends.

          call aptvdis (0., 0., 0., xverh(2), yverh(2), zverh(2),
     &                  1, tol, du2, dv2, dw2, distcv2, nerra)
cbugc___DEBUG begins.
cbug 9904 format ('phout DEBUG 4 du,dv,dw2,distcv2=',1p4e12.4)
cbug      write (   6, 9904) du2, dv2, dw2, distcv2
cbug      write (iout, 9904) du2, dv2, dw2, distcv2
cbugc___DEBUG ends.

c....     Find the scale factor to make the polyhedron the right size.

          factor = distcp / distcv1
cbugc___DEBUG begins.
cbug 9905 format ('phout DEBUG 5 scale factor=',1pe12.4)
cbug      write (   6, 9905) factor
cbug      write (iout, 9905) factor
cbugc___DEBUG ends.

c....     Scale the vertex coordinates, store in temporary array.

cbugc___DEBUG begins.
cbug 9931 format ('phout DEBUG 6A before scaling')
cbug          write (   6, 9931)
cbug          write (iout, 9931)
cbug          do 849 nn = 1, nverph(n)
cbug            write (   6, 8250) nn, uver(nn), vver(nn), wver(nn)
cbug            write (iout, 8250) nn, uver(nn), vver(nn), wver(nn)
cbug  849     continue
cbugc___DEBUG ends.
          do 210 nn = 1, nverph(n)
            xverh(nn) = xverh(nn) * factor
            yverh(nn) = yverh(nn) * factor
            zverh(nn) = zverh(nn) * factor
  210     continue
cbugc___DEBUG begins.
cbug 9906 format ('phout DEBUG 6B after scaling')
cbug          write (   6, 9906)
cbug          write (iout, 9906)
cbug          do 850 nn = 1, nverph(n)
cbug            write (   6, 8250) nn, xverh(nn), yverh(nn), zverh(nn)
cbug            write (iout, 8250) nn, xverh(nn), yverh(nn), zverh(nn)
cbug  850     continue
cbugc___DEBUG ends.

c....     Move the polyhedron to the specified center point, pcen.

          call apttran (xmove, ymove, zmove, xverh, yverh, zverh,
     &                  nverph(n), tol, nerra)
cbugc___DEBUG begins.
cbug 9907 format ('phout DEBUG 7 after centering')
cbug          write (   6, 9907)
cbug          write (iout, 9907)
cbug          do 857 nn = 1, nverph(n)
cbug            write (   6, 8250) nn, xverh(nn), yverh(nn), zverh(nn)
cbug            write (iout, 8250) nn, xverh(nn), yverh(nn), zverh(nn)
cbug  857     continue
cbugc___DEBUG ends.

c....     Find the rotation tensor to put vertex 1 at pver, and vertex 2
c....       in the plane of pcen, pver and pcop.

          call aptrott (du1, dv1, dw1, du2, dv2, dw2, dx1, dy1, dz1,
     &                  dx2, dy2, dz2, tol, rotm, nerra)
cbugc___DEBUG begins.
cbug 9920 format ('  Rotation tensor.')
cbug 9921 format (10x,1p3e20.12)
cbug      write (   6, 9920)
cbug      write (iout, 9920)
cbug      write (   6, 9921) ((rotm(i,j), j = 1, 3), i = 1, 3)
cbug      write (iout, 9921) ((rotm(i,j), j = 1, 3), i = 1, 3)
cbugc___DEBUG ends.

c....     Rotate the polyhedron to put vertex 1 at pver, and vertex 2
c....       in the plane of pcen, pver and pcop.

          call aptmopv (rotm, 0, xcenp, ycenp, zcenp,
     &                  xverh, yverh, zverh,
     &                  nverph(n), tol, nerra)
cbugc___DEBUG begins.
cbug 9908 format ('phout DEBUG 8 after rotating')
cbug          write (   6, 9908)
cbug          write (iout, 9908)
cbug          do 858 nn = 1, nverph(n)
cbug            write (   6, 8250) nn, xverh(nn), yverh(nn), zverh(nn)
cbug            write (iout, 8250) nn, xverh(nn), yverh(nn), zverh(nn)
cbug  858     continue
cbugc___DEBUG ends.

          do 240 nn = 1, nverph(n)
            uver(nn) = xverh(nn)
            vver(nn) = yverh(nn)
            wver(nn) = zverh(nn)
            call plotpt (xverh(nn), yverh(nn), zverh(nn), nerrg)
  240     continue

c....     Convert the units of vertices.

          if (asys .eq. 'cyl') then   ! Convert to cylindrical coordinates.
            call aptcsys (0, 1, iunit, uver, vver, wver,
     &                    nverph(n), tol, nerra)
            call aptcsyv (0, 1, iunit, xcenp, ycenp, zcenp,
     &                    unorm, vnorm, wnorm, 1, tol, nerra)
          elseif (asys .eq. 'sph') then   ! Convert to spherical coordinates.
            call aptcsys (0, 2, iunit, uver, vver, wver,
     &                    nverph(n), tol, nerra)
            call aptcsyv (0, 2, iunit, xcenp, ycenp, zcenp,
     &                    unorm, vnorm, wnorm, 1, tol, nerra)
          endif

 8250     format ('  Vertex',i3,7x,1p3e20.12)
 8260     format ('  Edges (vertex pairs):' /
     &      (5x,i2,' (',i2,' to ',i2,')',5x,i2,' (',i2,' to ',i2,')',
     &       5x,i2,' (',i2,' to ',i2,')',5x,i2,' (',i2,' to ',i2,')'))

          do 250 nn = 1, nverph(n)
            write (   6, 8250) nn, uver(nn), vver(nn), wver(nn)
            write (iout, 8250) nn, uver(nn), vver(nn), wver(nn)
  250     continue

          write (   6, 8260) (nn, nver1(nn), nver2(nn),
     &      nn = 1, nedgph(n))
          write (iout, 8260) (nn, nver1(nn), nver2(nn),
     &      nn = 1, nedgph(n))

          edge    = edge    * factor
          edgetot = edgetot * factor
          area    = area    * factor**2
          areatot = areatot * factor**2
          volume  = volume  * factor**3
          radinsc = radinsc * factor
          radcirc = factor
          if (angunit .eq. 'degrees') angdih  = angdih  * 180.0 / pi
          if (angunit .eq. 'degrees') angcent = angcent * 180.0 / pi

          angver = 180.0 - 360.0 / nverph(n)
          if (angunit .eq. 'radians') angver = angver * pi / 180.0

 8270     format ('  Edge length     ',1pe20.12 /
     &            '  Edge total      ',1pe20.12 /
     &            '  Area of face    ',1pe20.12 /
     &            '  Area total      ',1pe20.12 /
     &            '  Volume          ',1pe20.12 /
     &            '  Inscribed radius',1pe20.12 /
     &            '  Circumscr radius',1pe20.12 /
     &            '  Dihedral angle  ',1pe20.12,2x,a /
     &            '  Central  angle  ',1pe20.12,2x,a )

          write (   6, 8270) edge, edgetot, area, areatot, volume,
     &      radinsc, radcirc, angdih, angunit, angcent, angunit
          write (iout, 8270) edge, edgetot, area, areatot, volume,
     &      radinsc, radcirc, angdih, angunit, angcent, angunit

        endif

  270 continue                        ! End of loop over regular polyhedrons.

      return

c.... End of subroutine phout.        (+1 line)
      end
