      subroutine ptoper

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  ptoper   Updated     2005 April 5 17:45.
c               ptoper   Originated  1994 August 2 14:30.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To create a family of new points by repeatedly rotating or
c                 scaling a base point.
c
c     Logic:    Entry points:  ptoper.
c ptoper        Called by:  point.
c
c ptoper        Calls:  aptmopv, incrsubs, namechk, opout, pout, ptfind,
c               ptstore, varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*24 anamex
      character*8  aopx               ! Operation type, "rotate" or "scale".

c.... Local variables.

      character*8 amount              ! Amount of output data.

c=======================================================================********

c.... Input line:  point PNAME02 rotate NUMPT INC PNAME01 OPNAME PINV
c.... Input line:  point PNAME02 scale  NUMPT INC PNAME01 OPNAME PINV

      nerr   = 0

 8605 format ('  ptoper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 8 words in input line:  ',i3)

      if (nword .ne. 8) then
        write (   6, 8605) ainput, ninput, nword
        write (iout, 8605) ainput, ninput, nword
        nerr = 1
      endif

 8620 format ('  ptoper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word NUMPT is not a positive integer:  ',a)
 8625 format ('  ptoper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word NUMPT is not an integer or integer variable:  ',a)

      call varfind (4, 4)

      if (mtype(4) .eq. 2) then
        numpt = iword(4)
        if (numpt .lt. 1) then
          write (   6, 8620) ainput, ninput, aword(4)
          write (iout, 8620) ainput, ninput, aword(4)
          nerr = 1
        endif
      else
        write (   6, 8625) ainput, ninput, aword(4)
        write (iout, 8625) ainput, ninput, aword(4)
        nerr = 1
      endif

 8610 format ('  ptoper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word INC is zero.')
 8615 format ('  ptoper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word INC is not an integer or integer variable:  ',a)

      call varfind (5, 5)

      if (mtype(5) .eq. 2) then
        inc = iword(5)
        if (inc .eq. 0) then
          write (   6, 8610) ainput, ninput
          write (iout, 8610) ainput, ninput
          nerr = 1
        endif
      else
        write (   6, 8615) ainput, ninput, aword(5)
        write (iout, 8615) ainput, ninput, aword(5)
        nerr = 1
      endif

c.... Find the base point.

      call ptfind (6, nbase)

      if (nbase .eq. 0) nerr = 1

c.... Find the operator.

 8650 format ('  ptoper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named operator does not exist:  ',a)

  640 do 650 n = 1, nopers
        if (aword(7) .eq. aoper(n)) then
          nop = n
          call opout (nop, nop, 'min')
          go to 660
        endif
  650 continue
      write (   6, 8650) ainput, ninput, aword(7)
      write (iout, 8650) ainput, ninput, aword(7)
      nerr = 1

c.... See if the operator type agrees with scale or rotate.

 8660 format ('  ptoper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Operator type is not ',a,':  ',a)

  660 if ((aword(3) .eq. 'rotate') .or.
     &    (aword(3) .eq. 'rot'))   then

        aopx = 'rotate'
        if (aoptype(nop) .ne. aopx) then
          write (   6, 8660) ainput, ninput, aopx, aoptype(nop)
          write (iout, 8660) ainput, ninput, aopx, aoptype(nop)
          nerr = 1
        endif

      elseif ((aword(3) .eq. 'scale') .or.
     &        (aword(3) .eq. 'sc'))   then

        aopx = 'scale'
        if (aoptype(nop) .ne. aopx) then
          write (   6, 8660) ainput, ninput, aopx, aoptype(nop)
          write (iout, 8660) ainput, ninput, aopx, aoptype(nop)
          nerr = 1
        endif

      endif

c.... Find the invariant point.

      call ptfind (8, ninv)

      if (ninv .eq. 0) nerr = 1

c.... See if the point names can be generated uniquely.

 8680 format ('  ptoper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not generate',i5,' point names from:  ',a)

 8905 format ('  ptoper ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of point too dangerous to use:  ',a)

  680 anamex  = aword(2)
      lanamex = lword(2)
      namex   = inc * (numpt - 1)

      call incrsubs (namex, anamex, 1, lanamex, nerra)

      if (nerra .ne. 0) then
        write (   6, 8680) ainput, ninput, numpt, aword(2)
        write (iout, 8680) ainput, ninput, numpt, aword(2)
        nerr = 1
      endif

      call namechk (anamex, lanamex, nerrchk)

      if (nerrchk .ne. 0) then
        nerr = 1
        write (   6, 8905) ainput, ninput, anamex
        write (iout, 8905) ainput, ninput, anamex
      endif

      if (nerr .ne. 0) go to 999

c.... Initialize the base point data.

      ax = x(nbase)
      ay = y(nbase)
      az = z(nbase)

c.... Create the rotated or scaled points.

      anamex  = aword(2)              ! Initial point name.
      lanamex = lword(2)              ! Initial point name.

      do 690 npt = 1, numpt            ! Loop over the new points.

c....   Find the index in the point list for the new point.

        aword(2) = anamex
        lword(2) = lanamex

        call ptstore (nrep, n)

        if (n .eq. 0) go to 999

c....   Rotate or scale the base point.

        call aptmopv (toper(1,1,nop), 0, x(ninv), y(ninv), z(ninv),
     &                ax, ay, az, 1, tol, nerr)

        apoint(n)  = anamex
        lapoint(n) = lanamex

        apoints    = anamex
        lapoints   = lanamex

        x(n) = ax
        y(n) = ay
        z(n) = az

        amount = 'maximum'
        call pout (n, n, amount)

c....   Find the next point name, by incrementing by inc.

        call incrsubs (inc, anamex, 1, lanamex, nerra)

        call namechk (anamex, lanamex, nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, anamex
          write (iout, 8905) ainput, ninput, anamex
          go to 999
        endif

  690 continue                        ! End of loop over the new points.

  999 return

c.... End of subroutine ptoper.       (+1 line)
      end
