      subroutine pttet (kopt, np, nt, nside, wa, wb, wc, wd, nerra)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  pttet    Updated     2004 March 8 14:10.
c               pttet    Originated  1994 March 22 13:20.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find whether or not a point is in a tetrahedron, and the
c               distances from the point to each of the four faces of the
c               tetrahedron.
c
c     Logic:    Entry points:  pttet.
c pttet         Called by:  cluster, distance, where.
c
c pttet         Calls:  aptetrn, apttrip, aptvdis, plotpt.
c
c     Biblio:   ?
c
c     Glossary:
c
c     kopt      Input    Face distance display option, 0 for none, 1 for all.
c
c     nerra     Output   1 if an error occurred.  Otherwise, 0.
c
c     np        Input    Index of point.
c
c     nside     Output   1 if point is in tetrahedron.
c
c     nt        Input    Index of tetrahedron.
c
c     wa        Output   Fractional distance of point from opposite face to
c                          vertex "a".
c
c     wb        Output   Fractional distance of point from opposite face to
c                          vertex "b".
c
c     wc        Output   Fractional distance of point from opposite face to
c                          vertex "c".
c
c     wd        Output   Fractional distance of point from opposite face to
c                          vertex "d".
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c-----------------------------------------------------------------------********

c.... Initialize.

      nerra = 0

      proxptx = -1.e99
      proxpty = -1.e99
      proxptz = -1.e99

      proxvx = -1.e99
      proxvy = -1.e99
      proxvz = -1.e99

c-----------------------------------------------------------------------********

c.... Find the vertex points of the tetrahedron.

      do 120 npt = 1, npoints
        if (apoint(npt) .eq. atetpt(1,nt)) then
          xa = x(npt)
          ya = y(npt)
          za = z(npt)
          go to 130
        endif
  120 continue

 8120 format ('  pttet ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named vertex point does not exist:  ',a)

      write (   6, 8120) ainput, ninput, atetpt(1,nt)
      write (iout, 8120) ainput, ninput, atetpt(1,nt)
      nerra = 1
      go to 999

  130 do 140 npt = 1, npoints
        if (apoint(npt) .eq. atetpt(2,nt)) then
          xb = x(npt)
          yb = y(npt)
          zb = z(npt)
          go to 150
        endif
  140 continue
      write (   6, 8120) ainput, ninput, atetpt(2,nt)
      write (iout, 8120) ainput, ninput, atetpt(2,nt)
      nerra = 1
      go to 999

  150 do 160 npt = 1, npoints
        if (apoint(npt) .eq. atetpt(3,nt)) then
          xc = x(npt)
          yc = y(npt)
          zc = z(npt)
          go to 170
        endif
  160 continue
      write (   6, 8120) ainput, ninput, atetpt(3,nt)
      write (iout, 8120) ainput, ninput, atetpt(3,nt)
      nerra = 1
      go to 999

  170 do 180 npt = 1, npoints
        if (apoint(npt) .eq. atetpt(4,nt)) then
          xd = x(npt)
          yd = y(npt)
          zd = z(npt)
          go to 190
        endif
  180 continue
      write (   6, 8120) ainput, ninput, atetpt(4,nt)
      write (iout, 8120) ainput, ninput, atetpt(4,nt)
      nerra = 1
      go to 999

c-----------------------------------------------------------------------********

c.... See if the point is in the tetrahedron.

  190 call aptetrn (x(np), y(np), z(np), xa, ya, za, xb, yb, zb,
     &              xc, yc, zc, xd, yd, zd, 1, tol, 1,
     &              pabc, pbcd, pcda, pdab, pmin, wa, wb, wc, wd,
     &              nside, nerra)

 8190 format ('  Distance to face',1pe20.12 /
     &        '    with vertex points',2x,3(2x,a))

      if (kopt .eq. 1) then

      write (   6, 8190) pabc, atetpt(1,nt), atetpt(2,nt), atetpt(3,nt)
      write (iout, 8190) pabc, atetpt(1,nt), atetpt(2,nt), atetpt(3,nt)
      write (   6, 8190) pbcd, atetpt(2,nt), atetpt(3,nt), atetpt(4,nt)
      write (iout, 8190) pbcd, atetpt(2,nt), atetpt(3,nt), atetpt(4,nt)
      write (   6, 8190) pcda, atetpt(3,nt), atetpt(4,nt), atetpt(1,nt)
      write (iout, 8190) pcda, atetpt(3,nt), atetpt(4,nt), atetpt(1,nt)
      write (   6, 8190) pdab, atetpt(4,nt), atetpt(1,nt), atetpt(2,nt)
      write (iout, 8190) pdab, atetpt(4,nt), atetpt(1,nt), atetpt(2,nt)
      pdist = pmin

      endif

c-----------------------------------------------------------------------********

c.... Find the proximal point on the tetrahedron.

      if     (pabc .eq. pmin) then

        call apttrip (x(np), y(np), z(np),
     &                xa, ya, za, xb, yb, zb, xc, yc, zc, -1, tol,
     &                dpmin, wa, wb, wc, cx, cy, cz,
     &                nlima, nlimb, nlimc, itrun, nsidex, nerra)

      elseif (pbcd .eq. pmin) then

        call apttrip (x(np), y(np), z(np),
     &                xb, yb, zb, xc, yc, zc, xd, yd, zd, -1, tol,
     &                dpmin, wb, wc, wd, cx, cy, cz,
     &                nlimb, nlimc, nlimd, itrun, nsidex, nerra)

      elseif (pcda .eq. pmin) then

        call apttrip (x(np), y(np), z(np),
     &                xc, yc, zc, xd, yd, zd, xa, ya, za, -1, tol,
     &                dpmin, wc, wd, wa, cx, cy, cz,
     &                nlimc, nlimd, nlima, itrun, nsidex, nerra)

      elseif (pdab .eq. pmin) then

        call apttrip (x(np), y(np), z(np),
     &                xd, yd, zd, xa, ya, za, xb, yb, zb, -1, tol,
     &                dpmin, wd, wa, wb, cx, cy, cz,
     &                nlima, nlimb, nlimc, itrun, nsidex, nerra)

      endif

      call aptvdis (x(np), y(np), z(np), cx, cy, cz, 1, tol,
     &              dx, dy, dz, vlength, nerra)

      proxptx = cx
      proxpty = cy
      proxptz = cz

      proxvx = dx
      proxvy = dy
      proxvz = dz

 8200 format ('  Prox  x, y, z   ',1p3e20.12 /
     &        '  Prox dx,dy,dz   ',1p3e20.12 )

      write (   6, 8200) cx, cy, cz, dx, dy, dz
      write (iout, 8200) cx, cy, cz, dx, dy, dz
      call plotpt (cx, cy, cz, nerrg)

c-----------------------------------------------------------------------********

  999 return

c.... End of subroutine pttet.        (+1 line)
      end
