      subroutine ratio

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  ratio    Updated     1994 August 25 13:40.
c               ratio    Originated  1994 August 8 12:10.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find the ratio for a geometric series, given the first term,
c               the sum, and the number of terms.
c
c     Logic:    Entry points:  ratio.
c ratio         Called by:  helpall, main.
c
c ratio         Calls:  aptgrat, varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Input line:  ratio [help]

 8110 format ('  Command "ratio":  acceptable input lines include:' /
     &  '  "ratio [help]"  Display this message.' //
     &  '  "ratio TERM1 SUM NUMT"' /
     &  '     Find the ratio for a geometric series beginning with',
     &  ' TERM1,' /
     &  '     with sum SUM, and NUMT terms.' //
     &  '  Synonyms:  [help, h], [ratio, rat].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c=======================================================================********

c.... Input line:  ratio TERM1 SUM NUMT

c.... Test for wrong number of words on "ratio" line.

 8120 format ('  ratio ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 4 words on "ratio" line:  ', i3)
      if (nword .ne. 4) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        go to 999
      endif

c.... Find the first term and sum of the series.

 8130 format ('  ratio ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer or floating point value or variable:  ',a)

      call varfind (2, 3)

      do 140 nw = 2, 3
        if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
          write (   6, 8130) ainput, ninput, aword(nw)
          write (iout, 8130) ainput, ninput, aword(nw)
          go to 999
        endif
  140 continue

 8140 format ('  ratio ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  First term is not positive:  ',1pe20.12)

      if (fword(2) .le. 0.0) then
        write (   6, 8140) ainput, ninput, fword(2)
        write (iout, 8140) ainput, ninput, fword(2)
        go to 999
      endif

 8150 format ('  ratio ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Sum is not greater than first term:  ',1pe20.12)

      if (fword(3) .le. fword(2)) then
        write (   6, 8150) ainput, ninput, fword(3)
        write (iout, 8150) ainput, ninput, fword(3)
        go to 999
      endif

c.... Find the number of terms in the series.

 8160 format ('  ratio ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of terms is not an integer or integer variable:  ',a)

      call varfind (4, 4)

      if (mtype(4) .ne. 2) then
        write (   6, 8160) ainput, ninput, aword(4)
        write (iout, 8160) ainput, ninput, aword(4)
        go to 999
      endif

 8170 format ('  ratio ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of terms is less than two:  ',i5)

      if (iword(4) .lt. 2) then
        write (   6, 8170) ainput, ninput, iword(4)
        write (iout, 8170) ainput, ninput, iword(4)
        go to 999
      endif

c.... Find the ratio of consecutive terms for the geometric series.

      call aptgrat (fword(2), iword(4), fword(3), tol, ratt, nerr)

 8180 format ('  ratio ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Bad arguments.')
 8190 format ('  Ratio of terms  ',1pe20.12)

      if (nerr .ne. 0) then
        write (   6, 8180)
        write (iout, 8180)
      else
        write (   6, 8190) ratt
        write (iout, 8190) ratt
      endif

c=======================================================================********

  999 return

c.... End of subroutine ratio.        (+1 line)
      end
