      subroutine redo

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  redo     Updated     2006 January 2 15:40.
c               redo     Originated  1994 August 17 17:20.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To repeat a specified saved input line, with any number of
c               arguments replaced.
c
c     Logic:    Entry points:  redo.
c redo          Called by:  helpall, main.
c
c redo          Calls:  aptchap, aptchmv.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Variables needed for temporary storage of "redo" input line.

      common /aredo/ nwork            ! Number of input line fields.
      common /aredo/ awork(nwordm)    ! User message fields.  ASCII (1).
      character*80   awork            ! User message fields.  ASCII (1).
      common /aredo/ iwork(nwordm)    ! User message fields.  Integer (2).
      common /aredo/ fwork(nwordm)    ! User message fields.  Real (3).
      common /aredo/ lwork(nwordm)    ! Input line field lengths.
      common /aredo/ mtypk(nwordm)    ! Input line field types.
                                      !   1 for ASCII, 2 for integer,
                                      !   3 for floating point.

c=======================================================================********

c.... Input line:  redo help

 8110 format ('  Command "redo":  acceptable input lines include:' //
     &  '  "redo help"  Display this message.' //
     &  '  "redo [$]"' /
     &  '     Redo the last saved input line, with no replacements.' /
     &  '     (display with command "input command $ ?")' /
     &  '  "redo $ [R1 R2 R3 ...]"' /
     &  '     Redo the last saved input line, with arguments 1, 2, 3,',
     &  ' ...,' /
     &  '     replaced by R1, R2 R3, ...' /
     &  '     (display with command "input command $ ?")' /
     &  '  "redo LINE [R1 R2 R3 ...]"' /
     &  '     Redo line LINE in the list of saved input lines,' /
     &  '     with arguments 1, 2, 3, ..., replaced by R1, R2,',
     &  ' R3, ...' /
     &  '     (display with command "input command LINE ?")' )

 8112 format (
     &  '  "redo STRING [R1 R2 R3 ...]"' /
     &  '     Redo the last saved input line that begins with STRING,' /
     &  '     with arguments 1, 2, 3, ..., replaced by R1, R2',
     &  ', R3, ...' /
     &  '     (display with command "input command ?")' )

 8114 format (/
     &  '  Note:  "." means no replacement (all trailing "." may be',
     &  ' omitted).' /
     &  '  A negative LINE means the number of lines before the last'
     &  ' saved input line.' //
     &  '  Synonyms:  [help, h].' )

      if ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))  then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        write (   6, 8114)
        write (iout, 8114)
        nword = 0
        go to 999
      endif

c=======================================================================********

c.... Store redo input line in temporary location.

      nwork = nword

      do 120 n = 1, nwordm
        awork(n) = aword(n)
        iwork(n) = iword(n)
        fwork(n) = fword(n)
        lwork(n) = lword(n)
        mtypk(n) = mtype(n)
  120 continue

c=======================================================================********

c.... Input line:  redo
c.... Input line:  redo $      [R1 R2 R3 ...]
c.... Input line:  redo LINE   [R1 R2 R3 ...]
c.... Input line:  redo STRING [R1 R2 R3 ...]

c.... Find the saved input line in which to replace arguments.

 8140 format ('  redo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Saved input line does not exist:  ',i5)

      if (ncmds .le. 0) then
        write (   6, 8140) ainput, ninput, ncmds
        write (iout, 8140) ainput, ninput, ncmds
        nword = 0
        go to 999
      endif

  140 if ((nwork .eq. 1)       .or.
     &    (awork(2) .eq. '$')) then
        nmod = ncmds - 1              ! Option null or $ used.

        nn = 1 + mod (nmod - 1, ncmdm)

        call aptchap (acmd(1,nn), 1, ncharm, nwordm, afd,
     &                iamax, idmax, iemax, aword, iword, fword,
     &                mtype, lword, nword, nerra)

      elseif (mtypk(2) .eq. 2) then

        if (iwork(2) .lt. 0) then
          iwork(2) = ncmds + iwork(2)
        endif

        nmod = iwork(2)               ! Option LINE used.

        nmodmin = max (1, ncmds - ncmdm + 1)

        if ((nmod .ge. nmodmin) .and.
     &      (nmod .le. ncmds))  then

          nn = 1 + mod (nmod - 1, ncmdm)

          call aptchap (acmd(1,nn), 1, ncharm, nwordm, afd,
     &                  iamax, idmax, iemax, aword, iword, fword,
     &                  mtype, lword, nword, nerra)

        else
          write (   6, 8140) ainput, ninput, nmod
          write (iout, 8140) ainput, ninput, nmod
          nword = 0
          go to 999
        endif

      elseif (mtypk(2) .eq. 1) then   ! Option STRING used.

        nmodmin = max (1, ncmds - ncmdm + 1)

        do 150 n = ncmds, nmodmin, -1

          nn = 1 + mod (n - 1, ncmdm)

          call aptchap (acmd(1,nn), 1, ncharm, nwordm, afd,
     &                  iamax, idmax, iemax, aword, iword, fword,
     &                  mtype, lword, nword, nerra)

          if (awork(2) .eq.  aword(1)) then
            nmod = n
            go to 170
          endif

  150   continue

 8150   format ('  redo ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  There is no such saved input line:  ',a)

        write (   6, 8150) ainput, ninput, awork(2)
        write (iout, 8150) ainput, ninput, awork(2)
        nword = 0
        go to 999

      else                            ! Word 2 is floating point.

 8160   format ('  redo ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word 2 is not ASCII or integer:  ',a)

        write (   6, 8160) ainput, ninput, awork(2)
        write (iout, 8160) ainput, ninput, awork(2)
        nword = 0
        go to 999

      endif                           ! Found saved input line to modify.

 8185   format ('  redo ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  More replacements than arguments:  ',i3,' > ',i3)

  170 narg = nwork - 2
      if (narg .gt. nword) then
        write (   6, 8185) ainput, ninput, narg, nword
        write (iout, 8185) ainput, ninput, narg, nword
        nword = 0
        go to 999
      endif

c.... Save the unaltered saved input line in achar.

      do 225 n = 1, 80
        nn = 1 + mod (nmod - 1, ncmdm)
        achar(n) = acmd(n,nn)
  225 continue

c.... Replace arguments in the saved input line nmod, make new achar array.

      nrep = 0

      if (nwork .gt. 2) then

c....   Blank out achar, refill with new arguments.

        do 230 n = 1, 80
          achar(n) = ' '
  230   continue

        ibeg = 1

        do 240 narg = 1, nword        ! Loop over arguments.
          nw = narg + 2

          if ((nw .gt. nwork)       .or.
     &        (awork(nw) .eq. '.')) then
            go to 235
          endif

          nrep = nrep + 1

          aword(narg) = awork(nw)
          mtype(narg) = mtypk(nw)
          iword(narg) = iwork(nw)
          fword(narg) = fwork(nw)
          lword(narg) = lwork(nw)

 8235   format ('  redo ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  New command length exceeds 80 characters.')

  235     iend = ibeg + lword(narg) - 1
          if (iend .gt. 80) then
            write (   6, 8235) ainput, ninput
            write (iout, 8235) ainput, ninput
            nword = 0
            go to 999
          endif

          call aptchmv (aword(narg), 1, lword(narg), achar, ibeg, nerra)

          ibeg = ibeg + lword(narg) + 1

  240   continue                      ! End of loop over new arguments.

      endif                           ! Modified saved input line nmod.

 8250 format ('  Arguments replaced = **NONE**')
 8260 format ('  Arguments replaced = ',i3)

      if (nrep .eq. 0) then
        write (   6, 8250)
        write (iout, 8250)
      else
        write (   6, 8260) nrep
        write (iout, 8260) nrep
      endif

 8280 format ('  Original command:')
 8285 format ('  Modified command:')
 8290 format (80a1)

      write (   6, 8280)
      write (iout, 8280)
      write (   6, 8290) (acmd(n,nn), n = 1, 80)
      write (iout, 8290) (acmd(n,nn), n = 1, 80)
      write (   6, 8285)
      write (iout, 8285)
      write (   6, 8290) (achar(n), n = 1, 80)
      write (iout, 8290) (achar(n), n = 1, 80)

  999 return

c.... End of subroutine redo.         (+1 line)
      end
