      subroutine renamea

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  renamea  Updated     2007 January 19 15:30.
c               renamea  Originated  2005 April 5 17:45.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To rename by replacing the stem of subscripted names:
c               aliases, markers, variables, vectors, operators,
c               points, clusters, lines, triangles, annular disks,
c               regular polygons, planes, spheres, circular cylinders,
c               circular cones, hyperbolic paraboloids, ellipsoids, axisymmetric
c               quadric surfaces, general quadric surfaces, regular polyhedrons,
c               tetrahedrons, bricks, zones, bins, probability distribution
c               functions (pdfs) and symbols.
c
c     Logic:    Entry points:  renamea.
c renamea       Called by:  renamex.
c
c renamea       Calls:  alout, aptchfp, aptsubi, aptsubm, binout, brout,
c               clout, dkout, lout, markout, opout, pdfout, pgout, phout,
c               plout, pout, quout, qutype, symbdisp, tetout, trout, varout,
c               vout, zout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables 

      character*64 asrce              ! Characters $, %, @, A-Z, a-z.
      character*80 astemr             ! Replacement stem, aword(4).
      character*80 astemx             ! Stem of original object name.
      character*80 anamex             ! Temporary value of aword(3).
      character*80 anamer             ! Temporary value of aword(4).
      character*1  afirst             ! First character of point name.

c.... Find the original stem, astem, and the replacement stem, astemr.

      astem   = ' '
      astemr  = ' '
      astemx  = ' '
      anamer  = ' '
      anamex  = ' '
      lastem  = 0
      lastemr = 0
      lastemx = 0
      lanamer = 0
      lanamex = 0

 8102 format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Bad subscript(s):  ',a)
 8105 format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Stem name can not be subscripted:  ',a)

      call aptsubm (aword(3), 1, lword(3), iamax, idmax, iemax,
     &  '(', ',', ')', 16,
     &        astem,  istem, fstem, mstem, lastem,
     &  nsub, asub,   isub,  fsub,  msub,  lsub, nerra)

      if (nerra .ne. 0) then
        write (   6, 8102) ainput, ninput, aword(3)
        write (iout, 8102) ainput, ninput, aword(3)
        go to 999
      endif

      if (nsub .gt. 0) then
        write (   6, 8105) ainput, ninput, aword(3)
        write (iout, 8105) ainput, ninput, aword(3)
        go to 999
      endif

      call aptsubm (aword(4), 1, lword(4), iamax, idmax, iemax,
     &  '(', ',', ')', 16,
     &        astemr, istem, fstem, mstem, lastemr,
     &  nsub, asub,   isub,  fsub,  msub,  lsub, nerra)

      if (nerra .ne. 0) then
        write (   6, 8102) ainput, ninput, aword(4)
        write (iout, 8102) ainput, ninput, aword(4)
        go to 999
      endif

      if (nsub .gt. 0) then
        write (   6, 8105) ainput, ninput, aword(4)
        write (iout, 8105) ainput, ninput, aword(4)
        go to 999
      endif

c.... Test for the lengths of the geometric object names.

 8110 format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of object not from 1 to ',i3,' characters:  ',a)

      lenmax = 24

      if ((aword(2) .eq. 'marker') .or.
     &    (aword(2) .eq. 'mark')   .or.
     &    (aword(2) .eq. 'm'))     then
        lenmax = 1
      endif

      do 110 n = 3, 4
        if ((lword(n) .lt. 1) .or. (lword(n) .gt. lenmax)) then
          write (   6, 8110) ainput, ninput, lenmax, aword(n)
          write (iout, 8110) ainput, ninput, lenmax, aword(n)
          go to 999
        endif
  110 continue

c.... Test to see if geometric object names are different.

 8115 format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Source and sink objects have same name:  ',a)

      if (aword(3) .eq. aword(4)) then
        write (   6, 8115) ainput, ninput, aword(3)
        write (iout, 8115) ainput, ninput, aword(3)
        go to 999
      endif

c.... See if new name is acceptable.

 8120 format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Name too dangerous to use:  ',a)

c===============================================================================

c.... Input line:  rename variable VARSTEM VARSTEMR array

      if ((aword(2) .eq. 'variable') .or.
     &    (aword(2) .eq. 'var')      .or.
     &    (aword(2) .eq. 'set')      .or.
     &    (aword(2) .eq. 's'))       then

        numr = 0

c....   Find all occurrances of old stem.

        do 140 n1 = 1, nvars          ! Loop over variables.

          nerr = 0

          call aptsubm (avarbl(n1), 1, lvar(n1), iamax, idmax, iemax,
     &      '(', ',', ')', 16,
     &            astemx, istem, fstem, mstem, lastemx,
     &      nsub, asub,   isub,  fsub,  msub,  lsub, nerra)

          if (nerra .ne. 0) then
            write (   6, 8102) ainput, ninput, avarbl(n1)
            write (iout, 8102) ainput, ninput, avarbl(n1)
            go to 999
          endif

 8125     format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &            '  A subscript is not an integer:  ',a)

          do ns = 1, nsub
            if (msub(ns) .ne. 2) then
              write (   6, 8125) ainput, ninput, avarbl(n1)
              write (iout, 8125) ainput, ninput, avarbl(n1)
              go to 999
            endif
          enddo

          if (astemx .eq. astem) then ! Replace old stem with new stem.

            call aptsubi (astem,  lastem,  nsub, isub, 80,
     &                    '(', ',', ')', anamex, lanamex, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, avarbl(n1)
              write (iout, 8102) ainput, ninput, avarbl(n1)
              go to 999
            endif

            call aptsubi (astemr, lastemr, nsub, isub, 80,
     &                    '(', ',', ')', anamer, lanamer, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, avarbl(n1)
              write (iout, 8102) ainput, ninput, avarbl(n1)
              go to 999
            endif

            aword(3) = anamex
            lword(3) = lanamex

            aword(4) = anamer
            lword(4) = lanamer

c....       See if new variable already exists.

 8130       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  New variable name already exists:  ',a)

            do 120 n = 1, nvars
              if (aword(4) .eq. avarbl(n)) then
                write (   6, 8130) ainput, ninput, aword(4)
                write (iout, 8130) ainput, ninput, aword(4)
                go to 999
              endif
  120       continue

c....       Test for acceptable variable name.

 8132       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Unacceptable name.  Too dangerous:  ',a)
 8134       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Unacceptable name.  Use command "tol".')
 8136       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Name must begin with a-z, A-Z, $, % or @:  ',a)

            if (mtype(4) .ne. 1) then
              write (   6, 8132) ainput, ninput, aword(4)
              write (iout, 8132) ainput, ninput, aword(4)
              go to 999
            endif

            if (aword(4) .eq. 'random') then
              write (   6, 8132) ainput, ninput, aword(4)
              write (iout, 8132) ainput, ninput, aword(4)
              go to 999
            endif

            if (aword(4) .eq. 'tol') then
              write (   6, 8134) ainput, ninput
              write (iout, 8134) ainput, ninput
              go to 999
            endif

            asrce =
     &         '$%@ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'

            call aptchfp (asrce, 1, 55, aword(4), 1, ipat, nerra)

            if (ipat .eq. 0) then
              write (   6, 8136) ainput, ninput, aword(4)
              write (iout, 8136) ainput, ninput, aword(4)
              go to 999
            endif

 8138       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Name of variable not from 1 to 24 characters:  ',a)

            if ((lword(4) .lt. 1) .or. (lword(4) .gt. lenmax)) then
              write (   6, 8138) ainput, ninput, aword(4)
              write (iout, 8138) ainput, ninput, aword(4)
              go to 999
            endif

            if (nerr .ne. 0) go to 140

 8140       format ('  Renamed variable ',a,' to ',a)

            numr = numr + 1

            write (   6, 8140) avarbl(n1), aword(4)
            write (iout, 8140) avarbl(n1), aword(4)

            avarbl(n1)  = aword(4)
            avarbls     = aword(4)
            lvar(n1)    = lword(4)
            lvarbls     = lword(4)

            call varout (n1, n1)

          endif                       ! Replaced old stem with new stem.

  140   continue                      ! End of loop over variables.

 8145   format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  No replacement possible:  ',a)

        if (numr .eq. 0) then
          write (   6, 8145) ainput, ninput, aword(3)
          write (iout, 8145) ainput, ninput, aword(3)
        endif

        go to 999

      endif                           ! Renamed an array of variables.

c===============================================================================

c.... Input line:  rename point PTSTEM PTSTEMR array

      if ((aword(2) .eq. 'point') .or.
     &    (aword(2) .eq. 'pnt')   .or.
     &    (aword(2) .eq. 'pt')    .or.
     &    (aword(2) .eq. 'p'))    then

        numr = 0

c....   Find all occurrances of old stem.

        do 250 n1 = 1, npoints        ! Loop over points.

          nerr = 0

          call aptsubm (apoint(n1), 1, lapoint(n1), iamax, idmax, iemax,
     &      '(', ',', ')', 16,
     &            astemx, istem, fstem, mstem, lastemx,
     &      nsub, asub,   isub,  fsub,  msub,  lsub, nerra)

          if (nerra .ne. 0) then
            write (   6, 8102) ainput, ninput, apoint(n1)
            write (iout, 8102) ainput, ninput, apoint(n1)
            go to 999
          endif

 8150     format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &            '  A subscript is not an integer:  ',a)

          do ns = 1, nsub
            if (msub(ns) .ne. 2) then
              go to 999
              write (   6, 8150) ainput, ninput, apoint(n1)
              write (iout, 8150) ainput, ninput, apoint(n1)
            endif
          enddo

          if (astemx .eq. astem) then ! Replace old stem with new stem.

            call aptsubi (astem,  lastem,  nsub, isub, 80,
     &                    '(', ',', ')', anamex, lanamex, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, apoint(n1)
              write (iout, 8102) ainput, ninput, apoint(n1)
              go to 999
            endif

            call aptsubi (astemr, lastemr, nsub, isub, 80,
     &                    '(', ',', ')', anamer, lanamer, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, apoint(n1)
              write (iout, 8102) ainput, ninput, apoint(n1)
              go to 999
            endif

            aword(3) = anamex
            lword(3) = lanamex

            aword(4) = anamer
            lword(4) = lanamer

c....       See if new point already exists.

 8155       format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  New point name already exists:  ',a)

            do 150 n = 1, npoints
              if (aword(4) .eq. apoint(n)) then
                write (   6, 8155) ainput, ninput, aword(4)
                write (iout, 8155) ainput, ninput, aword(4)
                go to 999
              endif
  150       continue

c....       Test for acceptable point name.

            if (mtype(4) .ne. 1) then
              write (   6, 8132) ainput, ninput, aword(4)
              write (iout, 8132) ainput, ninput, aword(4)
              go to 999
            endif

            afirst = aword(4)

            if (afirst .eq. '-')  then
              write (   6, 8120) ainput, ninput, aword(4)
              write (iout, 8120) ainput, ninput, aword(4)
              go to 999
            endif

 8160       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Name of point not from 1 to 24 characters:  ',a)

            if ((lword(4) .lt. 1) .or. (lword(4) .gt. lenmax)) then
              write (   6, 8160) ainput, ninput, aword(4)
              write (iout, 8160) ainput, ninput, aword(4)
              go to 999
            endif

c....       See if new subscripts, if any, match mesh indices, if any.

            call aptsubm (aword(4), 1, lword(4), iamax, idmax, iemax,
     &        '(', ',', ')', 16,
     &              anamer, istem, fstem, mstem, lanamer,
     &        nsub, asub,  isub,  fsub,  msub,  lsub, nerra)

 8165       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Point name subscripts mismatch mesh indices:  ',a24)

            if (nsub .eq. 3) then
            if (((kpoint(n1) .ne. 0) .and.
     &           (kpoint(n1) .ne. isub(1)))  .or.
     &          ((lpoint(n1) .ne. 0) .and.
     &           (lpoint(n1) .ne. isub(2)))  .or.
     &          ((mpoint(n1) .ne. 0) .and.
     &           (mpoint(n1) .ne. isub(3)))) then
              write (   6, 8165) ainput, ninput, aword(4)
              write (iout, 8165) ainput, ninput, aword(4)
              go to 999
            endif
            endif

            if (nerr .ne. 0) go to 250

 8170       format ('  Renamed point ',a,' to ',a)

            numr = numr + 1

            write (   6, 8170) apoint(n1), aword(4)
            write (iout, 8170) apoint(n1), aword(4)

            apoint(n1)  = aword(4)
            apoints     = aword(4)
            lapoint(n1) = lword(4)
            lapoints    = lword(4)

c....       Rename same point in definitions of clusters, lines, triangles,
c....         annular disks, regular polygons, tetrahedrons.

            do 170 npr = 1, npairs
              if (apairpt(npr) .eq. aword(3)) apairpt(npr) = aword(4)
  170       continue

            do 180 nn = 1, nlines
              if (alipt1(nn)  .eq. aword(3)) alipt1(nn)   = aword(4)
              if (alipt2(nn)  .eq. aword(3)) alipt2(nn)   = aword(4)
              if (alipt1(nn)  .eq. aword(3)) lalipt1(nn)  = lword(4)
              if (alipt2(nn)  .eq. aword(3)) lalipt2(nn)  = lword(4)
  180       continue

            do 190 nt = 1, ntrigs
              if (atript1(nt) .eq. aword(3)) atript1(nt)  = aword(4)
              if (atript2(nt) .eq. aword(3)) atript2(nt)  = aword(4)
              if (atript3(nt) .eq. aword(3)) atript3(nt)  = aword(4)
              if (atript1(nt) .eq. aword(3)) latript1(nt) = lword(4)
              if (atript2(nt) .eq. aword(3)) latript2(nt) = lword(4)
              if (atript3(nt) .eq. aword(3)) latript3(nt) = lword(4)
  190       continue

            do 200 npg = 1, npolygs
              if (acenpg(npg) .eq. aword(3)) acenpg(npg) = aword(4)
              if (averpg(npg) .eq. aword(3)) averpg(npg) = aword(4)
              if (acoppg(npg) .eq. aword(3)) acoppg(npg) = aword(4)
  200       continue

            do 210 nd = 1, ndisks
              if (acendk(nd)  .eq. aword(3)) acendk(nd)  = aword(4)
  210       continue

            do 220 nt = 1, ntets
              if (atetpt(1,nt) .eq. aword(3)) atetpt(1,nt) = aword(4)
              if (atetpt(2,nt) .eq. aword(3)) atetpt(2,nt) = aword(4)
              if (atetpt(3,nt) .eq. aword(3)) atetpt(3,nt) = aword(4)
              if (atetpt(4,nt) .eq. aword(3)) atetpt(4,nt) = aword(4)
  220       continue

            do 230 nt = 1, ntets
              if (atetpt(1,nt) .eq. aword(3)) latetpt(1,nt) = lword(4)
              if (atetpt(2,nt) .eq. aword(3)) latetpt(2,nt) = lword(4)
              if (atetpt(3,nt) .eq. aword(3)) latetpt(3,nt) = lword(4)
              if (atetpt(4,nt) .eq. aword(3)) latetpt(4,nt) = lword(4)
  230       continue

            do 240 nph = 1, npolyhs
              if (acenph(nph) .eq. aword(3)) acenph(nph) = aword(4)
              if (averph(nph) .eq. aword(3)) averph(nph) = aword(4)
              if (acopph(nph) .eq. aword(3)) acopph(nph) = aword(4)
  240       continue

            call pout (n1, n1, 'min')

          endif                       ! Replaced old stem with new stem.

  250   continue                      ! End of loop over points.

 8250   format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  No replacement possible:  ',a)

        if (numr .eq. 0) then
          write (   6, 8250) ainput, ninput, aword(3)
          write (iout, 8250) ainput, ninput, aword(3)
        endif

        go to 999

      endif                           ! Renamed an array of points.

c===============================================================================

c.... Input line:  rename cluster CLSTEM CLSTEMR array

      if ((aword(2) .eq. 'cluster') .or.
     &    (aword(2) .eq. 'cl'))     then

        numr = 0

c....   Find all occurrances of old stem.

        do 290 n1 = 1, nclusts        ! Loop over clusters.

          nerr = 0

          call aptsubm (aclust(n1), 1, laclust(n1), iamax, idmax, iemax,
     &      '(', ',', ')', 16,
     &            astemx, istem, fstem, mstem, lastemx,
     &      nsub, asub,   isub,  fsub,  msub,  lsub, nerra)

          if (nerra .ne. 0) then
            write (   6, 8102) ainput, ninput, aclust(n1)
            write (iout, 8102) ainput, ninput, aclust(n1)
            go to 999
          endif

          do ns = 1, nsub
            if (msub(ns) .ne. 2) then
              write (   6, 8150) ainput, ninput, aclust(n1)
              write (iout, 8150) ainput, ninput, aclust(n1)
              go to 999
            endif
          enddo

          if (astemx .eq. astem) then ! Replace old stem with new stem.

            call aptsubi (astem,  lastem,  nsub, isub, 80,
     &                    '(', ',', ')', anamex, lanamex, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, aclust(n1)
              write (iout, 8102) ainput, ninput, aclust(n1)
              go to 999
            endif

            call aptsubi (astemr, lastemr, nsub, isub, 80,
     &                    '(', ',', ')', anamer, lanamer, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, aclust(n1)
              write (iout, 8102) ainput, ninput, aclust(n1)
              go to 999
            endif

            aword(3) = anamex
            lword(3) = lanamex

            aword(4) = anamer
            lword(4) = lanamer

c....       See if new cluster already exists.

 8260       format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  New cluster name already exists:  ',a)

            do 260 n = 1, nclusts
              if (aword(4) .eq. aclust(n)) then
                write (   6, 8260) ainput, ninput, aword(4)
                write (iout, 8260) ainput, ninput, aword(4)
                go to 999
              endif
  260       continue

c....       Test for acceptable cluster name.

            if (mtype(4) .ne. 1) then
              write (   6, 8132) ainput, ninput, aword(4)
              write (iout, 8132) ainput, ninput, aword(4)
              go to 999
            endif

            afirst = aword(4)

            if (afirst .eq. '-')  then
              go to 999
              write (   6, 8120) ainput, ninput, aword(4)
              write (iout, 8120) ainput, ninput, aword(4)
            endif

 8270       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Name of cluster not from 1 to 24 characters:  ',a)

            if ((lword(4) .lt. 1) .or. (lword(4) .gt. lenmax)) then
              write (   6, 8270) ainput, ninput, aword(4)
              write (iout, 8270) ainput, ninput, aword(4)
              go to 999
            endif

            if (nerr .ne. 0) go to 290

 8275       format ('  Renamed cluster ',a,' to ',a)

            numr = numr + 1

            write (   6, 8275) aclust(n1), aword(4)
            write (iout, 8275) aclust(n1), aword(4)

c....       Rename same cluster in table of point and cluster pairs.

            do 280 npr = 1, npairs
              if (apaircl(npr) .eq. aword(3)) then
                apaircl(npr) = aword(4)
              endif
  280       continue

            aclust(n1)  = aword(4)
            aclusts     = aword(4)
            laclust(n1) = lword(4)
            laclusts    = lword(4)

            call clout (n1, n1, 'min')

          endif                       ! Replaced old stem with new stem.

  290   continue                      ! End of loop over clusters.

 8280   format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  No replacement possible:  ',a)

        if (numr .eq. 0) then
          write (   6, 8280) ainput, ninput, aword(3)
          write (iout, 8280) ainput, ninput, aword(3)
        endif

        go to 999

      endif                           ! Renamed an array of clusters.

c===============================================================================

c.... Input line:  rename line ALSTEM ALSTEMR array

      if ((aword(2) .eq. 'line') .or.
     &    (aword(2) .eq. 'ln')   .or.
     &    (aword(2) .eq. 'l'))   then

        numr = 0

c....   Find all occurrances of old stem.

        do 320 n1 = 1, nlines         ! Loop over lines.

          nerr = 0

          call aptsubm (aline(n1), 1, 24, iamax, idmax, iemax,
     &      '(', ',', ')', 16,
     &            astemx, istem, fstem, mstem, lastemx,
     &      nsub, asub,   isub,  fsub,  msub,  lsub, nerra)

          if (nerra .ne. 0) then
            write (   6, 8102) ainput, ninput, aline(n1)
            write (iout, 8102) ainput, ninput, aline(n1)
            go to 999
          endif

 8292     format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &            '  A subscript is not an integer:  ',a)

          do ns = 1, nsub
            if (msub(ns) .ne. 2) then
              write (   6, 8292) ainput, ninput, aline(n1)
              write (iout, 8292) ainput, ninput, aline(n1)
              go to 999
            endif
          enddo

          if (astemx .eq. astem) then ! Replace old stem with new stem.

            call aptsubi (astem,  lastem,  nsub, isub, 80,
     &                    '(', ',', ')', anamex, lanamex, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, aline(n1)
              write (iout, 8102) ainput, ninput, aline(n1)
              go to 999
            endif

            call aptsubi (astemr, lastemr, nsub, isub, 80,
     &                    '(', ',', ')', anamer, lanamer, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, aline(n1)
              write (iout, 8102) ainput, ninput, aline(n1)
              go to 999
            endif

            aword(3) = anamex
            lword(3) = lanamex

            aword(4) = anamer
            lword(4) = lanamer

c....       See if new line already exists.

 8310       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  New line name already exists:  ',a)

            do 300 n = 1, nlines
              if (aword(4) .eq. aline(n)) then
                write (   6, 8310) ainput, ninput, aword(4)
                write (iout, 8310) ainput, ninput, aword(4)
                go to 999
              endif
  300       continue

c....       Test for acceptable line name.

 8312       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Unacceptable name.  Too dangerous:  ',a)
 8314       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Unacceptable name.  Use command "tol".')

            if (mtype(4) .ne. 1) then
              write (   6, 8312) ainput, ninput, aword(4)
              write (iout, 8312) ainput, ninput, aword(4)
              go to 999
            endif

 8316       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Name of line not from 1 to 24 characters:  ',a)

            if ((lword(4) .lt. 1) .or. (lword(4) .gt. lenmax)) then
              write (   6, 8316) ainput, ninput, aword(4)
              write (iout, 8316) ainput, ninput, aword(4)
              go to 999
            endif

            if (nerr .ne. 0) go to 320

 8320       format ('  Renamed line ',a,' to ',a)

            numr = numr + 1

            write (   6, 8320) aline(n1), aword(4)
            write (iout, 8320) aline(n1), aword(4)

            aline(n1) = aword(4)
            alines    = aword(4)
            lalines   = lword(4)

            call lout (n1, n1)

          endif                       ! Replaced old stem with new stem.

  320   continue                      ! End of loop over lines.

 8325   format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  No replacement possible:  ',a)

        if (numr .eq. 0) then
          write (   6, 8325) ainput, ninput, aword(3)
          write (iout, 8325) ainput, ninput, aword(3)
        endif

        go to 999

      endif                           ! Renamed an array of lines.

c===============================================================================

c.... Input line:  rename triangle TRSTEM TRSTEMR array

      if ((aword(2) .eq. 'triangle') .or.
     &    (aword(2) .eq. 'tri'))     then

        numr = 0

c....   Find all occurrances of old stem.

        do 350 n1 = 1, ntrigs         ! Loop over triangles.

          nerr = 0

          call aptsubm (atrig(n1), 1, 24, iamax, idmax, iemax,
     &      '(', ',', ')', 16,
     &            astemx, istem, fstem, mstem, lastemx,
     &      nsub, asub,   isub,  fsub,  msub,  lsub, nerra)

          if (nerra .ne. 0) then
            write (   6, 8102) ainput, ninput, atrig(n1)
            write (iout, 8102) ainput, ninput, atrig(n1)
            go to 999
          endif

 8330     format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &            '  A subscript is not an integer:  ',a)

          do ns = 1, nsub
            if (msub(ns) .ne. 2) then
              write (   6, 8330) ainput, ninput, atrig(n1)
              write (iout, 8330) ainput, ninput, atrig(n1)
              go to 999
            endif
          enddo

          if (astemx .eq. astem) then ! Replace old stem with new stem.

            call aptsubi (astem,  lastem,  nsub, isub, 80,
     &                    '(', ',', ')', anamex, lanamex, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, atrig(n1)
              write (iout, 8102) ainput, ninput, atrig(n1)
              go to 999
            endif

            call aptsubi (astemr, lastemr, nsub, isub, 80,
     &                    '(', ',', ')', anamer, lanamer, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, atrig(n1)
              write (iout, 8102) ainput, ninput, atrig(n1)
              go to 999
            endif

            aword(3) = anamex
            lword(3) = lanamex

            aword(4) = anamer
            lword(4) = lanamer

c....       See if new triangle already exists.

 8335       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  New triangle name already exists:  ',a)

            do 330 n = 1, ntrigs
              if (aword(4) .eq. atrig(n)) then
                write (   6, 8335) ainput, ninput, aword(4)
                write (iout, 8335) ainput, ninput, aword(4)
                go to 999
                go to 340
              endif
  330       continue
  340       continue

c....       Test for acceptable triangle name.

 8340       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Unacceptable name.  Too dangerous:  ',a)
 8345       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Unacceptable name.  Use command "tol".')

            if (mtype(4) .ne. 1) then
              write (   6, 8340) ainput, ninput, aword(4)
              write (iout, 8340) ainput, ninput, aword(4)
              go to 999
            endif

 8348       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Name of triangle not from 1 to 24 characters:  ',a)

            if ((lword(4) .lt. 1) .or. (lword(4) .gt. lenmax)) then
              write (   6, 8348) ainput, ninput, aword(4)
              write (iout, 8348) ainput, ninput, aword(4)
              go to 999
            endif

            if (nerr .ne. 0) go to 350

 8350       format ('  Renamed triangle ',a,' to ',a)

            numr = numr + 1

            write (   6, 8350) atrig(n1), aword(4)
            write (iout, 8350) atrig(n1), aword(4)

            atrig(n1)  = aword(4)
            atrigs     = aword(4)
            latrigs    = lword(4)

            call trout (n1, n1, 'min')

          endif                       ! Replaced old stem with new stem.

  350   continue                      ! End of loop over triangles.

 8352   format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  No replacement possible:  ',a)

        if (numr .eq. 0) then
          write (   6, 8352) ainput, ninput, aword(3)
          write (iout, 8352) ainput, ninput, aword(3)
        endif

        go to 999

      endif                           ! Renamed an array of triangles.

c===============================================================================

c.... Input line:  rename polygon PGSTEM PGSTEMR array

      if ((aword(2) .eq. 'polygon') .or.
     &    (aword(2) .eq. 'polyg')   .or.
     &    (aword(2) .eq. 'pg'))     then

        numr = 0

c....   Find all occurrances of old stem.

        do 380 n1 = 1, npolygs        ! Loop over polygons.

          nerr = 0

          call aptsubm (apolyg(n1), 1, 24, iamax, idmax, iemax,
     &      '(', ',', ')', 16,
     &            astemx, istem, fstem, mstem, lastemx,
     &      nsub, asub,   isub,  fsub,  msub,  lsub, nerra)

          if (nerra .ne. 0) then
            write (   6, 8102) ainput, ninput, apolyg(n1)
            write (iout, 8102) ainput, ninput, apolyg(n1)
            go to 999
          endif

 8355     format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &            '  A subscript is not an integer:  ',a)

          do ns = 1, nsub
            if (msub(ns) .ne. 2) then
              write (   6, 8355) ainput, ninput, apolyg(n1)
              write (iout, 8355) ainput, ninput, apolyg(n1)
              go to 999
            endif
          enddo

          if (astemx .eq. astem) then ! Replace old stem with new stem.

            call aptsubi (astem,  lastem,  nsub, isub, 80,
     &                    '(', ',', ')', anamex, lanamex, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, apolyg(n1)
              write (iout, 8102) ainput, ninput, apolyg(n1)
              go to 999
            endif

            call aptsubi (astemr, lastemr, nsub, isub, 80,
     &                    '(', ',', ')', anamer, lanamer, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, apolyg(n1)
              write (iout, 8102) ainput, ninput, apolyg(n1)
              go to 999
            endif

            aword(3) = anamex
            lword(3) = lanamex

            aword(4) = anamer
            lword(4) = lanamer

c....       See if new polygon already exists.

 8360       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  New polygon name already exists:  ',a)

            do 360 n = 1, npolygs
              if (aword(4) .eq. apolyg(n)) then
                write (   6, 8360) ainput, ninput, aword(4)
                write (iout, 8360) ainput, ninput, aword(4)
                go to 999
                go to 370
              endif
  360       continue
  370       continue

c....       Test for acceptable polygon name.

 8365       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Unacceptable name.  Too dangerous:  ',a)
 8370       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Unacceptable name.  Use command "tol".')

            if (mtype(4) .ne. 1) then
              write (   6, 8365) ainput, ninput, aword(4)
              write (iout, 8365) ainput, ninput, aword(4)
              go to 999
            endif

 8375       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Name of polygon not from 1 to 24 characters:  ',a)

            if ((lword(4) .lt. 1) .or. (lword(4) .gt. lenmax)) then
              write (   6, 8375) ainput, ninput, aword(4)
              write (iout, 8375) ainput, ninput, aword(4)
              go to 999
            endif

            if (nerr .ne. 0) go to 380

 8380       format ('  Renamed polygon ',a,' to ',a)

            numr = numr + 1

            write (   6, 8380) apolyg(n1), aword(4)
            write (iout, 8380) apolyg(n1), aword(4)

            apolyg(n1)  = aword(4)
            apolygs     = aword(4)
            lapolygs    = lword(4)

            call pgout (n1, n1, 'min')

          endif                       ! Replaced old stem with new stem.

  380   continue                      ! End of loop over polygons.

 8385   format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  No replacement possible:  ',a)

        if (numr .eq. 0) then
          write (   6, 8385) ainput, ninput, aword(3)
          write (iout, 8385) ainput, ninput, aword(3)
        endif

        go to 999

      endif                           ! Renamed an array of polygons.

c===============================================================================

c.... Input line:  rename disk DKSTEM DKSTEMR array

      if ((aword(2) .eq. 'disk') .or.
     &    (aword(2) .eq. 'disc') .or.
     &    (aword(2) .eq. 'dk'))  then

        numr = 0

c....   Find all occurrances of old stem.

        do 410 n1 = 1, ndisks         ! Loop over disks.

          nerr = 0

          call aptsubm (adisk(n1), 1, 24, iamax, idmax, iemax,
     &      '(', ',', ')', 16,
     &            astemx, istem, fstem, mstem, lastemx,
     &      nsub, asub,   isub,  fsub,  msub,  lsub, nerra)

          if (nerra .ne. 0) then
            write (   6, 8102) ainput, ninput, adisk(n1)
            write (iout, 8102) ainput, ninput, adisk(n1)
            go to 999
          endif

 8392     format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &            '  A subscript is not an integer:  ',a)

          do ns = 1, nsub
            if (msub(ns) .ne. 2) then
              write (   6, 8392) ainput, ninput, adisk(n1)
              write (iout, 8392) ainput, ninput, adisk(n1)
              go to 999
            endif
          enddo

          if (astemx .eq. astem) then ! Replace old stem with new stem.

            call aptsubi (astem,  lastem,  nsub, isub, 80,
     &                    '(', ',', ')', anamex, lanamex, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, adisk(n1)
              write (iout, 8102) ainput, ninput, adisk(n1)
              go to 999
            endif

            call aptsubi (astemr, lastemr, nsub, isub, 80,
     &                    '(', ',', ')', anamer, lanamer, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, adisk(n1)
              write (iout, 8102) ainput, ninput, adisk(n1)
              go to 999
            endif

            aword(3) = anamex
            lword(3) = lanamex

            aword(4) = anamer
            lword(4) = lanamer

c....       See if new disk already exists.

 8400       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  New disk name already exists:  ',a)

            do 390 n = 1, ndisks
              if (aword(4) .eq. adisk(n)) then
                write (   6, 8400) ainput, ninput, aword(4)
                write (iout, 8400) ainput, ninput, aword(4)
                go to 999
                go to 400
              endif
  390       continue
  400       continue

c....       Test for acceptable disk name.

 8405       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Unacceptable name.  Too dangerous:  ',a)

            if (mtype(4) .ne. 1) then
              write (   6, 8405) ainput, ninput, aword(4)
              write (iout, 8405) ainput, ninput, aword(4)
              go to 999
            endif

 8410       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Name of disk not from 1 to 24 characters:  ',a)

            if ((lword(4) .lt. 1) .or. (lword(4) .gt. lenmax)) then
              write (   6, 8410) ainput, ninput, aword(4)
              write (iout, 8410) ainput, ninput, aword(4)
              go to 999
            endif

            if (nerr .ne. 0) go to 410

 8415       format ('  Renamed disk ',a,' to ',a)

            numr = numr + 1

            write (   6, 8415) adisk(n1), aword(4)
            write (iout, 8415) adisk(n1), aword(4)

            adisk(n1)  = aword(4)
            adisks     = aword(4)
            ladisks    = lword(4)

            call dkout (n1, n1, 'min')

          endif                       ! Replaced old stem with new stem.

  410   continue                      ! End of loop over disks.

 8420   format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  No replacement possible:  ',a)

        if (numr .eq. 0) then
          write (   6, 8420) ainput, ninput, aword(3)
          write (iout, 8420) ainput, ninput, aword(3)
        endif

        go to 999

      endif                           ! Renamed an array of disks.

c===============================================================================

c.... Input line:  rename polyhedron PHSTEM PHSTEMR array

      if ((aword(2) .eq. 'polyhedron') .or.
     &    (aword(2) .eq. 'polyh')   .or.
     &    (aword(2) .eq. 'ph'))     then

        numr = 0

c....   Find all occurrances of old stem.

        do 440 n1 = 1, npolyhs        ! Loop over polyhedrons.

          nerr = 0

          call aptsubm (apolyh(n1), 1, 24, iamax, idmax, iemax,
     &      '(', ',', ')', 16,
     &            astemx, istem, fstem, mstem, lastemx,
     &      nsub, asub,   isub,  fsub,  msub,  lsub, nerra)

          if (nerra .ne. 0) then
            write (   6, 8102) ainput, ninput, apolyh(n1)
            write (iout, 8102) ainput, ninput, apolyh(n1)
            go to 999
          endif

 8425     format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &            '  A subscript is not an integer:  ',a)

          do ns = 1, nsub
            if (msub(ns) .ne. 2) then
              write (   6, 8425) ainput, ninput, apolyh(n1)
              write (iout, 8425) ainput, ninput, apolyh(n1)
              go to 999
            endif
          enddo

          if (astemx .eq. astem) then ! Replace old stem with new stem.

            call aptsubi (astem,  lastem,  nsub, isub, 80,
     &                    '(', ',', ')', anamex, lanamex, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, apolyh(n1)
              write (iout, 8102) ainput, ninput, apolyh(n1)
              go to 999
            endif

            call aptsubi (astemr, lastemr, nsub, isub, 80,
     &                    '(', ',', ')', anamer, lanamer, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, apolyh(n1)
              write (iout, 8102) ainput, ninput, apolyh(n1)
              go to 999
            endif

            aword(3) = anamex
            lword(3) = lanamex

            aword(4) = anamer
            lword(4) = lanamer

c....       See if new polyhedron already exists.

 8428       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  New polyhedron name already exists:  ',a)

            do 420 n = 1, npolyhs
              if (aword(4) .eq. apolyh(n)) then
                write (   6, 8428) ainput, ninput, aword(4)
                write (iout, 8428) ainput, ninput, aword(4)
                go to 999
                go to 430
              endif
  420       continue
  430       continue

c....       Test for acceptable polyhedron name.

 8430       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Unacceptable name.  Too dangerous:  ',a)
 8435       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Unacceptable name.  Use command "tol".')

            if (mtype(4) .ne. 1) then
              write (   6, 8430) ainput, ninput, aword(4)
              write (iout, 8430) ainput, ninput, aword(4)
              go to 999
            endif

 8442       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Name of polyhedron not from 1 to 24 characters:  ',a)

            if ((lword(4) .lt. 1) .or. (lword(4) .gt. lenmax)) then
              write (   6, 8442) ainput, ninput, aword(4)
              write (iout, 8442) ainput, ninput, aword(4)
              go to 999
            endif

            if (nerr .ne. 0) go to 440

 8445       format ('  Renamed polyhedron ',a,' to ',a)

            numr = numr + 1

            write (   6, 8445) apolyh(n1), aword(4)
            write (iout, 8445) apolyh(n1), aword(4)

            apolyh(n1)  = aword(4)
            apolyhs     = aword(4)
            lapolyhs    = lword(4)

            call phout (n1, n1, 'min')

          endif                       ! Replaced old stem with new stem.

  440   continue                      ! End of loop over polyhedrons.

 8450   format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  No replacement possible:  ',a)

        if (numr .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(3)
          write (iout, 8450) ainput, ninput, aword(3)
        endif

        go to 999

      endif                           ! Renamed an array of polyhedrons.

c===============================================================================

c.... Input line:  rename tetrahedron TETSTEM TETSTEMR array

      if ((aword(2) .eq. 'tetrahedron') .or.
     &    (aword(2) .eq. 'tetra')       .or.
     &    (aword(2) .eq. 'tet'))        then

        numr = 0

c....   Find all occurrances of old stem.

        do 470 n1 = 1, ntets        ! Loop over tetrahedrons.

          nerr = 0

          call aptsubm (atetra(n1), 1, 24, iamax, idmax, iemax,
     &      '(', ',', ')', 16,
     &            astemx, istem, fstem, mstem, lastemx,
     &      nsub, asub,   isub,  fsub,  msub,  lsub, nerra)

          if (nerra .ne. 0) then
            write (   6, 8102) ainput, ninput, atetra(n1)
            write (iout, 8102) ainput, ninput, atetra(n1)
            go to 999
          endif

 8455     format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &            '  A subscript is not an integer:  ',a)

          do ns = 1, nsub
            if (msub(ns) .ne. 2) then
              write (   6, 8455) ainput, ninput, atetra(n1)
              write (iout, 8455) ainput, ninput, atetra(n1)
              go to 999
            endif
          enddo

          if (astemx .eq. astem) then ! Replace old stem with new stem.

            call aptsubi (astem,  lastem,  nsub, isub, 80,
     &                    '(', ',', ')', anamex, lanamex, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, atetra(n1)
              write (iout, 8102) ainput, ninput, atetra(n1)
              go to 999
            endif

            call aptsubi (astemr, lastemr, nsub, isub, 80,
     &                    '(', ',', ')', anamer, lanamer, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, atetra(n1)
              write (iout, 8102) ainput, ninput, atetra(n1)
              go to 999
            endif

            aword(3) = anamex
            lword(3) = lanamex

            aword(4) = anamer
            lword(4) = lanamer

c....       See if new tetrahedron already exists.

 8460       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  New tetrahedron name already exists:  ',a)

            do 450 n = 1, ntets
              if (aword(4) .eq. atetra(n)) then
                write (   6, 8460) ainput, ninput, aword(4)
                write (iout, 8460) ainput, ninput, aword(4)
                go to 999
                go to 460
              endif
  450       continue
  460       continue

c....       Test for acceptable tetrahedron name.

 8470       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Unacceptable name.  Too dangerous:  ',a)
 8472       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Unacceptable name.  Use command "tol".')

            if (mtype(4) .ne. 1) then
              write (   6, 8470) ainput, ninput, aword(4)
              write (iout, 8470) ainput, ninput, aword(4)
              go to 999
            endif

 8475       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Name of tetrahedron not from 1 to 24 characters:  ',a)

            if ((lword(4) .lt. 1) .or. (lword(4) .gt. lenmax)) then
              write (   6, 8475) ainput, ninput, aword(4)
              write (iout, 8475) ainput, ninput, aword(4)
              go to 999
            endif

            if (nerr .ne. 0) go to 470

 8478       format ('  Renamed tetrahedron ',a,' to ',a)

            numr = numr + 1

            write (   6, 8478) atetra(n1), aword(4)
            write (iout, 8478) atetra(n1), aword(4)

            atetra(n1)  = aword(4)
            atetras     = aword(4)
            latetras    = lword(4)

            call tetout (n1, n1, 'min')

          endif                       ! Replaced old stem with new stem.

  470   continue                      ! End of loop over tetrahedrons.

 8480   format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  No replacement possible:  ',a)

        if (numr .eq. 0) then
          write (   6, 8480) ainput, ninput, aword(3)
          write (iout, 8480) ainput, ninput, aword(3)
        endif

        go to 999

      endif                           ! Renamed an array of tetrahedrons.

c===============================================================================

c.... Input line:  rename brick BRSTEM BRSTEMR array

      if ((aword(2) .eq. 'brick') .or.
     &    (aword(2) .eq. 'br'))   then

        numr = 0

c....   Find all occurrances of old stem.

        do 500 n1 = 1, nbricks         ! Loop over bricks.

          nerr = 0

          call aptsubm (abrick(n1), 1, 24, iamax, idmax, iemax,
     &      '(', ',', ')', 16,
     &            astemx, istem, fstem, mstem, lastemx,
     &      nsub, asub,   isub,  fsub,  msub,  lsub, nerra)

          if (nerra .ne. 0) then
            write (   6, 8102) ainput, ninput, abrick(n1)
            write (iout, 8102) ainput, ninput, abrick(n1)
            go to 999
          endif

 8482     format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &            '  A subscript is not an integer:  ',a)

          do ns = 1, nsub
            if (msub(ns) .ne. 2) then
              write (   6, 8482) ainput, ninput, abrick(n1)
              write (iout, 8482) ainput, ninput, abrick(n1)
              go to 999
            endif
          enddo

          if (astemx .eq. astem) then ! Replace old stem with new stem.

            call aptsubi (astem,  lastem,  nsub, isub, 80,
     &                    '(', ',', ')', anamex, lanamex, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, abrick(n1)
              write (iout, 8102) ainput, ninput, abrick(n1)
              go to 999
            endif

            call aptsubi (astemr, lastemr, nsub, isub, 80,
     &                    '(', ',', ')', anamer, lanamer, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, abrick(n1)
              write (iout, 8102) ainput, ninput, abrick(n1)
              go to 999
            endif

            aword(3) = anamex
            lword(3) = lanamex

            aword(4) = anamer
            lword(4) = lanamer

c....       See if new brick already exists.

 8485       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  New brick name already exists:  ',a)

            do 480 n = 1, nbricks
              if (aword(4) .eq. abrick(n)) then
                write (   6, 8485) ainput, ninput, aword(4)
                write (iout, 8485) ainput, ninput, aword(4)
                go to 999
                go to 490
              endif
  480       continue
  490       continue

c....       Test for acceptable brick name.

 8488       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Unacceptable name.  Too dangerous:  ',a)

            if (mtype(4) .ne. 1) then
              write (   6, 8488) ainput, ninput, aword(4)
              write (iout, 8488) ainput, ninput, aword(4)
              go to 999
            endif

 8490       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Name of brick not from 1 to 24 characters:  ',a)

            if ((lword(4) .lt. 1) .or. (lword(4) .gt. lenmax)) then
              write (   6, 8490) ainput, ninput, aword(4)
              write (iout, 8490) ainput, ninput, aword(4)
              go to 999
            endif

            if (nerr .ne. 0) go to 500

 8492       format ('  Renamed brick ',a,' to ',a)

            numr = numr + 1

            write (   6, 8492) abrick(n1), aword(4)
            write (iout, 8492) abrick(n1), aword(4)

            abrick(n1)  = aword(4)
            abricks     = aword(4)
            labricks    = lword(4)

            call brout (n1, n1, 'min')

          endif                       ! Replaced old stem with new stem.

  500   continue                      ! End of loop over bricks.

 8495   format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  No replacement possible:  ',a)

        if (numr .eq. 0) then
          write (   6, 8495) ainput, ninput, aword(3)
          write (iout, 8495) ainput, ninput, aword(3)
        endif

        go to 999

      endif                           ! Renamed an array of bricks.

c===============================================================================

c.... Input vector:  rename vector VSTEM VSTEMR array

      if ((aword(2) .eq. 'vector') .or.
     &    (aword(2) .eq. 'vect')   .or.
     &    (aword(2) .eq. 'v'))     then

        numr = 0

c....   Find all occurrances of old stem.

        do 530 n1 = 1, nvectors       ! Loop over vectors.

          nerr = 0

          call aptsubm (avector(n1), 1, 24, iamax, idmax, iemax,
     &      '(', ',', ')', 16,
     &            astemx, istem, fstem, mstem, lastemx,
     &      nsub, asub,   isub,  fsub,  msub,  lsub, nerra)

          if (nerra .ne. 0) then
            write (   6, 8102) ainput, ninput, avector(n1)
            write (iout, 8102) ainput, ninput, avector(n1)
            go to 999
          endif

 8500     format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &            '  A subscript is not an integer:  ',a)

          do ns = 1, nsub
            if (msub(ns) .ne. 2) then
              write (   6, 8500) ainput, ninput, avector(n1)
              write (iout, 8500) ainput, ninput, avector(n1)
              go to 999
            endif
          enddo

          if (astemx .eq. astem) then ! Replace old stem with new stem.

            call aptsubi (astem,  lastem,  nsub, isub, 80,
     &                    '(', ',', ')', anamex, lanamex, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, avector(n1)
              write (iout, 8102) ainput, ninput, avector(n1)
              go to 999
            endif

            call aptsubi (astemr, lastemr, nsub, isub, 80,
     &                    '(', ',', ')', anamer, lanamer, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, avector(n1)
              write (iout, 8102) ainput, ninput, avector(n1)
              go to 999
            endif

            aword(3) = anamex
            lword(3) = lanamex

            aword(4) = anamer
            lword(4) = lanamer

c....       See if new vector already exists.

 8520       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  New vector name already exists:  ',a)

            do 510 n = 1, nvectors
              if (aword(4) .eq. avector(n)) then
                write (   6, 8520) ainput, ninput, aword(4)
                write (iout, 8520) ainput, ninput, aword(4)
                go to 999
                go to 520
              endif
  510       continue
  520       continue

c....       Test for acceptable vector name.

            if (mtype(4) .ne. 1) then
              write (   6, 8365) ainput, ninput, aword(4)
              write (iout, 8365) ainput, ninput, aword(4)
              go to 999
            endif

            if (aword(4) .eq. 'random') then
              write (   6, 8365) ainput, ninput, aword(4)
              write (iout, 8365) ainput, ninput, aword(4)
              go to 999
            endif

            if (aword(4) .eq. 'tol') then
              write (   6, 8370) ainput, ninput
              write (iout, 8370) ainput, ninput
              go to 999
            endif

 8525       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Name of vector not from 1 to 24 characters:  ',a)

            if ((lword(4) .lt. 1) .or. (lword(4) .gt. lenmax)) then
              write (   6, 8525) ainput, ninput, aword(4)
              write (iout, 8525) ainput, ninput, aword(4)
              go to 999
            endif

            if (nerr .ne. 0) go to 530

 8528       format ('  Renamed vector ',a,' to ',a)

            numr = numr + 1

            write (   6, 8528) avector(n1), aword(4)
            write (iout, 8528) avector(n1), aword(4)

            avector(n1)  = aword(4)
            avectors     = aword(4)
            lavector(n1) = lword(4)
            lavectors    = lword(4)

            call vout (n1, n1)

          endif                       ! Replaced old stem with new stem.

  530   continue                      ! End of loop over vectors.

 8530   format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  No replacement possible:  ',a)

        if (numr .eq. 0) then
          write (   6, 8530) ainput, ninput, aword(3)
          write (iout, 8530) ainput, ninput, aword(3)
        endif

        go to 999

      endif                           ! Renamed an array of vectors.

c===============================================================================

c.... Input line:  rename operator OPSTEM OPSTEMR array

      if ((aword(2) .eq. 'operator') .or.
     &    (aword(2) .eq. 'oper')     .or.
     &    (aword(2) .eq. 'op'))      then

        numr = 0

c....   Find all occurrances of old stem.

        do 560 n1 = 1, nopers         ! Loop over operators.

          nerr = 0

          call aptsubm (aoper(n1), 1, 24, iamax, idmax, iemax,
     &      '(', ',', ')', 16,
     &            astemx, istem, fstem, mstem, lastemx,
     &      nsub, asub,   isub,  fsub,  msub,  lsub, nerra)

          if (nerra .ne. 0) then
            write (   6, 8102) ainput, ninput, aoper(n1)
            write (iout, 8102) ainput, ninput, aoper(n1)
            go to 999
          endif

 8535     format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &            '  A subscript is not an integer:  ',a)

          do ns = 1, nsub
            if (msub(ns) .ne. 2) then
              write (   6, 8535) ainput, ninput, aoper(n1)
              write (iout, 8535) ainput, ninput, aoper(n1)
              go to 999
            endif
          enddo

          if (astemx .eq. astem) then ! Replace old stem with new stem.

            call aptsubi (astem,  lastem,  nsub, isub, 80,
     &                    '(', ',', ')', anamex, lanamex, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, aoper(n1)
              write (iout, 8102) ainput, ninput, aoper(n1)
              go to 999
            endif

            call aptsubi (astemr, lastemr, nsub, isub, 80,
     &                    '(', ',', ')', anamer, lanamer, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, aoper(n1)
              write (iout, 8102) ainput, ninput, aoper(n1)
              go to 999
            endif

            aword(3) = anamex
            lword(3) = lanamex

            aword(4) = anamer
            lword(4) = lanamer

c....       See if new operator already exists.

 8540       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  New operator name already exists:  ',a)

            do 540 n = 1, nopers
              if (aword(4) .eq. aoper(n)) then
                write (   6, 8540) ainput, ninput, aword(4)
                write (iout, 8540) ainput, ninput, aword(4)
                go to 999
                go to 550
              endif
  540       continue
  550       continue

c....       Test for acceptable operator name.

 8545       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Unacceptable name.  Too dangerous:  ',a)

            if (mtype(4) .ne. 1) then
              write (   6, 8545) ainput, ninput, aword(4)
              write (iout, 8545) ainput, ninput, aword(4)
              go to 999
            endif

 8550       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Name of operator not from 1 to 24 characters:  ',a)

            if ((lword(4) .lt. 1) .or. (lword(4) .gt. lenmax)) then
              write (   6, 8550) ainput, ninput, aword(4)
              write (iout, 8550) ainput, ninput, aword(4)
              go to 999
            endif

            if (nerr .ne. 0) go to 560

 8555       format ('  Renamed operator ',a,' to ',a)

            numr = numr + 1

            write (   6, 8555) aoper(n1), aword(4)
            write (iout, 8555) aoper(n1), aword(4)

            aoper(n1)  = aword(4)
            aopers     = aword(4)
            laopers    = lword(4)

            call opout (n1, n1, 'min')

          endif                       ! Replaced old stem with new stem.

  560   continue                      ! End of loop over operators.

 8560   format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  No replacement possible:  ',a)

        if (numr .eq. 0) then
          write (   6, 8560) ainput, ninput, aword(3)
          write (iout, 8560) ainput, ninput, aword(3)
        endif

        go to 999

      endif                           ! Renamed an array of operators.

c===============================================================================

c.... Input line:  rename plane     PLSTEM PLSTEMR array
c.... Input line:  rename sphere    SPHSTEM SPHSTEMR array
c.... Input line:  rename cylinder  CYLSTEM CYLSTEMR array
c.... Input line:  rename cone      CNSTEM CNSTEMR array
c.... Input line:  rename ellipsoid ELSTEM ELSTEMR array
c.... Input line:  rename axisym    AXSTEM AXSTEMR array
c.... Input line:  rename quadric   QSTEM QSTEMR array

      if ((aword(2) .eq. 'quadric')   .or.
     &    (aword(2) .eq. 'quad')      .or.
     &    (aword(2) .eq. 'q')         .or.
     &    (aword(2) .eq. 'plane')     .or.
     &    (aword(2) .eq. 'pl')        .or.
     &    (aword(2) .eq. 'sphere')    .or.
     &    (aword(2) .eq. 'sph')       .or.
     &    (aword(2) .eq. 'cylinder')  .or.
     &    (aword(2) .eq. 'cyl')       .or.
     &    (aword(2) .eq. 'cone')      .or.
     &    (aword(2) .eq. 'hyperb')    .or.
     &    (aword(2) .eq. 'saddle')    .or.
     &    (aword(2) .eq. 'ellipsoid') .or.
     &    (aword(2) .eq. 'ell')       .or.
     &    (aword(2) .eq. 'el')        .or.
     &    (aword(2) .eq. 'axisym'))   then

        numr = 0

c....   Find all occurrances of old stem.

        do 610 n1 = 1, nquads         ! Loop over quadrics.

          nerr = 0

          call aptsubm (aquad(n1), 1, 24, iamax, idmax, iemax,
     &      '(', ',', ')', 16,
     &            astemx, istem, fstem, mstem, lastemx,
     &      nsub, asub,   isub,  fsub,  msub,  lsub, nerra)

          if (nerra .ne. 0) then
            write (   6, 8102) ainput, ninput, aquad(n1)
            write (iout, 8102) ainput, ninput, aquad(n1)
            go to 999
          endif

 8565     format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &            '  A subscript is not an integer:  ',a)

          do ns = 1, nsub
            if (msub(ns) .ne. 2) then
              write (   6, 8565) ainput, ninput, aquad(n1)
              write (iout, 8565) ainput, ninput, aquad(n1)
              go to 999
            endif
          enddo

          if (astemx .eq. astem) then ! Replace old stem with new stem.

      call qutype (aword(2), n1, 1, nerrq)

            if (nerrq .ne. 0) go to 999

            call aptsubi (astem,  lastem,  nsub, isub, 80,
     &                    '(', ',', ')', anamex, lanamex, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, aquad(n1)
              write (iout, 8102) ainput, ninput, aquad(n1)
              go to 999
            endif

            call aptsubi (astemr, lastemr, nsub, isub, 80,
     &                    '(', ',', ')', anamer, lanamer, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, aquad(n1)
              write (iout, 8102) ainput, ninput, aquad(n1)
              go to 999
            endif

            aword(3) = anamex
            lword(3) = lanamex

            aword(4) = anamer
            lword(4) = lanamer

c....       See if new quadric already exists.

 8575       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  New quadric name already exists:  ',a)

            do 570 n = 1, nquads
              if (aword(4) .eq. aquad(n)) then
                write (   6, 8575) ainput, ninput, aword(4)
                write (iout, 8575) ainput, ninput, aword(4)
                go to 999
                go to 580
              endif
  570       continue
  580       continue

c....       Test for acceptable quadric name.

 8580       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Unacceptable name.  Too dangerous:  ',a)

            if (mtype(4) .ne. 1) then
              write (   6, 8580) ainput, ninput, aword(4)
              write (iout, 8580) ainput, ninput, aword(4)
              go to 999
            endif

 8582       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Name of quadric not from 1 to 24 characters:  ',a)

            if ((lword(4) .lt. 1) .or. (lword(4) .gt. lenmax)) then
              write (   6, 8582) ainput, ninput, aword(4)
              write (iout, 8582) ainput, ninput, aword(4)
              go to 999
            endif

            if (nerr .ne. 0) go to 610

 8570       format ('  Renamed quadric ',a,' to ',a)

            numr = numr + 1

            write (   6, 8570) aquad(n1), aword(4)
            write (iout, 8570) aquad(n1), aword(4)

            aquad(n1)  = aword(4)
            aquads     = aword(4)

            laquad(n1) = lword(4)
            laquads    = lword(4)

            if  (aword(2) .eq. 'cone')      then
              acones     = aword(4)
              lacones    = lword(4)
            endif

            if ((aword(2) .eq. 'cylinder')  .or.
     &      (aword(2) .eq. 'cyl'))      then
              acyls      = aword(4)
              lacyls     = lword(4)
            endif

            if ((aword(2) .eq. 'ellipsoid') .or.
     &      (aword(2) .eq. 'ellipse')   .or.
     &      (aword(2) .eq. 'ell')       .or.
     &      (aword(2) .eq. 'el'))       then
              aellips    = aword(4)
              laellips   = lword(4)
            endif

            if ((aword(2) .eq. 'hyperb')    .or.
     &      (aword(2) .eq. 'saddle'))   then
              ahyperbs   = aword(4)
              lahyperbs  = lword(4)
            endif

            if ((aword(2) .eq. 'plane')     .or.
     &      (aword(2) .eq. 'pl'))       then
              aplanes    = aword(4)
              laplanes   = lword(4)
            endif

            if ((aword(2) .eq. 'sphere')    .or.
     &      (aword(2) .eq. 'sph'))      then
              aspheres   = aword(4)
              laspheres  = lword(4)
            endif

            if  (aword(2) .eq. 'axisym')    then
              axisyms    = aword(4)
              laxisyms   = lword(4)
            endif

            if (asym(n1) .eq. 'axisymmetric') then
              axisyms  = aword(4)
              laxisyms = lword(4)
            endif

c....       Rename quadric used to define a zone.

            do 600 nz = 1, nzones
              do 590 nq = 1, nzquads(nz)
                if (azquad(nq,nz) .eq. aword(3) ) then
                  azquad(nq,nz)  = aword(4)
                  lazquad(nq,nz) = lword(4)
                endif
  590         continue
  600       continue

            if ((aword(2) .eq. 'plane') .or.
     &          (aword(2) .eq. 'pl'))  then
              call plout (n1, n1)
            else
              call quout (n1, n1, 'min', nerrq)
            endif

          endif                       ! Replaced old stem with new stem.

  610   continue                      ! End of loop over quadrics.

 8590   format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  No replacement possible:  ',a)

        if (numr .eq. 0) then
          write (   6, 8590) ainput, ninput, aword(3)
          write (iout, 8590) ainput, ninput, aword(3)
        endif

        go to 999

      endif                           ! Renamed an array of quadrics.

c===============================================================================

c.... Input line:  rename zone ZSTEM ZSTEMR array

      if ((aword(2) .eq. 'zone') .or.
     &    (aword(2) .eq. 'zn')   .or.
     &    (aword(2) .eq. 'z'))   then

        numr = 0

c....   Find all occurrances of old stem.

        do 640 n1 = 1, nzones         ! Loop over zones.

          nerr = 0

          call aptsubm (azone(n1), 1, 24, iamax, idmax, iemax,
     &      '(', ',', ')', 16,
     &            astemx, istem, fstem, mstem, lastemx,
     &      nsub, asub,   isub,  fsub,  msub,  lsub, nerra)

          if (nerra .ne. 0) then
            write (   6, 8102) ainput, ninput, azone(n1)
            write (iout, 8102) ainput, ninput, azone(n1)
            go to 999
          endif

 8640     format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &            '  A subscript is not an integer:  ',a)

          do ns = 1, nsub
            if (msub(ns) .ne. 2) then
              write (   6, 8640) ainput, ninput, azone(n1)
              write (iout, 8640) ainput, ninput, azone(n1)
              go to 999
            endif
          enddo

          if (astemx .eq. astem) then ! Replace old stem with new stem.

            call aptsubi (astem,  lastem,  nsub, isub, 80,
     &                    '(', ',', ')', anamex, lanamex, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, azone(n1)
              write (iout, 8102) ainput, ninput, azone(n1)
              go to 999
            endif

            call aptsubi (astemr, lastemr, nsub, isub, 80,
     &                    '(', ',', ')', anamer, lanamer, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, azone(n1)
              write (iout, 8102) ainput, ninput, azone(n1)
              go to 999
            endif

            aword(3) = anamex
            lword(3) = lanamex

            aword(4) = anamer
            lword(4) = lanamer

c....       See if new zone already exists.

 8605       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  New zone name already exists:  ',a)

            do 620 n = 1, nzones
              if (aword(4) .eq. azone(n)) then
                write (   6, 8605) ainput, ninput, aword(4)
                write (iout, 8605) ainput, ninput, aword(4)
                go to 999
                go to 630
              endif
  620       continue
  630       continue

c....       Test for acceptable zone name.

 8610       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Unacceptable name.  Too dangerous:  ',a)

            if (mtype(4) .ne. 1) then
              write (   6, 8610) ainput, ninput, aword(4)
              write (iout, 8610) ainput, ninput, aword(4)
              go to 999
            endif

 8615       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Name of zone not from 1 to 24 characters:  ',a)

            if ((lword(4) .lt. 1) .or. (lword(4) .gt. lenmax)) then
              write (   6, 8615) ainput, ninput, aword(4)
              write (iout, 8615) ainput, ninput, aword(4)
              go to 999
            endif

            if (nerr .ne. 0) go to 640

 8622       format ('  Renamed zone ',a,' to ',a)

            numr = numr + 1

            write (   6, 8622) azone(n1), aword(4)
            write (iout, 8622) azone(n1), aword(4)

            azone(n1)  = aword(4)
            azones     = aword(4)

            call zout (n1, n1, 'min')

          endif                       ! Replaced old stem with new stem.

  640   continue                      ! End of loop over zones.

 8625   format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  No replacement possible:  ',a)

        if (numr .eq. 0) then
          write (   6, 8625) ainput, ninput, aword(3)
          write (iout, 8625) ainput, ninput, aword(3)
        endif

        go to 999

      endif                           ! Renamed an array of zones.

c===============================================================================

c.... Input line:  rename bin BISTEM BISTEMR array

      if (aword(2) .eq. 'bin') then

        numr = 0

c....   Find all occurrances of old stem.

        do 680 n1 = 1, nbins         ! Loop over bins.

          nerr = 0

          call aptsubm (abin(n1), 1, 24, iamax, idmax, iemax,
     &      '(', ',', ')', 16,
     &            astemx, istem, fstem, mstem, lastemx,
     &      nsub, asub,   isub,  fsub,  msub,  lsub, nerra)

          if (nerra .ne. 0) then
            write (   6, 8102) ainput, ninput, abin(n1)
            write (iout, 8102) ainput, ninput, abin(n1)
            go to 999
          endif

 8628     format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &            '  A subscript is not an integer:  ',a)

          do ns = 1, nsub
            if (msub(ns) .ne. 2) then
              write (   6, 8628) ainput, ninput, abin(n1)
              write (iout, 8628) ainput, ninput, abin(n1)
              go to 999
            endif
          enddo

          if (astemx .eq. astem) then ! Replace old stem with new stem.

            call aptsubi (astem,  lastem,  nsub, isub, 80,
     &                    '(', ',', ')', anamex, lanamex, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, abin(n1)
              write (iout, 8102) ainput, ninput, abin(n1)
              go to 999
            endif

            call aptsubi (astemr, lastemr, nsub, isub, 80,
     &                    '(', ',', ')', anamer, lanamer, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, abin(n1)
              write (iout, 8102) ainput, ninput, abin(n1)
              go to 999
            endif

            aword(3) = anamex
            lword(3) = lanamex

            aword(4) = anamer
            lword(4) = lanamer

c....       See if new bin already exists.

 8650       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  New bin name already exists:  ',a)

            do 650 n = 1, nbins
              if (aword(4) .eq. abin(n)) then
                write (   6, 8650) ainput, ninput, aword(4)
                write (iout, 8650) ainput, ninput, aword(4)
                go to 999
                go to 660
              endif
  650       continue
  660       continue

c....       Test for acceptable bin name.

 8660       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Unacceptable name.  Too dangerous:  ',a)

            if (mtype(4) .ne. 1) then
              write (   6, 8660) ainput, ninput, aword(4)
              write (iout, 8660) ainput, ninput, aword(4)
              go to 999
            endif

 8680       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Name of bin not from 1 to 24 characters:  ',a)

            if ((lword(4) .lt. 1) .or. (lword(4) .gt. lenmax)) then
              write (   6, 8680) ainput, ninput, aword(4)
              write (iout, 8680) ainput, ninput, aword(4)
              go to 999
            endif

            if (nerr .ne. 0) go to 680

 8620       format ('  Renamed bin ',a,' to ',a)

            numr = numr + 1

            write (   6, 8620) abin(n1), aword(4)
            write (iout, 8620) abin(n1), aword(4)

            abin(n1) = aword(4)
            abins    = aword(4)
            labins   = lword(4)

c....   Rename same bin in definitions of pdfs.

            do 670 npx = 1, npdfxs
              if (abinx(npx) .eq. aword(3)) abinx(npx) = aword(4)
  670       continue

            call binout (n1, n1, 'min')

          endif                       ! Replaced old stem with new stem.

  680   continue                      ! End of loop over bins.

 8630   format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  No replacement possible:  ',a)

        if (numr .eq. 0) then
          write (   6, 8630) ainput, ninput, aword(3)
          write (iout, 8630) ainput, ninput, aword(3)
        endif

        go to 999

      endif                           ! Renamed an array of bins.

c===============================================================================

c.... Input line:  rename pdf PDFSTEM PDFSTEMR array

      if (aword(2) .eq. 'pdf') then

        numr = 0

c....   Find all occurrances of old stem.

        do 720 n1 = 1, npdfs         ! Loop over bins.

          nerr = 0

          call aptsubm (apdf(n1), 1, 24, iamax, idmax, iemax,
     &      '(', ',', ')', 16,
     &            astemx, istem, fstem, mstem, lastemx,
     &      nsub, asub,   isub,  fsub,  msub,  lsub, nerra)

          if (nerra .ne. 0) then
            write (   6, 8102) ainput, ninput, apdf(n1)
            write (iout, 8102) ainput, ninput, apdf(n1)
            go to 999
          endif

 8688     format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &            '  A subscript is not an integer:  ',a)

          do ns = 1, nsub
            if (msub(ns) .ne. 2) then
              write (   6, 8688) ainput, ninput, apdf(n1)
              write (iout, 8688) ainput, ninput, apdf(n1)
              go to 999
            endif
          enddo

          if (astemx .eq. astem) then ! Replace old stem with new stem.

            call aptsubi (astem,  lastem,  nsub, isub, 80,
     &                    '(', ',', ')', anamex, lanamex, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, apdf(n1)
              write (iout, 8102) ainput, ninput, apdf(n1)
              go to 999
            endif

            call aptsubi (astemr, lastemr, nsub, isub, 80,
     &                    '(', ',', ')', anamer, lanamer, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, apdf(n1)
              write (iout, 8102) ainput, ninput, apdf(n1)
              go to 999
            endif

            aword(3) = anamex
            lword(3) = lanamex

            aword(4) = anamer
            lword(4) = lanamer

c....       See if new pdf already exists.

 8690       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  New pdf name already exists:  ',a)

            do 690 n = 1, npdfs
              if (aword(4) .eq. apdf(n)) then
                write (   6, 8690) ainput, ninput, aword(4)
                write (iout, 8690) ainput, ninput, aword(4)
                go to 999
                go to 700
              endif
  690       continue
  700       continue

c....       Test for acceptable pdf name.

 8700       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Unacceptable name.  Too dangerous:  ',a)

            if (mtype(4) .ne. 1) then
              write (   6, 8700) ainput, ninput, aword(4)
              write (iout, 8700) ainput, ninput, aword(4)
              go to 999
            endif

 8720       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Name of pdf not from 1 to 24 characters:  ',a)

            if ((lword(4) .lt. 1) .or. (lword(4) .gt. lenmax)) then
              write (   6, 8720) ainput, ninput, aword(4)
              write (iout, 8720) ainput, ninput, aword(4)
              go to 999
            endif

            if (nerr .ne. 0) go to 720

 8730       format ('  Renamed pdf ',a,' to ',a)

            numr = numr + 1

            write (   6, 8730) apdf(n1), aword(4)
            write (iout, 8730) apdf(n1), aword(4)

            apdf(n1) = aword(4)
            apdfs    = aword(4)
            lapdfs   = lword(4)

c....       Rename same pdf in table of bins and pdf pairs.

            do 710 npx = 1, npdfxs
              if (apdfx(npx) .eq. aword(3)) then
                apdfx(npx) = aword(4)
              endif
  710       continue

            call pdfout (n1, n1, 'min')

          endif                       ! Replaced old stem with new stem.

  720   continue                      ! End of loop over pdfs.

 8750   format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  No replacement possible:  ',a)

        if (numr .eq. 0) then
          write (   6, 8750) ainput, ninput, aword(3)
          write (iout, 8750) ainput, ninput, aword(3)
        endif

        go to 999

      endif                           ! Renamed an array of pdfs.

c===============================================================================

c.... Input line:  rename alias ASTEM ASTEMR array

      if ((aword(2) .eq. 'alias') .or.
     &    (aword(2) .eq. 'al')    .or.
     &    (aword(2) .eq. 'a'))    then

        numr = 0

c....   Find all occurrances of old stem.

        do 750 n1 = 1, nalias         ! Loop over aliases.

          nerr = 0

          call aptsubm (alias1(n1), 1, 24, iamax, idmax, iemax,
     &      '(', ',', ')', 16,
     &            astemx, istem, fstem, mstem, lastemx,
     &      nsub, asub,   isub,  fsub,  msub,  lsub, nerra)

          if (nerra .ne. 0) then
            write (   6, 8102) ainput, ninput, alias1(n1)
            write (iout, 8102) ainput, ninput, alias1(n1)
            go to 999
          endif

 8670     format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &            '  A subscript is not an integer:  ',a)

          do ns = 1, nsub
            if (msub(ns) .ne. 2) then
              write (   6, 8670) ainput, ninput, alias1(n1)
              write (iout, 8670) ainput, ninput, alias1(n1)
              go to 999
            endif
          enddo

          if (astemx .eq. astem) then ! Replace old stem with new stem.

            call aptsubi (astem,  lastem,  nsub, isub, 80,
     &                    '(', ',', ')', anamex, lanamex, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, alias1(n1)
              write (iout, 8102) ainput, ninput, alias1(n1)
              go to 999
            endif

            call aptsubi (astemr, lastemr, nsub, isub, 80,
     &                    '(', ',', ')', anamer, lanamer, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, alias1(n1)
              write (iout, 8102) ainput, ninput, alias1(n1)
              go to 999
            endif

            aword(3) = anamex
            lword(3) = lanamex

            aword(4) = anamer
            lword(4) = lanamer

c....       See if new alias already exists.

 8740       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  New alias name already exists:  ',a)

            do 730 n = 1, nalias
              if (aword(4) .eq. alias1(n)) then
                write (   6, 8740) ainput, ninput, aword(4)
                write (iout, 8740) ainput, ninput, aword(4)
                go to 999
                go to 740
              endif
  730       continue
  740       continue

c....       Test for acceptable alias name.

 8745       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Unacceptable name.  Too dangerous:  ',a)

            if (mtype(4) .ne. 1) then
              write (   6, 8745) ainput, ninput, aword(4)
              write (iout, 8745) ainput, ninput, aword(4)
              go to 999
            endif

 8748       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Name of alias not from 1 to 24 characters:  ',a)

            if ((lword(4) .lt. 1) .or. (lword(4) .gt. lenmax)) then
              write (   6, 8748) ainput, ninput, aword(4)
              write (iout, 8748) ainput, ninput, aword(4)
              go to 999
            endif

            if (nerr .ne. 0) go to 750

 8755       format ('  Renamed alias ',a,' to ',a)

            numr = numr + 1

            write (   6, 8755) alias1(n1), aword(4)
            write (iout, 8755) alias1(n1), aword(4)

            alias1(n1) = aword(4)
            alias1s    = aword(4)
            lalias1s   = lword(4)

            call alout (n1, n1, 'min')

          endif                       ! Replaced old stem with new stem.

  750   continue                      ! End of loop over aliases.

 8758   format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  No replacement possible:  ',a)

        if (numr .eq. 0) then
          write (   6, 8758) ainput, ninput, aword(3)
          write (iout, 8758) ainput, ninput, aword(3)
        endif

        go to 999

      endif                           ! Renamed an array of aliases.

c===============================================================================

c.... Input line:  rename marker AMSTEM AMSTEMR array

      if ((aword(2) .eq. 'marker') .or.
     &    (aword(2) .eq. 'mark')   .or.
     &    (aword(2) .eq. 'm'))     then

 8760 format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named marker does not exist:  ',a1)

c....   See if original marker exists.

        do 760 n = 1, nmarks
          if (aword(3) .eq. amark(n)) then
            n1 = n
            go to 770
          endif
  760   continue
        write (   6, 8760) ainput, ninput, aword(3)
        write (iout, 8760) ainput, ninput, aword(3)
        go to 999

  770   continue

c....   See if new marker exists.

 8770 format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  New marker name already exists:  ',a1)

        do 780 n = 1, nmarks
          if (aword(4) .eq. amark(n)) then
            write (   6, 8770) ainput, ninput, aword(4)
            write (iout, 8770) ainput, ninput, aword(4)
            go to 999
          endif
  780   continue

 8762 format ('  Renamed marker ',a1,' to ',a1)

        write (   6, 8762) amark(n1), aword(4)
        write (iout, 8762) amark(n1), aword(4)

        amark(n1) = aword(4)

        call markout (n1, n1)

        go to 999

      endif                           ! Renamed one marker.

c===============================================================================

c.... Input line:  rename symbol SYMBSTEM SYMBSTEMR array

      if ((aword(2) .eq. 'symbol') .or.
     &    (aword(2) .eq. 'symb')   .or.
     &    (aword(2) .eq. 'sym'))   then

        numr = 0

c....   Find all occurrances of old stem.

        do 810 n1 = 1, nsymbs         ! Loop over symbols.

          nerr = 0

          call aptsubm (asymbl(n1), 1, 24, iamax, idmax, iemax,
     &      '(', ',', ')', 16,
     &            astemx, istem, fstem, mstem, lastemx,
     &      nsub, asub,   isub,  fsub,  msub,  lsub, nerra)

          if (nerra .ne. 0) then
            write (   6, 8102) ainput, ninput, asymbl(n1)
            write (iout, 8102) ainput, ninput, asymbl(n1)
            go to 999
          endif

 8765     format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &            '  A subscript is not an integer:  ',a)

          do ns = 1, nsub
            if (msub(ns) .ne. 2) then
              write (   6, 8765) ainput, ninput, asymbl(n1)
              write (iout, 8765) ainput, ninput, asymbl(n1)
              go to 999
            endif
          enddo

          if (astemx .eq. astem) then ! Replace old stem with new stem.

            call aptsubi (astem,  lastem,  nsub, isub, 80,
     &                    '(', ',', ')', anamex, lanamex, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, asymbl(n1)
              write (iout, 8102) ainput, ninput, asymbl(n1)
              go to 999
            endif

            call aptsubi (astemr, lastemr, nsub, isub, 80,
     &                    '(', ',', ')', anamer, lanamer, nerra)

            if (nerra .ne. 0) then
              write (   6, 8102) ainput, ninput, asymbl(n1)
              write (iout, 8102) ainput, ninput, asymbl(n1)
              go to 999
            endif

            aword(3) = anamex
            lword(3) = lanamex

            aword(4) = anamer
            lword(4) = lanamer

c....       See if new symbol already exists.

 8775       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  New symbol name already exists:  ',a)

            do 790 n = 1, nsymbs
              if (aword(4) .eq. asymbl(n)) then
                write (   6, 8775) ainput, ninput, aword(4)
                write (iout, 8775) ainput, ninput, aword(4)
                go to 999
                go to 800
              endif
  790       continue
  800       continue

c....       Test for acceptable symbol name.

 8780       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Unacceptable name.  Too dangerous:  ',a)

            if (mtype(4) .ne. 1) then
              write (   6, 8780) ainput, ninput, aword(4)
              write (iout, 8780) ainput, ninput, aword(4)
              go to 999
            endif

 8785       format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Name of symbol not from 1 to 24 characters:  ',a)

            if ((lword(4) .lt. 1) .or. (lword(4) .gt. lenmax)) then
              write (   6, 8785) ainput, ninput, aword(4)
              write (iout, 8785) ainput, ninput, aword(4)
              go to 999
            endif

            if (nerr .ne. 0) go to 810

 8790       format ('  Renamed symbol ',a,' to ',a)

            numr = numr + 1

            write (   6, 8790) asymbl(n1), aword(4)
            write (iout, 8790) asymbl(n1), aword(4)

            asymbl(n1) = aword(4)
            asymbls    = aword(4)
            lasymls    = lword(4)

            call symbdisp (n1)

          endif                       ! Replaced old stem with new stem.

  810   continue                      ! End of loop over symbols.

 8810   format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  No replacement possible:  ',a)

        if (numr .eq. 0) then
          write (   6, 8810) ainput, ninput, aword(3)
          write (iout, 8810) ainput, ninput, aword(3)
        endif

        go to 999

      endif                           ! Renamed an array of symbols.

c===============================================================================

c.... Input line:  (not recognized)

 
 8820 format ('  renamea ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Specified object type does not exist:  ',a24)

      write (   6, 8820) ainput, ninput, aword(2)
      write (iout, 8820) ainput, ninput, aword(2)

  999 return

c.... End of subroutine renamea.      (+1 line)
      end
