      subroutine renamex

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  renamex  Updated     2006 January 6 17:00.
c               renamex  Originated  1993 November 9 15:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To rename aliases, markers, variables, vectors, operators,
c               points, clusters, lines, triangles, annular disks,
c               regular polygons, planes, spheres, circular cylinders,
c               circular cones, hyperbolic paraboloids, ellipsoids, axisymmetric
c               quadric surfaces, general quadric surfaces, regular polyhedrons,
c               tetrahedrons, bricks, zones, bins, probability distribution
c               functions (pdfs) and symbols.
c
c     Logic:    Entry points:  renamex.
c renamex       Called by:  helpall, main, switch* .
c
c renamex       Calls:  alout, aptchfp, aptsubm, bigout, binout, brout, clout,
c               dkout, incrsubs, lout, markout, namechk, opout, pdfout, pgout,
c               phout, plout, pout, ptfind, quout, qutype, renamea, symbdisp,
c               tetout, trout, varout, vectfind, vout, zout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables 

      character*64 asrce              ! Characters $, %, @, A-Z, a-z.
      character*24 awordx             ! Temporary value of aword(4).
      character*24 awordy             ! Temporary value of aword(4).
      character*1  afirst             ! First character of point name.

c.... Input line:  rename [help]

 8110 format ('  Command "rename":  acceptable input lines include:' /
     &  '  "rename [help]"  Display this message.' //
     &  '  "rename alias    ANAME   ANAME2" ',
     &  '   Rename alias    ANAME   to ANAME2.'/
     &  '  "rename marker   SYNNAME SYNNAME2" ',
     &  ' Rename marker   SYNNAME to SYNNAME2.'/
     &  '  "rename variable VARNAME VARNAME2" ',
     &  ' Rename variable VARNAME to VARNAME2.'/
     &  '  "rename big      BIGNAME BIGNAME2" ',
     &  ' Rename big integer BIGNAME to BIGNAME2.' /
     &  '  "rename point    PNAME   PNAME2" ',
     &  '   Rename point    PNAME   to PNAME2.'/
     &  '  "rename cluster  CLNAME  CLNAME2"',
     &  '   Rename cluster  CLNAME  to CLNAME2.'/
     &  '  "rename line     ALNAME  ALNAME2"',
     &  '   Rename line     ALNAME  to ALNAME2.'/
     &  '  "rename vector   VNAME   VNAME2" ',
     &  '   Rename vector   VNAME   to VNAME2.'/
     &  '  "rename operator OPNAME  OPNAME2"',
     &  '   Rename operator OPNAME  to OPNAME2.' /
     &  '  "rename triangle TRNAME  TRNAME2"',
     &  '   Rename triangle TRNAME  to TRNAME2.')

 8112 format (
     &  '  "rename polygon  PGNAME  PGNAME2"',
     &  '   Rename regular polygon PGNAME to PGNAME2.'/
     &  '  "rename disk     DKNAME  DKNAME2"',
     &  '   Rename annular disk DKNAME to DKNAME2.'/
     &  '  "rename plane    PLNAME  PLNAME2"',
     &  '   Rename plane    PLNAME  to PLNAME2.' /
     &  '  "rename sphere   SPHNAME SPHNAME2" ',
     &  ' Rename sphere   SPHNAME to SPHNAME2.' /
     &  '  "rename cylinder CYLNAME CYLNAME2" ',
     &  ' Rename cylinder CYLNAME to CYLNAME2.' /
     &  '  "rename cone     CNNAME  CNNAME2"',
     &  '   Rename cone     CNNAME  to CNNAME2.' /
     &  '  "rename ellipsoid ELNAME ELNAME2"',
     &  '   Rename ellipsoid ELNAME to ELNAME2.' /
     &  '  "rename hyperb   HPNAME  HPNAME2"',
     &  '   Rename hyperbolic paraboloid  HPNAME' /
     &  '                                      to HPNAME2.' /
     &  '  "rename axisym   AXNAME  AXNAME2"',
     &  '   Rename axisymmetric quadric AXNAME' /
     &  '                                      to AXNAME2.' /
     &  '  "rename quadric  QNAME   QNAME2"',
     &  '    Rename quadric  QNAME   to QNAME2.')

 8114 format (
     &  '  "rename polyhedron PHNAME PHNAME2"',
     &  '  Rename regular polyhedron PHNAME' /
     &  '                                      to PHNAME2.' /
     &  '  "rename tetrahedron TETNAME TETNAME2"',
     &  ' Rename tetrahedron TETNAME to TETNAME2.'/
     &  '  "rename brick    BRNAME  BRNAME2"',
     &  '   Rename brick    BRNAME  to BRNAME2.'/
     &  '  "rename zone     ZNAME   ZNAME2"',
     &  '    Rename zone     ZNAME   to ZNAME2.' /
     &  '  "rename bin      BINAME  BINAME2"',
     &  '   Rename probability bin BINAME to BINAME2.' /
     &  '  "rename pdf      PDFNAME PDFNAME2"',
     &  '  Rename pdf      PDFNAME to PDFNAME2.' /
     &  '  "rename symbol   SYMBNAME SYMBNAME2"',
     &  '  Rename symbol SYMBNAME to SYMBNAME2.' /
     &  '  "rename OBJTYPE  OBJNAME OBJNAME2"',
     &  '  Rename OBJTYPE  OBJNAME to OBJNAME2,' /
     &  '    where OBJTYPE is any of the object types used above.' //
     &  '  Note:  use +/- for OBJNAME2 to incr/decr the last OBJTYPE',
     &  ' name.' //
     &  '  Type <return> for more help:  ',$)

 8115 format (a)

 8116 format (/
     &  '  "rename OBJTYPE  OBJSTEM OBJSTEM2 array"' /
     &  '    Rename OBJTYPE stem name OBJSTEM to OBJSTEM2,' /
     &  '    where OBJTYPE is any of the object types used above.' //
     &  '  Synonyms:  [alias, al, a], [brick, br], [cluster, cl],',
     &  ' [cylinder, cyl],' /
     &  '    [disk, disc, dk], [ellipsoid, ellipse, ell, el],' /
     &  '    [help, h], [hyperb, saddle], [line, ln, l],',
     &  ' [marker, mark, m],' /
     &  '    [operator, oper, op, o], [plane, pl],' /
     &  '    [point, pnt, pt, p], [polygon, polyg, pg],' /
     &  '    [polyhedron, polyh, ph], [quadric, quad, q],',
     &  ' [rename, switch, sw],' /
     &  '    [sphere, sph], [symbol, symb, sym],',
     &  ' [tetrahedron, tetra, tet],' /
     &  '    [triangle, tri], [variable, var, set, s],',
     &  ' [vector, vect, v],' /
     &  '    [zone, zn, z].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        write (   6, 8114)
        write (iout, 8114)
        if (ainput .eq. 'tty') then
          read  (   5, 8115) ans
        endif
        write (   6, 8116)
        write (iout, 8116)
        go to 999
      endif

c.... Test for the wrong number of words on "rename" line.

 8120 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 4 or 5 words on "rename" line:  ', i3)

      if ((nword .ne. 4) .and. (nword .ne. 5)) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        go to 999
      endif

c.... Test for the lengths of the geometric object names.

 8125 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of object not from 1 to ',i3,' characters:  ',a)

      lenmax = 24

      if ((aword(2) .eq. 'marker') .or.
     &    (aword(2) .eq. 'mark')   .or.
     &    (aword(2) .eq. 'm'))     then
        lenmax = 1
      endif

      do 125 n = 3, 4
        if ((lword(n) .lt. 1) .or. (lword(n) .gt. lenmax)) then
          write (   6, 8125) ainput, ninput, lenmax, aword(n)
          write (iout, 8125) ainput, ninput, lenmax, aword(n)
          go to 999
        endif
  125 continue

c.... Test to see if geometric object names are different.

 8126 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Source and sink objects have same name:  ',a)

      if (aword(3) .eq. aword(4)) then
        write (   6, 8126) ainput, ninput, aword(3)
        write (iout, 8126) ainput, ninput, aword(3)
        go to 999
      endif

c.... See if new name is acceptable.

 8905 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Name too dangerous to use:  ',a)

      call namechk (aword(4), lword(4), nerrchk)

      if (nerrchk .ne. 0) then
        write (   6, 8905) ainput, ninput, aword(4)
        write (iout, 8905) ainput, ninput, aword(4)
        go to 999
      endif

c.... Test for replacing only the stems of the geometric objects.

 8122 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word 5 on "rename" line is not "array":  ', a)

      if (nword .eq. 5) then
        if (aword(5) .eq. 'array') then

          if (aword(2) .ne. 'big') then
            call renamea

            go to 999
          endif

        else
          write (   6, 8122) ainput, ninput, aword(5)
          write (iout, 8122) ainput, ninput, aword(5)
          go to 999
        endif
      endif

c===============================================================================

c.... Input line:  rename variable VARNAME VARNAME2

      if ((aword(2) .eq. 'variable') .or.
     &    (aword(2) .eq. 'var')      .or.
     &    (aword(2) .eq. 'set')      .or.
     &    (aword(2) .eq. 's'))       then

        nerr = 0

c....   See if original variable exists.

 8140   format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Named variable does not exist:  ',a)

        do 130 n = 1, nvars
          if (aword(3) .eq. avarbl(n)) then
            n1 = n
            go to 140
          endif
  130   continue
        write (   6, 8140) ainput, ninput, aword(3)
        write (iout, 8140) ainput, ninput, aword(3)
        go to 999

c....   See if last variable name should be incremented and used.

 8128   format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Name of last ',a24,' can not be incremented:  ',a)
 8129   format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Name of last ',a24,' can not be decremented:  ',a)

  140   if (aword(4) .eq. '+') then
          aword(4) = avarbls
          lword(4) = lavarbls
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), avarbls
            write (iout, 8128) ainput, ninput, aword(2), avarbls
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = avarbls
          lword(4) = lavarbls
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), avarbls
            write (iout, 8129) ainput, ninput, aword(2), avarbls
            go to 999
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new variable already exists.

 8150   format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  New variable name already exists:  ',a)

        do 150 n = 1, nvars
          if (aword(4) .eq. avarbl(n)) then
            write (   6, 8150) ainput, ninput, aword(4)
            write (iout, 8150) ainput, ninput, aword(4)
            go to 999
          endif
  150   continue

c....   Test for acceptable variable name.

 8152   format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Unacceptable name.  Too dangerous:  ',a)
 8155   format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Unacceptable name.  Use command "tol".')
 8156   format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Name must begin with a-z, A-Z, $, % or @:  ',a)
 8157   format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Single-character name must be lower case a to z:  ',a)

        if (mtype(4) .ne. 1) then
          write (   6, 8152) ainput, ninput, aword(4)
          write (iout, 8152) ainput, ninput, aword(4)
          go to 999
        endif

        if ((aword(4) .eq. 'random') .or. (aword(4) .eq. '?')) then
          write (   6, 8152) ainput, ninput, aword(4)
          write (iout, 8152) ainput, ninput, aword(4)
          go to 999
        endif

        if (aword(4) .eq. 'tol') then
          write (   6, 8155) ainput, ninput
          write (iout, 8155) ainput, ninput
          go to 999
        endif

        asrce='$%@ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'

        call aptchfp (asrce, 1, 55, aword(4), 1, ipat, nerra)

        if (ipat .eq. 0) then
          write (   6, 8156) ainput, ninput, aword(4)
          write (iout, 8156) ainput, ninput, aword(4)
          go to 999
        endif
  
        if (lword(4) .eq. 1) then       ! Single-character name must be a-z.
  
          asrce = 'abcdefghijklmnopqrstuvwxyz'
  
          call aptchfp (asrce, 1, 26, aword(4), 1, ipat, nerra)
  
          if (ipat .eq. 0) then
            write (   6, 8157) ainput, ninput, aword(4)
            write (iout, 8157) ainput, ninput, aword(4)
            go to 999
          endif

        endif

 8158   format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Name of variable not from 1 to 24 characters:  ',a)

        if ((lword(4) .lt. 1) .or. (lword(4) .gt. lenmax)) then
          write (   6, 8158) ainput, ninput, aword(4)
          write (iout, 8158) ainput, ninput, aword(4)
          go to 999
        endif

        if (nerr .ne. 0) go to 999

 8130   format ('  Renamed variable ',a,' to ',a)

        write (   6, 8130) avarbl(n1), aword(4)
        write (iout, 8130) avarbl(n1), aword(4)

        avarbl(n1)  = aword(4)
        lvar(n1)    = lword(4)
        avarbls     = aword(4)
        lavarbls    = lword(4)

        call varout (n1, n1)

        go to 999

      endif                           ! Renamed one variable.

c===============================================================================

c.... Input line:  rename big BIGNAME BIGNAME2

      if (aword(2) .eq. 'big') then

        nerr = 0

c....   See if original big integer exists.

 8142   format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Named big integer does not exist:  ',a)

        n1 = 0
        do 132 n = 1, nbigm
          if (aword(3) .eq. abig(n)) then
            n1 = n
            go to 142
          endif
  132   continue
        write (   6, 8142) ainput, ninput, aword(3)
        write (iout, 8142) ainput, ninput, aword(3)
        go to 999
  142   continue

c....   See if new big integer already exists.

 8153   format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  New big integer name already exists:  ',a)

        n2 = 0
        do 153 n = 1, nbigm
          if (aword(4) .eq. abig(n)) then
            n2 = n
            if ((ndig(n2) .gt. 1) .or.
     &         ((ndig(n2) .eq. 1) .and. (idig(1,n2) .gt. 0))) then
              write (   6, 8153) ainput, ninput, aword(4)
              write (iout, 8153) ainput, ninput, aword(4)
              go to 999
            endif
            go to 162
          endif
  153   continue
        write (   6, 8142) ainput, ninput, aword(4)
        write (iout, 8142) ainput, ninput, aword(4)
        go to 999
  162   continue

        if (nerr .ne. 0) go to 999

 8132   format ('  Renamed big integer ',a,' to ',a)

        write (   6, 8132) abig(n1), aword(4)
        write (iout, 8132) abig(n1), aword(4)

        do i = 1, ndig(n1)
          idig(i,n2) = idig(i,n1)
          idig(i,n1) = 0
        enddo

        ndig(n2) = ndig(n1)
        ndig(n1) = 1

        abigtag(n2) = abigtag(n1)
        abigtag(n1) = ' '

        call bigout (n2, n2, 'min')

        go to 999

      endif                           ! Renamed big integer BIGNAME to BIGNAME2.

c===============================================================================

c.... Input line:  rename point PNAME PNAME2

      if ((aword(2) .eq. 'point') .or.
     &    (aword(2) .eq. 'pnt')   .or.
     &    (aword(2) .eq. 'pt')    .or.
     &    (aword(2) .eq. 'p'))    then

c....   See if original point exists.

        call ptfind (3, n1)

        if (n1 .eq. 0) go to 999

c....   See if last point name should be incremented and used.

        if (aword(4) .eq. '+') then
          aword(4) = apoints
          lword(4) = lapoints
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), apoints
            write (iout, 8128) ainput, ninput, aword(2), apoints
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = apoints
          lword(4) = lapoints
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), apoints
            write (iout, 8129) ainput, ninput, aword(2), apoints
            go to 999
          endif
        endif

        afirst = aword(4)

        if (afirst .eq. '-')  then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new point already exists.

 8180   format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  New point name already exists:  ',a)

        do 180 n = 1, npoints
          if (aword(4) .eq. apoint(n)) then
            write (   6, 8180) ainput, ninput, aword(4)
            write (iout, 8180) ainput, ninput, aword(4)
            go to 999
          endif
  180   continue

c....   See if new subscripts, if any, match mesh indices, if any.

        call aptsubm (aword(4), 1, lword(4), iamax, idmax, iemax,
     &    '(', ',', ')', 16,
     &          astem, istem, fstem, mstem, lstem,
     &    nsub, asub,  isub,  fsub,  msub,  lsub, nerra)

 8339   format ('  rename ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Point name subscripts mismatch mesh indices:  ',a24)

        if (nsub .eq. 3) then
        if (((kpoint(n1) .ne. 0) .and. (kpoint(n1) .ne. isub(1)))  .or.
     &      ((lpoint(n1) .ne. 0) .and. (lpoint(n1) .ne. isub(2)))  .or.
     &      ((mpoint(n1) .ne. 0) .and. (mpoint(n1) .ne. isub(3)))) then
          write (   6, 8339) ainput, ninput, aword(4)
          write (iout, 8339) ainput, ninput, aword(4)
          go to 999
        endif
        endif

 8160   format ('  Renamed point ',a,' to ',a)

        write (   6, 8160) apoint(n1), aword(4)
        write (iout, 8160) apoint(n1), aword(4)

        apoint(n1)  = aword(4)
        lapoint(n1) = lword(4)
        apoints     = aword(4)
        lapoints    = lword(4)

c....   Rename same point in definitions of clusters, lines, triangles,
c....     annular disks, regular polygons, tetrahedrons.

        do 182 npr = 1, npairs
          if (apairpt(npr) .eq. aword(3)) apairpt(npr) = aword(4)
  182   continue

        do 185 nn = 1, nlines
          if (alipt1(nn)  .eq. aword(3)) alipt1(nn)   = aword(4)
          if (alipt2(nn)  .eq. aword(3)) alipt2(nn)   = aword(4)
          if (alipt1(nn)  .eq. aword(3)) lalipt1(nn)  = lword(4)
          if (alipt2(nn)  .eq. aword(3)) lalipt2(nn)  = lword(4)
  185   continue

        do 190 nt = 1, ntrigs
          if (atript1(nt) .eq. aword(3)) atript1(nt)  = aword(4)
          if (atript2(nt) .eq. aword(3)) atript2(nt)  = aword(4)
          if (atript3(nt) .eq. aword(3)) atript3(nt)  = aword(4)
          if (atript1(nt) .eq. aword(3)) latript1(nt) = lword(4)
          if (atript2(nt) .eq. aword(3)) latript2(nt) = lword(4)
          if (atript3(nt) .eq. aword(3)) latript3(nt) = lword(4)
  190   continue

        do 192 npg = 1, npolygs
          if (acenpg(npg) .eq. aword(3)) acenpg(npg) = aword(4)
          if (averpg(npg) .eq. aword(3)) averpg(npg) = aword(4)
          if (acoppg(npg) .eq. aword(3)) acoppg(npg) = aword(4)
  192   continue

        do 193 nd = 1, ndisks
          if (acendk(nd)  .eq. aword(3)) acendk(nd)  = aword(4)
  193   continue

        do 194 nt = 1, ntets
          if (atetpt(1,nt) .eq. aword(3)) atetpt(1,nt) = aword(4)
          if (atetpt(2,nt) .eq. aword(3)) atetpt(2,nt) = aword(4)
          if (atetpt(3,nt) .eq. aword(3)) atetpt(3,nt) = aword(4)
          if (atetpt(4,nt) .eq. aword(3)) atetpt(4,nt) = aword(4)
  194   continue

        do 195 nt = 1, ntets
          if (atetpt(1,nt) .eq. aword(3)) latetpt(1,nt) = lword(4)
          if (atetpt(2,nt) .eq. aword(3)) latetpt(2,nt) = lword(4)
          if (atetpt(3,nt) .eq. aword(3)) latetpt(3,nt) = lword(4)
          if (atetpt(4,nt) .eq. aword(3)) latetpt(4,nt) = lword(4)
  195   continue

        do 196 nph = 1, npolyhs
          if (acenph(nph) .eq. aword(3)) acenph(nph) = aword(4)
          if (averph(nph) .eq. aword(3)) averph(nph) = aword(4)
          if (acopph(nph) .eq. aword(3)) acopph(nph) = aword(4)
  196   continue

        call pout (n1, n1, 'min')

        go to 999

      endif                           ! Renamed one point.

c===============================================================================

c.... Input line:  rename cluster CLNAME CLNAME2

      if ((aword(2) .eq. 'cluster') .or.
     &    (aword(2) .eq. 'cl'))     then

 8610 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named cluster does not exist:  ',a)

c....   See if original cluster exists.

        do 610 n = 1, nclusts
          if (aword(3) .eq. aclust(n)) then
            n1 = n
            go to 620
          endif
  610   continue
        write (   6, 8610) ainput, ninput, aword(3)
        write (iout, 8610) ainput, ninput, aword(3)
        go to 999

c....   See if last cluster name should be incremented and used.

  620   if (aword(4) .eq. '+') then
          aword(4) = aclusts
          lword(4) = laclusts
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), aclusts
            write (iout, 8128) ainput, ninput, aword(2), aclusts
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = aclusts
          lword(4) = laclusts
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), aclusts
            write (iout, 8129) ainput, ninput, aword(2), aclusts
            go to 999
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new cluster already exists.

 8620 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  New cluster name already exists:  ',a)

        do 630 n = 1, nclusts
          if (aword(4) .eq. aclust(n)) then
            write (   6, 8620) ainput, ninput, aword(4)
            write (iout, 8620) ainput, ninput, aword(4)
            go to 999
          endif
  630   continue

 8630 format ('  Renamed cluster ',a,' to ',a)

        write (   6, 8630) aclust(n1), aword(4)
        write (iout, 8630) aclust(n1), aword(4)

        do 640 npr = 1, npairs
          if (apaircl(npr) .eq. aword(3)) then
            apaircl(npr) = aword(4)
          endif
  640   continue

        aclust(n1)  = aword(4)
        laclust(n1) = lword(4)
        aclusts     = aword(4)
        laclusts    = lword(4)

        call clout (n1, n1, 'min')

        go to 999

      endif                           ! Renamed one cluster.

c===============================================================================

c.... Input line:  rename line ALNAME ALNAME2

      if ((aword(2) .eq. 'line') .or.
     &    (aword(2) .eq. 'ln')   .or.
     &    (aword(2) .eq. 'l'))   then

 8280 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named line does not exist:  ',a)

c....   See if original line exists.

        do 240 n = 1, nlines
          if (aword(3) .eq. aline(n)) then
            n1 = n
            go to 250
          endif
  240   continue
        write (   6, 8280) ainput, ninput, aword(3)
        write (iout, 8280) ainput, ninput, aword(3)
        go to 999

c....   See if last line name should be incremented and used.

  250   if (aword(4) .eq. '+') then
          aword(4) = alines
          lword(4) = lalines
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), alines
            write (iout, 8128) ainput, ninput, aword(2), alines
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = alines
          lword(4) = lalines
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), alines
            write (iout, 8129) ainput, ninput, aword(2), alines
            go to 999
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new line already exists.

 8290 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  New line name already exists:  ',a)

        do 260 n = 1, nlines
          if (aword(4) .eq. aline(n)) then
            write (   6, 8290) ainput, ninput, aword(4)
            write (iout, 8290) ainput, ninput, aword(4)
            go to 999
          endif
  260   continue

 8270 format ('  Renamed line ',a,' to ',a)

        write (   6, 8270) aline(n1), aword(4)
        write (iout, 8270) aline(n1), aword(4)

        aline(n1) = aword(4)
        alines    = aword(4)
        lalines   = lword(4)

        call lout (n1, n1)

        go to 999

      endif                           ! Renamed one line.

c===============================================================================

c.... Input line:  rename triangle TRNAME TRNAME2

      if ((aword(2) .eq. 'triangle') .or.
     &    (aword(2) .eq. 'tri'))   then

 8380 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named triangle does not exist:  ',a)

c....   See if original triangle exists.

        do 340 n = 1, ntrigs
          if (aword(3) .eq. atrig(n)) then
            n1 = n
            go to 350
          endif
  340   continue
        write (   6, 8380) ainput, ninput, aword(3)
        write (iout, 8380) ainput, ninput, aword(3)
        go to 999

c....   See if last triangle name should be incremented and used.

  350   if (aword(4) .eq. '+') then
          aword(4) = atrigs
          lword(4) = latrigs
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), atrigs
            write (iout, 8128) ainput, ninput, aword(2), atrigs
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = atrigs
          lword(4) = latrigs
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), atrigs
            write (iout, 8129) ainput, ninput, aword(2), atrigs
            go to 999
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new triangle already exists.

 8390 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  New triangle name already exists:  ',a)

        do 360 n = 1, ntrigs
          if (aword(4) .eq. atrig(n)) then
            write (   6, 8390) ainput, ninput, aword(4)
            write (iout, 8390) ainput, ninput, aword(4)
            go to 999
          endif
  360   continue

 8370 format ('  Renamed triangle ',a,' to ',a)

        write (   6, 8370) atrig(n1), aword(4)
        write (iout, 8370) atrig(n1), aword(4)

        atrig(n1) = aword(4)
        atrigs    = aword(4)
        latrigs   = lword(4)

        call trout (n1, n1, 'min')

        go to 999

      endif                           ! Renamed one triangle.

c===============================================================================

c.... Input line:  rename polygon PGNAME PGNAME2

      if ((aword(2) .eq. 'polygon') .or.
     &    (aword(2) .eq. 'polyg')   .or.
     &    (aword(2) .eq. 'pg'))     then

 8550 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named regular polygon does not exist:  ',a)

c....   See if original polygon exists.

        do 550 n = 1, npolygs
          if (aword(3) .eq. apolyg(n)) then
            n1 = n
            go to 560
          endif
  550   continue
        write (   6, 8550) ainput, ninput, aword(3)
        write (iout, 8550) ainput, ninput, aword(3)
        go to 999

c....   See if last polygon name should be incremented and used.

  560   if (aword(4) .eq. '+') then
          aword(4) = apolygs
          lword(4) = lapolygs
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), apolygs
            write (iout, 8128) ainput, ninput, aword(2), apolygs
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = apolygs
          lword(4) = lapolygs
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), apolygs
            write (iout, 8129) ainput, ninput, aword(2), apolygs
            go to 999
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new polygon already exists.

 8560 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  New polygon name already exists:  ',a)

        do 570 n = 1, npolygs
          if (aword(4) .eq. apolyg(n)) then
            write (   6, 8560) ainput, ninput, aword(4)
            write (iout, 8560) ainput, ninput, aword(4)
            go to 999
          endif
  570   continue

 8570 format ('  Renamed polygon ',a,' to ',a)

        write (   6, 8570) apolyg(n1), aword(4)
        write (iout, 8570) apolyg(n1), aword(4)

        apolyg(n1) = aword(4)
        apolygs    = aword(4)
        lapolygs   = lword(4)

        call pgout (n1, n1, 'min')

        go to 999

      endif                           ! Renamed one polygon.

c===============================================================================

c.... Input line:  rename disk DKNAME DKNAME2

      if ((aword(2) .eq. 'disk') .or.
     &    (aword(2) .eq. 'disc') .or.
     &    (aword(2) .eq. 'dk'))  then

 8650 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named annular disk does not exist:  ',a)

c....   See if original annular disk exists.

        do 650 n = 1, ndisks
          if (aword(3) .eq. adisk(n)) then
            n1 = n
            go to 660
          endif
  650   continue
        write (   6, 8650) ainput, ninput, aword(3)
        write (iout, 8650) ainput, ninput, aword(3)
        go to 999

c....   See if last annular disk name should be incremented and used.

  660   if (aword(4) .eq. '+') then
          aword(4) = adisks
          lword(4) = ladisks
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), adisks
            write (iout, 8128) ainput, ninput, aword(2), adisks
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = adisks
          lword(4) = ladisks
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), adisks
            write (iout, 8129) ainput, ninput, aword(2), adisks
            go to 999
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new annular disk already exists.

 8660 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  New annular disk name already exists:  ',a)

        do 670 n = 1, ndisks
          if (aword(4) .eq. adisk(n)) then
            write (   6, 8660) ainput, ninput, aword(4)
            write (iout, 8660) ainput, ninput, aword(4)
            go to 999
          endif
  670   continue

 8670 format ('  Renamed annular disk ',a,' to ',a)

        write (   6, 8670) adisk(n1), aword(4)
        write (iout, 8670) adisk(n1), aword(4)

        adisk(n1) = aword(4)
        adisks    = aword(4)
        ladisks   = lword(4)

        call dkout (n1, n1)

        go to 999

      endif                           ! Renamed one disk.

c===============================================================================

c.... Input line:  rename polyhedron PHNAME PHNAME2

      if ((aword(2) .eq. 'polyhedron') .or.
     &    (aword(2) .eq. 'polyh')   .or.
     &    (aword(2) .eq. 'ph'))     then

 8580 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named regular polyhedron does not exist:  ',a)

c....   See if original polyhedron exists.

        do 580 n = 1, npolyhs
          if (aword(3) .eq. apolyh(n)) then
            n1 = n
            go to 590
          endif
  580   continue
        write (   6, 8580) ainput, ninput, aword(3)
        write (iout, 8580) ainput, ninput, aword(3)
        go to 999

c....   See if last polyhedron name should be incremented and used.

  590   if (aword(4) .eq. '+') then
          aword(4) = apolyhs
          lword(4) = lapolyhs
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), apolyhs
            write (iout, 8128) ainput, ninput, aword(2), apolyhs
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = apolyhs
          lword(4) = lapolyhs
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), apolyhs
            write (iout, 8129) ainput, ninput, aword(2), apolyhs
            go to 999
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new polyhedron already exists.

 8590 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  New polyhedron name already exists:  ',a)

        do 600 n = 1, npolyhs
          if (aword(4) .eq. apolyh(n)) then
            write (   6, 8590) ainput, ninput, aword(4)
            write (iout, 8590) ainput, ninput, aword(4)
            go to 999
          endif
  600   continue

 8600 format ('  Renamed polyhedron ',a,' to ',a)

        write (   6, 8600) apolyh(n1), aword(4)
        write (iout, 8600) apolyh(n1), aword(4)

        apolyh(n1) = aword(4)
        apolyhs    = aword(4)
        lapolyhs   = lword(4)

        call phout (n1, n1, 'min')

        go to 999

      endif                           ! Renamed one polyhedron.

c===============================================================================

c.... Input line:  rename tetrahedron TETNAME TETNAME2

      if ((aword(2) .eq. 'tetrahedron') .or.
     &    (aword(2) .eq. 'tetra')       .or.
     &    (aword(2) .eq. 'tet'))        then

 8410 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named tetrahedron does not exist:  ',a)

c....   See if original tetrahedron exists.

        do 410 n = 1, ntets
          if (aword(3) .eq. atetra(n)) then
            n1 = n
            go to 420
          endif
  410   continue
        write (   6, 8410) ainput, ninput, aword(3)
        write (iout, 8410) ainput, ninput, aword(3)
        go to 999

c....   See if last tetrahedron name should be incremented and used.

  420   if (aword(4) .eq. '+') then
          aword(4) = atetras
          lword(4) = latetras
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), atetras
            write (iout, 8128) ainput, ninput, aword(2), atetras
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = atetras
          lword(4) = latetras
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), atetras
            write (iout, 8129) ainput, ninput, aword(2), atetras
            go to 999
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new tetrahedron already exists.

 8420 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  New tetrahedron name already exists:  ',a)

        do 430 n = 1, ntets
          if (aword(4) .eq. atetra(n)) then
            write (   6, 8420) ainput, ninput, aword(4)
            write (iout, 8420) ainput, ninput, aword(4)
            go to 999
          endif
  430   continue

 8430 format ('  Renamed tetrahedron ',a,' to ',a)

        write (   6, 8430) atetra(n1), aword(4)
        write (iout, 8430) atetra(n1), aword(4)

        atetra(n1) = aword(4)
        atetras    = aword(4)
        latetras   = lword(4)

        call tetout (n1, n1, 'min')

        go to 999

      endif                           ! Renamed one tetrahedron.

c===============================================================================

c.... Input line:  rename brick BRNAME BRNAME2

      if ((aword(2) .eq. 'brick') .or.
     &    (aword(2) .eq. 'br'))   then

 8440 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named brick does not exist:  ',a)

c....   See if original brick exists.

        do 440 n = 1, nbricks
          if (aword(3) .eq. abrick(n)) then
            n1 = n
            go to 460
          endif
  440   continue
        write (   6, 8440) ainput, ninput, aword(3)
        write (iout, 8440) ainput, ninput, aword(3)
        go to 999

c....   See if last brick name should be incremented and used.

  460   if (aword(4) .eq. '+') then
          aword(4) = abricks
          lword(4) = labricks
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), abricks
            write (iout, 8128) ainput, ninput, aword(2), abricks
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = abricks
          lword(4) = labricks
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), abricks
            write (iout, 8129) ainput, ninput, aword(2), abricks
            go to 999
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new brick already exists.

 8460 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  New brick name already exists:  ',a)

        do 470 n = 1, nbricks
          if (aword(4) .eq. abrick(n)) then
            write (   6, 8460) ainput, ninput, aword(4)
            write (iout, 8460) ainput, ninput, aword(4)
            go to 999
          endif
  470   continue

 8470 format ('  Renamed brick ',a,' to ',a)

        write (   6, 8470) abrick(n1), aword(4)
        write (iout, 8470) abrick(n1), aword(4)

        abrick(n1) = aword(4)
        abricks    = aword(4)
        labricks   = lword(4)

        call brout (n1, n1, 'min')

        go to 999

      endif                           ! Renamed one brick.

c===============================================================================

c.... Input line:  rename vector VNAME VNAME2

      if ((aword(2) .eq. 'vector') .or.
     &    (aword(2) .eq. 'vect')   .or.
     &    (aword(2) .eq. 'v'))     then

c....   See if original vector exists.

        call vectfind (3, n1)

        if (n1 .eq. 0) go to 999

c....   See if last vector name should be incremented and used.

  220   if (aword(4) .eq. '+') then
          aword(4) = avectors
          lword(4) = lavectors
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), avectors
            write (iout, 8128) ainput, ninput, aword(2), avectors
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = avectors
          lword(4) = lavectors
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), avectors
            write (iout, 8129) ainput, ninput, aword(2), avectors
            go to 999
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

        if ((aword(4) .eq. 'random') .or. (aword(4) .eq. '?')) then
          write (   6, 8152) ainput, ninput, aword(4)
          write (iout, 8152) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new vector already exists.

 8230 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  New vector name already exists:  ',a)

        do 230 n = 1, nvectors
          if (aword(4) .eq. avector(n)) then
            write (   6, 8230) ainput, ninput, aword(4)
            write (iout, 8230) ainput, ninput, aword(4)
            go to 999
          endif
  230   continue

 8210 format ('  Renamed vector ',a,' to ',a)

        write (   6, 8210) avector(n1), aword(4)
        write (iout, 8210) avector(n1), aword(4)

        avector(n1)  = aword(4)
        lavector(n1) = lword(4)
        avectors     = aword(4)
        lavectors    = lword(4)

        call vout (n1, n1)

        go to 999

      endif                           ! Renamed one vector.

c===============================================================================

c.... Input line:  rename operator OPNAME OPNAME2

      if ((aword(2) .eq. 'operator') .or.
     &    (aword(2) .eq. 'oper')     .or.
     &    (aword(2) .eq. 'op'))      then

 8310 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named operator does not exist:  ',a)

c....   See if original operator exists.

        do 270 n = 1, nopers
          if (aword(3) .eq. aoper(n)) then
            n1 = n
            go to 280
          endif
  270   continue
        write (   6, 8310) ainput, ninput, aword(3)
        write (iout, 8310) ainput, ninput, aword(3)
        go to 999

c....   See if last operator name should be incremented and used.

  280   if (aword(4) .eq. '+') then
          aword(4) = aopers
          lword(4) = laopers
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), aopers
            write (iout, 8128) ainput, ninput, aword(2), aopers
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = aopers
          lword(4) = laopers
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), aopers
            write (iout, 8129) ainput, ninput, aword(2), aopers
            go to 999
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new operator already exists.

 8320 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  New operator name already exists:  ',a)

        do 290 n = 1, nopers
          if (aword(4) .eq. aoper(n)) then
            write (   6, 8320) ainput, ninput, aword(4)
            write (iout, 8320) ainput, ninput, aword(4)
            go to 999
          endif
  290   continue

 8300 format ('  Renamed operator ',a,' to ',a)

        write (   6, 8300) aoper(n1), aword(4)
        write (iout, 8300) aoper(n1), aword(4)

        aoper(n1) = aword(4)
        aopers    = aword(4)
        laopers   = lword(4)

        call opout (n1, n1, 'min')

        go to 999

      endif                           ! Renamed one operator.

c===============================================================================

c.... Input line:  rename plane     PLNAME PLNAME2
c.... Input line:  rename sphere    SPHNAME SPHNAME2
c.... Input line:  rename cylinder  CYLNAME CYLNAME2
c.... Input line:  rename cone      CNNAME CNNAME2
c.... Input line:  rename ellipsoid ELNAME ELNAME2
c.... Input line:  rename axisym    AXNAME AXNAME2
c.... Input line:  rename quadric   QNAME QNAME2

      if ((aword(2) .eq. 'quadric')   .or.
     &    (aword(2) .eq. 'quad')      .or.
     &    (aword(2) .eq. 'q')         .or.
     &    (aword(2) .eq. 'plane')     .or.
     &    (aword(2) .eq. 'pl')        .or.
     &    (aword(2) .eq. 'sphere')    .or.
     &    (aword(2) .eq. 'sph')       .or.
     &    (aword(2) .eq. 'cylinder')  .or.
     &    (aword(2) .eq. 'cyl')       .or.
     &    (aword(2) .eq. 'cone')      .or.
     &    (aword(2) .eq. 'hyperb')    .or.
     &    (aword(2) .eq. 'saddle')    .or.
     &    (aword(2) .eq. 'ellipsoid') .or.
     &    (aword(2) .eq. 'ellipse')   .or.
     &    (aword(2) .eq. 'ell')       .or.
     &    (aword(2) .eq. 'el')        .or.
     &    (aword(2) .eq. 'axisym'))   then

 8472 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named quadric surface does not exist:  ',a)

c....   See if original quadric exists, and is the right type.

        do 472 n = 1, nquads
          if (aword(3) .eq. aquad(n)) then

            call qutype (aword(2), n, 1, nerrq)

            if (nerrq .ne. 0) go to 999

            n1 = n

            go to 475
          endif
  472   continue
        write (   6, 8472) ainput, ninput, aword(3)
        write (iout, 8472) ainput, ninput, aword(3)
        go to 999

c....   See if last quadric surface name should be incremented and used.

  475   if ((aword(4) .eq. '+')  .or.
     &      (aword(4) .eq. '-')) then

          awordx = aword(4)

          if  (aword(2) .eq. 'cone')      then
            aword(4) = acones
            lword(4) = lacones
          endif

          if ((aword(2) .eq. 'cylinder')  .or.
     &        (aword(2) .eq. 'cyl'))      then
            aword(4) = acyls
            lword(4) = lacyls
          endif

          if ((aword(2) .eq. 'ellipsoid') .or.
     &        (aword(2) .eq. 'ellipse')   .or.
     &        (aword(2) .eq. 'ell')       .or.
     &        (aword(2) .eq. 'el'))       then
            aword(4) = aellips
            lword(4) = laellips
          endif

          if ((aword(2) .eq. 'hyperb')    .or.
     &        (aword(2) .eq. 'saddle'))   then
            aword(4) = ahyperbs
            lword(4) = lahyperbs
          endif

          if ((aword(2) .eq. 'plane')     .or.
     &        (aword(2) .eq. 'pl'))       then
            aword(4) = aplanes
            lword(4) = laplanes
          endif

          if ((aword(2) .eq. 'quadric')   .or.
     &        (aword(2) .eq. 'quad')      .or.
     &        (aword(2) .eq. 'q'))        then
            aword(4) = aquads
            lword(4) = laquads
          endif

          if ((aword(2) .eq. 'sphere')    .or.
     &        (aword(2) .eq. 'sph'))      then
            aword(4) = aspheres
            lword(4) = laspheres
          endif

          if  (aword(2) .eq. 'axisym')    then
            aword(4) = axisyms
            lword(4) = laxisyms
          endif

          if (awordx .eq. '+') then
            awordy = aword(4)
            lwordy = lword(4)
            call incrsubs (1, aword(4), 1, lword(4), nerra)
            if (nerra .ne. 0) then
              write (   6, 8128) ainput, ninput, aword(2), awordy
              write (iout, 8128) ainput, ninput, aword(2), awordy
              go to 999
            endif
          endif

          if (awordx .eq. '-') then
            awordy = aword(4)
            lwordy = lword(4)
            call incrsubs (-1, aword(4), 1, lword(4), nerra)
            if (nerra .ne. 0) then
              write (   6, 8129) ainput, ninput, aword(2), awordy
              write (iout, 8129) ainput, ninput, aword(2), awordy
              go to 999
            endif
          endif

          call namechk (aword(4), lword(4), nerrchk)

          if (nerrchk .ne. 0) then
            write (   6, 8905) ainput, ninput, aword(4)
            write (iout, 8905) ainput, ninput, aword(4)
            go to 999
          endif

        endif

c....   See if new quadric already exists.

 8475 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  New quadric name already exists:  ',a)

        do 480 n = 1, nquads
          if (aword(4) .eq. aquad(n)) then
            write (   6, 8475) ainput, ninput, aword(4)
            write (iout, 8475) ainput, ninput, aword(4)
            go to 999
          endif
  480   continue

 8478 format ('  Renamed ',a9,' ',a,' to ',a)

        write (   6, 8478) aword(2), aquad(n1), aword(4)
        write (iout, 8478) aword(2), aquad(n1), aword(4)

        aquads     = aword(4)
        laquads    = lword(4)
        aquad(n1)  = aword(4)
        laquad(n1) = lword(4)

        if  (aword(2) .eq. 'cone')      then
          acones     = aword(4)
          lacones    = lword(4)
        endif

        if ((aword(2) .eq. 'cylinder')  .or.
     &      (aword(2) .eq. 'cyl'))      then
          acyls      = aword(4)
          lacyls     = lword(4)
        endif

        if ((aword(2) .eq. 'ellipsoid') .or.
     &      (aword(2) .eq. 'ellipse')   .or.
     &      (aword(2) .eq. 'ell')       .or.
     &      (aword(2) .eq. 'el'))       then
          aellips    = aword(4)
          laellips   = lword(4)
        endif

        if ((aword(2) .eq. 'hyperb')    .or.
     &      (aword(2) .eq. 'saddle'))   then
          ahyperbs   = aword(4)
          lahyperbs  = lword(4)
        endif

        if ((aword(2) .eq. 'plane')     .or.
     &      (aword(2) .eq. 'pl'))       then
          aplanes    = aword(4)
          laplanes   = lword(4)
        endif

        if ((aword(2) .eq. 'sphere')    .or.
     &      (aword(2) .eq. 'sph'))      then
          aspheres   = aword(4)
          laspheres  = lword(4)
        endif

        if  (aword(2) .eq. 'axisym')    then
          axisyms    = aword(4)
          laxisyms   = lword(4)
        endif

        if (asym(n1) .eq. 'axisymmetric') then
          axisyms  = aword(4)
          laxisyms = lword(4)
        endif

c....   Rename quadric used to define a zone.

        do 485 nz = 1, nzones
          do 482 nq = 1, nzquads(nz)
            if (azquad(nq,nz) .eq. aword(3) ) azquad(nq,nz)  = aword(4)
            if (azquad(nq,nz) .eq. aword(3) ) lazquad(nq,nz) = lword(4)
  482     continue
  485   continue

        if ((aword(2) .eq. 'plane') .or.
     &      (aword(2) .eq. 'pl'))  then
          call plout (n1, n1)
        else
          call quout (n1, n1, 'min', nerrq)
        endif

        go to 999

      endif                           ! Renamed one quadric.

c===============================================================================

c.... Input line:  rename zone ZNAME ZNAME2

      if ((aword(2) .eq. 'zone') .or.
     &    (aword(2) .eq. 'zn')   .or.
     &    (aword(2) .eq. 'z'))   then

 8505 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named zone does not exist:  ',a)

c....   See if original zone exists.

        do 505 n = 1, nzones
          if (aword(3) .eq. azone(n)) then

            n1 = n

            go to 515
          endif
  505   continue
        write (   6, 8505) ainput, ninput, aword(3)
        write (iout, 8505) ainput, ninput, aword(3)
        go to 999

c....   See if last zone name should be incremented and used.

  515   if (aword(4) .eq. '+') then
          aword(4) = azones
          lword(4) = lazones
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), azones
            write (iout, 8128) ainput, ninput, aword(2), azones
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = azones
          lword(4) = lazones
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), azones
            write (iout, 8129) ainput, ninput, aword(2), azones
            go to 999
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new zone already exists.

 8515 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  New zone name already exists:  ',a)

        do 520 n = 1, nzones
          if (aword(4) .eq. azone(n)) then
            write (   6, 8515) ainput, ninput, aword(4)
            write (iout, 8515) ainput, ninput, aword(4)
            go to 999
          endif
  520   continue

 8530 format ('  Renamed ',a9,' ',a,' to ',a)

        write (   6, 8530) aword(2), azone(n1), aword(4)
        write (iout, 8530) aword(2), azone(n1), aword(4)

        azone(n1) = aword(4)
        azones    = aword(4)
        lazones   = lword(4)

        call zout (n1, n1, 'min')

        go to 999

      endif                           ! Renamed one zone.

c===============================================================================

c.... Input line:  rename bin BINAME BINAME2

      if (aword(2) .eq. 'bin') then

 8720 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named probability bin does not exist:  ',a)

c....   See if original bin exists.

        do 710 n = 1, nbins
          if (aword(3) .eq. abin(n)) then
            n1 = n
            go to 720
          endif
  710   continue
        write (   6, 8720) ainput, ninput, aword(3)
        write (iout, 8720) ainput, ninput, aword(3)
        go to 999

c....   See if last probability bin name should be incremented and used.

  720   if (aword(4) .eq. '+') then
          aword(4) = abins
          lword(4) = labins
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), abins
            write (iout, 8128) ainput, ninput, aword(2), abins
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = abins
          lword(4) = labins
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), abins
            write (iout, 8129) ainput, ninput, aword(2), abins
            go to 999
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new bin already exists.

 8730 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  New bin name already exists:  ',a)

        do 730 n = 1, nbins
          if (aword(4) .eq. abin(n)) then
            write (   6, 8730) ainput, ninput, aword(4)
            write (iout, 8730) ainput, ninput, aword(4)
            go to 999
          endif
  730   continue

 8710 format ('  Renamed bin ',a,' to ',a)

        write (   6, 8710) abin(n1), aword(4)
        write (iout, 8710) abin(n1), aword(4)

c....   Rename same bin in definitions of pdfs.

        do 735 npx = 1, npdfxs
          if (abinx(npx) .eq. aword(3)) abinx(npx) = aword(4)
  735   continue

        abin(n1) = aword(4)
        abins    = aword(4)
        labins   = lword(4)

        call binout (n1, n1, 'min')

        go to 999

      endif                           ! Renamed one bin.

c===============================================================================

c.... Input line:  rename pdf PDFNAME PDFNAME2

      if (aword(2) .eq. 'pdf') then

 8740 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named pdf does not exist:  ',a)

c....   See if original pdf exists.

        do 740 n = 1, npdfs
          if (aword(3) .eq. apdf(n)) then
            n1 = n
            go to 750
          endif
  740   continue
        write (   6, 8740) ainput, ninput, aword(3)
        write (iout, 8740) ainput, ninput, aword(3)
        go to 999

c....   See if last pdf name should be incremented and used.

  750   if (aword(4) .eq. '+') then
          aword(4) = apdfs
          lword(4) = lapdfs
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), apdfs
            write (iout, 8128) ainput, ninput, aword(2), apdfs
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = apdfs
          lword(4) = lapdfs
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), apdfs
            write (iout, 8129) ainput, ninput, aword(2), apdfs
            go to 999
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new pdf already exists.

 8750 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  New pdf name already exists:  ',a)

        do 760 n = 1, npdfs
          if (aword(4) .eq. apdf(n)) then
            write (   6, 8750) ainput, ninput, aword(4)
            write (iout, 8750) ainput, ninput, aword(4)
            go to 999
          endif
  760   continue

 8765 format ('  Renamed pdf ',a,' to ',a)

        write (   6, 8765) apdf(n1), aword(4)
        write (iout, 8765) apdf(n1), aword(4)

        do 780 npx = 1, npdfxs
          if (apdfx(npx) .eq. aword(3)) then
            apdfx(npx) = aword(4)
          endif
  780   continue

        apdf(n1) = aword(4)
        apdfs    = aword(4)
        lapdfs   = lword(4)

        call pdfout (n1, n1, 'min')

        go to 999

      endif                           ! Renamed one pdf.

c===============================================================================

c.... Input line:  rename alias ANAME ANAME2

      if ((aword(2) .eq. 'alias') .or.
     &    (aword(2) .eq. 'al')    .or.
     &    (aword(2) .eq. 'a'))    then

 8810 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named alias does not exist:  ',a)

c....   See if original alias exists.

        do 810 n = 1, nalias
          if (aword(3) .eq. alias1(n)) then
            n1 = n
            go to 820
          endif
  810   continue
        write (   6, 8810) ainput, ninput, aword(3)
        write (iout, 8810) ainput, ninput, aword(3)
        go to 999

c....   See if last alias name should be incremented and used.

  820   if (aword(4) .eq. '+') then
          aword(4) = alias1s
          lword(4) = lalias1s
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), alias1s
            write (iout, 8128) ainput, ninput, aword(2), alias1s
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = alias1s
          lword(4) = lalias1s
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), alias1s
            write (iout, 8129) ainput, ninput, aword(2), alias1s
            go to 999
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new alias already exists.

 8820 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  New alias name already exists:  ',a)

        do 830 n = 1, nalias
          if (aword(4) .eq. alias1(n)) then
            write (   6, 8820) ainput, ninput, aword(4)
            write (iout, 8820) ainput, ninput, aword(4)
            go to 999
          endif
  830   continue

 8835 format ('  Renamed alias ',a,' to ',a)

        write (   6, 8835) alias1(n1), aword(4)
        write (iout, 8835) alias1(n1), aword(4)

        alias1(n1) = aword(4)
        alias1s    = aword(4)
        lalias1s   = lword(4)

        call alout (n1, n1)

        go to 999

      endif                           ! Renamed one alias.

c===============================================================================

c.... Input line:  rename marker AMNAME AMNAME2

      if ((aword(2) .eq. 'marker') .or.
     &    (aword(2) .eq. 'mark')   .or.
     &    (aword(2) .eq. 'm'))     then

 8840 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named marker does not exist:  ',a1)

c....   See if original marker exists.

        do 840 n = 1, nmarks
          if (aword(3) .eq. amark(n)) then
            n1 = n
            go to 850
          endif
  840   continue
        write (   6, 8840) ainput, ninput, aword(3)
        write (iout, 8840) ainput, ninput, aword(3)
        go to 999

c....   See if last marker name should be incremented and used.

  850   if (aword(4) .eq. '+') then
          aword(4) = amarks
          lword(4) = lamarks
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), amarks
            write (iout, 8128) ainput, ninput, aword(2), amarks
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = amarks
          lword(4) = lamarks
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), amarks
            write (iout, 8129) ainput, ninput, aword(2), amarks
            go to 999
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new marker already exists.

 8850 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  New marker name already exists:  ',a1)

        do 860 n = 1, nmarks
          if (aword(4) .eq. amark(n)) then
            write (   6, 8850) ainput, ninput, aword(4)
            write (iout, 8850) ainput, ninput, aword(4)
            go to 999
          endif
  860   continue

 8870 format ('  Renamed marker ',a1,' to ',a1)

        write (   6, 8870) amark(n1), aword(4)
        write (iout, 8870) amark(n1), aword(4)

        amark(n1) = aword(4)
        amarks    = aword(4)
        lamarks   = lword(4)

        call markout (n1, n1)

        go to 999

      endif                           ! Renamed one marker.

c===============================================================================

c.... Input line:  rename symbol SYMBNAME SYMBNAME2

      if ((aword(2) .eq. 'symbol') .or.
     &    (aword(2) .eq. 'symb')   .or.
     &    (aword(2) .eq. 'sym'))   then

        nerr = 0

c....   See if original symbol exists.

 8940   format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Named symbol does not exist:  ',a)

        do 930 n = 1, nsymbs
          if (aword(3) .eq. asymbl(n)) then
            n1 = n
            go to 940
          endif
  930   continue

        write (   6, 8940) ainput, ninput, aword(3)
        write (iout, 8940) ainput, ninput, aword(3)
        go to 999

c....   See if last symbol name should be incremented and used.

  940   if (aword(4) .eq. '+') then
          aword(4) = asymbls
          lword(4) = lasymbls
          call incrsubs (1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8128) ainput, ninput, aword(2), asymbls
            write (iout, 8128) ainput, ninput, aword(2), asymbls
            go to 999
          endif
        endif

        if (aword(4) .eq. '-') then
          aword(4) = asymbls
          lword(4) = lasymbls
          call incrsubs (-1, aword(4), 1, lword(4), nerra)
          if (nerra .ne. 0) then
            write (   6, 8129) ainput, ninput, aword(2), asymbls
            write (iout, 8129) ainput, ninput, aword(2), asymbls
            go to 999
          endif
        endif

        call namechk (aword(4), lword(4), nerrchk)

        if (nerrchk .ne. 0) then
          write (   6, 8905) ainput, ninput, aword(4)
          write (iout, 8905) ainput, ninput, aword(4)
          go to 999
        endif

c....   See if new symbol already exists.

 8950   format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  New symbol name already exists:  ',a)

        do 950 n = 1, nsymbs

          if (aword(4) .eq. asymbl(n)) then
            write (   6, 8950) ainput, ninput, aword(4)
            write (iout, 8950) ainput, ninput, aword(4)
            go to 999
          endif

  950   continue

        if (nerr .ne. 0) go to 999

 8930   format ('  Renamed symbol ',a,' to ',a)

        write (   6, 8930) asymbl(n1), aword(4)
        write (iout, 8930) asymbl(n1), aword(4)

        asymbl(n1) = aword(4)
        lsymb(n1)  = lword(4)
        asymbls    = aword(4)
        lasymbls   = lword(4)

        call symbdisp (n1)

        go to 999

      endif                           ! Renamed one symbol.

c===============================================================================

c.... Input line:  (not recognized)

 8880 format ('  renamex ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Specified object type does not exist:  ',a24)

      write (   6, 8880) ainput, ninput, aword(2)
      write (iout, 8880) ainput, ninput, aword(2)

  999 return

c.... End of subroutine renamex.      (+1 line)
      end
