      subroutine sample

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  sample   Updated     2003 May 16 14:50.
c               sample   Originated  1994 June 23 13:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To randomly sample from a pdf, with or without replacement.
c
c     Logic:    Entry point:  sample.
c sample        Called by:  helpall, main.
c
c sample        Calls:  aptexpl, aptmaxw, aptnorm, aptslid, aptspod, aptxnup,
c               pdfout, varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c===============================================================================

c.... Input line:  sample [help]

 8110 format (
     &  '  Command "sample":  acceptable input lines include:' /
     &  '  "sample [help]"  Display this message.' //
     &  '  "sample PDFNAME [NSAMP|1]"' /
     &  '     Randomly sample from pdf PDFNAME, NSAMP times.' //
     &  '  "sample PDFNAME NSAMP [norepl|repl]"' /
     &  '     Randomly sample from pdf PDFNAME, NSAMP times,' /
     &  '     with [no replacement | replacement].' //
     &  '  "sample PDFNAME NSAMP [norepl,repl] [NSET|1]"' /
     &  '     Randomly sample from pdf PDFNAME, NSAMP times,' /
     &  '     and repeat for NSET sets, with',
     &  ' [no replacement, replacement].' //
     &  '  Note:  see commands "bin" and "pdf".' //
     &  '  Synonyms:  [help, h].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c===============================================================================

c.... See if pdf exists.

 8120 format ('  sample ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named pdf does not exist:  ',a)

      do 120 npdf = 1, npdfs
        if (aword(2) .eq. apdf(npdf)) then
          npsamp = npdf
          call pdfout (npsamp, npsamp, 'min')
          go to 130
        endif
  120 continue
      write (   6, 8120) ainput, ninput, aword(2)
      write (iout, 8120) ainput, ninput, aword(2)
      go to 999

c===============================================================================

c.... Test for positive probability.

 8130 format ('  sample ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named pdf has non-positive probability:  ',1pe20.12)

  130 if (prpdf(npsamp) .le. 0.0) then
        write (   6, 8130) ainput, ninput, prpdf(npsamp)
        write (iout, 8130) ainput, ninput, prpdf(npsamp)
        go to 999
      endif

c===============================================================================

c.... Test for a positive integer number of samples.

      nsamp = 1

 8140 format ('  sample ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of samples is not positive:  ',a)
 8145 format ('  sample ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of samples is not an integer or integer',
     &  ' variable:  ',a)


      if (nword .gt. 2) then

        call varfind (3, 3)

        if (mtype(3) .eq. 2) then
          if (iword(3) .le. 0) then
            write (   6, 8140) ainput, ninput, aword(3)
            write (iout, 8140) ainput, ninput, aword(3)
            go to 999
          endif
          nsamp = iword(3)
        else
          write (   6, 8145) ainput, ninput, aword(3)
          write (iout, 8145) ainput, ninput, aword(3)
          go to 999
        endif
      endif

c===============================================================================

c.... Test for too many words in command.

 8150 format ('  sample ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Too many words in "pdf" command:  ',i3)

      if (nword .gt. 5) then
        write (   6, 8150) ainput, ninput, nword
        write (iout, 8150) ainput, ninput, nword
        go to 999
      endif

c===============================================================================

c.... Test for sampling without replacement.

 8152 format ('  sample ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Fourth word not "repl" or "norepl":  ',a)

      norepl = 0
      if (nword .ge. 4) then
        if (aword(4) .eq. 'norepl') then
          norepl = 1
        elseif (aword(4) .eq. 'repl') then
          norepl = 0
        else
          write (   6, 8152) ainput, ninput, aword(4)
          write (iout, 8152) ainput, ninput, aword(4)
          go to 999
        endif
      endif

c===============================================================================

c.... Test for multiple sets of NSAMP samples.

      nset = 1

 8154 format ('  sample ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of sets is not positive:  ',a)
 8155 format ('  sample ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Fifth word not integer:  ',a)

      if (nword .eq. 5) then

        call varfind (5, 5)

        if (mtype(5) .eq. 2) then
          if (iword(5) .le. 0) then
            write (   6, 8154) ainput, ninput, aword(5)
            write (iout, 8154) ainput, ninput, aword(5)
            go to 999
          endif
          nset = iword(5)
        else
          write (   6, 8155) ainput, ninput, aword(5)
          write (iout, 8155) ainput, ninput, aword(5)
          go to 999
        endif
      endif

c===============================================================================

c.... Test for too many samples, when there is no replacement.

 8148 format ('  sample ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of samples (',i9,') exceeds number of bins (',i5,').')

      if (norepl .eq. 1) then
        nbintot = nsamp * nset
        if (nbintot .gt. nbinpdf(npsamp)) then
          write (   6, 8148) ainput, ninput, nbintot, nbinpdf(npsamp)
          write (iout, 8148) ainput, ninput, nbintot, nbinpdf(npsamp)
          go to 999
        endif
      endif

c===============================================================================

c.... Input line:  sample PDFNAME [NSAMP|1]
c.... Input line:  sample PDFNAME NSAMP [norepl|repl]
c.... Input line:  sample PDFNAME NSAMP [norepl,repl] [NSET|1]

c.... Initialize.

      nvalt    = 0
      valtmean = 0.0
      valtmnsq = 0.0
      valtmin  =  1.e99
      valtmax  = -1.e99

      prpdft(npsamp) = prpdf(npsamp)  ! Total relative probability of pdf.

      do 150 nb = 1, nbins
        prbint(nb) = prbin(nb)        ! Temporary probability of bin.
        nsampb(nb) = 0                ! Number of times bin sampled (all sets).
  150 continue

      do 300 ns = 1, nset             ! Loop over NSET sets of NSAMP samples.

 8156 format (/ 'Set ',i9,'.  norepl = ',i1,'.')
      write (   6, 8156) ns, norepl
      write (iout, 8156) ns, norepl

      nval    = 0
      valmean = 0.0
      valmnsq = 0.0
      valmin  =  1.e99
      valmax  = -1.e99

      do 155 nb = 1, nbins
        nsampb(nb) = 0                ! Number of times bin sampled (this set).
  155 continue

      do 250 n = 1, nsamp             ! Loop over samples for this set.

c.... See if any bins are left in this pdf.

 8158 format ('  sample ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  No bins left in this pdf:  ',a)

      if (prpdft(npsamp) .le. 0.0) then
        write (   6, 8158) ainput, ninput, apdf(nsamp)
        write (iout, 8158) ainput, ninput, apdf(nsamp)
        go to 999
      endif

c.... Randomly sample a bin in this pdf.

      prsamp = prpdft(npsamp) * ranf ()
      prtest = 0.0

      do 170 npx = 1, npdfxs          ! Loop over pdf/bin pairs for this pdf.
        if (apdfx(npx) .eq. ' ') go to 170

        if (apdfx(npx) .eq. apdf(npsamp)) then  ! Found a pair for this pdf.
          if (abinx(npx) .ne. ' ') then  ! Search for this bin name.
            do 160 nb = 1, nbins      ! Loop over bins.
              if (abin(nb) .eq. abinx(npx)) then  ! Found a bin in this pdf.
                prtest = prtest + prbint(nb)
                if (prtest .ge. prsamp) then  ! Sampled a bin.
                  nbsamp = nb
                  go to 180
                else                  ! Try next bin.
                  go to 170
                endif
              endif                   ! Tested a bin.
  160       continue                  ! End of loop over bins.

 8160       format ('  sample ERROR.  Reading ',a40,' line ',i5,'.' /
     &        '  Bin named in pdf does not exist:  ',a)

            write (   6, 8160) ainput, ninput, abinx(npx)
            write (iout, 8160) ainput, ninput, abinx(npx)
            go to 999
          endif                       ! Searched for this bin name.
        endif                         ! Found a pair for this pdf.

  170 continue                        ! End of loop over pdf/bin pairs.

 8170 format ('  sample ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Failed to sample a bin for this pdf.')

      write (   6, 8160) ainput, ninput
      write (iout, 8160) ainput, ninput
      go to 999

c===============================================================================

c.... Found randomly sampled bin.

 8180 format ('  Sample',i8,2x,a,2x,'  bin ',a)   

  180 nb = nbsamp
      nsampb(nb) = nsampb(nb) + 1

      if     (atypeb(nb) .eq. 'discrete event') then
        if (n .le. 26) then
          write (   6, 8180) n, abin(nb), abin(nb)
        endif
        write (iout, 8180) n, abin(nb), abin(nb)
        go to 210

      elseif (atypeb(nb) .eq. 'discrete value') then
        valsamp = vranl(nb)
        valtsamp = valsamp

      elseif (atypeb(nb) .eq. 'uniform') then
        valsamp = vranl(nb) + ranf () * (vranr(nb) - vranl(nb))
        valtsamp = valsamp

      elseif (atypeb(nb) .eq. 'linear') then
        call aptslid (vranl(nb), probl(nb), vranr(nb), probr(nb), 1,
     &                valsamp, nerra)
        valtsamp = valsamp

      elseif (atypeb(nb) .eq. 'exponential') then
        call aptexpl (0, conbin(nb), vranl(nb), vranr(nb), 1,
     &                valsamp, nerr)
        valtsamp = valsamp

      elseif (atypeb(nb) .eq. 'power-law') then
        call aptspod (conbin(nb), vranl(nb), vranr(nb), 1,
     &                valsamp, nerr)
        valtsamp = valsamp

      elseif (atypeb(nb) .eq. 'normal') then
        call aptnorm (0, conbin(nb), cosbin(nb), 1, valsamp, nerr)
        valtsamp = valsamp

      elseif (atypeb(nb) .eq. 'rel Maxwellian') then

 8190   format ('  Sample',i8,1pe20.12,'  bin ',a,2x,1pe20.12)

        call aptmaxw (conbin(nb), 1, 1.0, valsamp, gamma, nerr)
        valtsamp = valsamp

        if (n .le. 26) then
          write (   6, 8190) n, valsamp, abin(nb), gamma
        endif

        write (iout, 8190) n, valsamp, abin(nb), gamma

        nval    = nval + 1
        valmean = valmean + valsamp
        valmnsq = valmnsq + valsamp**2
        if (valsamp .lt. valmin) valmin = valsamp
        if (valsamp .gt. valmax) valmax = valsamp

        nvalt    = nvalt + 1
        valtmean = valtmean + valtsamp
        valtmnsq = valtmnsq + valtsamp**2
        if (valtsamp .lt. valtmin) valtmin = valtsamp
        if (valtsamp .gt. valtmax) valtmax = valtsamp

        go to 210

      elseif (atypeb(nb) .eq. 'Planck') then
        call aptxnup (0, conbin(nb), 1, valsamp, nerr)
        valtsamp = valsamp

      elseif (atypeb(nb) .eq. 'Wien') then
        call aptxnup (1, conbin(nb), 1, valsamp, nerr)
        valtsamp = valsamp

      else

 8220   format ('  sample ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Probability bin type not recognized:  ',a)

        write (   6, 8220) atypeb(nb)
        write (iout, 8220) atypeb(nb)
        go to 210
      endif

c.... Display sampled value, accumulate averages.

 8230 format ('  Sample',i8,1pe20.12,'  bin ',a)

      if (n .le. 26) then
        write (   6, 8230) n, valsamp, abin(nb)
      endif
      write (iout, 8230) n, valsamp, abin(nb)

      nval    = nval + 1
      valmean = valmean + valsamp
      valmnsq = valmnsq + valsamp**2
      if (valsamp .lt. valmin) valmin = valsamp
      if (valsamp .gt. valmax) valmax = valsamp

      nvalt    = nvalt + 1
      valtmean = valtmean + valtsamp
      valtmnsq = valtmnsq + valtsamp**2
      if (valtsamp .lt. valtmin) valtmin = valtsamp
      if (valtsamp .gt. valtmax) valtmax = valtsamp

c.... Test for sampling without replacement.

  210 if (norepl .eq. 1) then         ! No replacement.
cbugc***DEBUG begins.
cbug 9901 format ('sample DEBUG.' /
cbug     &        '  npsamp   =',i3,' apdf=',a,' prpdft=',1pe20.12 /
cbug     &        '  sample # =',i3 /
cbug     &        '  nb   =',i3,' abin=',a,' prbint=',1pe20.12 )
cbug      write (iout, 9901) npsamp, apdf(npsamp), prpdft(npsamp),
cbug     &                   n,
cbug     &                   nb, abin(nb), prbint(nb)
cbugc***DEBUG ends.
        prpdft(npsamp) = prpdft(npsamp) - prbint(nb)
        prerr          = tol * (abs (prpdft(npsamp)) + abs (prbint(nb)))
        prbint(nb)     = 0.0
        if (prpdft(npsamp) .le. prerr) then
cbugc***DEBUG begins.
cbug 9902 format ('sample DEBUG.' /
cbug     &        '  reduced prbint to zero.' )
cbug      write (iout, 9902)
cbugc***DEBUG ends.
          prpdft(npsamp) = 0.0
        endif
      endif

  250 continue                        ! End of loop over samples for this set.

 8250 format (/ )
 8255 format (/ '  Total           ',1pe20.12,'  Samples =',i9 )

      if (nval .lt. 2) then
        write (   6, 8250)
        write (iout, 8250)
      else
        total   = valmean
        write (   6, 8255) total, nval
        write (iout, 8255) total, nval
      endif

 8260 format (  '  Expected total  ',1pe20.12 )

      if (vexpdf(npsamp) .ne. -1.e99) then
        vexpdft = nval * vexpdf(npsamp)
        write (   6, 8260) vexpdft
        write (iout, 8260) vexpdft
      endif

 8265 format (/ '  Mean, Std Dev   ',1p2e20.12 )

      if (nval .ge. 2) then
        valmean = valmean / nval
        valmnsq = valmnsq / nval
        stddev  = sqrt (valmnsq - valmean**2)
        write (   6, 8265) valmean, stddev
        write (iout, 8265) valmean, stddev
      endif

 8270 format (  '  Expected value  ',1pe20.12 )    

      if (vexpdf(npsamp) .ne. -1.e99) then
        write (   6, 8270) vexpdf(npsamp)
        write (iout, 8270) vexpdf(npsamp)
      endif

 8275 format (  '  Minimum, Maximum',1p2e20.12 )

      if (nval .ge. 2) then
        write (   6, 8275) valmin, valmax
        write (iout, 8275) valmin, valmax
      endif

 8280 format (/ '  Samples displayed in geom.hsp:  ',i8)

      if (nsamp .gt. 26) then
        write (   6, 8280) nsamp
      endif

c.... Display number of times each bin sampled.

 8285 format (/ 'Total number of times each bin sampled:')
 8290 format ('bin  ',a,x,a,' Prob',1pe20.12 / '  #:  ',i8)

        write (   6, 8285)
        write (iout, 8285)

      do 290 nb = 1, nbins
        if (nsampb(nb) .le. 0) go to 290
        write (   6, 8290) abin(nb), atypeb(nb), prbin(nb), nsampb(nb)
        write (iout, 8290) abin(nb), atypeb(nb), prbin(nb), nsampb(nb)
  290 continue

  300 continue                    ! End of loop over NSET sets of NSAMP samples.

 8310 format (/ '  Total, all sets ',1pe20.12,'  Samples =',i9 )

      if ((nset .ge. 2) .and. (nvalt .ge. 2)) then

        total = valtmean
        write (   6, 8310) total, nvalt

        if (vexpdf(npsamp) .ne. -1.e99) then
          vexpdft = nvalt * vexpdf(npsamp)
          write (   6, 8260) vexpdft
          write (iout, 8260) vexpdft
        endif

        valtmean = valtmean / nvalt
        valtmnsq = valtmnsq / nvalt
        stddev   = sqrt (valtmnsq - valtmean**2)
        write (   6, 8265) valtmean, stddev
        write (iout, 8265) valtmean, stddev

        if (vexpdf(npsamp) .ne. -1.e99) then
          write (   6, 8270) vexpdf(npsamp)
          write (iout, 8270) vexpdf(npsamp)
        endif

        write (   6, 8275) valtmin, valtmax
        write (iout, 8275) valtmin, valtmax
      endif

c===============================================================================

  999 return

c.... End of subroutine sample.       (+1 line)
      end
