      subroutine slice

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  slice    Updated     2004 November 23 13:30.
c               slice    Originated  1994 April 6 16:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To slice thru a zone with a plane parallel to a major plane,
c                 finding the equations and any extreme values of any
c                 intersections with the quadric surfaces bounding the zone.
c
c     Logic:    Entry points:  slice.
c slice         Called by:  helpall, main.
c
c slice         Calls:  aptaxis, aptplis, aptqexc, plotpt, sliceq, varfind,
c               zout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      common /lslice/ rotm(3,3)       ! Principle axis rotation tensor.
      common /lslice/ ex(4)           ! Coordinate x of an extreme point.
      common /lslice/ ey(4)           ! Coordinate y of an extreme point.
      common /lslice/ ez(4)           ! Coordinate z of an extreme point.
      common /lslice/ adir(4)         ! Direction of normal of extreme point.
      character*8     adir            ! Direction of normal of extreme point.

c.... Input line:  slice [help]

 8110 format (
     &  '  Command "slice":  acceptable input lines include:' /
     &  '  "slice [help]"  Display this message.' //
     &  '  "slice quadric QNAME [x,y,z|x] [VAL|0] [[y,z,x]',
     &  ' VMIN DV VMAX]' /
     &  '     Slice quadric QNAME at [x,y,z|x] = [VAL|0],' /
     &  '     and optionally, find any points in the resulting',
     &  ' quadric curve' /
     &  '     at [y, z, x] = VMIN (DV) VMAX.' /
     &  '  "slice zone ZNAME [x,y,z|x] [VAL|0.0]"' /
     &  '     Slice zone ZNAME at [x,y,z|x] = [VAL|0].' )
 8112 format (/
     &  '  Synonyms:  [help, h], [quadric, quad, q], [slice, cut],',
     &  ' [zone, zn, z].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        go to 999
      endif

c=======================================================================********

c.... Input line:  slice quadric QNAME [x,y,z|x] [VAL|0] [[y,z,x] VMIN DV VMAX]

      if ((aword(2) .eq. 'quadric') .or.
     &    (aword(2) .eq. 'quad')    .or.
     &    (aword(2) .eq. 'q'))      then

        call sliceq
        go to 999
      endif

c=======================================================================********

c.... Input line:  slice zone ZNAME [x,y,z|x] [VAL|0]

 8130 format ('  slice ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words not 3-5 on input line',i4)

      if ((nword .lt. 3) .or. (nword .gt. 5)) then
        write (   6, 8130) ainput, ninput, nword
        write (iout, 8130) ainput, ninput, nword
        go to 999
      endif

c.... See if the zone exists.

 8140 format ('  slice ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named zone does not exist:  ',a)

      do 140 nz = 1, nzones
        if (aword(3) .eq. azone(nz)) then
          call zout (nz, nz, 'min')
          go to 150
        endif
  140 continue
      write (   6, 8140) ainput, ninput, aword(3)
      write (iout, 8140) ainput, ninput, aword(3)
      go to 999

c.... Change missing words to default values.

  150 if (nword .lt. 4) then
        nword = 4
        mtype(4) = 1
        aword(4) = 'x'
      endif

      if (nword .lt. 5) then
        nword = 5
        mtype(5) = 3
        fword(5) = 0.0
      endif

c.... Find the value of word 5.

 8160 format ('  slice ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word 5 not integer or floating point value or variable:  ',a)

      call varfind (5, 5)

      if ((mtype(5) .ne. 2) .and. (mtype(5) .ne. 3)) then
        write (   6, 8160) ainput, ninput, aword(5)
        write (iout, 8160) ainput, ninput, aword(5)
        go to 999
      endif

c.... Find the major plane parallel to the slice.

 8170 format ('  slice ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word 4 is not x, y or z')

      if ((aword(4) .eq. 'x')  .or.
     &    (aword(4) .eq. 'X')) then
        kpl = 1
      elseif ((aword(4) .eq. 'y')  .or.
     &        (aword(4) .eq. 'Y')) then
        kpl = 2
      elseif ((aword(4) .eq. 'z')  .or.
     &        (aword(4) .eq. 'Z')) then
        kpl = 3
      else
        write (   6, 8170) ainput, ninput, aword(4)
        write (iout, 8170) ainput, ninput, aword(4)
        go to 999
      endif

c.... Find the quadric surfaces bounding this zone.

 8180 format ('  slice ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Zone ',a,' has undefined boundary',i3,':  ',a)

      do 360 nn = 1, nzquads(nz)      ! Loop over zone boundaries.

        do 180 nq = 1, nquads
          if (aquad(nq) .eq. azquad(nn,nz)) go to 190
  180   continue
        write (   6, 8180) ainput, ninput, azone(nz), nn,
     &    azquad(nn,nz)
        write (iout, 8180) ainput, ninput, azone(nz), nn,
     &    azquad(nn,nz)
        go to 999

  190   continue

c=======================================================================********

c.... Find the slice thru the quadric.

        call aptplis (kpl, fword(5),
     &                qc(nq), qx(nq), qy(nq), qz(nq),
     &                qxy(nq), qyz(nq), qzx(nq),
     &                qxx(nq), qyy(nq), qzz(nq), 1, tol,
     &                ac, ax, ay, az, axy, ayz, azx,
     &                axx, ayy, azz, nerra)

 8200   format (/ '  Slice thru quadric ',a,' at ',a1,
     &    ' = ',1pe20.12 )

        write (   6, 8200) aquad(nq), aword(4), fword(5)
        write (iout, 8200) aquad(nq), aword(4), fword(5)

 8220   format ('    qc            ',1pe20.12 )
 8230   format ('    qx,  qy,  qz  ',1p3e20.12 )
 8240   format ('    qxy, qyz, qzx ',1p3e20.12 )
 8250   format ('    qxx, qyy, qzz ',1p3e20.12 )

        alinear = ax**2  + ay**2  + az**2
        across  = axy**2 + ayz**2 + azx**2
        asquare = axx**2 + ayy**2 + azz**2

        write (   6, 8220) ac
        write (iout, 8220) ac
        if (alinear .gt. 0.0) then
          write (   6, 8230) ax, ay, az
          write (iout, 8230) ax, ay, az
        endif
        if (across .gt. 0.0) then
          write (   6, 8240) axy, ayz, azx
          write (iout, 8240) axy, ayz, azx
        endif
        if (asquare .gt. 0.0) then
          write (   6, 8250) axx, ayy, azz
          write (iout, 8250) axx, ayy, azz
        endif

c=======================================================================********

c....   Find any extremities of the curve.

        if (kpl .eq. 1) then          ! At constant x.

          call aptqexc (ac, ay, az, ayz, ayy, azz, tol,
     &                  np, ey, ez, adir, nerra)

          do 310 n = 1, np
            ex(n) = fword(5)
            if (adir(n) .eq. 'ucon') adir(n) = 'Const y'
            if (adir(n) .eq. 'uint') adir(n) = 'Inter y'
            if (adir(n) .eq. 'umax') adir(n) = 'Max y'
            if (adir(n) .eq. 'umin') adir(n) = 'Min y'
            if (adir(n) .eq. 'vcon') adir(n) = 'Const z'
            if (adir(n) .eq. 'vint') adir(n) = 'Inter z'
            if (adir(n) .eq. 'vmax') adir(n) = 'Max z'
            if (adir(n) .eq. 'vmin') adir(n) = 'Min z'
  310     continue

        elseif (kpl .eq. 2) then      ! At constant y.

          call aptqexc (ac, az, ax, azx, azz, axx, tol,
     &                  np, ez, ex, adir, nerra)

          do 320 n = 1, np
            ey(n) = fword(5)
            if (adir(n) .eq. 'ucon') adir(n) = 'Const z'
            if (adir(n) .eq. 'uint') adir(n) = 'Inter z'
            if (adir(n) .eq. 'umax') adir(n) = 'Max z'
            if (adir(n) .eq. 'umin') adir(n) = 'Min z'
            if (adir(n) .eq. 'vcon') adir(n) = 'Const x'
            if (adir(n) .eq. 'vint') adir(n) = 'Inter x'
            if (adir(n) .eq. 'vmax') adir(n) = 'Max x'
            if (adir(n) .eq. 'vmin') adir(n) = 'Min x'
  320     continue

        else                          ! At constant z.

          call aptqexc (ac, ax, ay, axy, axx, ayy, tol,
     &                  np, ex, ey, adir, nerra)

          do 330 n = 1, np
            ez(n) = fword(5)
            if (adir(n) .eq. 'ucon') adir(n) = 'Const x'
            if (adir(n) .eq. 'uint') adir(n) = 'Inter x'
            if (adir(n) .eq. 'umax') adir(n) = 'Max x'
            if (adir(n) .eq. 'umin') adir(n) = 'Min x'
            if (adir(n) .eq. 'vcon') adir(n) = 'Const y'
            if (adir(n) .eq. 'vint') adir(n) = 'Inter y'
            if (adir(n) .eq. 'vmax') adir(n) = 'Max y'
            if (adir(n) .eq. 'vmin') adir(n) = 'Min y'
  330     continue

        endif                         ! Tested kpl.

 8340   format ('  ',a,'   (xyz)',1p3e20.12 )

        do 340 n = 1, np
          write (   6, 8340) adir(n), ex(n), ey(n), ez(n)
          write (iout, 8340) adir(n), ex(n), ey(n), ez(n)
          call plotpt (ex(n), ey(n), ez(n), nerrg)
  340   continue

c=======================================================================********

c....   Do the principle axis transformation on the quadric surface.

        call aptaxis (ac, ax, ay, az, axy, ayz, azx, axx, ayy, azz,
     &                tol, xcen, ycen, zcen, rotm, ntype, nerra)

 8345   format ('  ***NO INTERSECTION***' )
 8350   format ('  ***NO INTERSECTION***' / 18x,a)
 8355   format ('  ***TANGENT PLANE***' )
 8360   format ('  ***TANGENT PLANE***' / 18x,a)

        if (nerra .ne. 0) then
          if (np .eq. 0) then
            write (   6, 8345)
            write (iout, 8345)
          else
            write (   6, 8355)
            write (iout, 8355)
          endif
        endif

        if (ntype .gt. 16) then
          if (np .eq. 0) then
            write (   6, 8350) aqtypes(ntype)
            write (iout, 8350) aqtypes(ntype)
          else
            write (   6, 8360) aqtypes(ntype)
            write (iout, 8360) aqtypes(ntype)
          endif
        endif

c=======================================================================********

  360 continue                        ! End of loop over zone boundaries.

c=======================================================================********

  999 return

c.... End of subroutine slice.        (+1 line)
      end
