      subroutine sliceq

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  sliceq   Updated     2004 November 23 13:30.
c               sliceq   Originated  1994 April 6 16:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find points in a quadric surface at a specified value of one
c                 coordinate, for a range of values of a second coordinate.
c
c     Logic:    Entry points:  sliceq.
c sliceq        Called by:  slice.
c
c sliceq        Calls:  aptaxis, aptplis, aptqexc, aptqrts, plotpt, quout,
c               varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      common /lsliceq/ rotm(3,3)      ! Principle axis rotation tensor.
      common /lsliceq/ ex(4)          ! Coordinate x of an extreme point.
      common /lsliceq/ ey(4)          ! Coordinate y of an extreme point.
      common /lsliceq/ ez(4)          ! Coordinate z of an extreme point.
      common /lsliceq/ adir(4)        ! Direction of normal of extreme point.
      character*8      adir           ! Direction of normal of extreme point.

c.... Input line:  slice [help]

 8110 format (
     &  '  Command "slice":  acceptable input lines include:' /
     &  '  "slice [help]"  Display this message.' //
     &  '  "slice quadric QNAME [x,y,z|x] [VAL|0] [[y,z,x]',
     &  ' VMIN DV VMAX]' /
     &  '     Slice quadric QNAME at [x,y,z|x] = [VAL|0],' /
     &  '     and optionally, find any points in the resulting',
     &  ' quadric curve' /
     &  '     at [y, z, x] = VMIN (DV) VMAX.' /
     &  '  "slice zone ZNAME [x,y,z|x] [VAL|0.0]"' /
     &  '     Slice zone ZNAME at [x,y,z|x] = [VAL|0].' )
 8112 format (/
     &  '  Synonyms:  [help, h], [quadric, quad, q], [slice, cut],',
     &  ' [zone, zn, z].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        go to 999
      endif

c=======================================================================********

c.... Input line:  slice quadric QNAME [x,y,z|x] [VAL|0] [y,z,x] VMIN DV VMAX

 8130 format ('  sliceq ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words not 3-9 on input line',i4)

      if ((nword .lt. 3) .or. (nword .gt. 9)) then 
        write (   6, 8130) ainput, ninput, nword
        write (iout, 8130) ainput, ninput, nword
        go to 999
      endif

c.... See if the quadric exists.

 8140 format ('  sliceq ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named quadric surface does not exist:  ',a)

      do 140 nq = 1, nquads
        if (aword(3) .eq. aquad(nq)) then
          call quout (nq, nq, 'min', nerrq)
          go to 150
        endif
  140 continue
      write (   6, 8140) ainput, ninput, aword(3)
      write (iout, 8140) ainput, ninput, aword(3)
      go to 999

c.... Find the constant coordinate.

 8150 format ('  sliceq ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Constant coordinate not x, y or z:  ',a)

  150 if (nword .lt. 4) then
        nword = 4
        aword(4) = 'x'
      endif

      if ((aword(4) .ne. 'x')  .and.
     &    (aword(4) .ne. 'X')  .and.
     &    (aword(4) .ne. 'y')  .and.
     &    (aword(4) .ne. 'Y')  .and.
     &    (aword(4) .ne. 'z')  .and.
     &    (aword(4) .ne. 'Z')) then
        write (   6, 8140) ainput, ninput, aword(4)
        write (iout, 8140) ainput, ninput, aword(4)
        go to 999
      endif

c.... Find the value of word 5, the constant coordinate.

       if (nword .lt. 5) then
         nword    = 5
         mtype(5) = 3
         fword(5) = 0.0
       endif

 8160 format ('  sliceq ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word 5 not integer or floating point value or variable:  ',a)

      call varfind (5, 5)

      if ((mtype(5) .ne. 2) .and. (mtype(5) .ne. 3)) then
        write (   6, 8160) ainput, ninput, aword(5)
        write (iout, 8160) ainput, ninput, aword(5)
        go to 999
      endif

      if (nword .eq. 5) go to 210

c=======================================================================********

c.... Find the varying coordinate.

 8170 format ('  sliceq ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Variable coordinate not x, y or z:  ',a)

      if ((aword(6) .ne. 'x')  .and.
     &    (aword(6) .ne. 'X')  .and.
     &    (aword(6) .ne. 'y')  .and.
     &    (aword(6) .ne. 'Y')  .and.
     &    (aword(6) .ne. 'z')  .and.
     &    (aword(6) .ne. 'Z')) then
        write (   6, 8170) ainput, ninput, aword(4)
        write (iout, 8170) ainput, ninput, aword(4)
        go to 999
      endif

 8180 format ('  sliceq ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Constant and varying coordinates the same:  ',a)

      if (aword(6) .eq. aword(4)) then
        write (   6, 8180) ainput, ninput, aword(6)
        write (iout, 8180) ainput, ninput, aword(6)
        go to 999
      endif

c.... Find the values of words 7-9, the range and increment of the varying
c....   coordinate.

      if (nword .eq. 6) then
        nword    = 7
        mtype(7) = 3
        fword(7) = 0.0
      endif

      if (nword .eq. 7) then
        nword    = 8
        mtype(8) = 3
        fword(8) = 1.e99
      endif

      if (nword .eq. 8) then
        nword    = 9
        mtype(9) = mtype(7)
        aword(9) = aword(7)
        fword(9) = fword(7)
      endif

 8190 format ('  sliceq ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word 7, 8 or 9 not integer or floating point value or',
     &  ' variable:  ',a)

      call varfind (7, 9)

      do 200 nw = 7, 9
        if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
          write (   6, 8190) ainput, ninput, aword(nw)
          write (iout, 8190) ainput, ninput, aword(nw)
          go to 999
        endif
  200 continue

 8200 format ('  sliceq ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word 8 (DV) is not positive:  ',1pe20.12)

      if (fword(8) .le. 0.0) then
        write (   6, 8200) ainput, ninput, fword(8)
        write (iout, 8200) ainput, ninput, fword(8)
        go to 999
      endif

 8205 format ('  sliceq ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word 7 (VMIN)',1pe20.12,' > word 9 (VMAX)',1pe20.12)

      if (fword(9) .lt. fword(7)) then
        write (   6, 8205) ainput, ninput, fword(7), fword(9)
        write (iout, 8205) ainput, ninput, fword(7), fword(9)
        go to 999
      endif

c=======================================================================********

c.... Initialize point counters.

  210 nx = 0
      ny = 0
      nz = 0

c.... Find the flag for the constant coordinate.

      if     ((aword(4) .eq. 'x')  .or.
     &        (aword(4) .eq. 'X')) then
        kcon = 1
        x1   = fword(5)
        nx   = 1
      elseif ((aword(4) .eq. 'y')  .or.
     &        (aword(4) .eq. 'Y')) then
        kcon = 2
        y1   = fword(5)
        ny   = 1
      elseif ((aword(4) .eq. 'z')  .or.
     &        (aword(4) .eq. 'Z')) then
        kcon = 3
        z1   = fword(5)
        nz   = 1
      endif

c.... Find the coefficients of the quadric curve at the value of the
c....   constant coordinate.

      call aptplis (kcon, fword(5), qc(nq), qx(nq), qy(nq), qz(nq),
     &              qxy(nq), qyz(nq), qzx(nq),
     &              qxx(nq), qyy(nq), qzz(nq), 1, tol,
     &              sc, sx, sy, sz, sxy, syz, szx, sxx, syy, szz, nerr)

 8214   format (/ '  Slice thru quadric ',a,' at ',a1,
     &    ' = ',1pe20.12 )

        write (   6, 8214) aquad(nq), aword(4), fword(5)
        write (iout, 8214) aquad(nq), aword(4), fword(5)

 8215 format ('  Slice       sc  ',1pe20.12 /
     &        '    sx,  sy,  sz  ',1p3e20.12 /
     &        '    sxy, syz, szx ',1p3e20.12 /
     &        '    sxx, syy, szz ',1p3e20.12 )

      write (   6, 8215) sc, sx, sy, sz, sxy, syz, szx, sxx, syy, szz
      write (iout, 8215) sc, sx, sy, sz, sxy, syz, szx, sxx, syy, szz

c=======================================================================********

c.... Find any extremities of the quadric curve at the value of the
c....   constant coordinate.

      if (kcon .eq. 1) then          ! At constant x.

        call aptqexc (sc, sy, sz, syz, syy, szz, tol,
     &                np, ey, ez, adir, nerra)

        do 220 n = 1, np
          ex(n) = fword(5)
          if (adir(n) .eq. 'ucon') adir(n) = 'Const y'
          if (adir(n) .eq. 'uint') adir(n) = 'Inter y'
          if (adir(n) .eq. 'umax') adir(n) = 'Max y'
          if (adir(n) .eq. 'umin') adir(n) = 'Min y'
          if (adir(n) .eq. 'vcon') adir(n) = 'Const z'
          if (adir(n) .eq. 'vint') adir(n) = 'Inter z'
          if (adir(n) .eq. 'vmax') adir(n) = 'Max z'
          if (adir(n) .eq. 'vmin') adir(n) = 'Min z'
  220   continue

      elseif (kcon .eq. 2) then      ! At constant y.

        call aptqexc (sc, sz, sx, szx, szz, sxx, tol,
     &                np, ez, ex, adir, nerra)

        do 230 n = 1, np
          ey(n) = fword(5)
          if (adir(n) .eq. 'ucon') adir(n) = 'Const z'
          if (adir(n) .eq. 'uint') adir(n) = 'Inter z'
          if (adir(n) .eq. 'umax') adir(n) = 'Max z'
          if (adir(n) .eq. 'umin') adir(n) = 'Min z'
          if (adir(n) .eq. 'vcon') adir(n) = 'Const x'
          if (adir(n) .eq. 'vint') adir(n) = 'Inter x'
          if (adir(n) .eq. 'vmax') adir(n) = 'Max x'
          if (adir(n) .eq. 'vmin') adir(n) = 'Min x'
  230   continue

      else                          ! At constant z.

        call aptqexc (sc, sx, sy, sxy, sxx, syy, tol,
     &                np, ex, ey, adir, nerra)

        do 240 n = 1, np
          ez(n) = fword(5)
          if (adir(n) .eq. 'ucon') adir(n) = 'Const x'
          if (adir(n) .eq. 'uint') adir(n) = 'Inter x'
          if (adir(n) .eq. 'umax') adir(n) = 'Max x'
          if (adir(n) .eq. 'umin') adir(n) = 'Min x'
          if (adir(n) .eq. 'vcon') adir(n) = 'Const y'
          if (adir(n) .eq. 'vint') adir(n) = 'Inter y'
          if (adir(n) .eq. 'vmax') adir(n) = 'Max y'
          if (adir(n) .eq. 'vmin') adir(n) = 'Min y'
  240   continue

      endif                         ! Tested kcon.

 8250 format ('  ',a,'   (xyz)',1p3e20.12 )

      do 250 n = 1, np
        write (   6, 8250) adir(n), ex(n), ey(n), ez(n)
        write (iout, 8250) adir(n), ex(n), ey(n), ez(n)
        call plotpt (ex(n), ey(n), ez(n), nerrg)
  250 continue

c=======================================================================********

c.... Do the principle axis transformation on the quadric surface.

      ac  = sc
      ax  = sx
      ay  = sy
      az  = sz
      axy = sxy
      ayz = syz
      azx = szx
      axx = sxx
      ayy = syy
      azz = szz

      call aptaxis (ac, ax, ay, az, axy, ayz, azx, axx, ayy, azz,
     &              tol, xcen, ycen, zcen, rotm, ntype, nerra)

 8260 format ('  ***NO INTERSECTION***' / 18x,a)
 8265 format ('  ***NO QUADRIC SURFACE***' )
 8270 format ('  ***TANGENT PLANE***' / 18x,a)

      if (nerra .ne. 0) then
        if (ntype .lt. 0) then
          write (   6, 8265)
          write (iout, 8265)
        elseif (ntype .gt. 16) then
          if (np .eq. 0) then
            write (   6, 8260) aqtypes(ntype)
            write (iout, 8260) aqtypes(ntype)
          else
            write (   6, 8270) aqtypes(ntype)
            write (iout, 8270) aqtypes(ntype)
          endif
        endif
      endif

      if (nword .eq. 5) go to 300

c=======================================================================********

 8275   format (/ '  Slice thru quadric ',a,' at ',a1,
     &    ' = ',1pe20.12 /
     &    '  ',a1,'  (min,int,max)',1p3e20.12 /)

      write (   6, 8275) aquad(nq), aword(4), fword(5),
     &  aword(6), fword(7), fword(8), fword(9)
      write (iout, 8275) aquad(nq), aword(4), fword(5),
     &  aword(6), fword(7), fword(8), fword(9)

c.... Find the flag for the varying coordinate.

      if     ((aword(6) .eq. 'x')  .or.
     &        (aword(6) .eq. 'X')) then
        kxyz = 1
      elseif ((aword(6) .eq. 'y')  .or.
     &        (aword(6) .eq. 'Y')) then
        kxyz = 2
      elseif ((aword(6) .eq. 'z')  .or.
     &        (aword(6) .eq. 'Z')) then
        kxyz = 3
      endif

c.... Loop over the values of the varying coordinate.
c....   Find the value of the dependent coordinate.

 8280 format (/ '  Points of intersection:' //
     &          '  coord  rect',7x,'x',19x,'y',19x,'z')

      write (   6, 8280)
      write (iout, 8280)

      val    = fword(7)
      valerr = tol * abs (fword(8))
      nr     = 0                      ! Number of coord values with roots.

      do 280 nv = 1, 10000            ! Loop over varying coordinate values.

        if (kxyz .eq. 1) then
          x1 = val
          nx = nx + 1
        elseif (kxyz .eq. 2) then
          y1 = val
          ny = ny + 1
        else
          z1 = val
          nz = nz + 1
        endif

c....   Find the coefficients of the equation at the value of the
c....     varying coordinate.

        call aptplis (kxyz, val, sc, sx, sy, sz, sxy, syz, szx,
     &                sxx, syy, szz, 1, tol,
     &                ac, ax, ay, az, axy, ayz, azx, axx, ayy, azz,
     &                nerr)
cbugc____ DEBUG begins.
cbug 8278 format ('  Equation    ac  ',1pe20.12 /
cbug     &        '    ax,  ay,  az  ',1p3e20.12 /
cbug     &        '    axy, ayz, azx ',1p3e20.12 /
cbug     &        '    axx, ayy, azz ',1p3e20.12 )
cbug
cbug      write (   6, 8278) ac, ax, ay, az, axy, ayz, azx, axx, ayy, azz
cbug      write (iout, 8278) ac, ax, ay, az, axy, ayz, azx, axx, ayy, azz
cbugc____ DEBUG ends.

c....   Find any remaining unknown coordinate and solve for it.

        if     ((ax  .ne. 0.0)  .or.
     &          (axx .ne. 0.0)) then

          aa = axx
          bb = ax
          cc = ac

          call aptqrts (0, aa, bb, cc, qq, tol, nroots, x1, x2, itrun)

 8281     format ('  p qa(',i1,')       = ',1p3e20.12)            
 8282     format ('  p qa(',i2,')      = ',1p3e20.12)            
 8283     format ('  p qa(',i3,')     = ',1p3e20.12)            
 8284     format ('  p qa(',i4,')    = ',1p3e20.12)            
 8285     format ('  p qa(',i5,')   = ',1p3e20.12)            
 8286     format ('  p qa(',i6,')  = ',1p3e20.12)            
 8287     format ('  p qa(',i7,') = ',1p3e20.12)            

 8291     format ('  p qb(',i1,')       = ',1p3e20.12)            
 8292     format ('  p qb(',i2,')      = ',1p3e20.12)            
 8293     format ('  p qb(',i3,')     = ',1p3e20.12)            
 8294     format ('  p qb(',i4,')    = ',1p3e20.12)            
 8295     format ('  p qb(',i5,')   = ',1p3e20.12)            
 8296     format ('  p qb(',i6,')  = ',1p3e20.12)            
 8297     format ('  p qb(',i7,') = ',1p3e20.12)            

          if (nroots .ge. 1) then
            nx = nx + 1
            nr = nr + 1
            if (nr .lt. 10) then
              write (   6, 8281) nr, x1, y1, z1
              write (iout, 8281) nr, x1, y1, z1
            elseif (nr .lt. 100) then
              write (   6, 8282) nr, x1, y1, z1
              write (iout, 8282) nr, x1, y1, z1
            elseif (nr .lt. 1000) then
              write (   6, 8283) nr, x1, y1, z1
              write (iout, 8283) nr, x1, y1, z1
            elseif (nr .lt. 10000) then
              write (   6, 8284) nr, x1, y1, z1
              write (iout, 8284) nr, x1, y1, z1
            elseif (nr .lt. 100000) then
              write (   6, 8285) nr, x1, y1, z1
              write (iout, 8285) nr, x1, y1, z1
            elseif (nr .lt. 1000000) then
              write (   6, 8286) nr, x1, y1, z1
              write (iout, 8286) nr, x1, y1, z1
            else
              write (   6, 8287) nr, x1, y1, z1
              write (iout, 8287) nr, x1, y1, z1
            endif
            call plotpt (x1, y1, z1, nerrg)
          endif

          if (nroots .eq. 2) then
            nx = nx + 1
            if (nr .lt. 10) then
              write (   6, 8291) nr, x2, y1, z1
              write (iout, 8291) nr, x2, y1, z1
            elseif (nr .lt. 100) then
              write (   6, 8292) nr, x2, y1, z1
              write (iout, 8292) nr, x2, y1, z1
            elseif (nr .lt. 1000) then
              write (   6, 8293) nr, x2, y1, z1
              write (iout, 8293) nr, x2, y1, z1
            elseif (nr .lt. 10000) then
              write (   6, 8294) nr, x2, y1, z1
              write (iout, 8294) nr, x2, y1, z1
            elseif (nr .lt. 100000) then
              write (   6, 8295) nr, x2, y1, z1
              write (iout, 8295) nr, x2, y1, z1
            elseif (nr .lt. 1000000) then
              write (   6, 8296) nr, x2, y1, z1
              write (iout, 8296) nr, x2, y1, z1
            else
              write (   6, 8297) nr, x2, y1, z1
              write (iout, 8297) nr, x2, y1, z1
            endif
            call plotpt (x2, y1, z1, nerrg)
          endif

        elseif ((ay .ne. 0.0)   .or.
     &          (ayy .ne. 0.0)) then

          aa = ayy
          bb = ay
          cc = ac

          call aptqrts (0, aa, bb, cc, qq, tol, nroots, y1, y2, itrun)

          if (nroots .ge. 1) then
            ny = ny + 1
            nr = nr + 1
            if (nr .lt. 10) then
              write (   6, 8281) nr, x1, y1, z1
              write (iout, 8281) nr, x1, y1, z1
            elseif (nr .lt. 100) then
              write (   6, 8282) nr, x1, y1, z1
              write (iout, 8282) nr, x1, y1, z1
            elseif (nr .lt. 1000) then
              write (   6, 8283) nr, x1, y1, z1
              write (iout, 8283) nr, x1, y1, z1
            elseif (nr .lt. 10000) then
              write (   6, 8284) nr, x1, y1, z1
              write (iout, 8284) nr, x1, y1, z1
            elseif (nr .lt. 100000) then
              write (   6, 8285) nr, x1, y1, z1
              write (iout, 8285) nr, x1, y1, z1
            elseif (nr .lt. 1000000) then
              write (   6, 8286) nr, x1, y1, z1
              write (iout, 8286) nr, x1, y1, z1
            else
              write (   6, 8287) nr, x1, y1, z1
              write (iout, 8287) nr, x1, y1, z1
            endif
            call plotpt (x1, y1, z1, nerrg)
          endif

          if (nroots .eq. 2) then
            ny = ny + 1
            if (nr .lt. 10) then
              write (   6, 8291) nr, x1, y2, z1
              write (iout, 8291) nr, x1, y2, z1
            elseif (nr .lt. 100) then
              write (   6, 8292) nr, x1, y2, z1
              write (iout, 8292) nr, x1, y2, z1
            elseif (nr .lt. 1000) then
              write (   6, 8293) nr, x1, y2, z1
              write (iout, 8293) nr, x1, y2, z1
            elseif (nr .lt. 10000) then
              write (   6, 8294) nr, x1, y2, z1
              write (iout, 8294) nr, x1, y2, z1
            elseif (nr .lt. 100000) then
              write (   6, 8295) nr, x1, y2, z2
              write (iout, 8295) nr, x1, y2, z2
            elseif (nr .lt. 1000000) then
              write (   6, 8296) nr, x1, y2, z2
              write (iout, 8296) nr, x1, y2, z2
            else
              write (   6, 8297) nr, x1, y2, z1
              write (iout, 8297) nr, x1, y2, z1
            endif
            call plotpt (x1, y2, z1, nerrg)
          endif

        elseif ((az .ne. 0.0)   .or.
     &          (azz .ne. 0.0)) then

          aa = azz
          bb = az
          cc = ac

          call aptqrts (0, aa, bb, cc, qq, tol, nroots, z1, z2, itrun)

          if (nroots .ge. 1) then
            nz = nz + 1
            nr = nr + 1
            if (nr .lt. 10) then
              write (   6, 8281) nr, x1, y1, z1
              write (iout, 8281) nr, x1, y1, z1
            elseif (nr .lt. 100) then
              write (   6, 8282) nr, x1, y1, z1
              write (iout, 8282) nr, x1, y1, z1
            elseif (nr .lt. 1000) then
              write (   6, 8283) nr, x1, y1, z1
              write (iout, 8283) nr, x1, y1, z1
            elseif (nr .lt. 10000) then
              write (   6, 8284) nr, x1, y1, z1
              write (iout, 8284) nr, x1, y1, z1
            elseif (nr .lt. 100000) then
              write (   6, 8285) nr, x1, y1, z1
              write (iout, 8285) nr, x1, y1, z1
            elseif (nr .lt. 1000000) then
              write (   6, 8286) nr, x1, y1, z1
              write (iout, 8286) nr, x1, y1, z1
            else
              write (   6, 8287) nr, x1, y1, z1
              write (iout, 8287) nr, x1, y1, z1
            endif
            call plotpt (x1, y1, z1, nerrg)
          endif

          if (nroots .eq. 2) then
            nz = nz + 1
            if (nr .lt. 10) then
              write (   6, 8291) nr, x1, y1, z2
              write (iout, 8291) nr, x1, y1, z2
            elseif (nr .lt. 100) then
              write (   6, 8292) nr, x1, y1, z2
              write (iout, 8292) nr, x1, y1, z2
            elseif (nr .lt. 1000) then
              write (   6, 8293) nr, x1, y1, z2
              write (iout, 8293) nr, x1, y1, z2
            elseif (nr .lt. 10000) then
              write (   6, 8294) nr, x1, y1, z2
              write (iout, 8294) nr, x1, y1, z2
            elseif (nr .lt. 100000) then
              write (   6, 8295) nr, x1, y1, z2
              write (iout, 8295) nr, x1, y1, z2
            elseif (nr .lt. 1000000) then
              write (   6, 8296) nr, x1, y1, z2
              write (iout, 8296) nr, x1, y1, z2
            else
              write (   6, 8297) nr, x1, y1, z2
              write (iout, 8297) nr, x1, y1, z2
            endif
            call plotpt (x1, y1, z2, nerrg)
          endif

        else

        endif

        if (val .ge. fword(9)) go to 300
        val = val + fword(8)
        if (abs (val) .lt. valerr) val = 0.0
        if (val .gt. fword(9)) go to 300

  280 continue                        ! End of loop over varying coordinate.

 8300 format (/ '  Number of values:  nx =',i5,11x,'ny =',i5,11x,
     &  'nz =',i5)

  300 write (   6, 8300) nx, ny, nz
      write (iout, 8300) nx, ny, nz

c=======================================================================********

  999 return

c.... End of subroutine sliceq.       (+1 line)
      end
