      subroutine sudo

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  sudo     Updated     2006 March 13 16:20.
c               sudo     Originated  2006 Februry 27 14:30.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To solve a Sudoku puzzle.  Given a 9x9 grid of cells, with
c               9 rows, 9 columns, and 9 3x3 boxes, put one of the digits 1-9
c               in each cell, such that each row, column and box contains all
c               9 digits, with no repeats.  Some of the digits must initially
c               be specified.
c
c     Logic:    Entry points:  sudo.
c               Called by:  helpall, main.
c               Calls:  aptsudo, varfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      integer     icell(9,9)        ! Integer equivalent of acell.
      integer     icells(9,9)       ! Integer equivalent of acell.
      integer     ichar             ! Index of acell in achars.
      integer     idigs             ! Index of character inkperm.
      integer     kperm(9,9,9)      ! Permission matrix.
      integer     ncold             ! Column index of duplicated character.
      integer     nerra             ! Error flag from aptsudo.
      integer     nrowd             ! Row index of duplicated character.
      integer     npermit(9,9)      ! Number of digits permitted in a cell.
      integer     nwords            ! Initial number of command line words.

      character*1 acells(9,9)       ! Saved initial assignments.
      character*1 adup              ! Duplicated character.
      character*1 aperm(9,9,9)      ! Permission matrix.

c=======================================================================********

c.... Input line:  sudoku [help]

 8110 format ('  Command "sudoku":  acceptable input lines include:' //
     &  '  "sudoku [help]"               Display this message.' //
     &  '  "sudoku new"                  Start a new sudoku puzzle.' //
     &  '  "sudoku [row,col,box] N [=] I1 I2 I3 I4 I5 I6 I7 I8 I9"' /
     &  '     Assign up to 9 digits I1, I2, I3, ... to',
     &  ' [row, column, box] N.' //  
     &  '  "sudoku cell NROW NCOL [=] I1"' /
     &  '     Assign the digit I1 to row NROW, column NCOL.' //
     &  '  Note:  in the commands above, use zero for',
     &  ' cells with unassigned digits.' //
     &  '  "sudoku perm"                 Display the permitted digits.'/
     &  '  "sudoku save"                 Save and display the current',
     &  ' assignments.' /
     &  '  "sudoku reset"                Reset the assignments to',
     &  ' those of the last save.' /
     &  '  "sudoku solve"                Save the current assignments',
     &  ' and solve puzzle.' //
     &  '  Synonyms:  [help, h], [sudoku, su].')

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c=======================================================================********

c.... Initialize the new sudoku grid.

      if (nword .eq. 0) then

        achars(0) = ' '
        achars(1) = '1'
        achars(2) = '2'
        achars(3) = '3'
        achars(4) = '4'
        achars(5) = '5'
        achars(6) = '6'
        achars(7) = '7'
        achars(8) = '8'
        achars(9) = '9'

        nsave = 0

        do nrow = 1, 9
          do ncol = 1, 9
            acell(nrow,ncol) = ' '
            icell(nrow,ncol) = 0
          enddo
        enddo

        go to 999

      endif

c=======================================================================********

c.... Input line:  sudoku new

      if ((nword .eq. 2) .and. (aword(2) .eq. 'new')) then

        nsave = 0

        do nrow = 1, 9
          do ncol = 1, 9
            acell(nrow,ncol) = ' '
            icell(nrow,ncol) = 0
          enddo
        enddo

 8115   format (/ '  New Sudoku puzzle.')

        write (   6, 8115)
        write (iout, 8115)

        go to 999

      endif

c=======================================================================********

c.... Add missing optional "=" to option "row", "col" or "box".

      nwords = nword

 8122 format ('  sudo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not enough arguments:  ',i5)
 8124 format ('  sudo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Too many argumants:  ',i5)

      if ((aword(2) .eq. 'row')  .or.
     &    (aword(2) .eq. 'col')  .or.
     &    (aword(2) .eq. 'box')) then

        if (aword(4) .ne. '=') then   ! Equal sign is missing.

          nword = nword + 1

          if (nword .lt. 5) then
            write (   6, 8122) ainput, ninput, nwords
            write (iout, 8122) ainput, ninput, nwords
            go to 999
          endif

          if (nword .gt. 13) then
            write (   6, 8124) ainput, ninput, nwords
            write (iout, 8124) ainput, ninput, nwords
            go to 999
          endif

          do nw = nword, 5, -1
            aword(nw) = aword(nw-1)
            iword(nw) = iword(nw-1)
            fword(nw) = fword(nw-1)
            lword(nw) = lword(nw-1)
            mtype(nw) = mtype(nw-1)
          enddo

          aword(4) = '='
          iword(4) = 0
          fword(4) = 0
          lword(4) = 1
          mtype(4) = 0

        endif                         ! Equal sign was missing.

      endif

c=======================================================================********

c.... Input line:  sudoku row N [=] I1 I2 I3 I4 I5 I6 I7 I8 I9

 8120 format ('  sudo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer:  ',a)
 8136 format ('  sudo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Row number not from 1 to 9: ',i5)
 8138 format ('  sudo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Column number not from 1 to 9: ',i5)
 8142 format ('  sudo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Assigned digit not from 1 to 9: ',i5)
 8139 format ('  sudo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Box number not from 1 to 9: ',i5)

      if (aword(2) .eq. 'row') then

        call varfind (3, 3)

        if (mtype(3) .ne. 2) then
          write (   6, 8120) ainput, ninput, aword(3)
          write (iout, 8120) ainput, ninput, aword(3)
          go to 999
        endif

        call varfind (5, nword)

        do nw = 5, nword
          if (mtype(nw) .ne. 2) then
            write (   6, 8120) ainput, ninput, aword(nw)
            write (iout, 8120) ainput, ninput, aword(nw)
            go to 999
          endif
        enddo

        nrow = iword(3)
        if ((nrow .lt. 1) .or. (nrow .gt. 9))then
          write (   6, 8136) ainput, ninput, nrow
          write (iout, 8136) ainput, ninput, nrow
          go to 999
        endif

 8144   format (/ '  Assigning digits to Sudoku row',i2,'.')

        write (   6, 8144) nrow
        write (iout, 8144) nrow

        do nw = 5, nword
          ncol = nw - 4
          ichar = iword(nw)
          if ((ichar .ge. 0) .and. (ichar .le. 9)) then
            acell(nrow,ncol) = achars(ichar)
            icell(nrow,ncol) = ichar
          elseif ((ichar .lt. 0) .or. (ichar .gt. 9)) then
            write (   6, 8142) ainput, ninput, ichar
            write (iout, 8142) ainput, ninput, ichar
            go to 999
          endif
        enddo
  
 8140   format (/   'Sudoku puzzle assignments:')
 8146   format (/
     &    ' |-------|-------|-------|' /
     &    ' |',3(1x,a1),' |',3(1x,a1),' |',3(1x,a1),' |' /
     &    ' |',3(1x,a1),' |',3(1x,a1),' |',3(1x,a1),' |' /
     &    ' |',3(1x,a1),' |',3(1x,a1),' |',3(1x,a1),' |' /
     &    ' |-------|-------|-------|' /
     &    ' |',3(1x,a1),' |',3(1x,a1),' |',3(1x,a1),' |' /
     &    ' |',3(1x,a1),' |',3(1x,a1),' |',3(1x,a1),' |' /
     &    ' |',3(1x,a1),' |',3(1x,a1),' |',3(1x,a1),' |' /
     &    ' |-------|-------|-------|' /
     &    ' |',3(1x,a1),' |',3(1x,a1),' |',3(1x,a1),' |' /
     &    ' |',3(1x,a1),' |',3(1x,a1),' |',3(1x,a1),' |' /
     &    ' |',3(1x,a1),' |',3(1x,a1),' |',3(1x,a1),' |' /
     &    ' |-------|-------|-------|' )

        write (   6, 8140)
        write (iout, 8140)
        write (   6, 8146) ((acell(nrow,ncol), ncol = 1, 9),
     &    nrow = 1, 9)
        write (iout, 8146) ((acell(nrow,ncol), ncol = 1, 9),
     &    nrow = 1, 9)

c....   Check for duplicates.

        call aptsudu (achars, acell, adup, nrowd, ncold, nerra)

        if (nerra .eq. 0) then
          go to 999
        elseif (nerra .eq. 1) then
          write (   6, 8201) ainput, ninput
          write (iout, 8201) ainput, ninput
        elseif (nerra .eq. 3) then
          write (   6, 8203) ainput, ninput
          write (iout, 8203) ainput, ninput
        elseif (nerra .eq. 4) then
          write (   6, 8204) ainput, ninput
          write (iout, 8204) ainput, ninput
        elseif (nerra .eq. 5) then
          write (   6, 8205) ainput, ninput
          write (iout, 8205) ainput, ninput
        endif

 8147   format ('  Duplicate character',1x,a1,' in row',i2,
     &    ', col',i2,'.')
        write (   6, 8147) adup, nrowd, ncold
        write (iout, 8147) adup, nrowd, ncold

        go to 999

      endif

c=======================================================================********

c.... Input line:  sudoku col N [=] I1 I2 I3 I4 I5 I6 I7 I8 I9

      if (aword(2) .eq. 'col') then

        if (nword .lt. 5) then
          write (   6, 8122) ainput, ninput, nwords
          write (iout, 8122) ainput, ninput, nwords
          go to 999
        endif

        call varfind (5, nword)

        do nw = 5, nword
          if (mtype(nw) .ne. 2) then
            write (   6, 8120) ainput, ninput, aword(nw)
            write (iout, 8120) ainput, ninput, aword(nw)
            go to 999
          endif
        enddo

        ncol = iword(3)
        if ((ncol .lt. 1) .or. (ncol .gt. 9))then
          write (   6, 8136) ainput, ninput, ncol
          write (iout, 8136) ainput, ninput, ncol
          go to 999
        endif

 8145   format (/ '  Assigning digits to Sudoku column',i2,'.')

        write (   6, 8145) ncol
        write (iout, 8145) ncol

        do nw = 5, nword
          nrow = nw - 4
          ichar = iword(nw)
          if ((ichar .ge. 0) .and. (ichar .le. 9)) then
            acell(nrow,ncol) = achars(ichar)
            icell(nrow,ncol) = ichar
          elseif ((ichar .lt. 0) .or. (ichar .gt. 9)) then
            write (   6, 8142) ainput, ninput, ichar
            write (iout, 8142) ainput, ninput, ichar
            go to 999
          endif
        enddo

        write (   6, 8140)
        write (iout, 8140)
        write (   6, 8146) ((acell(nrow,ncol), ncol = 1, 9),
     &    nrow = 1, 9)
        write (iout, 8146) ((acell(nrow,ncol), ncol = 1, 9),
     &    nrow = 1, 9)

c....   Check for duplicates.

        call aptsudu (achars, acell, adup, nrowd, ncold, nerra)

        if (nerra .eq. 0) then
          go to 999
        elseif (nerra .eq. 1) then
          write (   6, 8201) ainput, ninput
          write (iout, 8201) ainput, ninput
        elseif (nerra .eq. 3) then
          write (   6, 8203) ainput, ninput
          write (iout, 8203) ainput, ninput
        elseif (nerra .eq. 4) then
          write (   6, 8204) ainput, ninput
          write (iout, 8204) ainput, ninput
        elseif (nerra .eq. 5) then
          write (   6, 8205) ainput, ninput
          write (iout, 8205) ainput, ninput
        endif

        write (   6, 8147) adup, nrowd, ncold
        write (iout, 8147) adup, nrowd, ncold

        go to 999

      endif

c=======================================================================********

c.... Input line:  sudoku box N [=] I1 I2 I3 I4 I5 I6 I7 I8 I9

      if (aword(2) .eq. 'box') then

        call varfind (3, 3)

        if (mtype(3) .ne. 2) then
          write (   6, 8120) ainput, ninput, aword(3)
          write (iout, 8120) ainput, ninput, aword(3)
          go to 999
        endif

        call varfind (5, nword)

        do nw = 5, nword
          if (mtype(nw) .ne. 2) then
            write (   6, 8120) ainput, ninput, aword(nw)
            write (iout, 8120) ainput, ninput, aword(nw)
            go to 999
          endif
        enddo

        nbox = iword(3)
        if ((nbox .lt. 1) .or. (nbox .gt. 9))then
          write (   6, 8136) ainput, ninput, nbox
          write (iout, 8136) ainput, ninput, nbox
          go to 999
        endif

 8143   format (/ '  Assigning digits to Sudoku box',i2,'.')

        write (   6, 8143) nbox
        write (iout, 8143) nbox

        nw = 4
        nrowbeg = 1 + 3 * ((nbox - 1) / 3)  
        nrowend = 3 + 3 * ((nbox - 1) / 3)
        ncolbeg = 1 + 3 * mod (nbox - 1, 3) 
        ncolend = 3 + 3 * mod (nbox - 1, 3)
        do nrow = nrowbeg,  nrowend
          do ncol = ncolbeg,  ncolend
            nw = nw + 1
            if (nw .gt. nword) go to 147
            ichar = iword(nw)
            if ((ichar .ge. 0) .and. (ichar .le. 9)) then
              acell(nrow,ncol) = achars(ichar)
              icell(nrow,ncol) = ichar
            elseif ((ichar .lt. 0) .or. (ichar .gt. 9)) then
              write (   6, 8142) ainput, ninput, ichar
              write (iout, 8142) ainput, ninput, ichar
              go to 999
            endif
          enddo
        enddo

 147    continue

        write (   6, 8140)
        write (iout, 8140)
        write (   6, 8146) ((acell(nrow,ncol), ncol = 1, 9),
     &    nrow = 1, 9)
        write (iout, 8146) ((acell(nrow,ncol), ncol = 1, 9),
     &    nrow = 1, 9)

c....   Check for duplicates.

        call aptsudu (achars, acell, adup, nrowd, ncold, nerra)

        if (nerra .eq. 0) then
          go to 999
        elseif (nerra .eq. 1) then
          write (   6, 8201) ainput, ninput
          write (iout, 8201) ainput, ninput
        elseif (nerra .eq. 3) then
          write (   6, 8203) ainput, ninput
          write (iout, 8203) ainput, ninput
        elseif (nerra .eq. 4) then
          write (   6, 8204) ainput, ninput
          write (iout, 8204) ainput, ninput
        elseif (nerra .eq. 5) then
          write (   6, 8205) ainput, ninput
          write (iout, 8205) ainput, ninput
        endif

        write (   6, 8147) adup, nrowd, ncold
        write (iout, 8147) adup, nrowd, ncold

        go to 999

      endif

c=======================================================================********

c.... Add missing optional "=" to option "cell".

      if (aword(2) .eq. 'cell') then
        if (aword(5) .ne. '=') then
          nword = nword + 1

          if (nword .lt. 6) then
            write (   6, 8122) ainput, ninput, nwords
            write (iout, 8122) ainput, ninput, nwords
            go to 999
          endif

          if (nword .gt. 6) then
            write (   6, 8124) ainput, ninput, nwords
            write (iout, 8124) ainput, ninput, nwords
            go to 999
          endif

          aword(6) = aword(5)
          iword(6) = iword(5)
          fword(6) = fword(5)
          lword(6) = lword(5)
          mtype(6) = mtype(5)
          aword(5) = '='
          iword(5) = 0
          fword(5) = 0
          lword(5) = 1
          mtype(5) = 0
        endif
      endif

c=======================================================================********

c.... Input line:  sudoku cell NROW NCOL [=] I1

      if ((nword .eq. 6) .and. (aword(2) .eq. 'cell')) then

        call varfind (3, 4)

        do nw = 3, 4
          if (mtype(nw) .ne. 2) then
            write (   6, 8120) ainput, ninput, aword(nw)
            write (iout, 8120) ainput, ninput, aword(nw)
            go to 999
          endif
        enddo

        call varfind (6, 6)

        if (mtype(6) .ne. 2) then
          write (   6, 8120) ainput, ninput, aword(nw)
          write (iout, 8120) ainput, ninput, aword(nw)
          go to 999
        endif

        nrow = iword(3)
        if ((nrow .lt. 1) .or. (nrow .gt. 9))then
          write (   6, 8136) ainput, ninput, nrow
          write (iout, 8136) ainput, ninput, nrow
          go to 999
        endif

        ncol = iword(4)
        if ((ncol .lt. 1) .or. (ncol .gt. 9))then
          write (   6, 8138) ainput, ninput, ncol
          write (iout, 8138) ainput, ninput, ncol
          go to 999
        endif

 8148   format (/ '  Assigning digits to Sudoku row',i2,
     &    ', column',i2,'.')

        write (   6, 8148) nrow, ncol
        write (iout, 8148) nrow, ncol

        ichar = iword(6)
        if ((ichar .ge. 0) .and. (ichar .le. 9)) then
          acell(nrow,ncol) = achars(ichar)
          icell(nrow,ncol) = ichar
        elseif ((ichar .lt. 0) .or. (ichar .gt. 9)) then
          write (   6, 8142) ainput, ninput, ichar
          write (iout, 8142) ainput, ninput, ichar
          go to 999
        endif

        write (   6, 8140)
        write (iout, 8140)
        write (   6, 8146) ((acell(nrow,ncol), ncol = 1, 9),
     &    nrow = 1, 9)
        write (iout, 8146) ((acell(nrow,ncol), ncol = 1, 9),
     &    nrow = 1, 9)

c....   Check for duplicates.

        call aptsudu (achars, acell, adup, nrowd, ncold, nerra)

        if (nerra .eq. 0) then
          go to 999
        elseif (nerra .eq. 1) then
          write (   6, 8201) ainput, ninput
          write (iout, 8201) ainput, ninput
        elseif (nerra .eq. 3) then
          write (   6, 8203) ainput, ninput
          write (iout, 8203) ainput, ninput
        elseif (nerra .eq. 4) then
          write (   6, 8204) ainput, ninput
          write (iout, 8204) ainput, ninput
        elseif (nerra .eq. 5) then
          write (   6, 8205) ainput, ninput
          write (iout, 8205) ainput, ninput
        endif

        write (   6, 8147) adup, nrowd, ncold
        write (iout, 8147) adup, nrowd, ncold

        go to 999

      endif

c=======================================================================********

c.... Input line:  sudoku perm

      if ((nword .eq. 2) .and. (aword(2) .eq. 'perm')) then

        call aptsudp (achars, acell, kperm, npermit, nerr)

        do nrow = 1, 9                 ! Loop over rows.
          do ncol = 1, 9               ! Loop over columns.
            ichar = icell(nrow,ncol)
            if (ichar .ne. 0) then
              npermit(nrow,ncol) = -9
            endif
            do idigs = 1, 9            ! Loop over permssions.  
              if (ichar .ne. 0) then
                aperm(nrow,ncol,idigs) = achars(ichar)
              elseif (kperm(nrow,ncol,idigs) .eq. 0) then
                aperm(nrow,ncol,idigs) = ' '
              elseif (kperm(nrow,ncol,idigs) .eq. 1) then
                aperm(nrow,ncol,idigs) = achars(idigs)
              else
                aperm(nrow,ncol,idigs) = '?'
              endif
            enddo                      ! End of loop over digits.
          enddo                        ! End of loop over columns.
        enddo                          ! End of loop over rows.

 8152   format (/ 'Permitted digits:' /)
 8153   format ('   Col  1',7x,'2',7x,'3   Col  4',7x,'5',7x,
     &    '6   Col  7',7x,'8',7x,'9  Col')
 8154   format ('Row|=',3('========'),'=',3('========'),'=',
     &    3('========'),'|')
 8156   format ('   ||',3('-------|'),'|',3('-------|'),'|',
     &    3('-------|'),'|')
 8158   format ('   |',3('|',3(1x,a1),1x),'|',3('|',3(1x,a1),1x),'|',
     &    3('|',3(1x,a1),1x),'||',i2)
 8159   format (i2,1x'|',3('|',3(1x,a1),1x),'|',3('|',3(1x,a1),1x),'|',
     &    3('|',3(1x,a1),1x),'||')

        write (   6, 8152)
        write (iout, 8152)
        write (   6, 8153)
        write (iout, 8153)

        do nrow = 1, 9                  ! Loop over rows.

          if (mod (nrow, 3) .eq. 1) then
            write (   6, 8154)
            write (iout, 8154)
          else
            write (   6, 8156)
            write (iout, 8156)
          endif

          write (   6, 8158) ((aperm(nrow,ncol,idigs), idigs = 1, 3),
     &                      ncol = 1,9)
          write (iout, 8158) ((aperm(nrow,ncol,idigs), idigs = 1, 3),
     &                      ncol = 1,9)
          write (   6, 8159) nrow, ((aperm(nrow,ncol,idigs),
     &                       idigs = 4, 6), ncol = 1,9)
          write (iout, 8159) nrow, ((aperm(nrow,ncol,idigs),
     &                       idigs = 4, 6), ncol = 1,9)
          write (   6, 8158) ((aperm(nrow,ncol,idigs), idigs = 7, 9),
     &                      ncol = 1,9)
          write (iout, 8158) ((aperm(nrow,ncol,idigs), idigs = 7, 9),
     &                      ncol = 1,9)

        enddo                           ! End of loop over rows.

        write (   6, 8154)
        write (iout, 8154)

c....   See of puzzle can be solved.

c....   Save current assignments.

        do nrow = 1, 9
          do ncol = 1, 9
            acells(nrow,ncol) = acell(nrow,ncol)
            do ichar = 0, 9
              if (acells(nrow,ncol) .eq. achars(ichar)) then
                icells(nrow,ncol) = ichar
              endif
            enddo
          enddo
        enddo

        call aptsudo (achars, acell, ncells, nerra)

c....   Restore current assignments.

        do nrow = 1, 9
          do ncol = 1, 9
            ichar = icells(nrow,ncol)
            icell(nrow,ncol) = ichar
            acell(nrow,ncol) = achars(ichar)
          enddo
        enddo

c....   Check for errors.

        if (nerra .eq. 0) then
          go to 999
        elseif (nerra .eq. 3) then
          write (   6, 8203) ainput, ninput
          write (iout, 8203) ainput, ninput
        elseif (nerra .eq. 4) then
          write (   6, 8204) ainput, ninput
          write (iout, 8204) ainput, ninput
        elseif (nerra .eq. 5) then
          write (   6, 8205) ainput, ninput
          write (iout, 8205) ainput, ninput
        endif

        go to 999

      endif

c=======================================================================********

c.... Input line:  sudoku save

 8175 format ('sudoku row',i2,' =',3i2,1x,3i2,1x,3i2)

      if ((nword .eq. 2) .and. (aword(2) .eq. 'save')) then

c....   Save current assignments.

        nsave = 1

        do nrow = 1, 9
          do ncol = 1, 9
            acells(nrow,ncol) = acell(nrow,ncol)
            do ichar = 0, 9
              if (acells(nrow,ncol) .eq. achars(ichar)) then
                icells(nrow,ncol) = ichar
              endif
            enddo
          enddo
        enddo

c....   Display current assignments

        do nrow = 1, 9
          write (   6, 8175) nrow, (icells(nrow,ncol), ncol = 1, 9)
          write (iout, 8175) nrow, (icells(nrow,ncol), ncol = 1, 9)
        enddo

        go to 999

      endif

c=======================================================================********

c.... Input line:  sudoku reset

 8170 format ('  sudo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not reset before saving or solving..')

      if ((nword .eq. 2) .and. (aword(2) .eq. 'reset')) then

c....   Reset initial assignments.

        if (nsave .ne. 1) then
          write (   6, 8170) ainput, ninput, aword(nw)
          write (iout, 8170) ainput, ninput, aword(nw)
          go to 999
        endif

        do nrow = 1, 9
          do ncol = 1, 9
            ichar = icells(nrow,ncol)
            icell(nrow,ncol) = ichar
            acell(nrow,ncol) = achars(ichar)
          enddo
        enddo

        write (   6, 8140)
        write (iout, 8140)
        write (   6, 8146) ((acell(nrow,ncol), ncol = 1, 9),
     &    nrow = 1, 9)
        write (iout, 8146) ((acell(nrow,ncol), ncol = 1, 9),
     &    nrow = 1, 9)

        go to 999

      endif

c=======================================================================********

c.... Input line:  sudoku solve
  
 8180 format (/   'Sudoku puzzle SOLVED.')
 8182 format (/   'Sudoku puzzle UNSOLVED.')
 8201 format ('  sudo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Initial digit not 1-9.')
 8202 format ('  sudo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Fewer than 17 cells have initial digits.')
 8203 format ('  sudo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  A digit appears more than once in same row.')
 8204 format ('  sudo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  A digit appears more than once in same column.')
 8205 format ('  sudo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  A digit appears more than once in same box.')
 8206 format ('  sudo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  No solution.  A cell can not be assigned.')
 8207 format ('  sudo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Duplicate digits in same row, cell or box.')
 8208 format ('  sudo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not solve Sudoku puzzle.')
 8209 format ('  sudo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Filled 81 cells, but not a valid solution.')

      if ((nword .eq. 2) .and. (aword(2) .eq. 'solve')) then

c....   Save current assignments.

        nsave = 1

        do nrow = 1, 9
          do ncol = 1, 9
            acells(nrow,ncol) = acell(nrow,ncol)
            do ichar = 0, 9
              if (acells(nrow,ncol) .eq. achars(ichar)) then
                icells(nrow,ncol) = ichar
              endif
            enddo
          enddo
        enddo

        call aptsudo (achars, acell, ncells, nerra)

        if (nerra .eq. 0) then
          write (   6, 8180)
          write (iout, 8180)
        else
          write (   6, 8182)
          write (iout, 8182)
        endif

        write (   6, 8146) ((acell(nrow,ncol), ncol = 1, 9),
     &    nrow = 1, 9)
        write (iout, 8146) ((acell(nrow,ncol), ncol = 1, 9),
     &    nrow = 1, 9)

        if (nerra .eq. 0) then
          go to 999
        elseif (nerra .eq. 1) then
          write (   6, 8201) ainput, ninput
          write (iout, 8201) ainput, ninput
        elseif (nerra .eq. 2) then
          write (   6, 8202) ainput, ninput
          write (iout, 8202) ainput, ninput
        elseif (nerra .eq. 3) then
          write (   6, 8203) ainput, ninput
          write (iout, 8203) ainput, ninput
        elseif (nerra .eq. 4) then
          write (   6, 8204) ainput, ninput
          write (iout, 8204) ainput, ninput
        elseif (nerra .eq. 5) then
          write (   6, 8205) ainput, ninput
          write (iout, 8205) ainput, ninput
        elseif (nerra .eq. 6) then
          write (   6, 8206) ainput, ninput
          write (iout, 8206) ainput, ninput
        elseif (nerra .eq. 7) then
          write (   6, 8207) ainput, ninput
          write (iout, 8207) ainput, ninput
        elseif (nerra .eq. 8) then
          write (   6, 8208) ainput, ninput
          write (iout, 8208) ainput, ninput
        elseif (nerra .eq. 9) then
          write (   6, 8209) ainput, ninput
          write (iout, 8209) ainput, ninput
        endif

        go to 999

      endif

c=======================================================================********


c=======================================================================********

 8810 format ('  sudo ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8810) ainput, ninput
      write (iout, 8810) ainput, ninput

c=======================================================================********

  999 return

c.... End of subroutine sudo.       (+1 line)
      end
