      subroutine symbrepl

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  symbrepl Updated     2004 June 8 18:00.
c               symbrepl Originated  2002 August 23 15:40.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To replace each symbol with its replacement word, except in
c               commands "symbol", "[copy, delete, rename] symbol", and
c               commands "alias", "[copy, delete, rename] alias".
c
c     Logic:    Entry points:  symbrepl.
c symbrepl      Called by:  main.
c
c symbrepl      Calls:  aptsubb, aptsubm, symbout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     aword     Output   ASCII value of argument.
c
c     fword     Output   Floating point value of argument.
c
c     iword     Output   Integer value of argument.
c
c     lword     Output   Number of characters in aword.
c
c     mtype     Output   Mode of argument.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Do not replace any symbols in commands
c....   "alias", "debug", "symbol",
c....   "[copy, delete, rename] alias",
c....   "[copy, delete, rename] symbol".

      if ((aword(1) .eq. 'debug')      .or.
     &    (aword(1) .eq. 'alias')      .or.
     &    (aword(1) .eq. 'al')         .or.
     &    (aword(1) .eq. 'a')          .or.
     &    (aword(1) .eq. 'symbol')     .or.
     &    (aword(1) .eq. 'symb')       .or.
     &    (aword(1) .eq. 'sym'))       go to 999

      if  ((aword(1) .eq. 'copy')      .or.
     &     (aword(1) .eq. 'cp')        .or.
     &     (aword(1) .eq. 'duplicate') .or.
     &     (aword(1) .eq. 'dup')       .or.
     &     (aword(1) .eq. 'delete')    .or.
     &     (aword(1) .eq. 'del')       .or.
     &     (aword(1) .eq. 'remove')    .or.
     &     (aword(1) .eq. 'rm')        .or.
     &     (aword(1) .eq. 'rename')    .or.
     &     (aword(1) .eq. 'switch')    .or.
     &     (aword(1) .eq. 'sw'))       then

        if ((aword(2) .eq. 'symbol')   .or.
     &      (aword(2) .eq. 'symb')     .or.
     &      (aword(2) .eq. 'sym')      .or.
     &      (aword(2) .eq. 'alias')    .or.
     &      (aword(2) .eq. 'al')       .or.
     &      (aword(2) .eq. 'a'))       then
          go to 999
        endif                         ! Option symbol or alias.
      endif                           ! Copy, delete or rename command.

      do 130 nw = 1, nword            ! Loop over command words.

 8110   format ('  symbrepl ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  More than 10 consecutive symbol replacements are not',
     &    ' allowed.')

c===============================================================================

c....   Replace entire word, either subscripted or unsubscripted.

        nrepl = 0                     ! Number of consecutive replacements.

  110   do 120 nsymb = 1, nsymbs      ! Loop over symbols.

          if (aword(nw) .eq. asymbl(nsymb)) then  ! A match was found.

            if (nrepl .eq. 0) then
              write (   6, "('')")
              write (iout, "('')")
            endif

            call symbout (nsymb, nsymb)

            nrepl = nrepl + 1         ! Number of consecutive replacements.

            if (nrepl .gt. 10) then
              write (   6, 8110) ainput, ninput
              write (iout, 8110) ainput, ninput
              go to 999
            endif

            aword(nw) = asymbr(nsymb)
            fword(nw) = fsymb(nsymb)
            iword(nw) = isymb(nsymb)
            lword(nw) = lsymb(nsymb)
            mtype(nw) = msymb(nsymb)

            go to 110

          endif                       ! A match was found.

  120   continue                      ! End of loop over symbols.

c===============================================================================

c.... Replace stems of subscripted words.

        call aptsubm (aword(nw), 1, lword(nw), iamax, idmax, iemax,
     &                '(', ',', ')', 16,
     &                astem, istem, fstem, mstem, lstem,
     &                nsub, asub, isub, fsub, msub, lsub, nerra)

        if (nsub .gt. 0) then         ! Word is subscripted.

        nrepl = 0                     ! Number of consecutive replacements.

  210   do 220 nsymb = 1, nsymbs      ! Loop over symbols.

          if (astem .eq. asymbl(nsymb)) then  ! A match was found.

            if (nrepl .eq. 0) then
              write (   6, "('')")
              write (iout, "('')")
            endif

            call symbout (nsymb, nsymb)

            nrepl = nrepl + 1         ! Number of consecutive replacements.

            if (nrepl .gt. 10) then
              write (   6, 8110) ainput, ninput
              write (iout, 8110) ainput, ninput
              go to 999
            endif

c....       Replace stem.

            astem = asymbr(nsymb)
            fstem = fsymb(nsymb)
            istem = isymb(nsymb)
            lstem = lsymb(nsymb)
            mstem = msymb(nsymb)

c....       Reconstruct word.

            call aptsubb (astem, lstem, nsub, asub, lsub, 80,
     &                    '(', ',', ')', aword(nw), lword(nw), nerr)

            go to 210

          endif                       ! A match was found.

  220   continue                      ! End of loop over symbols.

        endif                         ! Word is subscripted.

c=========================================================================

c....   Replace subscripts of subscripted words.

        call aptsubm (aword(nw), 1, lword(nw), iamax, idmax, iemax,
     &                '(', ',', ')', 16,
     &                astem, istem, fstem, mstem, lstem,
     &                nsub, asub, isub, fsub, msub, lsub, nerra)

        if (nsub .gt. 0) then         ! Word is subscripted.

        do nn = 1, nsub               ! Loop over subscripts.

        nrepl = 0                     ! Number of consecutive replacements.

  310   do 320 nsymb = 1, nsymbs      ! Loop over symbols.

          if (asub(nn) .eq. asymbl(nsymb)) then

            if (nrepl .eq. 0) then
              write (   6, "('')")
              write (iout, "('')")
            endif

            call symbout (nsymb, nsymb)

            nrepl = nrepl + 1         ! Number of consecutive replacements.

            if (nrepl .gt. 10) then
              write (   6, 8110) ainput, ninput
              write (iout, 8110) ainput, ninput
              go to 999
            endif

c....       Replace subscript.

            asub(nn) = asymbr(nsymb)
            fsub(nn) = fsymb(nsymb)
            isub(nn) = isymb(nsymb)
            lsub(nn) = lsymb(nsymb)
            msub(nn) = msymb(nsymb)

c....       Reconstruct word.

            call aptsubb (astem, lstem, nsub, asub, lsub, 80,
     &                    '(', ',', ')', aword(nw), lword(nw), nerr)

            go to 310

          endif                       ! A match was found.

  320   continue                      ! End of loop over symbols.

        enddo                         ! End of loop over subscripts.

        endif                         ! Word has subscripts.

c=========================================================================
 
  130 continue                        ! End of loop over command words.

  999 return

c.... End of subroutine symbrepl.     (+1 line)
      end
