      subroutine tettest (aname1, aname2, aname3, aname4,
     &                    n1, n2, n3, n4, nerrtet)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  tettest  Updated     2004 November 23 13:30.
c               tettest  Originated  1994 August 9 18:10.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To test tetrahedrons to see if their vertex points are good.
c
c     Logic:    Entry points:  tettest.
c tettest       Called by:  tetra.
c
c tettest       Calls:  apttetd.
c
c     Biblio:   ?
c
c     Glossary:
c
c     aname1-4  Input    Names of the vertex points.
c
c     n1-n4     Output   Indices in the point list of the vertex points.
c
c     nerrtet   Output   Indicates an error in the vertex points, if not 0.
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      character*24 aname1             ! Name of first vertex point.
      character*24 aname2             ! Name of second vertex point.
      character*24 aname3             ! Name of third vertex point.
      character*24 aname4             ! Name of fourth vertex point.

c.... Find the four points defining the tetrahedron.

      nerrtet = 0

 8140 format ('  tettest ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named point does not exist:  ',a)
      do 140 np = 1, npoints
        if (aname1 .eq. apoint(np)) then
          n1 = np
          go to 150
        endif
  140 continue
      nerrtet = 1
      write (   6, 8140) ainput, ninput, aname1
      write (iout, 8140) ainput, ninput, aname1

  150 do 160 np = 1, npoints
        if (aname2 .eq. apoint(np)) then
          n2 = np
          go to 170
        endif
  160 continue
      nerrtet = 1
      write (   6, 8140) ainput, ninput, aname2
      write (iout, 8140) ainput, ninput, aname2

  170 do 180 np = 1, npoints
        if (aname3 .eq. apoint(np)) then
          n3 = np
          go to 182
        endif
  180 continue
      nerrtet = 1
      write (   6, 8140) ainput, ninput, aname3
      write (iout, 8140) ainput, ninput, aname3

  182 do 184 np = 1, npoints
        if (aname4 .eq. apoint(np)) then
          n4 = np
          go to 190
        endif
  184 continue
      nerrtet = 1
      write (   6, 8140) ainput, ninput, aname4
      write (iout, 8140) ainput, ninput, aname4

c.... See if the points are unique.

 8190 format ('  tettest ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Points are not unique:  ' / a / a / a )

  190 if ((n1 .eq. n2)  .or.
     &    (n1 .eq. n3)  .or.
     &    (n1 .eq. n4)  .or.
     &    (n2 .eq. n3)  .or.
     &    (n2 .eq. n4)  .or.
     &    (n3 .eq. n4)) then
        nerrtet = 1
        write (   6, 8190) ainput, ninput, apoint(n1), apoint(n2),
     &    apoint(n3), apoint(n4)
        write (iout, 8190) ainput, ninput, apoint(n1), apoint(n2),
     &    apoint(n3), apoint(n4)
      endif

c.... See if the points are coincident or colinear.

c.... Find the centroid and volume.

      call apttetd (x(n1), y(n1), z(n1), x(n2), y(n2), z(n2),
     &              x(n3), y(n3), z(n3), x(n4), y(n4), z(n4),
     &              1, tol, vtet, cenx, ceny, cenz, nerra)

 8200 format ('  tettest ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Points are coplanar:    ' / a / a / a)

      if (vtet .eq. 0.0) then
        nerrtet = 1
        write (   6, 8200) ainput, ninput, apoint(n1), apoint(n2),
     &    apoint(n3), apoint(n4)
        write (iout, 8200) ainput, ninput, apoint(n1), apoint(n2),
     &    apoint(n3), apoint(n4)
      endif

      return

c.... End of subroutine tettest.      (+1 line)
      end
