      subroutine triple

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  triple   Updated     2004 March 15 17:30.
c               triple   Originated  1997 October 29 13:30.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To find any triple point of intersection of the three planes
c               with implicit equations:
c                 qc1 + qx1 * x + qy1 * y + qz1 * z = 0
c                 qc2 + qx2 * x + qy2 * y + qz2 * z = 0
c                 qc3 + qx3 * x + qy3 * y + qz3 * z = 0
c               or a triple point of intersection of any three general
c               quadric surfaces.
c
c     Logic:    Entry points:  triple.
c triple        Called by:  helpall, main.
c
c triple        Calls:  aptplpl, aptripq, aptrkis, aptscat, aptsolv, apttlod,
c               plotpt, ptfind, quadfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

      common /ltripl/ abc(3,3)        ! Coefficients of equations (row, column).
      common /ltripl/ cba(3,3)        ! Inverse of matrix abc     (row, column).
      common /ltripl/ d(3)            ! Right-hand side constants.
      common /ltripl/ dm(3)           ! Right-hand side constants check.
      common /ltripl/ dr(3)           ! Right-hand side constants residuals.
      common /ltripl/ work(3,4)       ! Working matrix.
      common /ltripl/ xyz(3)          ! Solution.
      common /ltripl/ xyzm(3)         ! Solution check.
      common /ltripl/ xyzr(3)         ! Solution residuals.
      common /ltripl/ trix(64)        ! Coordinate x of a triple point.
      common /ltripl/ triy(64)        ! Coordinate x of a triple point.
      common /ltripl/ triz(64)        ! Coordinate x of a triple point.

c=======================================================================********

c.... Input line:  triple [help]

 8110 format ('  Command "triple":  acceptable input lines include:' /
     &  '  "triple [help]"  Display this message.' //
     &  '  "triple QNAME1 QNAME2 QNAME3 [PNAME]"' /
     &  '     Find any triple intersection of the three quadric',
     &  ' surfaces' /
     &  '     QNAME1, QNAME2 and QNAME2 [near point PNAME].' //
     &  '  Synonyms:  [help, h], [point, pnt, pt, p].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        go to 999
      endif

c=======================================================================********

c.... Input line:  triple QNAME1 QNAME2 QNAME3 [point PNAME]

c.... Test for the wrong number of words on "triple" line.

 8120 format ('  triple ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not 4 or 6 words on "triple" line:  ', i3)
      if ((nword .ne. 4) .and. (nword .ne. 6)) then
        write (   6, 8120) ainput, ninput, nword
        write (iout, 8120) ainput, ninput, nword
        go to 999
      endif

      nerr = 0

c.... Find the first quadric surface.

      call quadfind (2, nq1)

      if (nq1 .eq. 0) then
        nerr = 1
        go to 120
      endif

c.... Find the second quadric surface.

  120 call quadfind (3, nq2)

      if (nq2 .eq. 0) then
        nerr = 1
        go to 130
      endif

c.... Find the third quadric surface.

  130 call quadfind (4, nq3)

      if (nq3 .eq. 0) then
        nerr = 1
        go to 140
      endif

c.... Find the initial point, if any.
 8140 format ('  triple ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Option not recognized:  ', a)

  140 if ((aword(5) .eq. 'point') .or.
     &    (aword(5) .eq. 'pnt')   .or.
     &    (aword(5) .eq. 'pt')    .or.
     &    (aword(5) .eq. 'p'))    then

        if (nword .eq. 6) then        ! Initial point specified.
  
          call ptfind (6, n1)
  
          if (n1 .eq. 0) then
            nerr = 1
            go to 150
          endif
  
        endif                         ! Initial point specified.
      elseif (nword .ne. 4) then
        write (   6, 8140) ainput, ninput, aword(5)
        write (iout, 8140) ainput, ninput, aword(5)
        nerr = 1
      endif

  150 if (nerr .ne. 0) go to 999

c.... Find any triple point for three planes.

      if ((aqtype(nq1) .eq. aqtypes(0))  .and.
     &    (aqtype(nq2) .eq. aqtypes(0))  .and.
     &    (aqtype(nq3) .eq. aqtypes(0))) then

        proxptx = -1.e99
        proxpty = -1.e99
        proxptz = -1.e99
  
        proxvx = -1.e99
        proxvy = -1.e99
        proxvz = -1.e99


        abc(1,1) = qx(nq1)
        abc(1,2) = qy(nq1)
        abc(1,3) = qz(nq1)
        d(1)     = -qc(nq1)

        abc(2,1) = qx(nq2)
        abc(2,2) = qy(nq2)
        abc(2,3) = qz(nq2)
        d(2)     = -qc(nq2)

        abc(3,1) = qx(nq3)
        abc(3,2) = qy(nq3)
        abc(3,3) = qz(nq3)
        d(3)     = -qc(nq3)

c....   Solve the three simultaneous equations.

        call aptsolv (abc, d, 3, work, 3, tol, xyz, cba, xyzm, xyzr,
     &                dm, dr, nerra)

 8210   format (/ 'No triple point.' )
 8220   format (/ '  triple WARNING.  Reading ',a40,' line ',i5,'.' /
     &    '  Large triple point residuals.' )
 8230   format (/ '  triple WARNING.  Reading ',a40,' line ',i5,'.' /
     &    '  Large qc residuals.' )
 8240   format (/ 'Triple point (xyz)',1p3e20.12 )

        if (nerra .eq. 1) then
          write (   6, 8210)
          write (iout, 8210)
          go to 999
        elseif (nerra .eq. 2) then
          write (   6, 8220) ainput, ninput
          write (iout, 8220) ainput, ninput
        elseif (nerra .eq. 3) then
          write (   6, 8230) ainput, ninput
          write (iout, 8230) ainput, ninput
        endif

        xint = xyz(1)
        yint = xyz(2)
        zint = xyz(3)

        write (   6, 8240) xint, yint, zint
        write (iout, 8240) xint, yint, zint
        call plotpt (xint, yint, zint, nerrg)

        proxptx = xint
        proxpty = yint
        proxptz = zint

        proxvx = xint
        proxvy = yint
        proxvz = zint

        go to 999

      endif                           ! Three planes.

c.... See if any two of the quadric surfaces are planes.

      if ((aqtype(nq1) .eq. aqtypes(0))  .and.
     &    (aqtype(nq2) .eq. aqtypes(0))) then

        npl1 = nq1
        npl2 = nq2
        nqq1 = nq3

      elseif ((aqtype(nq2) .eq. aqtypes(0))  .and.
     &        (aqtype(nq3) .eq. aqtypes(0))) then

        npl1 = nq2
        npl2 = nq3
        nqq1 = nq1

      elseif ((aqtype(nq3) .eq. aqtypes(0))  .and.
     &        (aqtype(nq1) .eq. aqtypes(0))) then

        npl1 = nq3
        npl2 = nq1
        nqq1 = nq2

      else                            ! Two or more nonplanar quadrics.

        scalex = 1.0
        scaley = 1.0
        scalez = 1.0

        ntrips = 0

        do 320 n = 1, 64

c....   Find a random point in the triangle formed by the centers of the
c....     three quadric surfaces.

        call apttlod (xcen(nq1), ycen(nq1), zcen(nq1),
     &                xcen(nq2), ycen(nq2), zcen(nq2),
     &                xcen(nq3), ycen(nq3), zcen(nq3), 1, 0, tol,
     &                px, py, pz, wa, wb, wc, nerra)

c....   Find a random vector and a scale factor.

        scalex = max (scalex,
     &               abs (xcen(nq1)), abs (xcen(nq2)), abs (xcen(nq3)))
        scaley = max (scaley,
     &               abs (ycen(nq1)), abs (ycen(nq2)), abs (ycen(nq3)))
        scalez = max (scalez,
     &               abs (zcen(nq1)), abs (zcen(nq2)), abs (zcen(nq3)))

        call aptscat (1, ax, ay, az, nerra)

c....   Add the scaled random vector to the random point in the triangle.

        if ((n .eq. 1) .and. (nword .eq. 6)) then
          px = x(n1)
          py = y(n1)
          pz = z(n1)
        else
          px = px + ax * scalex * 100.0
          py = py + ay * scaley * 100.0
          pz = pz + az * scalez * 100.0
        endif

c....   Try to find a triple point near the random point "p".

        call aptripq (px, py, pz,
     &                qc(nq1), qx(nq1), qy(nq1), qz(nq1),
     &                qxy(nq1), qyz(nq1), qzx(nq1),
     &                qxx(nq1), qyy(nq1), qzz(nq1),
     &                qc(nq2), qx(nq2), qy(nq2), qz(nq2),
     &                qxy(nq2), qyz(nq2), qzx(nq2),
     &                qxx(nq2), qyy(nq2), qzz(nq2),
     &                qc(nq3), qx(nq3), qy(nq3), qz(nq3),
     &                qxy(nq3), qyz(nq3), qzx(nq3),
     &                qxx(nq3), qyy(nq3), qzz(nq3),
     &                tol, tx, ty, tz, nerra)

        if (nerra .ne. 0) go to 320

        if (ntrips .eq. 0) go to 315  ! Found first triple point.

        do 310 nn = 1, ntrips         ! Loop over previous triple points.
          if ((abs (tx - trix(nn)) .le. tol * (10.0 + abs (tx)))  .and.
     &        (abs (ty - triy(nn)) .le. tol * (10.0 + abs (ty)))  .and.
     &        (abs (tz - triz(nn)) .le. tol * (10.0 + abs (tz)))) then
            go to 320
          endif
  310   continue                      ! End of loop over previous triple points.

c....   Found a unique new triple point.

  315   ntrips = ntrips + 1

        trix(ntrips) = tx
        triy(ntrips) = ty
        triz(ntrips) = tz

        scalex = max (scalex, abs (tx))
        scaley = max (scaley, abs (ty))
        scalez = max (scalez, abs (tz))

  320   continue                      ! Loop over random initial guesses.

 8250   format (/ 'Singular matrix.')
 8260   format (/ 'Failed to converge in 1000 iterations.')
 8270   format (/ 'Did not converge.')
 8320   format (/ 'Found',i3,' distinct triple points',
     &   ' (8 are possible).')
 8330   format ('Triple point (xyz)',1p3e20.12 )
 8420   format (/ 'No triple point.')

        if (ntrips .ne. 0) then       ! Found one or more triple points.
          write (   6, 8320) ntrips
          write (iout, 8320) ntrips
          write (   6, 8330) (trix(n), triy(n), triz(n), n = 1, ntrips)
          write (iout, 8330) (trix(n), triy(n), triz(n), n = 1, ntrips)
          call plotpt (trix(ntrips), triy(ntrips), triz(ntrips), nerrg)
  
          proxptx = trix(ntrips)
          proxpty = triy(ntrips)
          proxptz = triz(ntrips)
  
          proxvx = trix(ntrips)
          proxvy = triy(ntrips)
          proxvz = triz(ntrips)
        else
          if (nerra .eq. 1) then
            write (   6, 8250)
            write (iout, 8250)
          elseif (nerra .eq. 2) then
            write (   6, 8260)
            write (iout, 8260)
          elseif (nerra .eq. 3) then
            write (   6, 8270)
            write (iout, 8270)
          endif
          write (   6, 8420)
          write (iout, 8420)
        endif                         ! Found one or more triple points.

        go to 999

      endif                           ! Two planes and a nonplanar quadric.

c.... Find any intersection line of the two planes.
 
      call aptplpl (xcen(npl1), ycen(npl1), zcen(npl1),
     &              qx(npl1), qy(npl1), qz(npl1),
     &              xcen(npl2), ycen(npl2), zcen(npl2),
     &              qx(npl2), qy(npl2), qz(npl2), 1, tol,
     &              ex, ey, ez, fx, fy, fz, ux, uy, uz,
     &              ipar, dpmin, itrun, nerra)

      if ((nerra .ne. 0) .or. (ipar .ne. 0)) then
        write (   6, 8420)
        write (iout, 8420)
        go to 999
      endif

 8430 format ('Intersection point',1p3e20.12 /
     &        'Intersection point',1p3e20.12 /
     &        'Vector            ',1p3e20.12)

      write (   6, 8430) ex, ey, ez, fx, fy, fz, ux, uy, uz
      write (iout, 8430) ex, ey, ez, fx, fy, fz, ux, uy, uz

      call plotpt (ex, ey, ez, nerrg)
      call plotpt (fx, fy, fz, nerrg)

      pdist = 0.0

      proxptx = ex
      proxpty = ey
      proxptz = ez

      proxvx = ex
      proxvy = ey
      proxvz = ez

c.... Find any intersection between the intersection of the planes and
c....   the quadric surface.

      nints = 0

      tolp = tol * (abs (ex) + abs (ey) + abs (ez))
      if (tolp .eq. 0.0) tolp = tol

c.... Search in negative direction along intersection of planes.

      dintmn = -1.e99
      dintmx = -tolp

      call aptrkis (ex, ey, ez, ux, uy, uz,
     &              qc(nqq1), qx(nqq1), qy(nqq1), qz(nqq1),
     &              qxy(nqq1), qyz(nqq1), qzx(nqq1),
     &              qxx(nqq1), qyy(nqq1), qzz(nqq1),
     &              dintmn, dintmx, 1, tol,
     &              nint, xint, yint, zint, dtrack,
     &              snx, sny, snz, costh, nerra)

      if (nint .eq. 1) then           ! Found triple point.

        nints = nints + 1
        write (   6, 8240) xint, yint, zint
        write (iout, 8240) xint, yint, zint
        call plotpt (xint, yint, zint, nerrg)

      endif                           ! Found triple point.

c.... Search near initial point on intersection of planes.

      dintmn = -tolp
      dintmx =  tolp

      call aptrkis (ex, ey, ez, ux, uy, uz,
     &              qc(nqq1), qx(nqq1), qy(nqq1), qz(nqq1),
     &              qxy(nqq1), qyz(nqq1), qzx(nqq1),
     &              qxx(nqq1), qyy(nqq1), qzz(nqq1),
     &              dintmn, dintmx, 1, tol,
     &              nint, xint, yint, zint, dtrack,
     &              snx, sny, snz, costh, nerra)

      if (nint .eq. 1) then           ! Found triple point.

        nints = nints + 1
        write (   6, 8240) xint, yint, zint
        write (iout, 8240) xint, yint, zint
        call plotpt (xint, yint, zint, nerrg)

      endif                           ! Found triple point.

c.... Search in positive direction along intersection of planes.

      dintmn = tolp
      dintmx = 1.e99

      call aptrkis (ex, ey, ez, ux, uy, uz,
     &              qc(nqq1), qx(nqq1), qy(nqq1), qz(nqq1),
     &              qxy(nqq1), qyz(nqq1), qzx(nqq1),
     &              qxx(nqq1), qyy(nqq1), qzz(nqq1),
     &              dintmn, dintmx, 1, tol,
     &              nint, xint, yint, zint, dtrack,
     &              snx, sny, snz, costh, nerra)

      if (nint .eq. 1) then           ! Found triple point.

        nints = nints + 1
        write (   6, 8240) xint, yint, zint
        write (iout, 8240) xint, yint, zint
        call plotpt (xint, yint, zint, nerrg)

      endif                           ! Found triple point.

      if (nints .eq. 0) then          ! No triple point.

        write (   6, 8420)
        write (iout, 8420)

      else                            ! Found  one or more triple points.

        pdist = 0.0

        proxptx = xint
        proxpty = yint
        proxptz = zint

        proxvx  = xint
        proxvy  = yint
        proxvz  = zint

      endif                           ! Tested number of triple points.

  999 return

c.... End of subroutine triple.       (+1 line)
      end
