      subroutine variable

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  variable Updated     2006 October 11 17:50.
c               variable Originated  1993 November 4 11:50.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display and create named variables by specifying a name and
c               a value or a mathematical expression.
c               Initial values are pi, deg/rad, rad/deg, ebase, the golden ratio
c               rgold, and Euler's constant, euler.
c
c     Logic:    Entry points:  variable.
c variable      Called by:  doenddo, helpall, main.
c
c variable      Calls:  aptbtod, aptchfp, aptchmv, aptcmad, aptffip,
c               aptmord, aptpfac, aptprim, aptrips, aptsubr, aptvang, aptvdot
c               bigfind, compsubs, incrsubs, namechk, savecmd, symbdisp
c               varfind, varmode, varout, varstore, vectfind.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:  1997 January 9 10:30.  Changed input of ASCII variables.
c               Allow more than 8 characters in ASCII value.
c               Only need special input for ASCII values.  ASCII variables
c               with integer and floating-point values are recognizable from
c               their first character being "@".  See below.
c
c               1998 October 20 14:45.  Added commands:
c                 variable VARNAME + F1 F2 F3 ... FN
c                 variable VARNAME * F1 F2 F3 ... FN
c               to allow finding the sum or product of many factors.
c
c               2002 August 21 18:30.  Eliminated ASCII variables.
c               Replaced by symbols.
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      integer      ifact(18)          ! Prime factor of modulus.
      integer      ipow (18)          ! Exponent of ifact.
      integer      intarg(2)          ! Integer arguments to aptcmad.

      character*24 anamex             ! Temporary name of a variable.
      character*24 anamexx            ! Temporary name of a variable.
      character*8  ans                ! User's response.
      character*64 asrce              ! Characters $, %, @, A-Z, a-z.

      character*48 avartagx           ! Descriptive tag for variable.
      character*1  acharx             ! Single or double quote.


c===============================================================================

c.... Input line:  variable [help]

 8110 format ('  Command "variable":  acceptable input lines include:' /
     &  '  "variable [help]"              Display this message.' /
     &  '  "variable all"                 Display all named variables.'/
     &  '  "variable list [VARNAME1 VARNAME2 VARNAME3 ...]"' /
     &  '     Display all listed variables (default is all',
     &  ' variables).' /
     &  '  "variable list VARNAME1 thru VARNAME2"' /
     &  '     Display named variables VARNAME1 thru VARNAME2.' /
     &  '  "variable VARNAME"             Display named variable',
     &  ' VARNAME.' //
     &  '  "variable VARNAME [=] FVAR"    Create floating point',
     &  ' variable VARNAME = FVAR.' /
     &  '     FVAR may be numerical or a floating point variable.' /
     &  '  "variable VARNAME [=] IVAR"    Create integer variable',
     &  ' VARNAME = IVAR.' /
     &  '     IVAR may be numerical or an integer variable.' //
     &  '  In the 2 commands above, "=" is optional if the',
     &  ' next argument is numerical.' //
     &  '  "variable VARNAME big BIGNAME"    Create variable VARNAME',
     &  ' with the value of' /
     &  '     big integer BIGNAME, if it will fit in a machine word.' )

 8112 format (/
     &  '  "variable VARNAME pi"        Create variable VARNAME =',
     &  ' pi = acos(-1).' /
     &  '  "variable VARNAME deg/rad"   Create variable VARNAME =',
     &  ' 180 / pi.' /
     &  '  "variable VARNAME rad/deg"   Create variable VARNAME =',
     &  ' pi / 180.' /
     &  '  "variable VARNAME ebase"     Create variable VARNAME =',
     &  ' exp (1.0).' /
     &  '  "variable VARNAME rgold"     Create variable VARNAME =',
     &  ' (1+sqrt(5))/2.' /
     &  '  "variable VARNAME euler"     Create variable VARNAME =',
     &  ' 0.577215664901533861' //
     &  '  "variable VARNAME [mean,stddev,min,max]"' /
     &  '     Create variable VARNAME with the value of the last' /
     &  '     [mean value, standard deviation, minimum value,',
     &  ' maximum value]' /
     &  '     found by command "mean" for a group of variables.' //
     &  '  "variable VARNAME random"' /
     &  '     Create variable VARNAME with a random value uniformly' /
     &  '     distributed between 0 and 1.' //
     &  '  "variable VARNAME random VMIN VMAX"' /
     &  '     Create variable VARNAME with a random value uniformly' /
     &  '     distributed between VMIN and VMAX.' //
     &  '  "variable VAR(N) prime NUMPRIME"' /
     &  '     Create a series of NUMPRIME variables VAR(N), ...,' /
     &  '     with names incremented by 1 character,' /
     &  '     with the values of the first NUMPRIME prime numbers.' //
     &  '  Type <return> for more help:  ',$)

 8113 format (/
     &  '  "variable VARNAME vector VNAME"' /
     &  '     Create variable VARNAME with the value of the',
     &  ' length of' /
     &  '     the vector VNAME.' //
     &  '  "variable VARNAME tag ''VARTAG''"', /
     &  '     Give variable VARNAME the tag VARTAG (up to 48',
     &  ' characters).' //
     &  '  "variable VARNAME angle VNAME1 VNAME2"' /
     &  '     Create variable VARNAME with the value of the',
     &  ' angle between' /
     &  '     the two vectors VNAME1 and VNAME2.' //
     &  '  "variable VARNAME dot VNAME1 VNAME2"' /
     &  '     Create variable VARNAME with the value of the',
     &  ' dot product' /
     &  '     of the two vectors VNAME1 and VNAME2.' //
     &  '  "variable VARNAME triple VNAME1 VNAME2 VNAME3"' /
     &  '     Create variable VARNAME with the value of the',
     &  ' scalar triple product of' /
     &  '     the three vectors:  (VNAME1 cross VNAME2) dot VNAME3.' )

 8114 format (a)

 8115 format (/
     &  '  "variable VARNAME distance"' /
     &  '     Create variable VARNAME with the value of the last',
     &  ' distance' /
     &  '     found by command "distance", "side", "proximal",',
     &  ' "track" or "walk".' //
     &  '  "variable VAR(N) series NUMVAR INC VAR(N-1) FADD FMULT"' /
     &  '     Create a series of NUMVAR variables VAR(N), ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     by adding FADD to FMULT times the value of the',
     &  ' preceding variable,' /
     &  '     starting from variable VAR(N-1).' //
     &  '  "variable VAR(N) data F1 F2 F3 ... enddata"' /
     &  '     Create a series of variables VAR(N), VAR(N+1), ...,' /
     &  '     with names incremented by one character, with the',
     &  ' values of the entries' /
     &  '     F1 F2 F3 ... in the following data table, up to the word',
     &  ' "enddata".' /
     &  '     Everything after "data" may be on separate',
     &  ' input lines.' //
     &  '  Type <return> for more help:  ',$)

 8116 format (/
     &  '  "variable VARNAME [=] K [+,-,*,/,^] L mod M"' /
     &  '     Create variable VARNAME with integer value',
     &  ' K [+, -, *, /, ^] mod L,' /
     &  '     where K, L and M are integers.' //
     &  '  "variable VARNAME [=] VA FUNCTION VB"' /
     &  '     Create variable VARNAME with (FUNCTION, value):' /
     &  '     (+, VA+VB), (-, VA-VB),  (*, VA*VB), (/, VA/VB),',
     &  '  (**, VA**VB),' /
     &  '     (sqrt,  VA*sqrt(VB)),  (curt,  VA*VB**(1/3)),',
     &  ' (atan2, atan(VA/VB)),' /
     &  '     (exp,   VA*exp(VB)),   (log,   VA*alog(VB)),',
     &  '  (log10, VA*log10(VB)),' /
     &  '     (sin,   VA*sin(VB)),   (cos,   VA*cos(VB)),',
     &  '   (tan,   VA*tan(VB)),' /
     &  '     (asin,  VA*asin(VB)),  (acos,  VA*acos(VB)),',
     &  '  (atan,  VA*atan(VB)),' /
     &  '     (sinh,  VA*sinh(VB)),  (cosh,  VA*cosh(VB)),',
     &  '  (tanh,  VA*tanh(VB)),' /
     &  '     (asinh, VA*asinh(VB)), (acosh, VA*acosh(VB)),',
     &  ' (atanh, VA*atanh(VB)),' )

 8118 format (
     &  '     (int,   VA*int(VB)),   (abs,   VA*abs(VB)),',
     &  '   (sign,  sign(VA,VB)),' /
     &  '     (max,   amax1(VA,VB)), (min,   amin1(VA,VB)),',
     &  ' (mod,   amod(VA,VB)),' /
     &  '     (nint,  VA*nint(VB)),  (ran, random number from',
     &  ' VA to VB),' /
     &  '     (gcd,   gcd(VA,VB)),   (lcm,   lcm(VA,VB),' /
     &  '     (fact,  VA*VB!),       (avg,   (VA+VB)/2),   ',
     &  ' (root,  VA**(1/VB).' //
     &  '  In the command above, "=" is optional if the',
     &  ' next argument is numerical.' //
     &  '  "variable VARNAME [+,*,/] F1 F2 F3 ... FN"' /
     &  '     Create variable VARNAME with the value of the' /
     &  '     [sum, product, sum of reciprocals] of the values',
     &  ' F1, F2, F3, ..., FN."' //
     &  '  "variable VARNAME polynomial XARG A(0) A(1) A(2) ... A(N)"' /
     &  '     Create variable VARNAME with the value of the polynomial',
     &  ' function:' /
     &  '     A(0) + A(1) * XARG + A(2) * XARG^2 + ... +',
     &  ' A(N) * XARG^N.' //
     &  '  Note:  use +/- for VARNAME or VAR(N) to incr/decr the last',
     &  ' variable name:' / 2x,a //
     &  '  Note:  a numerical variable will be an integer variable if',
     &  ' its fractional' /
     &  '    difference from an integer is no more than the tolerance',
     &  ' limit tol.' //
     &  '  Synonyms:  [^, **], [angle, ang], [distance, dist],',
     &  ' [dot, inner],' /
     &  '    [increment, incr], [help, h], [polynomial, poly],',
     &  ' [random, ?],' /
     &  '    [tangent, tan], [variable, var, set, s],',
     &  ' [vector, vect, v].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then
        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        if (ainput .eq. 'tty') then
          read  (   5, 8114) ans
        endif
        write (   6, 8113)
        write (iout, 8113)
        write (   6, 8115)
        write (iout, 8115)
        if (ainput .eq. 'tty') then
          read  (   5, 8114) ans
        endif
        write (   6, 8116)
        write (iout, 8116)
        write (   6, 8118) avarbls
        write (iout, 8118) avarbls
        go to 999
      endif

c===============================================================================

c.... Initialize.

      if (nword .eq. 0) then
        fvarmean =  0.0
        fvarstdd =  0.0
        fvarmin  =  1.e308
        fvarmax  = -1.e308
        go to 999
      endif

c===============================================================================

c.... Input line:  variable all
c.... Input line:  variable list

 8120 format (/ 'Number of variables  =',i5)

      if ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'all')    .or.
     &    (aword(2) .eq. 'list'))) then

        nvarx = 0
        call varout (1, nvars)
        do 120 n = 1, nvars
          if (avarbl(n) .eq. ' ') go to 120
          nvarx = nvarx + 1
  120   continue
        write (   6, 8120) nvarx
        write (iout, 8120) nvarx

        go to 999

      endif                           ! Listed all variables.

c===============================================================================

c.... Input line:  variable list VARNAME

      if (aword(2) .eq. 'list') then  ! List one or more variables.

 8150   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &   '  Named variable does not exist:  ',a)

        if (nword .eq. 3) then        ! List one variable.
          nfind = 0
          do 125 nvar = 1, nvars
            call aptsubr (avarbl(nvar), 1, 24, 80, astem, lstem,
     &                    asub, lsub, nerra)
            if ((aword(3) .eq. avarbl(nvar)) .or.
     &          (aword(3) .eq. astem))       then
              nfind = nfind + 1
              call varout (nvar, nvar)
            endif
  125     continue
          if (nfind .eq. 0) then
            write (   6, 8150) ainput, ninput, aword(3)
            write (iout, 8150) ainput, ninput, aword(3)
          endif
          go to 999
        endif                         ! Listed one variable.

c===============================================================================

c.... Input line:  variable list VARNAME1 thru VARNAME2

 8136 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of variable not from 1 to 24 characters:  ',a)
 8138 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Min name > max name:  ',a24,2x,a24)
 8139 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word "thru" is duplicated.')

      if ((nword .eq. 5) .and.
     &    (aword(4) .eq. 'thru')) then

        if ((lword(3) .lt. 1) .or. (lword(3) .gt. 24)) then
          write (   6, 8136) ainput, ninput, aword(3)
          write (iout, 8136) ainput, ninput, aword(3)
          go to 999
        endif

        if ((lword(5) .lt. 1) .or. (lword(5) .gt. 24)) then
          write (   6, 8136) ainput, ninput, aword(5)
          write (iout, 8136) ainput, ninput, aword(5)
          go to 999
        endif

        call compsubs (aword(3), aword(5), nsmall, nerrc)
        if (nsmall .eq. 2) then
          write (   6, 8138) ainput, ninput, aword(3), aword(5)
          write (iout, 8138) ainput, ninput, aword(3), aword(5)
          go to 999
        endif

        if ((aword(3) .eq. 'thru')  .or.
     &      (aword(5) .eq. 'thru')) then
          write (   6, 8139) ainput, ninput
          write (iout, 8139) ainput, ninput
          go to 999
        endif

 8155   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &   '  No variables between ',a24,' and ',a24)

        nfind = 0
        do 132 nvar = 1, nvars
          if (avarbl(nvar) .eq. ' ') go to 132
          call compsubs (avarbl(nvar), aword(3), nsmall, nerrc)
          if (nsmall .eq. 1) go to 132
          call compsubs (avarbl(nvar), aword(5), nsmall, nerrc)
          call aptsubr (avarbl(nvar), 1, 24, 80, astem, lstem,
     &                  asub, lsub, nerra)
          if ((nsmall .eq. 2) .and.
     &        (astem .ne. aword(5))) go to 132
          nfind = nfind + 1
          call varout (nvar, nvar)
  132   continue                      ! End of loop over variables.

        if (nfind .eq. 0) then
          write (   6, 8155) ainput, ninput, aword(3), aword(5)
          write (iout, 8155) ainput, ninput, aword(3), aword(5)
        endif

        go to 999

      else                            ! Word 4 was not "thru".

c===============================================================================

c....   Input line:  variable list VARNAME1 VARNAME2 VARNAME3 ...

        do 140 nw = 3, nword          ! Loop over list names.
          nfind = 0
          do 135 nvar = 1, nvars      ! Loop over variables.
            call aptsubr (avarbl(nvar), 1, 24, 80, astem, lstem,
     &                    asub, lsub, nerra)
            if ((aword(nw) .eq. avarbl(nvar)) .or.
     &          (aword(nw) .eq. astem))       then
              nfind = nfind + 1
              call varout (nvar, nvar)
            endif
  135     continue                    ! End of loop over variables.
          if (nfind .eq. 0) then
            write (   6, 8150) ainput, ninput, aword(nw)
            write (iout, 8150) ainput, ninput, aword(nw)
          endif
  140   continue                      ! End of loop over list names.
        go to 999

      endif                           ! Tested word 4.

      endif                           ! Word 2 was "list"

c===============================================================================

c.... Input line:  variable VARNAME

      if (nword .eq. 2) then
        do 150 n = 1, nvars
          if (aword(2) .eq. avarbl(n)) then
            call varout (n, n)
            go to 999
          endif
  150   continue
        write (   6, 8150) ainput, ninput, aword(2)
        write (iout, 8150) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Input line:  variable VARNAME tag 'VARTAG'

 8158 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Tag not properly quoted.')
 8160 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Tag > 48 characters:  ',i3)

      if (aword(3) .eq. 'tag') then

c....   Find VARTAG.

        ibeg = lword(1) + lword(2) + lword(3) + 3
        acharx = '"'
        ibeg1  = 81
        do 152 n = ibeg, 80

          if (achar(n) .eq. '"') then
            ibeg1  =  n + 1
            acharx = '"'
            go to 154
          endif

          if (achar(n) .eq. '''') then
            ibeg1  =  n + 1
            acharx = ''''
            go to 154
          endif

  152   continue

  154   iend1 = 0
        do 156 n = 80, ibeg1 + 1, -1

          if (achar(n) .eq. acharx) then
            iend1 = n - 1
            go to 158
          endif

  156   continue

  158   if (ibeg1 .gt. iend1) then
          write (   6, 8158) ainput, ninput
          write (iout, 8158) ainput, ninput
          go to 999
        endif

        ltag = iend1 - ibeg1 + 1
        if (ltag .gt. 48) then
          write (   6, 8160) ainput, ninput, ltag
          write (iout, 8160) ainput, ninput, ltag
          go to 999
        endif

        avartagx = ' '
        call aptchmv (achar, ibeg1, ltag, avartagx, 1, nerra)

        do 160 n = 1, nvars
          if (aword(2) .eq. avarbl(n)) then
            avartag(n) = avartagx
            call varout (n, n)
            go to 999
          endif
  160   continue
        write (   6, 8150) ainput, ninput, aword(2)
        write (iout, 8150) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... See if last variable name should be incremented and used.

 8140 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last variable can not be incremented:  ',a)

      if (aword(2) .eq. '+') then
        aword(2) = avarbls
        lword(2) = lavarbls
        call incrsubs (1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8140) ainput, ninput, avarbls
          write (iout, 8140) ainput, ninput, avarbls
          go to 999
        endif
      endif

 8141 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last variable can not be decremented:  ',a)

      if (aword(2) .eq. '-') then
        aword(2) = avarbls
        lword(2) = lavarbls
        call incrsubs (-1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8141) ainput, ninput, avarbls
          write (iout, 8141) ainput, ninput, avarbls
          go to 999
        endif
      endif

 8905 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of variable too dangerous to use:  ',a)

      call namechk (aword(2), lword(2), nerrchk)

      if (nerrchk .ne. 0) then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Test for acceptable variable name.

 8122 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Unacceptable name.  Too dangerous:  ',a)
 8125 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Unacceptable name.  Use command "tol".')
 8128 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name must begin with a-z, A-Z, $, % or @:  ',a)
 8129 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Single-character name must be lower case a to z:  ',a)

      if (mtype(2) .ne. 1) then
        write (   6, 8122) ainput, ninput, aword(2)
        write (iout, 8122) ainput, ninput, aword(2)
        go to 999
      endif

      if ((aword(2) .eq. 'random') .or. (aword(2) .eq. '?')) then
        write (   6, 8122) ainput, ninput, aword(2)
        write (iout, 8122) ainput, ninput, aword(2)
        go to 999
      endif

      if (aword(2) .eq. 'tol') then
        write (   6, 8125) ainput, ninput
        write (iout, 8125) ainput, ninput
        go to 999
      endif

      asrce = '$%@ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'

      call aptchfp (asrce, 1, 55, aword(2), 1, ipat, nerr)

      if (ipat .eq. 0) then
        write (   6, 8128) ainput, ninput, aword(2)
        write (iout, 8128) ainput, ninput, aword(2)
        go to 999
      endif

      if (lword(2) .eq. 1) then       ! Single-character name must be a-z.

        asrce = 'abcdefghijklmnopqrstuvwxyz'

        call aptchfp (asrce, 1, 26, aword(2), 1, ipat, nerr)

        if (ipat .eq. 0) then
          write (   6, 8129) ainput, ninput, aword(2)
          write (iout, 8129) ainput, ninput, aword(2)
          go to 999
        endif

      endif

 8130 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of variable not from 1 to 24 characters:  ',a)

      if ((lword(2) .lt. 1) .or. (lword(2) .gt. 24)) then
        write (   6, 8130) ainput, ninput, aword(2)
        write (iout, 8130) ainput, ninput, aword(2)
        go to 999
      endif

c.... See if a symbol already exists with this name.

 8133 format ('  variable WARNING. Reading ',a40,' line ',i5,'.' /
     &  '  A symbol or its replacement has the same name:  ',a)

      do 133 nsymb = 1, nsymbs
        if ((aword(2) .eq. asymbl(nsymb))  .or.
     &      (aword(2) .eq. asymbr(nsymb))) then
          write (   6, 8133) ainput, ninput, aword(2)
          write (iout, 8133) ainput, ninput, aword(2)
          call symbdisp (nsymb)
        endif
  133 continue

c===============================================================================

c.... Find the index in the variable array at which to store this variable.
c....   Returns nrep = 1 if replacing an existing variable.

      call varstore (nrep, n)

      if (n .eq. 0) go to 999

      nerr = 0

c===============================================================================

c.... Input line:  variable VARNAME + F1 F1 F3 ... FN
c.... Input line:  variable VARNAME * F1 F1 F3 ... FN
c.... Input line:  variable VARNAME / F1 F1 F3 ... FN

      if ((aword(3) .eq. '+')  .or.
     &    (aword(3) .eq. '*')  .or.
     &    (aword(3) .eq. '/')) then

 8210   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Only 3 words in command "variable ... [+,*,/] ..."')

        if (nword .lt. 4) then
          write (   6, 8210) ainput, ninput
          write (iout, 8210) ainput, ninput
          nerr = 1
        endif

c....   Find the factors.

 8215 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  A value is zero.  Cannot divide by zero.  ',a)

        do 220 nw = 4, nword          ! Loop over input factors.

          call varfind (nw, nw)

          if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
            write (   6, 8172) ainput, ninput, aword(nw)
            write (iout, 8172) ainput, ninput, aword(nw)
            nerr = 1
          endif

          if ((aword(3) .eq. '/') .and. (fword(nw) .eq. 0.0)) then
            write (   6, 8215) ainput, ninput, aword(nw)
            write (iout, 8215) ainput, ninput, aword(nw)
            nerr = 2
          endif

  220   continue                      ! End of loop over input factors.

c....   See if any errors were found.

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nvars) nvars = nvars - 1
          endif
          go to 999
        endif

c....   Calculate the variable.

        avarbls   = aword(2)
        lavarbls  = lword(2)

        avarbl(n) = aword(2)
        avar(n)   = aword(2)
        lvar(n)   = lword(2)
        
 9701   format ('  variable WARNING.  Reading ',a38,' line ',i5,'.' /
     &   '  Replaced ',7x,1pe20.12,' with  0.  tol = ',1pe10.2,'.')

        if (aword(3) .eq. '+') then   ! Add arguments.

          fvarnew = 0.0
          errvar  = 0.0
          do 230 nw = 4, nword
            fvarnew = fvarnew + fword(nw)
            errvar  = errvar + abs (fword(nw))
  230     continue
          if (abs (fvarnew) .le. tol * errvar) then
            if (fvarnew .ne. 0.0) then
              write (   6, 9701), ainput, ninput, fvarnew, tol
              write (iout, 9701), ainput, ninput, fvarnew, tol
            endif
            fvarnew = 0.0
          endif

        elseif (aword(3) .eq. '*') then  ! Multiply arguments.

          fvarnew = 1.0
          do 240 nw = 4, nword

            if (fword(4) .eq. 0.0) then
              fvarnew = 0.0
            else
    
              fexp = log10 (abs (fvarnew)) + log10 (abs (fword(nw)))
              if (abs (fexp) .gt. iemax) then
                write (   6, 8342) ainput, ninput, fexp
                write (iout, 8342) ainput, ninput, fexp
                nerr = 1
                go to 900
              endif
    
              fvarnew = fvarnew * fword(nw)
            endif

  240     continue

        else                          ! Add reciprocals of arguments.

          fvarnew = 0.0
          errvar  = 0.0
          do 250 nw = 4, nword
            fvarnew = fvarnew + 1.0 / fword(nw)
            errvar  = errvar + 1.0 / abs (fword(nw))
  250     continue
          if (abs (fvarnew) .le. tol * errvar) then
            if (fvarnew .ne. 0.0) then
              write (   6, 9701), ainput, ninput, fvarnew, tol
              write (iout, 9701), ainput, ninput, fvarnew, tol
            endif
            fvarnew = 0.0
          endif

        endif

        fvar(n) = fvarnew

        call varmode (n)

        call varout (n, n)

        go to 999

      endif                           ! Arg 3 was [+, *, /].

c===============================================================================

c.... Input line:  variable VARNAME poly XARG A(0) A(1) A(2) ... A(N)

      if ((aword(3) .eq. 'polynomial') .or.
     &    (aword(3) .eq. 'poly'))      then

 8490   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Fewer than 5 words in command',
     &    ' "variable ... polynomial ...":  ',i3)

        if (nword .lt. 5) then
          write (   6, 8490) ainput, ninput, nword
          write (iout, 8490) ainput, ninput, nword
          nerr = 1
        endif

c....   Find the argument and polynomial coefficients.

        do 232 nw = 4, nword          ! Loop over input arguments.

          call varfind (nw, nw)

          if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
            write (   6, 8172) ainput, ninput, aword(nw)
            write (iout, 8172) ainput, ninput, aword(nw)
            nerr = 1
          endif

  232   continue                      ! End of loop over input arguments.

c....   See if any errors were found.

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nvars) nvars = nvars - 1
          endif
          go to 999
        endif

c....   Calculate the variable.

        avarbls   = aword(2)
        lavarbls  = lword(2)

        avarbl(n) = aword(2)
        avar(n)   = aword(2)
        lvar(n)   = lword(2)

        fvarnew   = 0.0
        errvar    = 0.0

        xarg      = fword(4)

        fvarnew   = fword(5)
        errvar    = abs (fvarnew)

        if (nword .ge. 6) then
          xpow = 1.0
          do 234 nw = 6, nword
            xpow    = xpow * xarg
            term    = fword(nw) * xpow
            fvarnew = fvarnew + term
            errvar  = errvar + abs (term)
  234     continue
        endif

        if (abs (fvarnew) .le. tol * errvar) then
          if (fvarnew .ne. 0.0) then
            write (   6, 9701), ainput, ninput, fvarnew, tol
            write (iout, 9701), ainput, ninput, fvarnew, tol
          endif
          fvarnew = 0.0
        endif

        fvar(n) = fvarnew

        call varmode (n)

        call varout (n, n)

        go to 999

      endif

c===============================================================================

c.... Input line:  variable VARNAME big BIGNAME

      if ((nword .eq. 4) .and. (aword(3) .eq. 'big')) then

c....   Find the big number.

 8450   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &   '  Named big integer does not exist:  ',a)

        call bigfind (4, nb1)

        if (nb1 .eq. 0) then
          write (   6, 8450) ainput, ninput, aword(4)
          write (iout, 8450) ainput, ninput, aword(4)
          go to 999

        else

        avarbls   = aword(2)
        lavarbls  = lword(2)

        avarbl(n) = aword(2)
        avar(n)   = aword(2)
        lvar(n)   = lword(2)

c....   Find the one-word value of the big integer.

        call aptbtod (10, idig(1,nb1), ndig(nb1), idmax, iemax,
     &                ideca, fdeca, floga, nerra)

 8500   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Big integer will not fit in one machine word:  ',a)

        if ((ideca .eq. -999999) .and. (fdeca .eq. -1.e99)) then
          nerr = 1
          write (   6, 8500) ainput, ninput, abig(nb1)
          write (iout, 8500) ainput, ninput, abig(nb1)
        elseif (ideca .ne. -999999) then
          ivar(n)  = 0
          mtype(n) = 3
          fvar(n)  = fdeca
          ivarx    = fdeca
          fvarx    = ivarx
          if (fvarx .eq. fvar(n)) then
            mtype(n) = 2
            ivar(n)  = ivarx
          endif
        elseif (fdeca .ne. -1.e99) then
          ivar(n)  = 0
          mtype(n) = 3
          fvar(n)  = fdeca
        endif

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nvars) nvars = nvars - 1
          endif
          go to 999
        endif

c....   Find the mode of the value of the variable.

        call varmode (n)

        call varout (n, n)

        go to 999

        endif                         ! Found big number BIGNAME.

      endif                         ! Input line:  variable VARNAME big BIGNAME

c===============================================================================

c.... Input line:  variable VARNAME vector VNAME

      if ((aword(3) .eq. 'vector') .or.
     &    (aword(3) .eq. 'vect')   .or.
     &    (aword(3) .eq. 'v'))     then

 8501   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Not 4 words in command "variable ... vector ...":  ',i3)

        if (nword .ne. 4) then
          write (   6, 8501) ainput, ninput, nword
          write (iout, 8501) ainput, ninput, nword
          nerr = 1
        endif

c....   Find the vector.

        call vectfind (4, n1)

        if (n1 .eq. 0) nerr = 1

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nvars) nvars = nvars - 1
          endif
          go to 999
        endif

        avarbls   = aword(2)
        lavarbls  = lword(2)

        avarbl(n) = aword(2)
        avar(n)   = aword(2)
        lvar(n)   = lword(2)
        fvar(n)   = vlen(n1)          ! Length of vector VNAME.

c....   Find the mode of the value of the variable.

        call varmode (n)

        call varout (n, n)

        go to 999

      endif                           ! Length of a vector.

c===============================================================================

c.... Input line:  variable VARNAME dot VNAME1 VNAME2

      if ((aword(3) .eq. 'dot')    .or.
     &    (aword(3) .eq. 'inner')) then

 8502   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Not 5 words in command "variable ... dot ...":  ',i3)

        if (nword .ne. 5) then
          write (   6, 8502) ainput, ninput, nword
          write (iout, 8502) ainput, ninput, nword
          nerr = 1
        endif

c....   Find the first vector.

        call vectfind (4, n1)

        if (n1 .eq. 0) nerr = 1

c....   Find the second vector.

        call vectfind (5, n2)

        if (n2 .eq. 0) nerr = 1

c....   Find the dot product of the two vectors.

        call aptvdot (xcom(n1), ycom(n1), zcom(n1),
     &                xcom(n2), ycom(n2), zcom(n2), 1, tol, vdot, nerra)

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nvars) nvars = nvars - 1
          endif
          go to 999
        endif

        avarbls   = aword(2)
        lavarbls  = lword(2)

        avarbl(n) = aword(2)
        avar(n)   = aword(2)
        lvar(n)   = lword(2)
        fvar(n)   = vdot

c....   Find the mode of the value of the variable.

        call varmode (n)

        call varout (n, n)

        go to 999

      endif                           ! Dot product of two vectors.

c===============================================================================

c.... Input line:  variable VARNAME triple VNAME1 VNAME2 VNAME3

      if (aword(3) .eq. 'triple') then

 8504   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Not 6 words in command "variable ... triple ...":  ',i3)

        if (nword .ne. 6) then
          write (   6, 8504) ainput, ninput, nword
          write (iout, 8504) ainput, ninput, nword
          nerr = 1
        endif

c....   Find the first vector.

        call vectfind (4, n1)

        if (n1 .eq. 0) nerr = 1

c....   Find the second vector.

        call vectfind (5, n2)

        if (n2 .eq. 0) nerr = 1

c....   Find the third vector.

        call vectfind (6, n3)

        if (n3 .eq. 0) nerr = 1

c....   Find the scalar triple product of the three vectors.

            call aptrips (xcom(n1), ycom(n1), zcom(n1),
     &                    xcom(n2), ycom(n2), zcom(n2),
     &                    xcom(n3), ycom(n3), zcom(n3), 1, tol,
     &                    vtrip, nerra)

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nvars) nvars = nvars - 1
          endif
          go to 999
        endif

        avarbls   = aword(2)
        lavarbls  = lword(2)

        avarbl(n) = aword(2)
        avar(n)   = aword(2)
        lvar(n)   = lword(2)
        fvar(n)   = vtrip

c....   Find the mode of the value of the variable.

        call varmode (n)

        call varout (n, n)

        go to 999

      endif                           ! Triple product of three vectors.

c===============================================================================

c.... Input line:  variable VARNAME angle VNAME1 VNAME2

      if ((aword(3) .eq. 'angle') .or.
     &    (aword(3) .eq. 'ang'))  then

 8503   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Not 5 words in command "variable ... angle ...":  ',i3)

        if (nword .ne. 5) then
          write (   6, 8503) ainput, ninput, nword
          write (iout, 8503) ainput, ninput, nword
          nerr = 1
        endif

c....   Find the first vector.

        call vectfind (4, n1)

        if (n1 .eq. 0) nerr = 1

c....   Find the second vector.

        call vectfind (5, n2)

        if (n2 .eq. 0) nerr = 1

c....   Find the cosine of the angle between the two vectors.

        call aptvang (xcom(n1), ycom(n1), zcom(n1),
     &                xcom(n2), ycom(n2), zcom(n2), 1, tol, vcos, nerra)

c....   Find the angle between the two vectors.

        vang = acos (vcos)
        if (abs (vang) .lt. 1.e-5) vang = 0.0
        if (abs (vang - pi) .lt. 1.e-5) vang = pi

        if (angunit .eq. 'degrees') then
          vang = vang * 180.0 / pi
        endif

 8165   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Can not find the angle if one vector has zero length.')

        if ((vlen(n1) .eq. 0.0) .or.
     &      (vlen(n2) .eq. 0.0)) then
          write (   6, 8165)
          write (iout, 8165)
          nerr = 1
        endif

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nvars) nvars = nvars - 1
          endif
          go to 999
        endif

        avarbls   = aword(2)
        lavarbls  = lword(2)

        avarbl(n) = aword(2)
        avar(n)   = aword(2)
        lvar(n)   = lword(2)
        fvar(n)   = vang

c....   Find the mode of the value of the variable.

        call varmode (n)

 8166   format ('  Angle is in ',a)

        write (   6, 8166) angunit
        write (iout, 8166) angunit

        call varout (n, n)

        go to 999

      endif                           ! Angle between two vectors.

c===============================================================================

c.... Input line:  variable VAR02 prime NUMVAR

      if (aword(3) .eq. 'prime') then

 8405   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Not 4 words in command "variable ... prime ...":  ',i3)

        if (nword .ne. 4) then
          write (   6, 8405) ainput, ninput, nword
          write (iout, 8405) ainput, ninput, nword
          nerr = 1
        endif

        call varfind (4, 4)

        if (mtype(4) .eq. 2) then
          numvar = iword(4)
          if (numvar .lt. 1) then
            write (   6, 8520) ainput, ninput, aword(4)
            write (iout, 8520) ainput, ninput, aword(4)
            nerr = 1
          endif
        else
          write (   6, 8525) ainput, ninput, aword(4)
          write (iout, 8525) ainput, ninput, aword(4)
          nerr = 1
        endif

c....   Find the base variable.

        do 430 nvar = 1, nvars
          if (aword(2) .eq. avarbl(nvar)) then
            nbase = nvar
            call varout (nbase, nbase)
            go to 440
          endif
  430   continue

c....   See if the variable names can be generated uniquely.

  440   anamex  = aword(2)
        lanamex = lword(2)
        namex   = numvar - 1

        call incrsubs (namex, anamex, 1, lanamex, nerra)

        if (nerra .ne. 0) then
          nerr = 1
          write (   6, 8570) ainput, ninput, numvar, aword(2)
          write (iout, 8570) ainput, ninput, numvar, aword(2)
        endif

        call namechk (anamex, lanamex, nerrchk)

        if (nerrchk .ne. 0) then
          nerr = 1
          write (   6, 8905) ainput, ninput, anamex
          write (iout, 8905) ainput, ninput, anamex
        endif

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nvars) nvars = nvars - 1
          endif
          go to 999
        endif

c....   Find the prime numbers.

 8475   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Can not generate prime numbers.  aptprim nerra =',i3 )
 8480   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Not enough room in temporary array for:  ',i5)

        if (ntempm .ge. numvar) then

          ntemps = numvar

          call aptprim (10000, numvar, 0, itempa, ftempa, np, nerra)

          if (nerra .gt. 0) then
            write (   6, 8475) ainput, ninput, nerra
            write (iout, 8475) ainput, ninput, nerra
            nerr = 1
          endif

        else
          write (   6, 8480) ainput, ninput, numvar
          write (iout, 8480) ainput, ninput, numvar
          nerr = 1
        endif

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nvars) nvars = nvars - 1
          endif
          go to 999
        endif

c....   Create the series of variables.

        anamex   = aword(2)           ! Initial variable name.
        lanamex  = lword(2)           ! Initial variable name.
        fvars    = fvar(nbase)

        do 480 num = 1, numvar        ! Loop over the new variables.

c....     Find the index in the variable list for the new variable.

          aword(2) = anamex
          lword(2) = lanamex

          call varstore (nrep, n)

          if (n .eq. 0) then
            if (nrep .eq. 0) then
              if (n .eq. nvars) nvars = nvars - 1
            endif
            go to 999
          endif

          avarbls    = anamex
          lavarbls   = lanamex

          avarbl(n)  = anamex

          ivar(n)    = itempa(num)
          fvar(n)    = itempa(num)

c....     Find the mode of the value of the variable.

          call varmode (n)

          call varout (n, n)

c....     Find the next variable name, by incrementing by 1.

          call incrsubs (1, anamex, 1, lanamex, nerra)

          call namechk (anamex, lanamex, nerrchk)

          if (nerrchk .ne. 0) then
            write (   6, 8905) ainput, ninput, anamex
            write (iout, 8905) ainput, ninput, anamex
            go to 999
          endif

  480   continue                      ! End of loop over the new variables.

        go to 999

      endif                           ! Created a series of variables.

c===============================================================================

c.... Input line:  variable VAR(N) series NUMVAR INC VAR(N-1) FADD FMULT

      if (aword(3) .eq. 'series') then

 8505   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Not 8 words in command "variable ... series ...":  ',i3)

        if (nword .ne. 8) then
          write (   6, 8505) ainput, ninput, nword
          write (iout, 8505) ainput, ninput, nword
          nerr = 1
        endif

 8520   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word NUMVAR is not a positive integer:  ',a)
 8525   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word NUMVAR is not an integer or integer variable:  ',a)

        call varfind (4, 4)

        if (mtype(4) .eq. 2) then
          numvar = iword(4)
          if (numvar .lt. 1) then
            write (   6, 8520) ainput, ninput, aword(4)
            write (iout, 8520) ainput, ninput, aword(4)
            nerr = 1
          endif
        else
          write (   6, 8525) ainput, ninput, aword(4)
          write (iout, 8525) ainput, ninput, aword(4)
          nerr = 1
        endif

 8510   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word INC is zero:  ',a)
 8515   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word INC is not an integer or integer variable:  ',a)

        call varfind (5, 5)

        if (mtype(5) .eq. 2) then
          inc = iword(5)
          if (inc .eq. 0) then
            write (   6, 8510) ainput, ninput, aword(5)
            write (iout, 8510) ainput, ninput, aword(5)
            nerr = 1
          endif
        else
          write (   6, 8515) ainput, ninput, aword(5)
          write (iout, 8515) ainput, ninput, aword(5)
          nerr = 1
        endif

c....   Find the base variable.

        do 530 nvar = 1, nvars
          if (aword(6) .eq. avarbl(nvar)) then
            nbase = nvar
            call varout (nbase, nbase)
            go to 540
          endif
  530   continue
        write (   6, 8150) ainput, ninput, aword(6)
        write (iout, 8150) ainput, ninput, aword(6)
        nerr = 1

c....   Find the coefficients in the recursion formula.

 8172 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer or floating point value or variable:  ',a)

  540   call varfind (7, 8)

        do 560 nw = 7, 8
          if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
            write (   6, 8172) ainput, ninput, aword(nw)
            write (iout, 8172) ainput, ninput, aword(nw)
            nerr = 1
          endif
  560   continue

        fadd  = fword(7)
        fmult = fword(8)

c....   See if the variable names can be generated uniquely.

 8570   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Can not generate',i5,' variable names from:  ',a)

        anamex  = aword(2)
        lanamex = lword(2)
        namex   = inc * (numvar - 1)

        call incrsubs (namex, anamex, 1, lanamex, nerra)

        if (nerra .ne. 0) then
          write (   6, 8570) ainput, ninput, numvar, aword(2)
          write (iout, 8570) ainput, ninput, numvar, aword(2)
          nerr = 1
        endif

        call namechk (anamex, lanamex, nerrchk)

        if (nerrchk .ne. 0) then
          nerr = 1
          write (   6, 8905) ainput, ninput, anamex
          write (iout, 8905) ainput, ninput, anamex
        endif

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nvars) nvars = nvars - 1
          endif
          go to 999
        endif

c....   Create the series of variables.

        nerr     = 0
        anamex   = aword(2)           ! Initial variable name.
        lanamex  = lword(2)           ! Initial variable name.
        fvars    = fvar(nbase)
        sumterm  = fvar(nbase)

        do 580 num = 1, numvar        ! Loop over the new variables.

c....     Find the index in the variable list for the new variable.

          aword(2) = anamex
          lword(2) = lanamex

          call varstore (nrep, n)

          if (n .eq. 0) then
            if (nrep .eq. 0) then
              if (n .eq. nvars) nvars = nvars - 1
            endif
            go to 999
          endif

          avarbls    = anamex
          lavarbls   = lanamex

          avarbl(n)  = anamex

          fvar(n)    = fadd + fmult * fvars
          serr       = tol * (abs (fadd) + abs (fmult * fvars))
          if (abs (fvar(n)) .le. serr) then
            if (fvar(n) .ne. 0.0) then
              write (   6, 9701), ainput, ninput, fvarnew, tol
              write (iout, 9701), ainput, ninput, fvarnew, tol
            endif
            fvar(n) = 0.0
          endif
          fvars      = fvar(n)
          sumterm    = sumterm + fvar(n)

c....     Find the mode of the value of the variable.

          call varmode (n)

          call varout (n, n)

c....     Find the next variable name, by incrementing by inc.

          call incrsubs (inc, anamex, 1, lanamex, nerra)

          call namechk (anamex, lanamex, nerrchk)

          if (nerrchk .ne. 0) then
            write (   6, 8905) ainput, ninput, anamex
            write (iout, 8905) ainput, ninput, anamex
            go to 999
          endif

  580   continue                      ! End of loop over the new variables.

 8580 format ('  Series sum      ',i16,6x,'Integer')
 8585 format ('  Series sum      ',1pe20.12,2x,'Floating point')

        msum = 2
        if (abs (sumterm) .gt. 10.0**idmax) then
          msum = 3
        else
          isumterm = nint (sumterm)
          sumtermx = isumterm
          errsum   = tol * abs (sumterm)
          if (abs (sumterm - sumtermx) .gt. errsum) msum = 3
        endif

        if (msum .eq. 2) then
          write (   6, 8580) isumterm
          write (iout, 8580) isumterm
        else
          write (   6, 8585) sumterm
          write (iout, 8585) sumterm
        endif

        go to 999

      endif                           ! Created a series of variables.

c===============================================================================

c.... Input line:  variable VAR(N) data

c.... Create variables from a data table.

      if (aword(3) .eq. 'data') then

        anamex  = aword(2)
        lanamex = lword(2)
        nfirst  = 1
cbugc***DEBUG begins.
cbug 9702 format ('DEBUG anamex=',a,' lanamex=',i2,' nfirst=',i2,' n=',i2)
cbug      write (   6, 9702) anamex, lanamex, nfirst, n
cbug      write (iout, 9702) anamex, lanamex, nfirst, n
cbugc***DEBUG ends.

        if (nword .gt. 3) then
          do nw = 4, nword
            aword(nw-3) = aword(nw)
            iword(nw-3) = iword(nw)
            fword(nw-3) = fword(nw)
            mtype(nw-3) = mtype(nw)
            lword(nw-3) = lword(nw)
            aword(nw-3) = aword(nw)
          enddo
          nword = nword - 3
          go to 320
        endif

  310   ninput = ninput + 1

cbugc***DEBUG begins.
cbug 9771 format ('var DEBUG iin=',i3,'  ainput=',a16,'  ninput=',i3)
cbug      write (   6, 9771) iin, ainput, ninput
cbug      write (iout, 9771) iin, ainput, ninput
cbugc***DEBUG begins.
        call aptffip (iin, afd, iamax, idmax, iemax, aword, iword,
     &                fword, mtype, lword, nword, achar, nerrf)
cbugc***DEBUG begins.
cbug 9101 format ('nbug=',i5,'  iin='i3)
cbug      nbug = 301
cbug      write (   6, 9101) nbug, iin
cbug      write (iout, 9101) nbug, iin
cbugc***DEBUG ends.
cbugc***DEBUG begins.
cbug 9777 format ('var DEBUG.  ninput =',i5,'  iin=',i5,'  achar:' / 80a1)
cbug 9778 format ('var DEBUG.  nerrf  =',i5,'  aword(1):' / a80)
cbug      write (   6, 9777) ninput, iin, (achar(n), n = 1, 80)
cbug      write (iout, 9777) ninput, iin, (achar(n), n = 1, 80)
cbug      write (   6, 9778) nerrf, aword(1)
cbug      write (iout, 9778) nerrf, aword(1)
cbugc***DEBUG ends.

        if (nword .eq. 0) go to 310

        call savecmd

 8350 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer or floating point value:  ',a)

  320   do nw = 1, nword              ! Loop over variable values.
cbugc***DEBUG begins.
cbug 9703 format ('DEBUG nw=',i2,' nword=',i2)
cbug      write (   6, 9703) nw, nword
cbug      write (iout, 9703) nw, nword
cbugc***DEBUG ends.

          if (aword(nw) .eq. 'enddata') go to 999

cbugc***DEBUG begins.
cbug 9704 format ('DEBUG n=',i2)
cbug      write (   6, 9704) n
cbug      write (iout, 9704) n
cbugc***DEBUG ends.

          if (nfirst .eq. 0) then     ! Not the first variable.

c....       Find the next variable name, by incrementing by 1.

            call incrsubs (1, anamex, 1, lanamex, nerra)
cbugc***DEBUG begins.
cbug 9705 format ('DEBUG anamex=',a,' lanamex=',i2)
cbug      write (   6, 9705) anamex, lanamex
cbug      write (iout, 9705) anamex, lanamex
cbugc***DEBUG ends.
            if (nerra .ne. 0) then
              write (   6, 8140) ainput, ninput, avarbls
              write (iout, 8140) ainput, ninput, avarbls
              go to 999
            endif

            call namechk (anamex, lanamex, nerrchk)
            if (nerrchk .ne. 0) then
              write (   6, 8905) ainput, ninput, anamex
              write (iout, 8905) ainput, ninput, anamex
              go to 999
            endif

            anamexx  = aword(2)
            aword(2) = anamex

            call varstore (nrep, n)

            aword(2) = anamexx

            if (n .eq. 0) then
              if (nrep .eq. 0) then
                if (n .eq. nvars) nvars = nvars - 1
              endif
              go to 999
            endif

          endif                       ! Not the first variable.

          nfirst = 0

          call varfind (nw, nw)

          if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
            write (   6, 8350) ainput, ninput, aword(nw)
            write (iout, 8350) ainput, ninput, aword(nw)
            if (nrep .eq. 0) then
              if (n .eq. nvars) nvars = nvars - 1
            endif
            go to 999
          endif
cbugc***DEBUG begins.
cbug      write (   6, 9705) anamex, lanamex
cbug      write (iout, 9705) anamex, lanamex
cbugc***DEBUG ends.

          avarbls    = anamex
          lavarbls   = lanamex

          avar(n)    = anamex
          avarbl(n)  = anamex
          avartag(n) = ' '
          fvar(n)    = fword(nw)
          ivar(n)    = iword(nw)
          lvar(n)    = lanamex
          mvar(n)    = mtype(nw)
cbugc***DEBUG begins.
cbug 9706 format ('DEBUG avar=',a,' ivar=',i9,' mvar=',i2)
cbug      write (   6, 9706) avar(n), ivar(n), mvar(n)
cbug      write (iout, 9706) avar(n), ivar(n), mvar(n)
cbugc***DEBUG ends.

          call varout (n, n)

        enddo

        go to 310

      endif



c===============================================================================

c.... Test for optional equal sign.

      if ((nword .ge. 3)      .and.
     &   ((mtype(3) .eq. 2)   .or.
     &    (mtype(3) .eq. 3))) then

        nword  = nword + 1

        do 802 nn = nword, 4, -1
          aword(nn) = aword(nn-1)
          mtype(nn) = mtype(nn-1)
          iword(nn) = iword(nn-1)
          fword(nn) = fword(nn-1)
          lword(nn) = lword(nn-1)
  802   continue

        aword(3) = '='
        mtype(3) = 1
        iword(3) = 0
        fword(3) = 0.0
        lword(3) = 1
      endif                           ! Added optional equal sign.

c=======================================================================********

c.... Input line:  variable VARNAME pi
c.... Input line:  variable VARNAME deg/rad
c.... Input line:  variable VARNAME rad/deg
c.... Input line:  variable VARNAME rgold
c.... Input line:  variable VARNAME euler
c.... Input line:  variable VARNAME ebase
c.... Input line:  variable VARNAME random
c.... Input line:  variable VARNAME distance
c.... Input line:  variable VARNAME mean
c.... Input line:  variable VARNAME stddev
c.... Input line:  variable VARNAME min
c.... Input line:  variable VARNAME max

c.... Test for special variables.

 8134 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      if (nword .eq. 3) then

        if (aword(3) .eq. 'pi') then
          fvarnew  = pi
        elseif (aword(3) .eq. 'deg/rad') then
          fvarnew  = 180.0 / pi
        elseif (aword(3) .eq. 'rad/deg') then
          fvarnew  = pi / 180.0
        elseif (aword(3) .eq. 'ebase') then
          fvarnew  = ebase
        elseif (aword(3) .eq. 'rgold') then
          fvarnew  = 0.5 * (sqrt (5.0) + 1.0)
        elseif ((aword(3) .eq. 'euler')  .or.
     &          (aword(3) .eq. 'Euler')) then
          fvarnew  = euler
        elseif ((aword(3) .eq. 'random') .or.
     &          (aword(3) .eq. '?'))     then
          fvarnew  = ranf ()
        elseif ((aword(3) .eq. 'distance') .or.
     &          (aword(3) .eq. 'dist'))    then
          fvarnew = pdist
        elseif (aword(3) .eq. 'mean') then
          fvarnew = fvarmean
        elseif (aword(3) .eq. 'stddev') then
          fvarnew = fvarstdd
        elseif (aword(3) .eq. 'min') then
          fvarnew = fvarmin
        elseif (aword(3) .eq. 'max') then
          fvarnew = fvarmax
        else
          write (   6, 8134) ainput, ninput
          write (iout, 8134) ainput, ninput
          nerr = 1
        endif

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nvars) nvars = nvars - 1
          endif
          go to 999
        endif

        avarbls   = aword(2)
        lavarbls  = lword(2)

        avarbl(n) = aword(2)
        avar(n)   = aword(2)
        lvar(n)   = lword(2)
        fvar(n)   = fvarnew

c....   Find the mode of the value of the variable.

        call varmode (n)

        call varout (n, n)

        go to 999

      endif                           ! Special variables.

c=======================================================================********

c.... Input line:  variable VARNAME random VARMIN VARMAX

      if ((aword(3) .eq. 'random') .or.
     &    (aword(3) .eq. '?'))     then
        if (nword. eq. 5) then
          call varfind (4, 5)
          if ((mtype(4) .ne. 2)  .and.
     &        (mtype(4) .ne. 3)) then
            nerr = 1
            write (   6, 8172) ainput, ninput, aword(4)
            write (iout, 8172) ainput, ninput, aword(4)
          elseif ((mtype(5) .ne. 2)  .and.
     &            (mtype(5) .ne. 3)) then
            nerr = 1
            write (   6, 8172) ainput, ninput, aword(5)
            write (iout, 8172) ainput, ninput, aword(5)
          else
            fvarnew  = fword(4) + ranf () * (fword(5) - fword(4))
          endif
        else
          nerr = 1
          write (   6, 8134) ainput, ninput
          write (iout, 8134) ainput, ninput
        endif

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nvars) nvars = nvars - 1
          endif
          go to 999
        endif

        avarbls   = aword(2)
        lavarbls  = lword(2)

        avarbl(n) = aword(2)
        avar(n)   = aword(2)
        lvar(n)   = lword(2)
        fvar(n)   = fvarnew

c....   Find the mode of the value of the variable.

        call varmode (n)

        call varout (n, n)

        go to 999

      endif                           ! Special variables.

c=======================================================================********

c.... Test input line format for errors.

 8135 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Wrong number of words on input line:  ',i3)

      if ((nword .ne. 4)  .and.
     &    (nword .ne. 6)  .and.
     &    (nword .ne. 8)) then
        write (   6, 8135) ainput, ninput, nword
        write (iout, 8135) ainput, ninput, nword
        nerr = 1
      endif

      if (aword(3) .ne. '=') then
        write (   6, 8134) ainput, ninput
        write (iout, 8134) ainput, ninput
        nerr = 1
      endif

c.... See if word 4 is integer or floating point.

      call varfind (4, 4)

      if ((mtype(4) .ne. 2) .and. (mtype(4) .ne. 3)) then
        write (   6, 8172) ainput, ninput, aword(4)
        write (iout, 8172) ainput, ninput, aword(4)
        nerr = 1
      endif

      if (nerr .ne. 0) then
        if (nrep .eq. 0) then
          if (n .eq. nvars) nvars = nvars - 1
        endif
        go to 999
      endif

c=======================================================================********

c.... Input line:  variable VARNAME = K [+,-,*,/,^] L mod M
c.... Input line:  variable VARNAME   K [+,-,*,/,^] L mod M  ! K numerical.
cbugc***DEBUG begins.
cbug 6601 format ('var DEBUG.  nbug=',i3)
cbug      nbug = 1
cbug      write (   6, 6601) nbug
cbug      write (iout, 6601) nbug
cbugc***DEBUG ends.

      if ((nword .eq. 8) .and. (aword(7) .eq. 'mod')) then
cbugc***DEBUG begins.
cbug      nbug = 2
cbug      write (   6, 6601) nbug
cbug      write (iout, 6601) nbug
cbugc***DEBUG ends.

 8305   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  function is not [+, -, *, /, ^]:  ',a)

        if ((aword(5) .ne. '+')   .and.
     &      (aword(5) .ne. '-')   .and.
     &      (aword(5) .ne. '*')   .and.
     &      (aword(5) .ne. '/')   .and.
     &      (aword(5) .ne. '^')   .and.
     &      (aword(5) .ne. '**')) then
          write (   6, 8305) ainput, ninput, aword(5)
          write (iout, 8305) ainput, ninput, aword(5)
          nerr = 1
        endif

        call varfind (4, 4)
        call varfind (6, 6)
        call varfind (8, 8)

 8310   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  K, L or M is not an integer or integer variable:' /
     &    a / a / a )

        if ((mtype(4) .ne. 2)  .or.
     &      (mtype(6) .ne. 2)  .or.
     &      (mtype(8) .ne. 2)) then
          write (   6, 8310) ainput, ninput, (aword(nw), nw = 4, 8, 2)
          write (iout, 8310) ainput, ninput, (aword(nw), nw = 4, 8, 2)
          nerr = 1
        endif

 8312 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Cannot do modulo < 2:  ',a)

        if (iword(8) .lt. 2) then
          write (   6, 8312) ainput, ninput, aword(8)
          write (iout, 8312) ainput, ninput, aword(8)
          nerr = 1
        endif

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nvars) nvars = nvars - 1
          endif
          go to 999
        endif

c....   Find the function K [+, -, *, /, ^] L.

 8315   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  K ',a1,' L and modulus M have a common factor,' /
     &          '  so K ',a1,' L has no order or inverse.')
 8320   format ('  Order   of',i20,' mod',i20,' =',i20 /
     &          '  Inverse of',i20,' mod',i20,' =',i20 )
 8325   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  Divisor L and modulus M have a common factor,' /
     &          '  so divisor L has no inverse, can not divide.')
 8330   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  Can not use a negative power L in modular arith:  '
     &          ,a)
 8335   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  Modulus M can not be factored:  ',i20 /
     &          '  Residue =',i20,'  nerra =',i2)
 8340   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  Too big for an integer.')
 8342   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  Floating point overflow.  Exponent of 10 =',1pe22.14)
 8343   format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &          '  Integer overflow.  Exponent of 10 =',1pe22.14)
cbugc***DEBUG begins.
cbug      nbug = 3
cbug      write (   6, 6601) nbug
cbug      write (iout, 6601) nbug
cbugc***DEBUG ends.

        if (aword(5) .eq. '+') then

          ivarnew = iword(4) + iword(6)

        elseif (aword(5) .eq. '-') then

          ivarnew = iword(4) - iword(6)

        elseif (aword(5) .eq. '*') then
           
          if (iword(4) .eq. 0) then
            ivarnew = 0.0
          else
  
            fexp = log10 (abs (fword(4))) + log10 (abs (fword(6)))
            if (abs (fexp) .gt. iemax) then
              write (   6, 8342) ainput, ninput, fexp
              write (iout, 8342) ainput, ninput, fexp
              nerr = 1
              go to 900
            endif
  
            ivarnew = iword(4) * iword(6)
          endif

          ivarnew = iword(4) * iword(6)

        elseif (aword(5) .eq. '/') then

          call aptmord (iword(6), iword(8), mord, minv, nerra)

          if (nerra .eq. -1) then
            write (   6, 8325) ainput, ninput
            write (iout, 8325) ainput, ninput
            nerr = 1
            go to 900
          endif

        elseif (aword(5) .eq. '^') then
cbugc***DEBUG begins.
cbug      nbug = 4
cbug      write (   6, 6601) nbug
cbug      write (iout, 6601) nbug
cbugc***DEBUG ends.

          if (iword(6) .lt. 0) then
            write (   6, 8330) ainput, ninput, aword(6)
            write (iout, 8330) ainput, ninput, aword(6)
            nerr = 1
            go to 900
          endif
cbugc***DEBUG begins.
cbug      nbug = 5
cbug      write (   6, 6601) nbug
cbug      write (iout, 6601) nbug
cbugc***DEBUG ends.

          call aptpfac (iword(8), 18, ifact, ipow, nfact, itot, inumr,
     &                  nerra)

          if(nerra .ne. 0) then
            write (   6, 8335) ainput, ninput, iword(8), inumr, nerra
            write (iout, 8335) ainput, ninput, iword(8), inumr, nerra
            nerr = 1
            go to 900
          endif
cbugc***DEBUG begins.
cbug      nbug = 6
cbug      write (   6, 6601) nbug
cbug      write (iout, 6601) nbug
cbugc***DEBUG ends.

          call aptmord (iword(6), iword(8), mord, minv, nerra)

          ipowm   = mod (iword(6), itot)
          if (mord .lt. 0) ipowm = ipowm + itot
          iarg    = mod (iword(4), iword(8))
          if (iarg .lt. 0) iarg = iarg + iword(8)
          farg    = iarg
          fpowm   = ipowm
          fexp    = fpowm * log10 (farg)
          if (fexp .gt. idmax) then
            write (   6, 8343) ainput, ninput, fexp
            write (iout, 8343) ainput, ninput, fexp
            nerr = 1
            go to 900
          endif
cbugc***DEBUG begins.
cbug      nbug = 7
cbug      write (   6, 6601) nbug
cbug      write (iout, 6601) nbug
cbugc***DEBUG ends.

          ivarnew = iarg**ipowm
        endif
cbugc***DEBUG begins.
cbug 6602 format ('var DEBUG.  nbug=',i3,'  ivarnew=',i20)
cbug      nbug = 8
cbug      write (   6, 6602) nbug, ivarnew
cbug      write (iout, 6602) nbug, ivarnew
cbugc***DEBUG ends.

c....   Find the function K [+, -, *, /, ^] L modulo M.

        ivarnew = mod (ivarnew, iword(8))
        if (ivarnew .lt. 0.0) ivarnew = ivarnew + iword(8)

        call aptmord (ivarnew, iword(8), mord, minv, nerra)

        if (nerra .eq. -1) then
          write (   6, 8315) ainput, ninput, aword(5), aword(5)
          write (iout, 8315) ainput, ninput, aword(5), aword(5)
        elseif (nerra .eq. 0) then
          write (   6, 8320) ivarnew, iword(8), mord,
     &                       ivarnew, iword(8), minv
          write (iout, 8320) ivarnew, iword(8), mord,
     &                       ivarnew, iword(8), minv
        endif
cbugc***DEBUG begins.
cbug      nbug = 9
cbug      write (   6, 6602) nbug, ivarnew
cbug      write (iout, 6602) nbug, ivarnew
cbugc***DEBUG ends.

        fvarnew = ivarnew
cbugc***DEBUG begins.
cbug 6603 format ('var DEBUG.  nbug=',i3,'  ivarnew=',i20,
cbug     &  '  fvarnew =',1pe22.14)
cbug      nbug = 10
cbug      write (   6, 6603) nbug, ivarnew, fvarnew
cbug      write (iout, 6603) nbug, ivarnew, fvarnew
cbugc***DEBUG ends.

        go to 920

      endif                           ! 8-word mod function.

c===============================================================================

c.... Input line:  variable VARNAME = A FUNCTION B
c.... Input line:  variable VARNAME   A FUNCTION B !Optional, if A is numerical.

 8142 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  FUNCTION is not a recognized operator:  ',a)
 8144 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  B is zero.  Cannot divide by zero.')

      if (nword .eq. 6) then

        if ((aword(5) .ne. '*')     .and.
     &      (aword(5) .ne. '**')    .and.
     &      (aword(5) .ne. '+')     .and.
     &      (aword(5) .ne. '-')     .and.
     &      (aword(5) .ne. '/')     .and.
     &      (aword(5) .ne. '^')     .and.
     &      (aword(5) .ne. 'abs')   .and.
     &      (aword(5) .ne. 'acos')  .and.
     &      (aword(5) .ne. 'acosh') .and.
     &      (aword(5) .ne. 'asin')  .and.
     &      (aword(5) .ne. 'asinh') .and.
     &      (aword(5) .ne. 'atan')  .and.
     &      (aword(5) .ne. 'atan2') .and.
     &      (aword(5) .ne. 'atanh') .and.
     &      (aword(5) .ne. 'avg')   .and.
     &      (aword(5) .ne. 'cos')   .and.
     &      (aword(5) .ne. 'cosh')  .and.
     &      (aword(5) .ne. 'curt')  .and.
     &      (aword(5) .ne. 'exp')   .and.
     &      (aword(5) .ne. 'fact')  .and.
     &      (aword(5) .ne. 'gcd'))  then

        if ((aword(5) .ne. 'int')       .and.
     &      (aword(5) .ne. 'lcm')       .and.
     &      (aword(5) .ne. 'log')       .and.
     &      (aword(5) .ne. 'log10')     .and.
     &      (aword(5) .ne. 'max')       .and.
     &      (aword(5) .ne. 'min')       .and.
     &      (aword(5) .ne. 'mod')       .and.
     &      (aword(5) .ne. 'nint')      .and.
     &      (aword(5) .ne. 'ran')       .and.
     &      (aword(5) .ne. 'root')      .and.
     &      (aword(5) .ne. 'sign')      .and.
     &      (aword(5) .ne. 'sin')       .and.
     &      (aword(5) .ne. 'sinh')      .and.
     &      (aword(5) .ne. 'sqrt')      .and.
     &      (aword(5) .ne. 'tan')       .and.
     &      (aword(5) .ne. 'tangent')   .and.
     &      (aword(5) .ne. 'tanh'))     then
          write (   6, 8142) ainput, ninput, aword(5)
          write (iout, 8142) ainput, ninput, aword(5)
          nerr = 1
        endif

        endif

        call varfind (6, 6)

        if ((mtype(6) .ne. 2) .and. (mtype(6) .ne. 3)) then
          write (   6, 8172) ainput, ninput, aword(6)
          write (iout, 8172) ainput, ninput, aword(6)
          nerr = 1
        endif

        if ((fword(6) .eq. 0.0)   .and.
     &      (aword(5) .eq. '/'))  then
          write (   6, 8144) ainput, ninput
          write (iout, 8144) ainput, ninput
          nerr = 1
        endif

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nvars) nvars = nvars - 1
          endif
          go to 999
        endif

      endif                           ! If nword = 6.

c===============================================================================

c.... Input line:  variable VARNAME   IVAR     ! Optional, if IVAR is numerical.
c.... Input line:  variable VARNAME   FVAR     ! Optional, if FVAR is numerical.
c.... Input line:  variable VARNAME   A FUNCTION B !Optional, if A is numerical.
c.... Input line:  variable VARNAME = IVAR
c.... Input line:  variable VARNAME = FVAR
c.... Input line:  variable VARNAME = A FUNCTION B

 8143 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  B is to big to find nearest integer:  ',a)
 8145 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  B is not a non-negative integer:  ',a)
 8152 format ('Integers A and B have a common factor,' /
     &        '  so integer A has no order or inverse.')
 8154 format ('Order   of',i20,' mod',i20,' =',i20 /
     &        'Inverse of',i20,' mod',i20,' =',i20 )
 8170 format ('angles ',a)
 8173 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not take square root of negative number:  ',a)
 8174 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not take log of non-positive number:  ',a)
 8175 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not raise negative number to a non-integer power:  ',a)
 8176 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not take inverse sine or cosine of number',
     &  ' < -1 or > 1:  ',a)
 8177 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not take inverse cosh of number < 1:  ',a)
 8178 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not take inverse tanh of number <= -1 or >= 1:  ',a)
 8179 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Zero to the zero''th power is undefined.')
 8180 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Zero to a negative power is infinite:  ',a)
 8181 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  First argument must be positive, second non-zero.' / a / a )
 8182 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not find greatest common divisor of arguments.' / a / a )
 8183 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not find least common multiple of arguments.' / a / a )
 8184 format ('  variable ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Can not do modulus zero:  ',a )

      if ((nword .eq. 4) .or. (nword .eq. 6)) then

      fact = 1.0
      if (angunit .eq. 'degrees') fact = 180.0 / pi

      mvarnew = 0

      if (nword .eq. 4) then
        fvarnew = fword(4)
        ivarnew = iword(4)
        mvarnew = mtype(4)
cbugc***DEBUG begins.
cbug 9910 format ('variable DEBUG.  iword=',i20)
cbug      write (iout, 9910) iword(4)
cbugc***DEBUG ends.

c.... Do function "+".

      elseif (aword(5) .eq. '+') then
cbugc***DEBUG begins.
cbug 9911 format ('variable DEBUG adding words 4 and 6.' /
cbug     &  1p2e22.14 )
cbug      write (   6, 9911) fword(4), fword(6)
cbug      write (iout, 9911) fword(4), fword(6)
cbugc***DEBUG ends.
        fvarnew = fword(4) + fword(6)
        serr    = tol * (abs (fword(4)) + abs (fword(6)))
        if (abs (fvarnew) .le. serr) then
          if (fvarnew .ne. 0.0) then
            write (   6, 9701), ainput, ninput, fvarnew, tol
            write (iout, 9701), ainput, ninput, fvarnew, tol
          endif
          fvarnew = 0.0
        endif

c.... Do function "-".

      elseif (aword(5) .eq. '-') then

        fvarnew = fword(4) - fword(6)
        serr    = tol * (abs (fword(4)) + abs (fword(6)))
        if (abs (fvarnew) .le. serr) then
          if (fvarnew .ne. 0.0) then
            write (   6, 9701), ainput, ninput, fvarnew, tol
            write (iout, 9701), ainput, ninput, fvarnew, tol
          endif
          fvarnew = 0.0
        endif

c.... Do function "*".

      elseif (aword(5) .eq. '*') then

        if ((fword(4) .eq. 0.0) .or. (fword(6) .eq. 0.0)) then
          fvarnew = 0.0
        else

          fexp = log10 (abs (fword(4))) + log10 (abs (fword(6)))
          if (abs (fexp) .gt. iemax) then
            write (   6, 8342) ainput, ninput, fexp
            write (iout, 8342) ainput, ninput, fexp
            nerr = 1
            go to 900
          endif

          fvarnew = fword(4) * fword(6)
        endif

c.... Do function "/".

      elseif (aword(5) .eq. '/') then

        if (fword(4) .eq. 0.0) then
          fvarnew = 0.0
        else

          fexp = log10 (abs (fword(4))) - log10 (abs (fword(6)))
cbugc***DEBUG begins.
cbug 6701 format ('var DEBUG.  fexp =',1pe22.14)
cbug      write (   6, 6701) fexp
cbug      write (iout, 6701) fexp
cbugc***DEBUG ends.
          if (abs (fexp) .gt. iemax) then
            write (   6, 8342) ainput, ninput, fexp
            write (iout, 8342) ainput, ninput, fexp
            nerr = 1
            go to 900
          endif

          fvarnew = fword(4) / fword(6)
        endif

c.... Do function "sqrt".

      elseif (aword(5) .eq. 'sqrt') then

        if (fword(6) .lt. 0.0) then
          write (   6, 8173) ainput, ninput, aword(6)
          write (iout, 8173) ainput, ninput, aword(6)
          nerr = 1
          go to 900
        endif
        fvarnew = fword(4) * sqrt (fword(6))

c.... Do function "curt".

      elseif (aword(5) .eq. 'curt') then

        fvarnew = fword(4) *
     &              sign (abs (fword(6))**(1.0/3.0), fword(6))

c.... Do function "root".

      elseif (aword(5) .eq. 'root') then

        if ((fword(4) .le. 0.0). or. (fword(6) .eq. 0.0)) then
          write (   6, 8181) ainput, ninput, aword(4), aword(6)
          write (iout, 8181) ainput, ninput, aword(4), aword(6)
          nerr = 1
          go to 900
        endif
        fvarnew = fword(4) ** (1.0 / fword(6))

c.... Do function "^".

      elseif ((aword(5) .eq. '^') .or.
     &        (aword(5) .eq. '**')) then

c....   See if arg fword(4) is negative.

        if (fword(4) .lt. 0.0) then
          if (mtype(6) .eq. 2) then   ! Negative arg to integer power.

            fmult = 1.0
            if (mod (iword(6), 2) .ne. 0) fmult = -1.0

            farg = abs (fword(4))
            fpow = iword(6)
            fexp = fpow * log10 (farg)
            if (abs (fexp) .gt. iemax) then
              write (   6, 8342) ainput, ninput, fexp
              write (iout, 8342) ainput, ninput, fexp
              nerr = 1
              go to 900
            endif
            fvarnew = fmult * farg**iword(6)
            go to 900

          else                        ! Negative arg to fl pt power.  BAD!
            write (   6, 8175) ainput, ninput, aword(4)
            write (iout, 8175) ainput, ninput, aword(4)
            nerr = 1
            go to 900
          endif
        endif

c....   See if arg fword(4) is zero.

        if (fword(4) .eq. 0.0) then

          if (fword(6) .lt. 0.0) then
            write (   6, 8180) ainput, ninput, aword(6)
            write (iout, 8180) ainput, ninput, aword(6)
            nerr = 1
            go to 900
          elseif (fword(6) .eq. 0.0) then
            write (   6, 8179) ainput, ninput
            write (iout, 8179) ainput, ninput
            nerr = 1
            go to 900
          elseif (fword(6) .gt. 0.0) then
            fvarnew = 0.0
            go to 900
          endif

        endif                         ! Arg fword(4) is zero.

c....   Arg fword(4) is positive.

        if (fword(6) .eq. 0.0) then
          fvarnew = 1.0
        else

          fexp = fword(6) * log10 (fword(4))
          if (abs (fexp) .gt. iemax) then
            write (   6, 8342) ainput, ninput, fexp
            write (iout, 8342) ainput, ninput, fexp
            nerr = 1
            go to 900
          endif

          fvarnew = fword(4)**fword(6)
        endif

c.... Do function "exp".

      elseif (aword(5) .eq. 'exp') then

        if (fword(4) .eq. 0.0) then
          fvarnew = 0.0
        else
          fexp = log10 (abs (fword(4))) + fword(6) / 2.302585092994
          if (abs (fexp) .gt. iemax) then
            write (   6, 8342) ainput, ninput, fexp
            write (iout, 8342) ainput, ninput, fexp
            nerr = 1
            go to 900
          endif
          fvarnew = fword(4) * exp (fword(6))
        endif

c.... Do function "log".

      elseif (aword(5) .eq. 'log') then

        if (fword(6) .le. 0.0) then
          write (   6, 8174) ainput, ninput, aword(6)
          write (iout, 8174) ainput, ninput, aword(6)
          nerr = 1
          go to 900
        endif
        fvarnew = fword(4) * log (fword(6))

c.... Do function "log10".

      elseif (aword(5) .eq. 'log10') then

        if (fword(6) .le. 0.0) then
          write (   6, 8174) ainput, ninput, aword(6)
          write (iout, 8174) ainput, ninput, aword(6)
          nerr = 1
          go to 900
        endif
        fvarnew = fword(4) * log10 (fword(6))

c.... Do function "sin".

      elseif (aword(5) .eq. 'sin') then

        write (   6, 8170) angunit
        write (iout, 8170) angunit
        fvarnew = fword(4) * sin (fword(6) / fact)

c.... Do function "cos".

      elseif (aword(5) .eq. 'cos') then

        write (   6, 8170) angunit
        write (iout, 8170) angunit
        fvarnew = fword(4) * cos (fword(6) / fact)

c.... Do function "tan".

      elseif ((aword(5) .eq. 'tan')      .or.
     &        (aword(5) .eq. 'tangent')) then

        write (   6, 8170) angunit
        write (iout, 8170) angunit
        fvarnew = fword(4) * tan (fword(6) / fact)

c.... Do function "asin".

      elseif (aword(5) .eq. 'asin') then

        if (abs (fword(6)) .gt. 1.0) then
          write (   6, 8176) ainput, ninput, aword(6)
          write (iout, 8176) ainput, ninput, aword(6)
          nerr = 1
          go to 900
        endif
        write (   6, 8170) angunit
        write (iout, 8170) angunit
        fvarnew = fact * fword(4) * asin (fword(6))

c.... Do function "acos".

      elseif (aword(5) .eq. 'acos') then

        if (abs (fword(6)) .gt. 1.0) then
          write (   6, 8176) ainput, ninput, aword(6)
          write (iout, 8176) ainput, ninput, aword(6)
          nerr = 1
          go to 900
        endif
        write (   6, 8170) angunit
        write (iout, 8170) angunit
        fvarnew = fact * fword(4) * acos (fword(6))

c.... Do function "atan".

      elseif (aword(5) .eq. 'atan') then

        write (   6, 8170) angunit
        write (iout, 8170) angunit
        fvarnew = fact * fword(4) * atan (fword(6))

c.... Do function "atan2".

      elseif (aword(5) .eq. 'atan2') then

        write (   6, 8170) angunit
        write (iout, 8170) angunit
        fvarnew = fact * atan2 (fword(4), fword(6))

c.... Do function "sinh".

      elseif (aword(5) .eq. 'sinh') then

        fvarnew = fword(4) * sinh (fword(6))

c.... Do function "cosh".

      elseif (aword(5) .eq. 'cosh') then

        fvarnew = fword(4) * cosh (fword(6))

c.... Do function "tanh".

      elseif (aword(5) .eq. 'tanh') then

        fvarnew = fword(4) * tanh (fword(6))

c.... Do function "asinh".

      elseif (aword(5) .eq. 'asinh') then

        if (fword(6) .lt. 0.0) then
          fvarnew =  fword(4) *
     &               log ( fword(6) + sqrt (fword(6)**2 + 1.0))
        else
          fvarnew = -fword(4) *
     &               log (-fword(6) + sqrt (fword(6)**2 + 1.0))
        endif

c.... Do function "acosh".

      elseif (aword(5) .eq. 'acosh') then

        if (fword(6) .lt. 1.0) then
          write (   6, 8177) ainput, ninput, aword(6)
          write (iout, 8177) ainput, ninput, aword(6)
          nerr = 1
          go to 900
        endif
        fvarnew = fword(4) * log (fword(6) + sqrt (fword(6)**2 - 1))

c.... Do function "atanh".

      elseif (aword(5) .eq. 'atanh') then

        if (fword(6)**2 .ge. 1.0) then
          write (   6, 8178) ainput, ninput, aword(6)
          write (iout, 8178) ainput, ninput, aword(6)
          nerr = 1
          go to 900
        endif
        fvarnew = 0.5 * fword(4) *
     &            log ((1.0 + fword(6)) / (1.0 - fword(6)))

c.... Do function "int".

      elseif (aword(5) .eq. 'int') then

        fvarnew = fword(4) * aint (fword(6))

c.... Do function "abs".

      elseif (aword(5) .eq. 'abs') then

        fvarnew = fword(4) * abs (fword(6))

c.... Do function "sign".

      elseif (aword(5) .eq. 'sign') then

        fvarnew = sign (fword(4), fword(6))

c.... Do function "max".

      elseif (aword(5) .eq. 'max') then

        fvarnew = amax1 (fword(4), fword(6))

c.... Do function "min".

      elseif (aword(5) .eq. 'min') then

        fvarnew = amin1 (fword(4), fword(6))

c.... Do function "mod".

      elseif (aword(5) .eq. 'mod') then

        if (fword(6) .eq. 0.0) then
          write (   6, 8184) ainput, ninput, aword(6)
          write (iout, 8184) ainput, ninput, aword(6)
          nerr = 1
          go to 900
        endif
        fvarnew = amod (fword(4), fword(6))
        if (fvarnew .lt. 0.0) fvarnew = fvarnew + fword(6)
        if ((mtype(4) .eq. 2) .and. (mtype(6) .eq. 2)) then

          iwordp = fvarnew
          call aptmord (iwordp, iword(6), mord, minv, nerra)

          if (nerra .eq. -1) then
            write (   6, 8152)
            write (iout, 8152)
          elseif (nerra .eq. 0) then
            write (   6, 8154) iword(4), iword(6), mord,
     &                         iword(4), iword(6), minv
            write (iout, 8154) iword(4), iword(6), mord,
     &                         iword(4), iword(6), minv
          endif

        endif

c.... Do function "nint".

      elseif (aword(5) .eq. 'nint') then

        if (abs (fword(6)) .gt. 10.0**idmax) then
          write (   6, 8143) ainput, ninput, aword(6)
          write (iout, 8143) ainput, ninput, aword(6)
          nerr = 1
          go to 900
        endif
        fvarnew = fword(4) * nint (fword(6))

c.... Do function "ran".  Random number.

      elseif (aword(5) .eq. 'ran') then

        fvarnew = fword(4) + (fword(6) - fword(4)) * ranf ()

c.... Do function "avg".  Arithmetic average.

      elseif (aword(5) .eq. 'avg') then

        fvarnew = 0.5 * (fword(4) + fword(6))
        serr    = 0.5 * tol * (abs (fword(4)) + abs (fword(6)))
        if (fvarnew .le. serr) then
          if (fvarnew .ne. 0.0) then
            write (   6, 9701), ainput, ninput, fvarnew, tol
            write (iout, 9701), ainput, ninput, fvarnew, tol
          endif
          fvarnew = 0.0
        endif

c.... Do function "fact".  Factorial.

      elseif (aword(5) .eq. 'fact') then

        if ((mtype(6) .ne. 2)  .or.
     &      (iword(6) .lt. 0)) then
          write (   6, 8145) ainput, ninput, aword(6)
          write (iout, 8145) ainput, ninput, aword(6)
          nerr = 1
          go to 900
        endif

        if (fword(4) .eq. 0.0) then
          fvarnew = 0.0
        else
          fvarnew = fword(4)
          do 888 nn = 1, iword(6)
              fnn   = nn
              fexp  = log10 (abs (fvarnew)) + log10 (fnn)
              if (fexp .gt. iemax) then
                write (   6, 8342) ainput, ninput, fexp
                write (iout, 8342) ainput, ninput, fexp
                nerr = 1
                go to 900
              endif
            fvarnew = fvarnew * fnn
  888     continue
        endif

c.... Do function "gcd".  Greatest common divisor.

      elseif (aword(5) .eq. 'gcd') then

        if ((mtype(4) .ne. 2) .or. (mtype(6) .ne. 2)) then
          write (   6, 8182) ainput, ninput, aword(4), aword(6)
          write (iout, 8182) ainput, ninput, aword(4), aword(6)
          nerr = 1
          go to 900
        endif
        intarg(1) = iword(4)
        intarg(2) = iword(6)
        call aptcmad (intarg, 2, intr, ijunk, nerra)
        if ((nerra .ne. 0) .and. (nerra .ne. 4)) then
          write (   6, 8182) ainput, ninput, aword(4), aword(6)
          write (iout, 8182) ainput, ninput, aword(4), aword(6)
          nerr = 1
          go to 900
        endif
        fvarnew = intr

c.... Do function "lcm".  Least common multiple.

      elseif (aword(5) .eq. 'lcm') then

        if ((mtype(4) .ne. 2) .or. (mtype(6) .ne. 2)) then
          write (   6, 8183) ainput, ninput, aword(4), aword(6)
          write (iout, 8183) ainput, ninput, aword(4), aword(6)
          nerr = 1
          go to 900
        endif
        intarg(1) = iword(4)
        intarg(2) = iword(6)
        call aptcmad (intarg, 2, ijunk, intr, nerra)
        if (nerra .ne. 0) then
          write (   6, 8183) ainput, ninput, aword(4), aword(6)
          write (iout, 8183) ainput, ninput, aword(4), aword(6)
          nerr = 1
          go to 900
        endif
        fvarnew = intr

c.... Function not recognized.

      else

        write (   6, 8142) ainput, ninput, aword(5)
        write (iout, 8142) ainput, ninput, aword(5)
        nerr = 1
        go to 900

      endif                           ! Tested for function type.
cbugc____ DEBUG begins.
cbug 9901 format ('  variable DEBUG.  ibug=1.  ivarnew='i20 /
cbug     &  '    fvarnew=',1pe20.12 /
cbug     &  '    mvarnew=',i3 )
cbug      write (   6, 9901) ivarnew, fvarnew, mvarnew
cbug      write (iout, 9901) ivarnew, fvarnew, mvarnew
cbugc____ DEBUG ends.

      endif                           ! 4 or 6 word input line.

c===============================================================================

  900 if (nerr .ne. 0) then
        if (nrep .eq. 0) then
          if (n .eq. nvars) nvars = nvars - 1
        endif
        go to 999
      endif

c===============================================================================

c.... Store the data for an integer or floating point variable.

  920 avarbls   = aword(2)            ! Last name used for a variable.
      lavarbls  = lword(2)            ! Last name used for a variable.

      avarbl(n) = aword(2)            ! Name of variable.
      avar(n) = aword(2)              ! Name of variable.
      lvar(n) = lword(2)              ! # of characters in avarbl.
      ivar(n) = ivarnew               ! Integer value of variable.
      fvar(n) = fvarnew               ! Floating point value of variable.
      mvar(n) = mvarnew               ! Mode of word 4.
cbugc____ DEBUG begins.
cbug 9902 format ('  variable DEBUG.  ibug=2.  mvar='i3)
cbug 9903 format ('  variable DEBUG.  ibug=2.  ivar='i20,
cbug     &  '  fvar=',1pe20.12)
cbug      write (   6, 9902) mvar(n)
cbug      write (iout, 9902) mvar(n)
cbug      write (   6, 9903) ivar(n), fvar(n)
cbug      write (iout, 9903) ivar(n), fvar(n)
cbugc____ DEBUG ends.

c.... Find the mode of the value of the variable.

      if (mvar(n) .ne. 2) then

        call varmode (n)

      endif

cbugc____ DEBUG begins.
cbug 9904 format ('  variable DEBUG.  ibug=3.  mvar='i3)
cbug 9905 format ('  variable DEBUG.  ibug=3.  ivar='i20,
cbug     &  '  fvar=',1pe20.12)
cbug      write (   6, 9904) mvar(n)
cbug      write (iout, 9904) mvar(n)
cbug      write (   6, 9905) ivar(n), fvar(n)
cbug      write (iout, 9905) ivar(n), fvar(n)
cbugc____ DEBUG ends.

c.... Print the value of the variable.

      call varout (n, n)

c===============================================================================

  999 return

c.... End of subroutine variable.     (+1 line)
      end
