      subroutine vector

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  vector   Updated     2007 January 19 15:30.
c               vector   Originated  1994 November 8 13:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To display and create vectors.
c
c     Logic:    Entry points:  vector.
c vector        Called by:  helpall, main.
c
c vector        Calls:  aptcsyv, aptetru, aptqnor, aptripv, aptscan, aptscat,
c               aptscav, aptsubr, aptvaxb, aptvdis, aptvplp, aptvsum, aptvunz,
c               aptvusz, incrsubs, namechk, plout, ptfind, quout, qutype,
c               varfind, vectfind, voper, vout, vstore.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      common /ccvect/ np              ! Index in table of points.

      character*8 ans                 ! User's response.

c.... Input line:  vector [help]

 8110 format (
     &  '  Command "vector":  acceptable input lines include:' //
     &  '  "vector [help]"      Display this message.' /
     &  '  "vector all"         Display all vectors.' /
     &  '  "vector list [VNAME1 VNAME2 VNAME3 ...]"' /
     &  '     Display all listed vectors (default is all',
     &  ' vectors).' /
     &  '  "vector VNAME"       Display vector VNAME.' //
     &  '  "vector VNAME = DU DV DW [U|0] [V|0] [W|0]"' /
     &  '     Create vector VNAME with components',
     &  ' DU, DV, DW.' /
     &  '     at bound point U, V, W, (defaults to [0, 0, 0]).' /
     &  '     The "=" is optional if argument DU is numerical.' //
     &  '  "vector VNAME unit"' /
     &  '     Change the length of vector VNAME to 1.' /
     &  '  "vector VNAME abs [ABSLEN|1]"' /
     &  '     Change the length of vector VNAME to ABSLEN',
     &  ' [1].' /
     &  '  "vector VNAME rel RATIO"' /
     &  '     Multiply the length of vector VNAME by the factor',
     &  ' RATIO.' )

 8112 format (
     &  '  "vector VNAME bound PNAME"' /
     &  '     Change the bound point of vector VNAME to point PNAME.' //
     &  '  "vector VNAME line ALNAME"' /
     &  '     Create vector VNAME same as line ALNAME.' //
     &  '  "vector VNAME point PNAME1 PNAME2 [rel [RELLEN|1]]"' /
     &  '     Create vector VNAME from point PNAME1 to',
     &  ' point PNAME2,' /
     &  '     but with length RELLEN [1] times that distance.'/
     &  '  "vector VNAME point PNAME1 PNAME2 unit"' /
     &  '     Create vector VNAME from point PNAME1 toward',
     &  ' point PNAME2,' /
     &  '     but with absolute length 1.' /
     &  '  "vector VNAME point PNAME1 PNAME2 [abs [ABSLEN|1]]"' /
     &  '     Create vector VNAME from point PNAME1 toward',
     &  ' point PNAME2,' /
     &  '     but with absolute length ABSLEN [1].' //
     &  '  "vector VNAME cross VNAME1 VNAME2"' /
     &  '     Create vector VNAME, the cross product of vectors',
     &  ' VNAME1 and VNAME2.' //
     &  '  Type <return> for more help.' )

 8114 format (
     &  '  "vector VNAME triple VNAME1 VNAME2 VNAME3"' /
     &  '     Create vector VNAME, the vector triple product of',
     &  ' vectors' /
     &  '     VNAME1, VNAME2 and VNAME3:  VNAME1 cross',
     &  ' (VNAME2 cross VNAME3) =' /
     &  '     (VNAME3 cross VNAME2) cross VNAME1.' //
     &  '  "vector VNAME project VNAME1 PLNAME"' /
     &  '     Create vector VNAME, the projection of vector VNAME1',
     &  ' on the plane PLNAME.' //
     &  '  "vector VNAME angle ANG1 VNAME1 ANG2 VNAME2"' /
     &  '     Create unit vector VNAME at angle ANG1 from vector',
     &  ' VNAME1' /
     &  '     and angle ANG2 from vector VNAME2.' //
     &  '  "vector VNAME sum VNAME1 VNAME2 [VMULT1|1 [VMULT2|1]]"' /
     &  '     Create vector VNAME, the weighted sum VMULT1 * VNAME1',
     &  ' + VMULT2 * VNAME2,' /
     &  '     where VMULT1 and VMULT2 default to 1.' //
     &  '  Type <return> for more help.' )

 8115 format (a)

 8116 format (/
     &  '  "vector VNAME proximal"' /
     &  '     Create vector VNAME, between the last pair of proximal',
     &  ' points found' /
     &  '     by command "distance", "proximal", "side" or "track",' /
     &  '     or the net movement found by command "walk".' //
     &  '  "vector VNAME random"  Create unit vector VNAME,' /
     &  '     randomly sampled from an isotropic distribution.' /
     &  '  "vector VNAME random VAXIS cosine [POWER|1]"' /
     &  '     Create unit vector VNAME, randomly sampled from a' /
     &  '     cosine-power distribution with power POWER, around the',
     &  ' axis vector VAXIS.' /
     &  '  "vector VNAME random VAXIS angle ANGLE"' /
     &  '     Create unit vector VNAME, randomly sampled from a' /
     &  '     uniform azimuthal distribution at angle ANGLE from the',
     &  ' axis vector VAXIS.' )

 8117 format (/
     &  '  "vector VNAME QTYPE QNAME"' /
     &  '     QTYPE must be:  plane, sphere, cylinder, cone,',
     &  ' hyperb, ellipsoid,' /
     &  '     axisym, quadric.' /
     &  '     Create vector VNAME from the center of quadric',
     &  ' QNAME to the origin.' /
     &  '  "vector VNAME normal PNAME QNAME"' /
     &  '     Create vector VNAME, normal at point PNAME to',
     &  ' quadric surface family QNAME.' //
     &  '  "vector VNAME02 rotate NUMV INC VNAME01 OPNAME PINV"' /
     &  '     Create a family of NUMV vectors VNAME02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     by rotating the preceding vector with operator OPNAME' /
     &  '     and invariant point PINV, starting from vector',
     &  ' VNAME01.' //
     &  '  "vector VNAME02 scale NUMV INC VNAME01 OPNAME PINV"' /
     &  '     Create a family of NUMV vectors VNAME02, ...,' /
     &  '     with names incremented by INC characters,' /
     &  '     by scaling the preceding vector with operator OPNAME' /
     &  '     and invariant point PINV, starting from vector',
     &  ' VNAME01.' )

 8118 format (/
     &  '  Note:  use +/- for VNAME or VNAME02 to incr/decr the last',
     &  ' vector name:  ',a //
     &  '  Synonyms:  [angle, ang], [cosine, cos], [cylinder, cyl],' /
     &  '    [ellipsoid, ell, el], [help, h], [hyperb, saddle],',
     &  ' [line, ln, l],' /
     &  '    [normal, norm], [plane, pl],' /
     &  '    [point, pnt, pt, p], [project, proj], [proximal, prox],',
     &  ' [quadric, quad, q],' /
     &  '    [random, ?], [rel, *], [rotate, rot], [scale, sc],',
     &  ' [sphere, sph],' /
     &  '    [sum, add], [vector, vect, v].' )

      if ((nword .eq. 1) .or.
     &   ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'help') .or.
     &    (aword(2) .eq. 'h')))) then

        write (   6, 8110)
        write (iout, 8110)
        write (   6, 8112)
        write (iout, 8112)
        if (ainput .eq. 'tty') then
          read  (   5, 8115) ans
        endif
        write (   6, 8114)
        write (iout, 8114)
        if (ainput .eq. 'tty') then
          read  (   5, 8115) ans
        endif
        write (   6, 8116)
        write (iout, 8116)
        write (   6, 8117)
        write (iout, 8117)
        write (   6, 8118) avectors
        write (iout, 8118) avectors
        go to 999

      endif                           ! Displayed help message.

c=======================================================================********

c.... Input line:  vector all
c.... Input line:  vector list

 8210 format (/ 'Number of vectors = ',i5)
 8212 format ('coord  ',a,5x,a,4x,a,4x,a)

      if ((nword .eq. 2) .and.
     &   ((aword(2) .eq. 'all')    .or.
     &    (aword(2) .eq. 'list'))) then

        write (   6, 8212) asys, acoordu, acoordv, acoordw
        write (iout, 8212) asys, acoordu, acoordv, acoordw
        nvectorx = 0
        call vout (1, nvectors)
        do 210 n = 1, nvectors
          if (avector(n) .eq. ' ') go to 210
          nvectorx = nvectorx + 1
  210   continue
        write (   6, 8210) nvectorx
        write (iout, 8210) nvectorx
        go to 999

      endif                           ! Displayed all vectors.

c=======================================================================********

c.... Input line:  vector list VNAME1 VNAME2 VNAME3 ...

      if (aword(2) .eq. 'list') then
        do 230 nw = 3, nword          ! Loop over list names.
          nfind = 0
          do 225 n = 1, nvectors      ! Loop over vectors.
            call aptsubr (avector(n), 1, 24, 80, astem, lstem,
     &                    asub, lsub, nerra)
            if ((aword(nw) .eq. avector(n)) .or.
     &          (aword(nw) .eq. astem))       then
              nfind = nfind + 1
              call vout (n, n)
            endif
  225     continue                    ! End of loop over vectors.
          if (nfind .eq. 0) then
            write (   6, 8390) ainput, ninput, aword(nw)
            write (iout, 8390) ainput, ninput, aword(nw)
          endif
  230   continue
        go to 999

      endif                           ! Displayed listed vectors.

c=======================================================================********

c.... Input line:  vector VNAME

      if (nword .eq. 2) then

        call vectfind (2, n)

        go to 999

      endif                           ! Displayed one vector.

c=======================================================================********

c.... See if last vector name should be incremented and used.

 8128 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last vector can not be incremented:  ',a)

      if (aword(2) .eq. '+') then
        aword(2) = avectors
        lword(2) = lavectors
        call incrsubs (1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8128) ainput, ninput, avectors
          write (iout, 8128) ainput, ninput, avectors
          go to 999
        endif
      endif                           ! Incremented last vector name.

 8129 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of last vector can not be decremented:  ',a)

      if (aword(2) .eq. '-') then
        aword(2) = avectors
        lword(2) = lavectors
        call incrsubs (-1, aword(2), 1, lword(2), nerra)
        if (nerra .ne. 0) then
          write (   6, 8129) ainput, ninput, avectors
          write (iout, 8129) ainput, ninput, avectors
          go to 999
        endif
      endif                           ! Incremented last vector name.

 8905 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of vector too dangerous to use:  ',a)

      call namechk (aword(2), lword(2), nerrchk)

      if (nerrchk .ne. 0) then
        write (   6, 8905) ainput, ninput, aword(2)
        write (iout, 8905) ainput, ninput, aword(2)
        go to 999
      endif

c===============================================================================

c.... Test for acceptable name.

 8690   format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Unacceptable name.  Too dangerous:  ',a)

        if ((aword(2) .eq. 'random') .or.
     &      (aword(2) .eq. '?'))     then
          write (   6, 8690) ainput, ninput, aword(2)
          write (iout, 8690) ainput, ninput, aword(2)
          go to 999
        endif

c===============================================================================

c.... Input line:  vector VNAME02 rotate NUMV INC VNAME01 OPNAME PINV
c.... Input line:  vector VNAME02 scale  NUMV INC VNAME01 OPNAME PINV

      if ((aword(3) .eq. 'scale')   .or.
     &    (aword(3) .eq. 'sc')      .or.
     &    (aword(3) .eq. 'rotate')  .or.
     &    (aword(3) .eq. 'rot'))    then

        call voper

        go to 999

      endif                           ! Created rotated or scaled vector(s).

c===============================================================================

c.... Test input for validity.

 8205 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on "vector VNAME quadric" line not 4:  ',i3)
 8206 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on "vector VNAME proximal" line not 3:  ',i3)
 8215 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on "vector VNAME unit" line not 3:  ',i3)
 8220 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on "vector VNAME =" line not 4 to 9:  ',i3)
 8230 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on "vector VNAME point PNAME1 PNAME2" line',
     &  ' not 5-7:  ',i3)
 8250 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on "vector VNAME line ALNAME2" line',
     &  ' not 4:  ',i3)
 8255 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on "vector VNAME cross VNAME1 VNAME2" line',
     &  ' not 5:  ',i3)
 8258 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on "vector VNAME triple VNAME1 ..." line',
     &  ' not 6:  ',i3)
 8260 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on "vector VNAME abs ABSLEN" line',
     &  ' not 3 or 4:  ',i3)
 8262 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on "vector VNAME rel RATIO" line',
     &  ' not 4:  ',i3)
 8264 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on "vector VNAME angle ..." line',
     &  ' not 7:  ',i3)
 8265 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on "vector VNAME bound PNAME" line',
     &  ' not 4:  ',i3)
 8268 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of words on "vector VNAME project VNAME PLNAME" line',
     &  ' not 5:  ',i3)

      if ((aword(3) .eq. 'unit') .and. (nword .ne. 3)) then
        write (   6, 8215) ainput, ninput, nword
        write (iout, 8215) ainput, ninput, nword
        go to 999
      endif

      if ((aword(3) .eq. 'abs') .and.
     &    ((nword .lt. 3) .or. (nword .gt. 4))) then
        write (   6, 8260) ainput, ninput, nword
        write (iout, 8260) ainput, ninput, nword
        go to 999
      endif

      if (((aword(3) .eq. 'rel') .or.
     &     (aword(3) .eq. '*'))  .and.
     &     (nword .ne. 4))        then
        write (   6, 8262) ainput, ninput, nword
        write (iout, 8262) ainput, ninput, nword
        go to 999
      endif

      if ((aword(3) .eq. '=') .and.
     &    ((nword .lt. 4) .or. (nword .gt. 9))) then
        write (   6, 8220) ainput, ninput, nword
        write (iout, 8220) ainput, ninput, nword
        go to 999
      endif

      if ((aword(3) .eq. 'bound') .and. (nword .ne. 4)) then
        write (   6, 8265) ainput, ninput, nword
        write (iout, 8265) ainput, ninput, nword
        go to 999
      endif

      if (((aword(3) .eq. 'proximal') .or.
     &     (aword(3) .eq. 'prox'))    .and.
     &     (nword .ne. 3))            then
        write (   6, 8206) ainput, ninput, nword
        write (iout, 8206) ainput, ninput, nword
        go to 999
      endif

      if (((aword(3) .eq. 'quadric')   .or.
     &     (aword(3) .eq. 'quad')      .or.
     &     (aword(3) .eq. 'q')         .or.
     &     (aword(3) .eq. 'plane')     .or.
     &     (aword(3) .eq. 'pl')        .or.
     &     (aword(3) .eq. 'sphere')    .or.
     &     (aword(3) .eq. 'sph')       .or.
     &     (aword(3) .eq. 'cylinder')  .or.
     &     (aword(3) .eq. 'cyl')       .or.
     &     (aword(3) .eq. 'cone')      .or.
     &     (aword(3) .eq. 'hyperb')    .or.
     &     (aword(3) .eq. 'saddle')    .or.
     &     (aword(3) .eq. 'ellipsoid') .or.
     &     (aword(3) .eq. 'ellipse')   .or.
     &     (aword(3) .eq. 'ell')       .or.
     &     (aword(3) .eq. 'el')        .or.
     &     (aword(3) .eq. 'axisym'))   .and.
     &     (nword .ne. 4)) then
        write (   6, 8205) ainput, ninput, nword
        write (iout, 8205) ainput, ninput, nword
        go to 999
      endif

      if (((aword(3) .eq. 'point') .or.
     &     (aword(3) .eq. 'pnt')   .or.
     &     (aword(3) .eq. 'pt')    .or.
     &     (aword(3) .eq. 'p'))    .and.
     &    ((nword .lt. 5) .or. (nword .gt. 7))) then
        write (   6, 8230) ainput, ninput, nword
        write (iout, 8230) ainput, ninput, nword
        go to 999
      endif

      if ((aword(3) .eq. 'line') .or.
     &    (aword(3) .eq. 'ln') .or.
     &    (aword(3) .eq. 'l')) then
        if (nword .ne. 4) then
          write (   6, 8250) ainput, ninput, nword
          write (iout, 8250) ainput, ninput, nword
          go to 999
        endif
      endif

      if (aword(3) .eq. 'cross') then
        if (nword .ne. 5) then
          write (   6, 8255) ainput, ninput, nword
          write (iout, 8255) ainput, ninput, nword
          go to 999
        endif
      endif

      if (aword(3) .eq. 'triple') then
        if (nword .ne. 6) then
          write (   6, 8258) ainput, ninput, nword
          write (iout, 8258) ainput, ninput, nword
          go to 999
        endif
      endif

      if ((aword(3) .eq. 'project') .or.
     &    (aword(3) .eq. 'proj'))   then
        if (nword .ne. 5) then
          write (   6, 8268) ainput, ninput, nword
          write (iout, 8268) ainput, ninput, nword
          go to 999
        endif
      endif

      if ((aword(3) .eq. 'angle') .or.
     &    (aword(3) .eq. 'ang'))  then
        if (nword .ne. 7) then
          write (   6, 8264) ainput, ninput, nword
          write (iout, 8264) ainput, ninput, nword
          go to 999
        endif
      endif

 8139 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Name of vector not from 1 to 24 characters:  ',a)

      if ((lword(2) .lt. 1) .or. (lword(2) .gt. 24)) then
        write (   6, 8139) ainput, ninput, aword(2)
        write (iout, 8139) ainput, ninput, aword(2)
        go to 999
      endif

c=======================================================================********

c.... Find the index in the vector array at which to store this vector.
c....   Set nrep = 1 if replacing an existing vector.

      call vstore (nrep, n)

      if (n .eq. 0) go to 999

      nerr = 0

c=======================================================================********

c.... Input line:  vector VNAME proximal

      if ((aword(3) .eq. 'proximal') .or.
     &    (aword(3) .eq. 'prox'))    then

        avectors    = aword(2)
        lavectors   = lword(2)

        avector(n)  = aword(2)
        lavector(n) = lword(2)
        xcom(n)     = proxvx
        ycom(n)     = proxvy
        zcom(n)     = proxvz

        vlen(n)     = sqrt (proxvx**2 + proxvy**2 + proxvz**2)

        xbnd(n)     = 0.0
        ybnd(n)     = 0.0
        zbnd(n)     = 0.0

c....   Print the vector specifications.

        call vout (n, n)

        go to 999

      endif

c=======================================================================********

c.... Input line:  vector VNAME project VNAME1 PLNAME

      if ((aword(3) .eq. 'project') .or.
     &    (aword(3) .eq. 'proj'))   then

c....   Find the vector.

        call vectfind (4, nv)

        if (nv .eq. 0) then
          nerr = 1
          go to 310
        endif

        dx1 = xcom(nv)
        dy1 = ycom(nv)
        dz1 = zcom(nv)

c....   Find the plane.

  310   npl = 0
        ans = 'plane'

        do 320 nq = 1, nquads

          if (aword(5) .ne. aquad(nq)) go to 320

          call qutype (ans, nq, 1, nerrq)

          if (nerrq .eq. 0) then
            npl = nq
            call plout (npl, npl)
            go to 330
          else
            nerr = 1
            go to 340
          endif

  320   continue
        write (   6, 8510) ainput, ninput, aword(5)
        write (iout, 8510) ainput, ninput, aword(5)
        nerr = 1
        go to 340

  330   dx2 = qx(npl)
        dy2 = qy(npl)
        dz2 = qz(npl)

  340   if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nvectors) nvectors = nvectors - 1
          endif
          go to 999
        endif

c....   Find the projection of the vector on the plane.

        call aptvplp (dx1, dy1, dz1, dx2, dy2, dz2, 1, tol,
     &                du, dv, dw, vlength, nerra)

        avectors    = aword(2)
        lavectors   = lword(2)

        avector(n)  = aword(2)
        lavector(n) = lword(2)
        xcom(n)     = du
        ycom(n)     = dv
        zcom(n)     = dw

        vlen(n)     = vlength

        xbnd(n)     = 0.0
        ybnd(n)     = 0.0
        zbnd(n)     = 0.0

c....   Print the vector specifications.

        call vout (n, n)

        go to 999

      endif                           ! Vector projected onto a plane.

c=======================================================================********

c.... Input line:  vector VNAME random
c.... Input line:  vector VNAME random VAXIS cosine [POWER|1]
c.... Input line:  vector VNAME random VAXIS angle ANGLE

      if ((aword(3) .eq. 'random') .or.
     &    (aword(3) .eq. '?'))     then

 8270   format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Input line is not recognized.')
 8272   format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Cosine power is negative:  ',a)
 8274   format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Axis vector has zero length:  ',a)
 8276 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word 6 not integer or floating point value or variable:  ',a)
 8278 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Scattering angle is not in range 0 to 180 degrees:  ',a)

        if (nword .eq. 3) then        ! Sample from an isotropic distribution.

c....     Input line:  vector VNAME random

          avectors    = aword(2)
          lavectors   = lword(2)

          avector(n)  = aword(2)
          lavector(n) = lword(2)
  
          call aptscat (1, xcom(n), ycom(n), zcom(n), nerra)
  
          vlen(n)    = 1.0
          xbnd(n)    = 0.0
          ybnd(n)    = 0.0
          zbnd(n)    = 0.0
  
          call vout (n, n)
  
          go to 999

        elseif ((nword .eq. 5) .or.
     &          (nword .eq. 6)) then

c....     Find the unit vector on the axis.

          call vectfind (4, nv)
  
          if (nv .eq. 0) then
            nerr = 1
          endif

          if (vlen(nv) .eq. 0.0) then
            write (   6, 8274) ainput, ninput, aword(4)
            write (iout, 8274) ainput, ninput, aword(4)
            nerr = 1
          else

            call aptvusz (xcom(nv), ycom(nv), zcom(nv), tol,
     &                    ux, uy, uz, vlength)

          endif

          if ((aword(5) .eq. 'cosine') .or.
     &        (aword(5) .eq. 'cos'))   then  ! Sample from cosine distribution.

c....       Input line:  vector VNAME random VAXIS cosine [POWER|1]

c....       Find the power of the cosine-power distribution.

            if (nword .eq. 6) then

              call varfind (6, 6)
  
              if ((mtype(6) .ne. 2) .and. (mtype(6) .ne. 3)) then
                write (   6, 8276) ainput, ninput, aword(6)
                write (iout, 8276) ainput, ninput, aword(6)
                nerr = 1
              endif

              power = fword(6)

            elseif (nword .eq. 5) then
              power = 1.0
            endif

            if (power .lt. 0.0) then
              write (   6, 8272) ainput, ninput, aword(6)
              write (iout, 8272) ainput, ninput, aword(6)
              go to 360
            endif

            if (nerr .ne. 0) go to 360

            avectors    = aword(2)
            lavectors   = lword(2)

            avector(n)  = aword(2)
            lavector(n) = lword(2)

            call aptscav (1, power, ux, uy, uz,
     &                    xcom(n), ycom(n), zcom(n), nerra)

            call aptvunz (xcom(n), ycom(n), zcom(n), tol, vlen(n))
  
            xbnd(n)    = xbnd(nv)
            ybnd(n)    = ybnd(nv)
            zbnd(n)    = zbnd(nv)
  
            call vout (n, n)

          elseif ((aword(5) .eq. 'angle') .or.
     &            (aword(5) .eq. 'ang'))   then  ! Sample around fixed angle.

c....       Input line:  vector VNAME random VAXIS angle ANGLE

c....       Find the scattering angle.

            call varfind (6, 6)

            if ((mtype(6) .ne. 2) .and. (mtype(6) .ne. 3)) then
              write (   6, 8276) ainput, ninput, aword(6)
              write (iout, 8276) ainput, ninput, aword(6)
              nerr = 1
            endif

            angle = fword(6)

            if (angunit .eq. 'degrees') then
              iunit = 0
              fact  = 1.0
            else
              iunit = 1
              fact  = 180.0 / pi
            endif

            if (((angle * fact) .lt. 0.0)    .or.
     &          ((angle * fact) .gt. 180.0)) then
              write (   6, 8278) ainput, ninput, aword(6)
              write (iout, 8278) ainput, ninput, aword(6)
              nerr = 1
            endif

            if (nerr .ne. 0) go to 360

            avectors    = aword(2)
            lavectors   = lword(2)

            avector(n)  = aword(2)
            lavector(n) = lword(2)

            call aptscan (iunit, ux, uy, uz, angle, 1,
     &                    xcom(n), ycom(n), zcom(n), nerra)

            call aptvunz (xcom(n), ycom(n), zcom(n), tol, vlen(n))
  
            xbnd(n)    = xbnd(nv)
            ybnd(n)    = ybnd(nv)
            zbnd(n)    = zbnd(nv)
  
            call vout (n, n)

          else                        ! Not option "cosine" or "angle".
            write (   6, 8270) ainput, ninput
            write (iout, 8270) ainput, ninput
            nerr = 1
          endif                       ! Tested for option "cosine" or "angle".

        else                          ! Number of words not 3, 5 or 6.
          write (   6, 8270) ainput, ninput
          write (iout, 8270) ainput, ninput
          nerr = 1
        endif                         ! Tested number of words in command.

  360   if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nvectors) nvectors = nvectors - 1
          endif
        endif

        go to 999

      endif                           ! Vector sampled randomly.

c=======================================================================********

c.... Input line:  vector VNAME unit
c.... Input line:  vector VNAME abs
c.... Input line:  vector VNAME abs ABSLEN

      if (((nword .eq. 3)           .and.
     &    (aword(3) .eq. 'abs')     .or.
     &    (aword(3) .eq. 'unit'))   .or.
     &   ((nword .eq. 4)            .and.
     &    (aword(3) .eq. 'abs')))   then

c....   See if the vector exists.

 8390   format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Named vector does not exist:  ',a)

        if (nrep .eq. 0) then
          write (   6, 8390) ainput, ninput, aword(2)
          write (iout, 8390) ainput, ninput, aword(2)
          if (n .eq. nvectors) nvectors = nvectors - 1
          go to 999
        endif

c....   Find the new vector length.

 8400 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word 4 not integer or floating point value or variable:  ',a)

        abslen = 1.0

        if (nword .eq. 4) then

          call varfind (4, 4)

          if ((mtype(4) .ne. 2) .and. (mtype(4) .ne. 3)) then
            write (   6, 8400) ainput, ninput, aword(4)
            write (iout, 8400) ainput, ninput, aword(4)
            go to 999
          endif

          abslen = fword(4)

        endif

c....   Convert the vector to an absolute vector.  Truncate small components.

 8410   format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Can not give a null vector an absolute length:  ',a)

  370   if (vlen(n) .eq. 0.0) then
          write (   6, 8410) ainput, ninput, avector(n)
          write (iout, 8410) ainput, ninput, avector(n)
          go to 999
        else

          call aptvunz (xcom(n), ycom(n), zcom(n), tol, vlength)

          xcom(n) = abslen * xcom(n)
          ycom(n) = abslen * ycom(n)
          zcom(n) = abslen * zcom(n)
        endif

        vlen(n)    = abslen

c....   Print the vector specifications.

        call vout (n, n)

        if (abs (vlen(n) - abslen) .gt. tol * abslen) go to 370

        go to 999

      endif                           ! Make existing vector an absolute vector.

c=======================================================================********

c.... Input line:  vector VNAME rel RATIO

      if ((nword .eq. 4)         .and.
     &    ((aword(3) .eq. 'rel') .or.
     &     (aword(3) .eq. '*'))) then

c....   See if the vector exists.

        if (nrep .eq. 0) then
          write (   6, 8390) ainput, ninput, aword(2)
          write (iout, 8390) ainput, ninput, aword(2)
          if (n .eq. nvectors) nvectors = nvectors - 1
          go to 999
        endif

c....   Find the ratio of the new length to the old length.

        call varfind (4, 4)

        if ((mtype(4) .ne. 2) .and. (mtype(4) .ne. 3)) then
          write (   6, 8400) ainput, ninput, aword(4)
          write (iout, 8400) ainput, ninput, aword(4)
          go to 999
        endif

c....   Scale the length of the vector..  Truncate small components.

        abslen = fword(4) * vlen(n)

 8412   format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Can not multiply the length of a null vector:  ',a)

        if (vlen(n) .eq. 0.0) then
          write (   6, 8412) ainput, ninput, avector(n)
          write (iout, 8412) ainput, ninput, avector(n)
          go to 999
        else

          call aptvunz (xcom(n), ycom(n), zcom(n), tol, vlength)

          xcom(n) = abslen * xcom(n)
          ycom(n) = abslen * ycom(n)
          zcom(n) = abslen * zcom(n)

        endif

        vlen(n)    = abslen

c....   Print the vector specifications.

        call vout (n, n)

        go to 999

      endif                           ! Scale the length of an existing vector.

c=======================================================================********

c.... Input line:  vector VNAME bound PNAME

      if (aword(3) .eq. 'bound') then

        if (nrep .eq. 0) then
          write (   6, 8390) ainput, ninput, aword(2)
          write (iout, 8390) ainput, ninput, aword(2)
          if (n .eq. nvectors) nvectors = nvectors - 1
          go to 999
        endif

        call ptfind (4, n1)

        if (n1 .eq. 0) then
          go to 999
        endif

        xbnd(n) = x(n1)
        ybnd(n) = y(n1)
        zbnd(n) = z(n1)

c....   Print the vector specifications.

        call vout (n, n)

        go to 999

      endif                           ! Vector bound point redefined.

c=======================================================================********

c.... Input line:  vector VNAME line ALNAME

 8420 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named line does not exist:  ',a)

      if ((aword(3) .eq. 'line') .or.
     &    (aword(3) .eq. 'ln')   .or.
     &    (aword(3) .eq. 'l'))   then

c....   Find the line for this vector.  Replace input with names of two points.

        do 420 nn = 1, nlines
          if (aline(nn) .eq. ' ') go to 420
          if (aword(4) .eq. aline(nn)) then
            nword = 5
            aword(3) = 'point'
            mtype(4) = 1
            aword(4) = alipt1(nn)
            mtype(5) = 1
            aword(5) = alipt2(nn)
            go to 530
          endif
  420   continue
        write (   6, 8420) ainput, ninput, aword(4)
        write (iout, 8420) ainput, ninput, aword(4)
        nerr = 1

        if (nrep .eq. 0) then
          if (n .eq. nvectors) nvectors = nvectors - 1
        endif
        go to 999

      endif                           ! Vector defined by a line.

c=======================================================================********

c.... Input line:  vector VNAME cross VNAME1 VNAME2

      if (aword(3) .eq. 'cross') then

c....   Find the first vector.

        call vectfind (4, nv)

        if (nv .eq. 0) then
          nerr = 1
          go to 430
        endif

        dx1 = xcom(nv)
        dy1 = ycom(nv)
        dz1 = zcom(nv)

c....   Find the second vector.

  430   call vectfind (5, nv)

        if (nv .eq. 0) then
          nerr = 1
          go to 440
        endif

        dx2 = xcom(nv)
        dy2 = ycom(nv)
        dz2 = zcom(nv)

  440   if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nvectors) nvectors = nvectors - 1
          endif
          go to 999
        endif

c....   Find the cross product of the two vectors.

        call aptvaxb (dx1, dy1, dz1, dx2, dy2, dz2, 1, tol,
     &                du, dv, dw, vlength, nerra)

        avectors    = aword(2)
        lavectors   = lword(2)

        avector(n)  = aword(2)
        lavector(n) = lword(2)

        xcom(n)     = du
        ycom(n)     = dv
        zcom(n)     = dw

        vlen(n)     = vlength

        xbnd(n)     = 0.0
        ybnd(n)     = 0.0
        zbnd(n)     = 0.0

c....   Print the vector specifications.

        call vout (n, n)

        go to 999

      endif                           ! Vector is cross product of two vectors.

c=======================================================================********

c.... Input line:  vector VNAME triple VNAME1 VNAME2 VNAME3

      if (aword(3) .eq. 'triple') then

c....   Find the first vector.

        call vectfind (4, nv)

        if (nv .eq. 0) then
          nerr = 1
          go to 450
        endif

        dx1 = xcom(nv)
        dy1 = ycom(nv)
        dz1 = zcom(nv)

c....   Find the second vector.

  450   call vectfind (5, nv)

        if (nv .eq. 0) then
          nerr = 1
          go to 460
        endif

        dx2 = xcom(nv)
        dy2 = ycom(nv)
        dz2 = zcom(nv)

c....   Find the third vector.

  460   call vectfind (6, nv)

        if (nv .eq. 0) then
          nerr = 1
          go to 470
        endif

        dx3 = xcom(nv)
        dy3 = ycom(nv)
        dz3 = zcom(nv)

  470   if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nvectors) nvectors = nvectors - 1
          endif
          go to 999
        endif

c....   Find the vector triple product of the three vectors.

        call aptripv (dx1, dy1, dz1, dx2, dy2, dz2, dx3, dy3, dz3,
     &                1, tol, du, dv, dw, vlength, nerra)

        avectors    = aword(2)
        lavectors   = lword(2)

        avector(n)  = aword(2)
        lavector(n) = lword(2)

        xcom(n)     = du
        ycom(n)     = dv
        zcom(n)     = dw

        vlen(n)     = vlength

        xbnd(n)     = 0.0
        ybnd(n)     = 0.0
        zbnd(n)     = 0.0

c....   Print the vector specifications.

        call vout (n, n)

        go to 999

      endif                           ! Vector is vector triple product.

c=======================================================================********

c.... Input line:  vector VNAME sum VNAME1 VNAME2 [VMULT1|1 [VMULT2|1]]

 8450 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  There are not 5-7 words on "sum" command line:  ',i1)

      if ((aword(3) .eq. 'add')  .or.
     &    (aword(3) .eq. 'sum')) then

        if ((nword .lt. 5) .or. (nword .gt. 7)) then
          write (   6, 8450) ainput, ninput, nword
          write (iout, 8450) ainput, ninput, nword
          nerr = 1
          go to 500
        endif

c....   Find the first vector.

        call vectfind (4, nv)

        if (nv .eq. 0) then
          nerr = 1
          go to 475
        endif

        dx1 = xcom(nv)
        dy1 = ycom(nv)
        dz1 = zcom(nv)

c....   Find the second vector.

  475   call vectfind (5, nv)

        if (nv .eq. 0) then
          nerr = 1
          go to 480
        endif

        dx2 = xcom(nv)
        dy2 = ycom(nv)
        dz2 = zcom(nv)

c....   Find the multipliers.

  480   vmult1 = 1.0
        vmult2 = 1.0

 8485 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer or floating point value or variable:  ',a)

        if (nword .gt. 5) then

          call varfind (6, 6)

          if ((mtype(6) .ne. 2) .and. (mtype(6) .ne. 3)) then
            write (   6, 8485) ainput, ninput, aword(6)
            write (iout, 8485) ainput, ninput, aword(6)
            nerr = 1
            go to 500
          endif

          vmult1 = fword(6)

        endif

        if (nword .eq. 7) then

          call varfind (7, 7)

          if ((mtype(7) .ne. 2) .and. (mtype(7) .ne. 3)) then
            write (   6, 8485) ainput, ninput, aword(7)
            write (iout, 8485) ainput, ninput, aword(7)
            nerr = 1
            go to 500
          endif

          vmult2 = fword(7)

        endif

  500   if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nvectors) nvectors = nvectors - 1
          endif
          go to 999
        endif

c....   Find the weighted sum of the two vectors.

        call aptvsum (0, vmult1, dx1, dy1, dz1, vmult2, dx2, dy2, dz2,
     &                1, tol, du, dv, dw, vlength, nerra)

        avectors    = aword(2)
        lavectors   = lword(2)

        avector(n)  = aword(2)
        lavector(n) = lword(2)

        xcom(n)     = du
        ycom(n)     = dv
        zcom(n)     = dw

        vlen(n)     = vlength

        xbnd(n)     = 0.0
        ybnd(n)     = 0.0
        zbnd(n)     = 0.0

c....   Print the vector specifications.

        call vout (n, n)

        go to 999

      endif                           ! Vector weighted sum of two vectors.

c=======================================================================********

c.... Input line:  vector VNAME angle ANG1 VNAME1 ANG2 VNAME2

      if ((aword(3) .eq. 'angle') .or.
     &    (aword(3) .eq. 'ang'))  then

 8700 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Word not integer or floating point value or variable:  ',a)
 8710   format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Vector has zero length:  ',a)

c....   Find the first angle.

          call varfind (4, 4)

          if ((mtype(4) .ne. 2) .and. (mtype(4) .ne. 3)) then
            write (   6, 8700) ainput, ninput, aword(4)
            write (iout, 8700) ainput, ninput, aword(4)
            nerr = 1
          endif

          ang1 = fword(4)

c....     Find the first vector.

          call vectfind (5, nv1)
  
          if (nv1 .eq. 0) then
            nerr = 1
          elseif (vlen(nv1) .eq. 0.0) then
            write (   6, 8710) ainput, ninput, aword(5)
            write (iout, 8710) ainput, ninput, aword(5)
            nerr = 1
          endif

c....   Find the second angle.

          call varfind (6, 6)

          if ((mtype(6) .ne. 2) .and. (mtype(6) .ne. 3)) then
            write (   6, 8700) ainput, ninput, aword(6)
            write (iout, 8700) ainput, ninput, aword(6)
            nerr = 1
          endif

          ang2 = fword(6)

c....     Find the second vector.

          call vectfind (7, nv2)
  
          if (nv2 .eq. 0) then
            nerr = 1
          elseif (vlen(nv2) .eq. 0.0) then
            write (   6, 8710) ainput, ninput, aword(7)
            write (iout, 8710) ainput, ninput, aword(7)
            nerr = 1
          endif

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nvectors) nvectors = nvectors - 1
          endif
          go to 999
        endif

c....   Find the angle units.

        if (angunit .eq. 'degrees') then
          iunit = 0
        else
          iunit = 1
        endif

c....   Find the unit vector at angle ANG1 from vector VNAME1 and
c....     at angle ANG2 from vector VNAME2.

        call aptetru (xcom(nv1), ycom(nv1), zcom(nv1),
     &                xcom(nv2), ycom(nv2), zcom(nv2),
     &                iunit, ang1, ang2, tol, ux, uy, uz, nerra)

c....   Test for errors.

 8720 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  No solution when vectors are parallel:  ' / a / a )
 8730 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  No solution with these angles.  Too close to 0 or 180.')

        if (nerra .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nvectors) nvectors = nvectors - 1
          endif
          if     (nerra .eq. 1) then  ! Vector VNAME1 has zero length.
            write (   6, 8710) ainput, ninput, aword(5)
            write (iout, 8710) ainput, ninput, aword(5)
          elseif (nerra .eq. 2) then  ! Vector VNAME2 has zero length.
            write (   6, 8710) ainput, ninput, aword(7)
            write (iout, 8710) ainput, ninput, aword(7)
          elseif (nerra .eq. 3) then  ! Vectors VNAME1 and VNAME2 parallel.
            write (   6, 8720) ainput, ninput, aword(5), aword(7)
            write (iout, 8720) ainput, ninput, aword(5), aword(7)
          elseif (nerra .eq. 4) then  ! Angles ANG1 and ANG2 have no solution.
            write (   6, 8730) ainput, ninput
            write (iout, 8730) ainput, ninput
          endif
          go to 999
        endif

        avectors    = aword(2)
        lavectors   = lword(2)

        avector(n)  = aword(2)
        lavector(n) = lword(2)

        xcom(n)     = ux
        ycom(n)     = uy
        zcom(n)     = uz

        vlen(n)     = sqrt (xcom(n)**2 + ycom(n)**2 + zcom(n)**2)

        xbnd(n)     = 0.0
        ybnd(n)     = 0.0
        zbnd(n)     = 0.0

c....   Print the vector specifications.

        call vout (n, n)

        go to 999

      endif                           ! Vector at two specified angles.

c=======================================================================********

c.... Input line:  vector VNAME plane     PLNAME
c.... Input line:  vector VNAME sphere    SPHNAME
c.... Input line:  vector VNAME cylinder  CYLNAME
c.... Input line:  vector VNAME cone      CNNAME
c.... Input line:  vector VNAME hyperb    HPNAME
c.... Input line:  vector VNAME ellipsoid ELNAME
c.... Input line:  vector VNAME axisym    AXNAME
c.... Input line:  vector VNAME quadric   QNAME

c.... Find the quadric surface.

 8510 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named quadric surface does not exist:  ',a)

      if ((aword(3) .eq. 'quadric')   .or.
     &    (aword(3) .eq. 'quad')      .or.
     &    (aword(3) .eq. 'q')         .or.
     &    (aword(3) .eq. 'plane')     .or.
     &    (aword(3) .eq. 'pl')        .or.
     &    (aword(3) .eq. 'sphere')    .or.
     &    (aword(3) .eq. 'sph')       .or.
     &    (aword(3) .eq. 'cylinder')  .or.
     &    (aword(3) .eq. 'cyl')       .or.
     &    (aword(3) .eq. 'cone')      .or.
     &    (aword(3) .eq. 'hyperb')    .or.
     &    (aword(3) .eq. 'saddle')    .or.
     &    (aword(3) .eq. 'ellipsoid') .or.
     &    (aword(3) .eq. 'ellipse')   .or.
     &    (aword(3) .eq. 'ell')       .or.
     &    (aword(3) .eq. 'el')        .or.
     &    (aword(3) .eq. 'axisym'))   then

        do 510 nq = 1, nquads
          if (aword(4) .eq. aquad(nq)) then

            call qutype (aword(3), nq, 1, nerrq)

            if (nerrq .ne. 0) go to 999

            n1 = nq

            call quout (n1, n1, 'min', nerrq)

            go to 520
          endif
  510   continue
        write (   6, 8510) ainput, ninput, aword(4)
        write (iout, 8510) ainput, ninput, aword(4)
        nerr = 1

        if (nrep .eq. 0) then
          if (n .eq. nvectors) nvectors = nvectors - 1
        endif
        go to 999

c....   Find the central point of the quadric, and the vector to the origin.

  520   du = -xcen(n1)
        dv = -ycen(n1)
        dw = -zcen(n1)
        vlength = sqrt (du**2 + dv**2 + dw**2)

        avectors    = aword(2)
        lavectors   = lword(2)

        avector(n)  = aword(2)
        lavector(n) = lword(2)

        xcom(n)     = -xcen(n1)
        ycom(n)     = -ycen(n1)
        zcom(n)     = -zcen(n1)

        vlen(n)     = sqrt (xcom(n)**2 + ycom(n)**2 + zcom(n)**2)

        xbnd(n)     = xcen(n1)
        ybnd(n)     = ycen(n1)
        zbnd(n)     = zcen(n1)

c....   Print the vector specifications.

        call vout (n, n)

        go to 999

      endif                           ! Vector defined by plane or quadric.
 
c=======================================================================********

c.... Input line:  vector VNAME point PNAME1 PNAME2 [abs [ABSLEN|1]]
c.... Input line:  vector VNAME point PNAME1 PNAME2 [rel [RELLEN|1]]

c.... Find the points for this vector.

  530 if ((aword(3) .eq. 'point') .or.
     &    (aword(3) .eq. 'pnt')   .or.
     &    (aword(3) .eq. 'pt')    .or.
     &    (aword(3) .eq. 'p'))    then

        call ptfind (4, n1)

        if (n1 .eq. 0) then
          nerr = 1
          go to 550
        endif

        u = x(n1)
        v = y(n1)
        w = z(n1)

  550   call ptfind (5, n2)

        if (n2 .eq. 0) then
          nerr = 1
          go to 570
        endif

        call aptvdis (x(n1), y(n1), z(n1), x(n2), y(n2), z(n2), 1,
     &                tol, du, dv, dw, vlength, nerra)

        if (nerra .ne. 0) nerr = 1

c....   Find the length factors, if no errors.

  570   if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nvectors) nvectors = nvectors - 1
          endif
          go to 999
        endif

c....   Find the length factors.

        if (nword .eq. 5) then
          abslen  = 0.0
          rellen  = 1.0
          go to 600
        endif

 8580   format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word 6 not "rel" or "abs":  ',a)

        if (nword .ge. 6) then
          if ((aword(6) .eq. 'abs') .or.
     &        ((nword .eq. 6) .and. (aword(6) .eq. 'unit'))) then
            abslen = 1.0
            rellen = 0.0
          elseif ((aword(6) .eq. 'rel') .or.
     &            (aword(6) .eq. '*'))  then
            abslen  = 0.0
            rellen  = 1.0
          else
            write (   6, 8580) ainput, ninput, aword(6)
            write (iout, 8580) ainput, ninput, aword(6)
            nerr = 1
          endif
        endif

 8590   format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word 7 not integer or floating point value',
     &    ' or variable:  ',a)

        if (nword .eq. 7) then        ! Find length factor.

          call varfind (7, 7)

          if ((mtype(7) .ne. 2) .and. (mtype(7) .ne. 3)) then
            write (   6, 8590) ainput, ninput, aword(7)
            write (iout, 8590) ainput, ninput, aword(7)
            nerr = 1
          endif

          if (aword(6) .eq. 'abs') then
            abslen = fword(7)
            rellen = 0.0
          elseif ((aword(6) .eq. 'rel') .or.
     &            (aword(6) .eq. '*'))  then
            abslen = 0.0
            rellen = fword(7)
          endif

        endif                         ! Found length factor.

c....   Find the vector length.  Truncate small components.

  600   if (abslen .eq. 0.0) then     ! Vector type is relative.
          du      = rellen * du
          dv      = rellen * dv
          dw      = rellen * dw
          vlength = rellen * vlength
        else                          ! Vector type is absolute.
          if (vlength .eq. 0.0) then
            if (abslen .ne. 0.0) then
              write (   6, 8410) ainput, ninput, avector(n)
              write (iout, 8410) ainput, ninput, avector(n)
            endif
          else

            call aptvunz (du, dv, dw, tol, vlength)

            du      = abslen * du
            dv      = abslen * dv
            dw      = abslen * dw
            vlength = abslen
          endif
        endif                         ! Adjusted vector length.

        if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nvectors) nvectors = nvectors - 1
          endif
          go to 999
        endif

        avectors    = aword(2)
        lavectors   = lword(2)

        avector(n)  = aword(2)
        lavector(n) = lword(2)

        xcom(n)     = du
        ycom(n)     = dv
        zcom(n)     = dw

        vlen(n)     = vlength

        xbnd(n)     = u
        ybnd(n)     = v
        zbnd(n)     = w

c....   Print the vector specifications.

        call vout (n, n)

        go to 999

      endif                           ! Vector defined by points.

c=======================================================================********

c.... Input line:  vector VNAME normal PNAME QNAME

      if ((nword .eq. 5)           .and.
     &   ((aword(3) .eq. 'normal') .or.
     &    (aword(3) .eq. 'norm'))) then

c....   Find the point.

        call ptfind (4, n1)
  
        if (n1 .eq. 0) then
          nerr = 1
          go to 720
        endif

c....   Find the quadric surface.

        do 710 nq = 1, nquads
          if (aword(5) .eq. aquad(nq)) then

            n2 = nq

            call quout (n2, n2, 'min', nerrq)

            go to 720
          endif
  710   continue
        write (   6, 8510) ainput, ninput, aword(5)
        write (iout, 8510) ainput, ninput, aword(5)
        nerr = 1

  720   if (nerr .ne. 0) then
          if (nrep .eq. 0) then
            if (n .eq. nvectors) nvectors = nvectors - 1
          endif
          go to 999
        endif

c....   Find the normal vector of the family of quadric surfaces at the point.
 
        call aptqnor (x(n1), y(n1), z(n1), qc(n2),
     &                qx(n2), qy(n2), qz(n2), qxy(n2), qyz(n2), qzx(n2),
     &                qxx(n2), qyy(n2), qzz(n2), tol,
     &                sc, xcom(n), ycom(n), zcom(n), slen, nerra)

        avectors    = aword(2)
        lavectors   = lword(2)

        avector(n)  = aword(2)
        lavector(n) = lword(2)

        vlen(n)     = slen

        xbnd(n)     = x(n1)
        ybnd(n)     = y(n1)
        zbnd(n)     = z(n1)

        call vout (n, n)

        go to 999

      endif                           ! Vector normal to quadric surface.

c=======================================================================********

c.... Input line:  vector VNAME   DU DV DW [U|0] [V|0] [W|0]     ! DU numerical.
c....   Equal sign is optional, if DU is numerical.
c.... Input line:  vector VNAME = DU DV DW [U|0] [V|0] [W|0]

      if ((nword .eq. 5)      .and.
     &   ((mtype(3) .eq. 2)   .or.
     &    (mtype(3) .eq. 3))) then
        nword    = 6
        aword(6) = aword(5)
        mtype(6) = mtype(5)
        iword(6) = iword(5)
        fword(6) = fword(5)
        lword(6) = lword(5)
        aword(5) = aword(4)
        mtype(5) = mtype(4)
        iword(5) = iword(4)
        fword(5) = fword(4)
        lword(5) = lword(4)
        aword(4) = aword(3)
        mtype(4) = mtype(3)
        iword(4) = iword(3)
        fword(4) = fword(3)
        lword(4) = lword(3)
        aword(3) = '='
        mtype(3) = 1
        iword(3) = 0
        fword(3) = 0.0
        lword(3) = 1
      endif

      if ((nword .eq. 8)      .and.
     &   ((mtype(3) .eq. 2)   .or.
     &    (mtype(3) .eq. 3))) then
        nword    = 9
        aword(9) = aword(8)
        mtype(9) = mtype(8)
        iword(9) = iword(8)
        fword(9) = fword(8)
        lword(9) = lword(8)
        aword(8) = aword(7)
        mtype(8) = mtype(7)
        iword(8) = iword(7)
        fword(8) = fword(7)
        lword(8) = lword(7)
        aword(7) = aword(6)
        mtype(7) = mtype(6)
        iword(7) = iword(6)
        fword(7) = fword(6)
        lword(7) = lword(6)
        aword(6) = aword(5)
        mtype(6) = mtype(5)
        iword(6) = iword(5)
        fword(6) = fword(5)
        lword(6) = lword(5)
        aword(5) = aword(4)
        mtype(5) = mtype(4)
        iword(5) = iword(4)
        fword(5) = fword(4)
        lword(5) = lword(4)
        aword(4) = aword(3)
        mtype(4) = mtype(3)
        iword(4) = iword(3)
        fword(4) = fword(3)
        lword(4) = lword(3)
        aword(3) = '='
        mtype(3) = 1
        iword(3) = 0
        fword(3) = 0.0
        lword(3) = 1
      endif

      if (aword(3) .eq. '=') then

c....   Substitute the default values for missing words.

        do 620 nw = 4, 9
          if (nw .gt. nword) then
            mtype(nw) = 3
            fword(nw) = 0.0
          endif
  620   continue

        nword = 9

c....   Find the values of words 4-9.

 8630   format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &    '  Word not integer or floating point value or variable:  ',a)

        call varfind (4, 9)

        do 640 nw = 4, 9
          if ((mtype(nw) .ne. 2) .and. (mtype(nw) .ne. 3)) then
            write (   6, 8630) ainput, ninput, aword(nw)
            write (iout, 8630) ainput, ninput, aword(nw)
            go to 999
          endif
  640   continue

c....   Find the vector components, bound point, and length.

        du = fword(4)
        dv = fword(5)
        dw = fword(6)

        u  = fword(7)
        v  = fword(8)
        w  = fword(9)

c....   Convert units, if needed.

        iunit = 0
        if (angunit .eq. 'radians') iunit = 1
  
        if     (asys .eq. 'cyl') then
          call aptcsyv (1, 0, iunit, u, v, w, du, dv, dw, 1, tol,
     &                  nerra)
          if (nerra .ne. 0) nerr = 1
        elseif (asys .eq. 'sph') then
          call aptcsyv (2, 0, iunit, u, v, w, du, dv, dw, 1, tol,
     &                  nerra)
          if (nerra .ne. 0) nerr = 1
        endif

        avectors    = aword(2)
        lavectors   = lword(2)

        avector(n)  = aword(2)
        lavector(n) = lword(2)

        xcom(n)     = du
        ycom(n)     = dv
        zcom(n)     = dw

        vlen(n)     = sqrt (du**2 + dv**2 + dw**2)

        xbnd(n)     = u
        ybnd(n)     = v
        zbnd(n)     = w

c....   Print the vector specifications.

        call vout (n, n)

        go to 999

      endif                           ! Vector defined by components.

c=======================================================================********

 8810 format ('  vector ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Input line is not recognized.')

      write (   6, 8810) ainput, ninput
      write (iout, 8810) ainput, ninput

c=======================================================================********

  999 return

c.... End of subroutine vector.       (+1 line)
      end
