      subroutine znmake (akey)

c*s
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Version:  znmake   Updated     2004 November 23 13:30.
c               znmake   Originated  1994 September 8 14:00.
c
c     Author:   Arthur L. Edwards, L-298, x-24123.
c
c     Purpose:  To create zones, bounded by quadric surfaces in 3-D space.
c
c     Logic:    Entry points:  znmake.
c znmake        Called by:  zone.
c
c znmake        Calls:  quout, znstore, zout.
c
c     Biblio:   ?
c
c     Glossary:
c
c     History:
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c/c

      include 'store.h'

c.... Local variables.

      common /llset/ nzquadx          ! Number of quadric surfaces around zone.
      common /llset/ lazquadx(nzquadm)  ! Number of characters in azquadx.
      common /llset/ azquadx(nzquadm) ! Name of a quadric surface.
      character*24   azquadx          ! Name of a quadric surface.
      common /llset/ azsignx(nzquadm) ! Exit side of a quadric surface (+ or -).
      character*1    azsignx          ! Exit side of a quadric surface (+ or -).

      character*8    amount           ! Amount of output data.
      character*8    akey             ! Key word option in command "zone".

c=======================================================================********

c.... Find the index in the zone array at which to store this zone.
c....   Set nrep = 1 if replacing an existing zone.

      call znstore (nrep, n)

      if (n .eq. 0) go to 999

c=======================================================================********

c.... Input line:  zone ZNAME = J1 Q1 J2 Q2 J3 Q3 ...
c.... Input line:  zone ZNAME & J1 Q1 J2 Q2 J3 Q3 ...

c.... Find the signs and quadric surfaces.

 8130 format ('  znmake ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named zone does not exist:  ',a)
 8200 format ('  znmake ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Extra + or - on input line.  Word ',i4)
 8210 format ('  znmake ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Named quadric surface does not exist:  ',a)
 8220 format ('  znmake ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Quadric surface is bad:  ',a)
 8225 format ('  znmake ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  one has no quadric boundaries.')
 8230 format ('  znmake ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Number of bounding quadric surfaces exceeds',i3)
 8240 format ('  znmake ERROR.  Reading ',a40,' line ',i5,'.' /
     &  '  Not enough words on input line:  ',i3)
c***DEBUG begins.
 9901 format ('znmake DEBUG.  nbug= ',i3,'  nzquadx=',i3,
     &        '  nzquads=',i3,'  lazquadx=',i3)
      nbug = 1
      write (   6, 9901) nbug, nzquadx, nzquads(n), lazquadx(nzquadx)
c***DEBUG ends.

      nerr = 0

      if (aword(3) .eq. '=') then

        nzquadx = 0
        nzfirst = 1
c***DEBUG begins.
      nbug = 2
      write (   6, 9901) nbug, nzquadx, nzquads(n), lazquadx(nzquadx)
c***DEBUG ends.

      elseif (aword(3) .eq. '&') then
c***DEBUG begins.
      nbug = 3
      write (   6, 9901) nbug, nzquadx, nzquads(n), lazquadx(nzquadx)
c***DEBUG ends.

        if (azone(n) .eq. ' ') then
          write (   6, 8130) ainput, ninput, aword(2)
          write (iout, 8130) ainput, ninput, aword(2)
          nerr = 1
          go to 240
        endif
        nzquadx = nzquads(n)
        nzfirst = nzquads(n) + 1
c***DEBUG begins.
      nbug = 4
      write (   6, 9901) nbug, nzquadx, nzquads(n), lazquadx(nzquadx)
c***DEBUG ends.

      endif
c***DEBUG begins.
      nbug = 5
      write (   6, 9901) nbug, nzquadx, nzquads(n), lazquadx(nzquadx)
c***DEBUG ends.

      if (nword .lt. 4) then
        write (   6, 8240) ainput, ninput, nword
        write (iout, 8240) ainput, ninput, nword
        go to 999
      endif
c***DEBUG begins.
      nbug = 6
      write (   6, 9901) nbug, nzquadx, nzquads(n), lazquadx(nzquadx)
c***DEBUG ends.

      nw = 4

  210 nzquadx = nzquadx + 1
c***DEBUG begins.
      nbug = 7
      write (   6, 9901) nbug, nzquadx, nzquads(n), lazquadx(nzquadx)
c***DEBUG ends.

      if (nzquadx .gt. nzquadm) then
        write (   6, 8230) ainput, ninput, nzquadm
        write (iout, 8230) ainput, ninput, nzquadm
        nerr = 1
        go to 240
      endif
c***DEBUG begins.
      nbug = 8
      write (   6, 9901) nbug, nzquadx, nzquads(n), lazquadx(nzquadx)
c***DEBUG ends.

      azsignx(nzquadx) = '+'

      if ((aword(nw) .eq. '+')  .or.
     &    (aword(nw) .eq. '-')) then
c***DEBUG begins.
      nbug = 9
      write (   6, 9901) nbug, nzquadx, nzquads(n), lazquadx(nzquadx)
c***DEBUG ends.

        azsignx(nzquadx) = aword(nw)

        if ((nw .eq. nword) .or.
     &      (aword(nw+1) .eq. '+')  .or.
     &      (aword(nw+1) .eq. '-')) then
          write (   6, 8200) ainput, ninput, nword
          write (iout, 8200) ainput, ninput, nword
          nerr = 1
          go to 230
        endif
c***DEBUG begins.
      nbug = 10
      write (   6, 9901) nbug, nzquadx, nzquads(n), lazquadx(nzquadx)
c***DEBUG ends.

        nw = nw + 1

      endif                           ! Sign is specified.

      do 220 nq = 1, nquads
        if (aword(nw) .eq. aquad(nq)) then

c***DEBUG begins.
 9902 format ('znmake DEBUG.  nbug= ',i3,'  laquad=',i3)
      nbug = 11
      write (   6, 9901) nbug, nzquadx, nzquads(n), lazquadx(nzquadx)
      write (   6, 9902) laquad(nq)
c***DEBUG ends.
          azquadx(nzquadx)  = aquad(nq)
          lazquadx(nzquadx) = laquad(nq)

          if ((akey .ne. 'family') .and.
     &        (akey .ne. 'fam'))   then
            call quout (nq, nq, 'min', nerrq)
          endif

          if (nerrq .ne. 0) then
c***DEBUG begins.
 8702 format ('znmake DEBUG nbug=',i3 )
      nbug = 1
      write (   6, 8702) nbug
      write (iout, 8702) nbug
c***DEBUG ends.
            write (   6, 8220) ainput, ninput, aquad(nq)
            write (iout, 8220) ainput, ninput, aquad(nq)
            nerr = 1
          endif
c***DEBUG begins.
      nbug = 12
      write (   6, 9901) nbug, nzquadx, nzquads(n), lazquadx(nzquadx)
      write (   6, 9902) nbug, laquad(nq)
c***DEBUG ends.

          go to 230
        endif
  220 continue
      write (   6, 8210) ainput, ninput, aword(nw)
      write (iout, 8210) ainput, ninput, aword(nw)
      nerr = 1

  230 nw = nw + 1
c***DEBUG begins.
      nbug = 13
      write (   6, 9901) nbug, nzquadx, nzquads(n), lazquadx(nzquadx)
c***DEBUG ends.

      if (nw .le. nword) go to 210
c***DEBUG begins.
      nbug = 14
      write (   6, 9901) nbug, nzquadx, nzquads(n), lazquadx(nzquadx)
c***DEBUG ends.

      if (nzquadx .le. 0) then
        write (   6, 8225) ainput, ninput
        write (iout, 8225) ainput, ninput
        nerr = 1
        go to 240
      endif
c***DEBUG begins.
      nbug = 15
      write (   6, 9901) nbug, nzquadx, nzquads(n), lazquadx(nzquadx)
c***DEBUG ends.

  240 if (nerr .ne. 0) then
        if (nrep .eq. 0) then
          if (n .eq. nzones) nzones = nzones - 1
        endif
        go to 999
      endif

c=======================================================================********

c.... Store the zone specifications.

c***DEBUG begins.
      nbug = 16
      write (   6, 9901) nbug, nzquadx, nzquads(n), lazquadx(nzquadx)
c***DEBUG ends.
      azone(n)   = aword(2)
      azones     = aword(2)
      lazones    = lword(2)
      nzquads(n) = nzquadx
c***DEBUG begins.
      nbug = 17
      write (   6, 9901) nbug, nzquadx, nzquads(n), lazquadx(nzquadx)
c***DEBUG ends.

      do 250 nq = nzfirst, nzquadx
        azquad(nq,n)  = azquadx(nq)
        lazquad(nq,n) = lazquadx(nq)
        azsign(nq,n)  = azsignx(nq)
  250 continue
c***DEBUG begins.
      nbug = 18
      write (   6, 9901) nbug, nzquadx, nzquads(n), lazquadx(nzquadx)
c***DEBUG ends.

      volz(n) = 0.0                   ! May be specified or calculated later.

c.... Print the zone specifications.

      amount = 'minimum'
      call zout (n, n, amount)
c***DEBUG begins.
      nbug = 18
      write (   6, 9901) nbug, nzquadx, nzquads(n), lazquadx(nzquadx)
c***DEBUG ends.

c=======================================================================********

  999 return

c.... End of subroutine znmake        (+1 line)
      end
